﻿using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using System;

namespace GAFNETBlazor.Constantes
{
    public static class URIsFaturamentos
    {
        static string UriBase => "https://www.faturamentos.gafnetapi.com.br/api/";
        //static string UriBase => "http://localhost:57038/api/";
        public static Uri PostAsync => new(UriBase + "faturamentos/post");
        public static Uri PostPedidoAsync => new(UriBase + "pedidos/post");
        public static Uri PostOrdemDeServicoAsync => new(UriBase + "ordensdeservico/post");
        public static Uri PostOrcamentoAsync => new(UriBase + "orcamentos/post");
        public static Uri PostCompraAsync => new(UriBase + "compras/post");
        public static Uri PostPedidoDeCompraAsync => new(UriBase + "pedidosdecompra/post");
        public static Uri PostNotaFiscalAsync => new(UriBase + "notasfiscais/post");
        public static Uri PostDevolucaoAsync => new(UriBase + "devolucoes/post");
        public static Uri PutAsync => new(UriBase + "faturamentos/put");
        public static Uri PutPedidoAsync => new(UriBase + "pedidos/put");
        public static Uri PutOrdemDeServicoAsync => new(UriBase + "ordensdeservico/put");
        public static Uri PutOrcamentoAsync => new(UriBase + "orcamentos/put");
        public static Uri PutCompraAsync => new(UriBase + "compras/put");
        public static Uri PutPedidoDeCompraAsync => new(UriBase + "pedidosdecompra/put");
        public static Uri GetAsync => new(UriBase + "faturamentos/get");
        public static Uri GetPedidosAsync => new(UriBase + "pedidos/get");
        public static Uri GetOrcamentosAsync => new(UriBase + "orcamentos/get");
        public static Uri GetComprasAsync => new(UriBase + "compras/get");
        public static Uri GetPedidosDeCompraAsync => new(UriBase + "pedidosdecompra/get");
        public static Uri GetNotasFiscaisAsync => new(UriBase + "notasfiscais/get");
        public static Uri GetOrdensDeServicoAsync => new(UriBase + "ordensdeservico/get");
        public static Uri GetProdutosAsync => new(UriBase + "produtos/get");
        public static Uri GetPessoasAsync => new(UriBase + "pessoas/get");
        public static Uri GetHistoricosAsync => new(UriBase + "historicos/get");
        public static Uri GetFormulariosAsync => new(UriBase + "formularios/get");
        public static Uri GetPlanosDeContaAsync => new(UriBase + "planosdeconta/get");
        public static Uri GetCentrosDeCustoAsync => new(UriBase + "centrosdecusto/get");
        public static Uri GetCondicoesDePagamentoAsync => new(UriBase + "condicoesdepagamento/get");
        public static Uri GetTiposDeOperacaoAsync => new(UriBase + "tiposdeoperacao/get");
        public static Uri PostCalculoFaturamentoAsync => new(UriBase + "calculofaturamento/post");
        public static Uri ImportacaoXMLCompra => new(UriBase + "importacaoXML/post");
        public static Uri LeituraXMLCompra => new(UriBase + "leituraXML/post");
        public static Uri PostCompraImportadaAsync => new(UriBase + "importacaocompra/post");
        public static Uri DuplicacaoPedidoAsync => new(UriBase + "duplicacaopedidos/post");
        public static Uri CancelamentoPedidoAsync => new(UriBase + "cancelamentopedidos/post");
        public static Uri CancelamentoNotaFiscalAsync => new(UriBase + "cancelamentonotasfiscais/post");
        public static Uri InutilizacaoNotaFiscalAsync => new(UriBase + "inutilizacaonotasfiscais/post");
        public static Uri DuplicacaoOSAsync => new(UriBase + "duplicacaooss/post");
        public static Uri CancelamentoOSAsync => new(UriBase + "cancelamentooss/post");
        public static Uri DuplicacaoOrcamentoAsync => new(UriBase + "duplicacaoorcamentos/post");
        public static Uri CancelamentoOrcamentoAsync => new(UriBase + "cancelamentoorcamentos/post");
        public static Uri PostRetornoNFeAsync => new(UriBase + "retornosnfe/post");
        public static Uri OcorrenciasNotaFiscalAsync => new(UriBase + "ocorrenciasnotafiscal/get");
        public static Uri PostEstornoNFeAsync => new(UriBase + "estornos/post");

        public static Uri UriOcorrencias(TipoFaturamento tipoFaturamento)
        {
            return tipoFaturamento switch
            {
                TipoFaturamento.PedidoDeVenda => throw new NotImplementedException(),
                TipoFaturamento.OrdemDeServico => throw new NotImplementedException(),
                TipoFaturamento.Orcamento => throw new NotImplementedException(),
                TipoFaturamento.NotaFiscal => OcorrenciasNotaFiscalAsync,
                TipoFaturamento.NotaCompra => throw new NotImplementedException(),
                TipoFaturamento.PedidoDeCompra => throw new NotImplementedException(),
                _ => throw new NotImplementedException()
            };
        }

        public static Uri UriCancelamento(TipoFaturamento tipoFaturamento)
        {
            return tipoFaturamento switch
            {
                TipoFaturamento.PedidoDeVenda => CancelamentoPedidoAsync,
                TipoFaturamento.OrdemDeServico => CancelamentoOSAsync,
                TipoFaturamento.Orcamento => CancelamentoOrcamentoAsync,
                TipoFaturamento.NotaFiscal => CancelamentoNotaFiscalAsync,
                TipoFaturamento.NotaCompra => throw new NotImplementedException(),
                TipoFaturamento.PedidoDeCompra => throw new NotImplementedException(),
                _ => throw new NotImplementedException()
            };
        }

        public static Uri UriDuplicacao(TipoFaturamento tipoFaturamento)
        {
            return tipoFaturamento switch
            {
                TipoFaturamento.PedidoDeVenda => DuplicacaoPedidoAsync,
                TipoFaturamento.OrdemDeServico => DuplicacaoOSAsync,
                TipoFaturamento.Orcamento => DuplicacaoOrcamentoAsync,
                TipoFaturamento.NotaFiscal => throw new NotImplementedException(),
                TipoFaturamento.NotaCompra => throw new NotImplementedException(),
                TipoFaturamento.PedidoDeCompra => throw new NotImplementedException(),
                _ => throw new NotImplementedException()
            };
        }

        public static Uri UriGet(TipoFaturamento tipoFaturamento)
        {
            return tipoFaturamento switch
            {
                TipoFaturamento.PedidoDeVenda => GetPedidosAsync,
                TipoFaturamento.OrdemDeServico => GetOrdensDeServicoAsync,
                TipoFaturamento.Orcamento => GetOrcamentosAsync,
                TipoFaturamento.NotaFiscal => GetNotasFiscaisAsync,
                TipoFaturamento.NotaCompra => GetComprasAsync,
                TipoFaturamento.PedidoDeCompra => GetPedidosDeCompraAsync,
                _ => throw new NotImplementedException()
            };
        }

        public static Uri UriPut(TipoFaturamento tipoFaturamento)
        {
            return tipoFaturamento switch
            {
                TipoFaturamento.PedidoDeVenda => PutPedidoAsync,
                TipoFaturamento.OrdemDeServico => PutOrdemDeServicoAsync,
                TipoFaturamento.Orcamento => PutOrcamentoAsync,
                TipoFaturamento.NotaFiscal => throw new NotImplementedException(),
                TipoFaturamento.NotaCompra => PutCompraAsync,
                TipoFaturamento.PedidoDeCompra => PutPedidoDeCompraAsync,
                _ => throw new NotImplementedException()
            };
        }

        public static Uri UriPost(FaturamentoModel faturamento)
        {
            var tipoFaturamento = (TipoFaturamento)faturamento.Tipo;

            return tipoFaturamento switch
            {
                TipoFaturamento.PedidoDeVenda => PostPedidoAsync,
                TipoFaturamento.OrdemDeServico => PostOrdemDeServicoAsync,
                TipoFaturamento.Orcamento => PostOrcamentoAsync,
                TipoFaturamento.NotaFiscal => PostNotaFiscalAsync,
                TipoFaturamento.NotaCompra => PostCompraAsync,
                TipoFaturamento.PedidoDeCompra => throw new NotImplementedException(),
                _ => throw new NotImplementedException()
            };
        }

    }
}
