﻿using BoldReports.Web.ReportViewer;
using GAFNETBlazor.Services.Interfaces;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Caching.Memory;
using Newtonsoft.Json;
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;

namespace GAFNETBlazor.Controllers
{
    [Route("api/{controller}/{action}/{id?}")]
    [ApiController]
    public class ReportViewerController : ControllerBase, IReportController
    {
        private readonly IMemoryCache _cache;
        private readonly IWebHostEnvironment _hostEnvironment;
        private readonly IDadosRelatService _dadosService;

        public static List<EmpresaModel> EmpresasUsuario { get; } = new List<EmpresaModel>();
        public static string Token { get; set; }

        public static EmpresaModel EmpresaLogada { get; set; }

        public SampleData customDatas = new();

        public class SampleData
        {
            public Uri UriBase { get; set; }
            public string Campos { get; set; }
            public string Ordenacao { get; set; }
            public string QtdRegistros { get; set; }
            public string Filtros { get; set; }
            public ServicoDados ServicoDados { get; set; }
        }

        private async Task<RespostaAPIModel> GetDados()
        {
            var data = await _dadosService.Dados(customDatas.ServicoDados, customDatas.Campos, customDatas.Filtros, customDatas.Ordenacao, customDatas.QtdRegistros);
            return data;
        }

        public ReportViewerController(IMemoryCache cache, IWebHostEnvironment hostEnvironment, IDadosRelatService dadosService)
        {
            _cache = cache;
            _hostEnvironment = hostEnvironment;
            _dadosService = dadosService;
        }

        [ActionName("GetResource")]
        [AcceptVerbs("GET")]
        public object GetResource(ReportResource resource)
        {
            return ReportHelper.GetResource(resource, this, _cache);
        }

        public void OnInitReportOptions(ReportViewerOptions reportOption)
        {
            reportOption.ReportModel.EnableVirtualEvaluation = true;
            reportOption.ReportModel.DisablePageSplitting = true;
            string basePath = _hostEnvironment.WebRootPath;
            reportOption.ReportModel.ProcessingMode = ProcessingMode.Local;
            FileStream inputStream = new(basePath + @"\Resources\Reports\" + reportOption.ReportModel.ReportPath, FileMode.Open, FileAccess.Read);
            MemoryStream reportStream = new();
            inputStream.CopyTo(reportStream);
            reportStream.Position = 0;
            inputStream.Close();
            reportOption.ReportModel.Stream = reportStream;
            var respostaAPI = GetDados().Result;
            var dados = (IList)respostaAPI.Objeto;
            reportOption.ReportModel.DataSources.Add(new BoldReports.Web.ReportDataSource { Name = "Dados", Value = dados });
        }

        public void OnReportLoaded(ReportViewerOptions reportOption)
        {

        }

        public object PostFormReportAction()
        {
            return ReportHelper.ProcessReport(null, this, _cache);
        }

        public object PostReportAction(Dictionary<string, object> jsonResult)
        {
            if (jsonResult.ContainsKey("customData"))
            {

                customDatas = JsonConvert.DeserializeObject<SampleData>(jsonResult["customData"].ToString());

            }

            return ReportHelper.ProcessReport(jsonResult, this, _cache);
        }
    }
}
