﻿using GAFNETBlazor.Services;
using GAFNETLibrary.Models;
using System;

namespace GAFNETBlazor.Models
{
    public class EnderecoPessoaModel : IEquatable<EnderecoPessoaModel>
    {
        public EnderecoPessoaModel()
        {
            Cidade = new CidadeModel();
        }

        public long Id { get; set; }

        public long IdPessoa { get; set; }

        public long IdPais { get; set; }

        public long IdCidade { get; set; }

        public string Logradouro { get; set; }

        public string Numero { get; set; }

        public string Bairro { get; set; }

        public string CEP { get; set; }

        public string CNPJ { get; set; }
        public string CNPJFormatado => UtilitarioService.CNPJCPFFormatado(CNPJ);

        public bool Ativo { get; set; }

        public long? IdGAF { get; set; }

        public string Complemento { get; set; }

        public bool EhPrincipal { get; set; }

        public bool EhEntrega { get; set; }

        public bool EhCobranca { get; set; }

        public string DescEhPrincipal => EhPrincipal ? "Sim" : "Não";
        public string DescEhEntrega => EhEntrega ? "Sim" : "Não";
        public string DescEhCobranca => EhCobranca ? "Sim" : "Não";
        public PessoaModel Pessoa { get; set; }

        public PaisModel Pais { get; set; }

        public CidadeModel Cidade { get; set; }

        public bool EnderecoCompleto()
        {
            return !string.IsNullOrEmpty(CEP) &&
                CEP.Length == 8 &&
                !string.IsNullOrEmpty(Logradouro) &&
                !string.IsNullOrEmpty(Numero) &&
                !string.IsNullOrEmpty(CNPJ) &&
                Cidade?.Id > 0 &&
                Pais?.Id > 0;
        }

        public override bool Equals(object obj)
        {
            return Equals(obj as EnderecoPessoaModel);
        }

        public bool Equals(EnderecoPessoaModel other)
        {
            return other != null &&
                   IdPais == other.IdPais &&
                   IdCidade == other.IdCidade &&
                   Logradouro == other.Logradouro &&
                   Numero == other.Numero &&
                   Bairro == other.Bairro &&
                   CEP == other.CEP &&
                   Complemento == other.Complemento &&
                   CNPJ == other.CNPJ;
        }

        public override int GetHashCode()
        {
            return HashCode.Combine(IdPais, IdCidade, Logradouro, Numero, Bairro, CEP, Complemento, CNPJ);
        }
    }
}
