﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETLibrary.Enums;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;

namespace GAFNETBlazor.Models
{
    public partial class FaturamentoModel : ObservableObject
    {
        public long Id { get; set; }
        public int Tipo { get; set; }
        public int Numero { get; set; }
        public int Serie { get; set; }
        public string NPedidoDeCompra { get; set; }
        public string NNotaDeEmpenho { get; set; }
        public string NContratoDeCompra { get; set; }
        public string Observacoes { get; set; }
        //[DisplayFormat(DataFormatString = "dd/MM/yyyy HH:mm")]
        public DateTime DtHrLancamento { get; set; }
        public DateTime? DtCancelamento { get; set; }
        public DateTime? DtEstorno { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal Valor { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrFrete { get; set; }
        public DateTime? DtSaida { get; set; }
        public DateTime? DtHoraEnvio { get; set; }
        public DateTime? DtRecebimento { get; set; }
        public int TipoDeFrete { get; set; } = (int)TipoFrete.SemOcorrenciaFrete;
        public bool GeradoPorContrato { get; set; }
        public bool GeraFinanceiro { get; set; }
        public bool GeraEstoque { get; set; }
        public bool AtualizaCusto { get; set; }
        public int EntradaSaida { get; set; }
        public TipoNFHistorico TipoNF { get; set; }
        public FinNFeHistorico FinNFe { get; set; }
        public string Placa { get; set; }
        public string EspecieVolume { get; set; }
        public decimal QtdVolume { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrOutDespAcess { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrSeguro { get; set; }
        public string UFVeiculo { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBaseRetIRF { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBaseRetINSS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBaseRetISS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBaseRetCSLL { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBaseRetCOFINS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBaseRetPIS { get; set; }
        public decimal PercRetIRF { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal PercRetINSS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal PercRetISS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal PercRetCSLL { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal PercRetCOFINS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal PercRetPIS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrICMS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrIPI { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrDesconto { get; set; }
        public decimal PesoBruto { get; set; }
        public decimal PesoLiquido { get; set; }
        public string ChaveEletronica { get; set; }
        public bool Denegada { get; set; }
        public bool OperacaoPresencial { get; set; }
        public DateTime? DtInutilizacao { get; set; }
        public string MotivoInutilizacao { get; set; }
        public string MotivoCancelamento { get; set; }
        public string MotivoContingencia { get; set; }
        public string MotivoEstorno { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrICMSST { get; set; }

        public string DescricaoTipoFaturamento => (TipoFaturamento)Tipo switch
        {
            TipoFaturamento.PedidoDeVenda => "Pedido de Venda",
            TipoFaturamento.OrdemDeServico => "Ordem de Serviço",
            TipoFaturamento.Orcamento => "Orçamento",
            TipoFaturamento.NotaFiscal => "Nota Fiscal",
            TipoFaturamento.NotaCompra => "Nota de Compra",
            TipoFaturamento.PedidoDeCompra => "Pedido de Compra",
            _ => throw new NotImplementedException()
        };

        public long? IdLancamentoOrigem { get; set; }
        public long? IdHistorico { get; set; }
        public long IdPessoa { get; set; }
        public long? IdCondicaoDePagamento { get; set; }
        public long? IdPlanoDeConta { get; set; }
        public long? IdCentroDeCusto { get; set; }
        public long? IdTipoDeOperacao { get; set; }
        public long? IdFormulario { get; set; }
        public long? IdGAF { get; set; }
        public long? IdTransportadora { get; set; }
        public long IdEndereco { get; set; }
        public long? IdEnderecoEntrega { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBasePIS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBaseCOFINS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrPIS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrCOFINS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBaseICMS { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBaseICMSST { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBaseICMSParaDIFA { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrFundoPobreza { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBaseFundoPobreza { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrICMSFundoPobreza { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrBaseIPI { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrICMSInterestUFDestino { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal DIFA { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrTribNacionalIBPT { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrTribEstadualIBPT { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrTribMunicipalIBPT { get; set; }
        [DisplayFormat(DataFormatString = "#,##0.00")]
        public decimal VlrTribImportadosIBPT { get; set; }
        public CentroDeCustoModel CentroDeCusto { get; set; } = new CentroDeCustoModel();
        public CondicaoDePagamentoModel CondicaoDePagamento { get; set; } = new CondicaoDePagamentoModel();
        public FormularioModel Formulario { get; set; } = new FormularioModel();
        [ObservableProperty]
        public HistoricoModel historico;
        
        public FaturamentoModel()
        {
            Historico = new();
        }
        partial void OnHistoricoChanged(HistoricoModel value)
        {
            if (value == null) return;
            if (!value.GeraFinanceiro)
            {
                LimparDadosFinanceiros();
            }
        }

        public PessoaModel Pessoa { get; set; } = new PessoaModel();
        public PlanoDeContaModel PlanoDeConta { get; set; } = new PlanoDeContaModel();
        public TipoDeOperacaoModel TipoDeOperacao { get; set; } = new TipoDeOperacaoModel();
        public PessoaModel Transportadora { get; set; } = new PessoaModel();
        public EnderecoPessoaModel Endereco { get; set; }
        public EnderecoPessoaModel EnderecoEntrega { get; set; }
        public ObservableCollection<ItemFaturamentoModel> Itens { get; set; } = new ObservableCollection<ItemFaturamentoModel>();
        public ICollection<FaturamentoModel> Referencias { get; set; }
        public FaturamentoModel LancamentoOrigem { get; set; }
        public ObservableCollection<VendedorFaturamentoModel> Vendedores { get; set; }
        public ICollection<RetornoNFeModel> RetornosNFe { get; set; }
        public string DescricaoStatusNFe => RetornosNFe == null || !RetornosNFe.Any(r => r.CStat == 100) ? "" : "Autorizado";
        public bool Faturado => Itens.Any(i => i.QtdFaturada > 0);
        public bool Devolvido => Itens.Any(i => i.QtdDevolvida > 0);
        public bool TudoDevolvidoRetornado => Itens.Sum(i => i.QtdLancada - i.QtdDevolvida) == 0;
        public bool Retornado => Itens.Any(i => i.QtdRetornada > 0);
        public bool UsaFormulario
        {
            get
            {
                List<TipoFaturamento> tiposComFormulario = new() { TipoFaturamento.PedidoDeVenda, TipoFaturamento.OrdemDeServico, TipoFaturamento.Orcamento, TipoFaturamento.NotaFiscal };
                return tiposComFormulario.Contains((TipoFaturamento)Tipo);
            }
        }
        public bool UsaHistorico => Tipo != (int)TipoFaturamento.PedidoDeCompra;
        public bool UsaPlanoDeConta => Tipo != (int)TipoFaturamento.PedidoDeCompra;
        public bool DocumentoFaturavel
        {
            get
            {
                var tipoFaturamento = (TipoFaturamento)Tipo;

                if (tipoFaturamento == TipoFaturamento.PedidoDeCompra ||
                    tipoFaturamento == TipoFaturamento.NotaCompra ||
                    tipoFaturamento == TipoFaturamento.NotaFiscal ||
                    tipoFaturamento == TipoFaturamento.OrdemDeServico ||
                    tipoFaturamento == TipoFaturamento.PedidoDeCompra)
                {
                    return false;
                }
                return true;
            }
        }
        public string DtHrLancamentoFormatado => $"{DtHrLancamento.ToShortDateString()} {DtHrLancamento.ToShortTimeString()}";
        public bool PodeInserirItens => Tipo != (int)TipoFaturamento.NotaFiscal && Tipo != (int)TipoFaturamento.NotaFiscal && !Itens.Any(i => i.QtdFaturada > 0) && !DtCancelamento.HasValue;
        public ObservableCollection<ProdutoFornecedorModel> ProdutosFornecedor { get; set; }

        public decimal VlrTotProdutos => Itens != null ? Itens.Sum(i => i.VlrTotLiquido) : 0;
        public decimal VlrTotTribIBPT => VlrTribEstadualIBPT + VlrTribMunicipalIBPT + VlrTribNacionalIBPT + VlrTribImportadosIBPT;

        [NotMapped]
        public decimal VlrISQN { get; set; }

        public void ZerarCamposTotalizados()
        {
            VlrICMSST = 0;
            VlrBaseIPI = 0;
            VlrIPI = 0;
            VlrDesconto = 0;
            VlrBaseFundoPobreza = 0;
            VlrICMSFundoPobreza = 0;
            VlrFundoPobreza = 0;
            VlrBaseCOFINS = 0;
            VlrCOFINS = 0;
            VlrBasePIS = 0;
            VlrPIS = 0;
            VlrBaseICMS = 0;
            VlrICMS = 0;
            VlrBaseICMSST = 0;
            VlrICMSST = 0;
            VlrBaseICMSParaDIFA = 0;
            VlrICMSInterestUFDestino = 0;
            DIFA = 0;
            Valor = 0;
            VlrTribNacionalIBPT = 0;
            VlrTribEstadualIBPT = 0;
            VlrTribMunicipalIBPT = 0;
            VlrTribImportadosIBPT = 0;
            VlrISQN = 0;
        }

        public void LimparDadosFinanceiros()
        {
            CondicaoDePagamento = new CondicaoDePagamentoModel();
            TipoDeOperacao = new TipoDeOperacaoModel();
            PlanoDeConta = new PlanoDeContaModel();
            CentroDeCusto = new CentroDeCustoModel();
            IdTipoDeOperacao = null;
            IdCondicaoDePagamento = null;
            IdCentroDeCusto = null;
            IdPlanoDeConta = null;
        }

    }
}
