﻿using GAFNETLibrary.Enums;

namespace GAFNETBlazor.Models
{
    public class HistoricoModel
    {
        public long Id { get; set; }

        public string Descricao { get; set; }

        public int EntradaSaida { get; set; }

        public int? CdHistoricoFiscal { get; set; }

        public int? CFOPDentroUF { get; set; }

        public string DescricaoNF { get; set; }

        public int? CFOPForaUF { get; set; }

        public string DescCFOPForaUF { get; set; }

        public int? CFOPDentroUFST60 { get; set; }
        public int? CFOPForaUFST60 { get; set; }

        public int? CdHistoricoFiscalST60 { get; set; }

        public string EspecieDocto { get; set; }

        public string EspecieNF { get; set; }

        public int? CFOPDentroUFServ { get; set; }

        public string DescCFOPDentroUFServ { get; set; }

        public int? CFOPForaUFServ { get; set; }

        public string DescCFOPForaUFServ { get; set; }

        public string ObsNFDentroUF { get; set; }

        public string ObsNFForaUF { get; set; }

        public int? CdHistoricoFiscalServ { get; set; }

        public int TipoNF { get; set; } = (int)TipoNFHistorico.Normal;
        public int FinNFe { get; set; } = (int)FinNFeHistorico.Normal;

        public int? CdHistoricoFiscalST10 { get; set; }

        public int? CFOPDentroUFST10 { get; set; }

        public int? CFOPForaUFST10 { get; set; }

        public int? TabelaA { get; set; }

        public int? SituacaoTributaria { get; set; }

        public int? CSTPISCOFINS { get; set; }

        public int? CSTIPI { get; set; }

        public int? CFOPProdDentroUF { get; set; }

        public int? CFOPProdForaUF { get; set; }

        public decimal PercPIS { get; set; }

        public decimal PercCOFINS { get; set; }

        public decimal PercIPI { get; set; }

        public int? CFOPProdDentroUFST10 { get; set; }

        public int? CFOPProdForaUFST10 { get; set; }

        public string CdListaServLC11603 { get; set; }

        public long? IdHistoricoEstorno { get; set; }

        public int? CFOPDentroUFEnt { get; set; }

        public int? CFOPForaUFEnt { get; set; }

        public int? CFOPForaUFST60Ent { get; set; }

        public int? CFOPDentroUFST60Ent { get; set; }

        public int? CFOPDentroUFServEnt { get; set; }

        public int? CFOPForaUFServEnt { get; set; }

        public int? CFOPDentroUFST10Ent { get; set; }

        public int? CFOPForaUFST10Ent { get; set; }

        public int? CFOPProdDentroUFST10Ent { get; set; }

        public int? CFOPProdForaUFST10Ent { get; set; }

        public int? CFOPProdDentroUFEnt { get; set; }

        public int? CFOPProdForaUFEnt { get; set; }

        public NaturezaOperacaoServico? NaturezaOperacaoServiço { get; set; }

        public bool GeraFinanceiro { get; set; }

        public bool GeraEstoque { get; set; }

        public bool GeraComissao { get; set; }

        public bool TemRetencao { get; set; }

        public bool AtualizaCusto { get; set; }

        public bool UsoConsumo { get; set; }

        public bool UsarCustoMedioNaVenda { get; set; }

        public bool NaoTributarQuantidade { get; set; }

        public bool EstoqueTerceiros { get; set; }

        public long? IdOperacaoANP { get; set; }
        public long? IdPlanoDeConta { get; set; }

        public bool Ativo { get; set; } = true;
        public bool OperacaoForaUF { get; set; }
        public int TipoCTE { get; set; } = (int)TipoCTEHistorico.Venda;
        public string EnquadramentoIPI { get; set; }
        public decimal PercRedICMSRetorno { get; set; }
        public bool UsaLote { get; set; }
        public bool ProdutorRural { get; set; }
        public bool ObrigaNFEmpenho { get; set; }
        public HistoricoModel HistoricoEstorno { get; set; }

        public string CST
        {
            get
            {
                if (SituacaoTributaria.HasValue && (int)SituacaoTributaria >= 0 && TabelaA.HasValue && (int)TabelaA >= 0)
                {
                    string sitTrib = ((int)SituacaoTributaria).ToString().Length <= 2 ? ((int)SituacaoTributaria).ToString("00") : ((int)SituacaoTributaria).ToString("000");
                    string tabA = ((int)TabelaA).ToString();
                    return tabA + sitTrib;
                }
                return string.Empty;
            }
        }
        public string DescricaoEntradaSaida => EntradaSaida == 0 ? "E" : "S";
        public string DescricaoGeraEstoque => GeraEstoque ? "Sim" : "Não";
        public string DescricaoGeraFinanceiro => GeraFinanceiro ? "Sim" : "Não";
        public string DescricaoAtivo => Ativo ? "Sim" : "Não";
        public string DescricaoStatus => Ativo ? "Ativo" : "Inativo";
        public PlanoDeContaModel PlanoDeConta { get; set; }

        public void LimparDados()
        {
            Ativo = true;
            AtualizaCusto = false;
            CFOPDentroUF = null;
            CFOPDentroUFEnt = null; 
            CFOPDentroUFServ = null; 
            CFOPDentroUFServEnt = null; 
            CFOPDentroUFST10 = null;
            CFOPDentroUFST10Ent = null; 
            CFOPDentroUFST60 = null; 
            CFOPDentroUFST60Ent = null; 
            CFOPForaUF = null; 
            CFOPForaUFEnt = null; 
            CFOPForaUFServ = null; 
            CFOPForaUFServEnt = null; 
            CFOPForaUFST10 = null; 
            CFOPForaUFST10Ent = null;
            CFOPForaUFST60 = null; 
            CFOPForaUFST60Ent = null; 
            CFOPProdDentroUF = null; 
            CFOPProdDentroUFEnt = null; 
            CFOPProdDentroUFST10 = null; 
            CFOPProdDentroUFST10Ent = null; 
            CFOPProdForaUF = null;
            CFOPProdForaUFEnt = null; 
            CFOPProdForaUFST10 = null; 
            CFOPProdForaUFST10Ent = null; 
            CdHistoricoFiscal=null; 
            CdHistoricoFiscalServ=null; 
            CdHistoricoFiscalST10=null; 
            CdHistoricoFiscalST60=null;
            CdListaServLC11603=null; 
            CSTIPI=null; 
            CSTPISCOFINS=null; 
            DescCFOPDentroUFServ=null; 
            DescCFOPForaUF=null; 
            DescCFOPForaUFServ=null;
            Descricao=null; 
            DescricaoNF=null; 
            TipoNF = (int)TipoNFHistorico.Normal; 
            EnquadramentoIPI=null; 
            EntradaSaida=(int)GAFNETLibrary.Enums.EntradaSaidaHistorico.Saida;
            EspecieDocto=null; 
            EspecieNF=null; 
            EstoqueTerceiros=false; 
            GeraComissao=false; 
            GeraEstoque=false; 
            GeraFinanceiro=false; 
            HistoricoEstorno=new HistoricoModel();
            Id=0; 
            IdHistoricoEstorno=null; 
            IdOperacaoANP=null; 
            NaoTributarQuantidade=false; 
            ObrigaNFEmpenho=false; 
            ObsNFDentroUF=null; 
            NaturezaOperacaoServiço=null;
            ObsNFForaUF=null; 
            OperacaoForaUF=false; 
            PercCOFINS=0; 
            PercIPI=0; 
            PercPIS=0; 
            PercRedICMSRetorno=0; 
            ProdutorRural=false; 
            SituacaoTributaria=null; 
            TabelaA=null; 
            TemRetencao=false; 
            TipoCTE=(int)TipoCTEHistorico.Venda; 
            TipoNF = (int)TipoNFHistorico.Normal; 
            UsaLote=false; 
            UsarCustoMedioNaVenda=false; 
            UsoConsumo = false;
            IdPlanoDeConta = null;
            PlanoDeConta = new PlanoDeContaModel();
        }
    }
}
