﻿using GAFNETLibrary.Enums;
using System;
using System.Collections.ObjectModel;
using System.Linq;

namespace GAFNETBlazor.Models;

public class PessoaModel
{
    public long Id { get; set; }

    public string Nome { get; set; }

    public string CNPJ { get; set; }

    public string InscEstadual { get; set; }
    public IndicadorIE IndicadorIE { get; set; }
    public string NomeFantasia { get; set; }

    public int? CdContabil { get; set; }

    //public long? IdRegiao { get; set; }

    public string Observacoes { get; set; }

    public DateTime? DtCadastro { get; set; }

    public long? IdCondicaoDePagamento { get; set; }

    public long? IdPlanoDeConta { get; set; }

    public long? IdCentroDeCusto { get; set; }

    public TipoFrete? TipoFrete { get; set; }

    public long? IdHistorico { get; set; }

    public long? IdTabelaPreco { get; set; }
    public long? IdEnderecoPrincipal { get; set; }

    public DateTime? DtNascimento { get; set; }

    public string CdAtivEconomica { get; set; }

    public string CdInstalacaoANP { get; set; }

    public long? IdModeloBoleto { get; set; }

    public TipoRegimeTributacao TipoRegimeTributacao { get; set; }

    public string InscMunicipal { get; set; }

    public long? IdTipoDeOperacao { get; set; }
    public long? IdGAF { get; set; }
    public bool Ativo { get; set; }

    public bool InformarXPed { get; set; }

    public bool MEI { get; set; }

    public string IdEstrangeiro { get; set; }

    public bool EhFornecedor { get; set; }

    public bool EhTransportadora { get; set; }

    public bool EhCliente { get; set; }
    public bool EhVendedor { get; set; }

    public int? DiaDeRota { get; set; }
    public string IdDebitoEmConta { get; set; }
    public string AgenciaDebitoEmConta { get; set; }
    public string ContaDebitoEmConta { get; set; }
    public string CNPJFormatado => !string.IsNullOrEmpty(CNPJ) ? CNPJ.Length > 11 ? Convert.ToUInt64(CNPJ).ToString(@"00\.000\.000\/0000\-00") : Convert.ToUInt64(CNPJ).ToString(@"000\.000\.000\-00") : "";

    public string DescricaoAtivo => Ativo ? "Ativo" : "Inativo";
    public CondicaoDePagamentoModel CondicaoDePagamento { get; set; }

    public HistoricoModel Historico { get; set; }

    public PlanoDeContaModel PlanoDeConta { get; set; }

    public TabelaDePrecoModel TabelaDePreco { get; set; }

    public TipoDeOperacaoModel TipoDeOperacao { get; set; }

    public CentroDeCustoModel CentroDeCusto { get; set; }

    public ModeloBoletoModel ModeloBoleto { get; set; }

    public ObservableCollection<EnderecoPessoaModel> Enderecos { get; set; }

    public ObservableCollection<EMailPessoaModel> EMails { get; set; }

    public ObservableCollection<TelefonePessoaModel> Telefones { get; set; }

    public ObservableCollection<VendedorPessoaModel> VendedoresPessoa { get; set; }
    public ObservableCollection<TransportadoraPessoaModel> TransportadorasPessoa { get; set; }

    public ObservableCollection<ProdutoPessoaModel> Produtos { get; set; }

    public EnderecoPessoaModel EnderecoPrincipal { get; set; } = new EnderecoPessoaModel { Cidade = new(), Pais = new() };

    public PessoaModel()
    {
        Enderecos = new ObservableCollection<EnderecoPessoaModel>();
    }
    public string PrimeiroTelefone
    {
        get
        {
            if (Telefones == null || !Telefones.Any())
            {
                return "";
            }
            return Telefones.First().TelefoneFormatado;
        }
    }
}
