﻿using GAFNETLibrary.Enums;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;

namespace GAFNETBlazor.Models
{
    public class ProdutoModel
    {
        public long Id { get; set; }

        [Display(Name = "Código")] 
        public string Codigo { get; set; }

        public int Tipo { get; set; }

        public long IdSubgrupo { get; set; }

        public long IdUnidade { get; set; }

        public long IdAliquota { get; set; }

        public long IdClassificacaoFiscal { get; set; }

        [Display(Name = "Descrição")]
        public string Nome { get; set; }

        [DisplayFormat(DataFormatString = "dd/MM/yyyy")]
        [Display(Name = "Data Cadastro")] 
        public DateTime? DataCadastro { get; set; }

        public string CodigoDeBarras { get; set; }

        public decimal PercCustoVar { get; set; }

        public string Observacoes { get; set; }

        public decimal PercComissao { get; set; }


        public string LocalEstoque { get; set; }

        public decimal PercISQN { get; set; }

        public decimal FracaoHora { get; set; }

        public string AcaoServico { get; set; }

        public string CodigoANP { get; set; }
        public string CodigoSCANCNF { get; set; }
        public string CodigoSCANCEstoque { get; set; }

        public decimal PercMargemLucro { get; set; }

        public string CaminhoImagem { get; set; }

        public long? IdGenero { get; set; }

        public decimal PesoBruto { get; set; }

        public decimal PesoLiq { get; set; }

        public long? IdEmbalagem { get; set; }

        public decimal CustoDigitado { get; set; }

        public decimal PercICMSST { get; set; }

        public string ItemListaServico { get; set; }

        public string CdTributacaoMunicipio { get; set; }

        public decimal CustoProd { get; set; }

        [DisplayFormat(DataFormatString = "#,##0.0000000")]
        [Display(Name = "Vlr. Venda")]
        public decimal VlrVenda { get; set; }

        public decimal Estoque { get; set; }

        public bool CompoeLMP { get; set; }

        public bool Ativo { get; set; } = true;

        public decimal EstoqueMinimo { get; set; }

        public bool CompoeNF { get; set; }

        [DisplayFormat(DataFormatString = "#,##0.0000000")]
        [Display(Name = "Custo Médio")]
        public decimal CustoMedio { get; set; }

        public decimal VlrBaseICMSST { get; set; }

        public decimal VlrICMSST { get; set; }

        public bool AssociaVeiculo { get; set; }

        public long? IdGAF { get; set; }

        public decimal VlrPromocional { get; set; }
        public string CEST { get; set; }
        public bool TributaServicoNoMunicipio { get; set; }
        public decimal FatorQTrib { get; set; }

        public ClassificacaoFiscalModel ClassificacaoFiscal { get; set; }
        public EmbalagemModel Embalagem { get; set; }
        public GeneroProdutoModel Genero { get; set; }
        public SubgrupoProdutoModel Subgrupo { get; set; }
        public UnidadeProdutoModel Unidade { get; set; }
        public AliquotaModel Aliquota { get; set; }
        [Display(Name = "Tipo")]
        public string DescricaoTipoItem =>
            Tipo switch
            {
                (int)TipoProduto.Mercadoria           => "00 - Mercadoria",
                (int)TipoProduto.MateriaPrima         => "01 - Matéria-Prima",
                (int)TipoProduto.Servico              => "02 - Serviço",
                (int)TipoProduto.Produto              => "03 - Produto",
                (int)TipoProduto.Embalagem            => "04 - Embalagem",
                (int)TipoProduto.ProdutoEmProcesso    => "05 - Produto em Processo",
                (int)TipoProduto.Subproduto           => "06 - Subproduto",
                (int)TipoProduto.ProdutoIntermediario => "07 - Produto Intermediário",
                (int)TipoProduto.MaterialDeUsoConsumo => "08 - Material de Uso e Consumo",
                (int)TipoProduto.AtivoImobilizado     => "09 - Ativo Imobilizado",
                (int)TipoProduto.OutrosInsumos        => "10 - Outros insumos",
                (int)TipoProduto.Outras               => "11 - Outras",
                _                                => string.Empty
            };

        public string DescricaoANP =>
            CodigoSCANCNF switch
            {
                "99600" => "ALCOOL ANIDRO",
                "64300" => "DIESEL",
                "62006" => "GASOLINA A",
                "62448" => "GASOLINA A PREMIUM, 62Y",
                "62103" => "GASOLINA B",
                "62154" => "GASOLINA C",
                "62155" => "GASOLINA C ADITIVADA",
                "62456" => "GASOLINA C PREMIUM",
                "62308" => "GASOLINA DE AVIACAO",
                "22403" => "GLP",
                "66575" => "OLEO COMBUSTIVEL TIPO 1A",
                "66664" => "OLEO COMBUSTIVEL TIPO 1B",
                "65583" => "OLEO COMBUSTIVEL TIPO 2A",
                "66591" => "OLEO COMBUSTIVEL TIPO 3A",
                "66680" => "OLEO COMBUSTIVEL TIPO 3B",
                "66605" => "OLEO COMBUSTIVEL TIPO 4A",
                "66699" => "OLEO COMBUSTIVEL TIPO 4B",
                "66702" => "OLEO COMBUSTIVEL TIPO 5B",
                "66621" => "OLEO COMBUSTIVEL TIPO 6A",
                "66710" => "OLEO COMBUSTIVEL TIPO 6B",
                "66630" => "OLEO COMBUSTIVEL TIPO 7A",
                "66729" => "OLEO COMBUSTIVEL TIPO 7B",
                "66648" => "OLEO COMBUSTIVEL TIPO 8A",
                "64106" => "QUEROSENE DE AVIACAO",
                "63991" => "QUEROSENE ILUMINANTE",
                "99900" => "OLEO FUSEL",
                "64432" => "BIODIESEL B100",
                "64440" => "BIODIESEL BX-INTERIOR COMUM",
                "64475" => "BIODIESEL BX-INTERIOR ADITIVADO",
                "64483" => "BIODIESEL BX-METROP. COMUM",
                "64491" => "BIODIESEL BX-METROP. ADITIVADO",
                _ => string.Empty
            };

        [Display(Name = "Status")]
        public string DescricaoAtivo => Ativo ? "Ativo" : "Inativo";

        public ICollection<ProdutoPessoaModel> Pessoas { get; set; }
    }
}
