﻿using FluentValidation;

namespace GAFNETBlazor.Models.Validations
{
    public class AliquotaModelValidator : AbstractValidator<AliquotaModel>
    {
        int tamanhoDescricao = 100;
        int tamanhoCdBenefFiscal = 8;
        public AliquotaModelValidator()
        {
            RuleFor(p => p.Descricao).NotEmpty().WithMessage("A descrição deve ser informada");
            RuleFor(p => p.Descricao).MaximumLength(tamanhoDescricao).WithMessage($"A descrição deve conter no máximo {tamanhoDescricao} caracteres");
            RuleFor(p => p.CodigoBeneficioFiscal).MaximumLength(tamanhoCdBenefFiscal).WithMessage($"O código de benefício fiscal deve conter no máximo {tamanhoCdBenefFiscal} caracteres");
            RuleFor(p => p.TabA).NotNull().WithMessage("A Tabela A deve ser informada");
            RuleFor(p => p.SituacaoTributaria).NotNull().WithMessage("A Situação Tributária deve ser informada");
            RuleFor(p => p.CSTPISCOFINS).NotEmpty().WithMessage("O CST do PISCOFINS deve ser informado");
            RuleFor(p => p.CSTIPI).NotEmpty().WithMessage("O CST do IPI deve ser informado");
            RuleFor(p => p.PercICMS).GreaterThan(0).When(a => a.ObrigaAliquotaICMS).WithMessage("Este CST obriga o preenchimento da alíquota de ICMS.");
            RuleFor(p => p.PercRedICMS).GreaterThan(0).When(a => a.ObrigaReducaoICMS).WithMessage("Este CST obriga o preenchimento da redução de ICMS.");
            RuleFor(p => p.PercMargem).GreaterThan(0).When(a => a.ObrigaMargem).WithMessage("Este CST obriga o preenchimento da MVA.");
        }
    }
}