﻿using FluentValidation;

namespace GAFNETBlazor.Models.Validations
{
    public class ClassificacaoFiscalModelValidator : AbstractValidator<ClassificacaoFiscalModel>
    {
        int tamanhoNumero = 8;
        int tamanhoDescricao = 80;
        int tamanhoEX = 3;
        int tamanhoCEST = 7;
        public ClassificacaoFiscalModelValidator()
        {
            RuleFor(p => p.Numero).NotEmpty().WithMessage("O número deve ser informado");
            RuleFor(p => p.Numero).LessThanOrEqualTo(99999999).WithMessage($"O número deve conter no máximo {tamanhoNumero} caracteres");
            RuleFor(p => p.Descricao).NotEmpty().WithMessage("A descrição deve ser informada");
            RuleFor(p => p.Descricao).MaximumLength(tamanhoDescricao).WithMessage($"A descrição deve conter no máximo {tamanhoDescricao} caracteres");
            RuleFor(p => p.EX).LessThanOrEqualTo(999).WithMessage($"O EX deve conter no máximo {tamanhoEX} caracteres");
            RuleFor(p => p.CEST).LessThanOrEqualTo(9999999).WithMessage($"O CEST deve conter no máximo {tamanhoCEST} caracteres");
        }

        int? StringToNullableInt(string value) 
        {
            return int.TryParse(value, out int val) ? (int?)val : null;
        }
        
    }
}