﻿using FluentValidation;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Models.Validations
{
    public class EmpresaModelValidator : AbstractValidator<EmpresaModel>
    {
        public EmpresaModelValidator()
        {
            RuleFor(p => p.Nome).NotEmpty().WithMessage("O nome da empresa deve ser informado");
            RuleFor(p => p.Nome).MaximumLength(100).WithMessage("O nome da empresa deve conter no máximo 100 caracteres");
            RuleFor(p => p.CNPJ).NotEmpty().WithMessage("O CNPJ da empresa deve ser informado");
            RuleFor(p => p.CNPJ).MaximumLength(14).WithMessage("O CNPJ da empresa deve conter no máximo 14 caracteres");
            RuleFor(p => p.IdRepresentante).NotEqual(0).WithMessage("O representante deve ser informado");
            RuleFor(p => p.IdCidade).NotEqual(0).WithMessage("A cidade deve ser informada");
            RuleFor(p => p.Logradouro).NotEmpty().WithMessage("O logradouro deve ser informado");
            RuleFor(p => p.NumeroEndereco).NotEmpty().WithMessage("O número deve ser informado");
            RuleFor(p => p.Bairro).NotEmpty().WithMessage("O bairro deve ser informado");
            RuleFor(p => p.CEP).Length(8).WithMessage("O CEP deve conter 8 caracteres");
        }
    }
}