﻿using FluentValidation;
using System.Linq;

namespace GAFNETBlazor.Models.Validations
{
    public class HistoricoModelValidator : AbstractValidator<HistoricoModel>
    {
        public HistoricoModelValidator()
        {
            RuleFor(p => p.Descricao).NotEmpty().WithMessage("A descrição deve ser informada");
            RuleFor(p => p.Descricao).MaximumLength(50).WithMessage("A descrição deve conter no máximo 50 caracteres");
            RuleFor(p => p.DescricaoNF).MaximumLength(60).WithMessage("A descrição da NF deve conter no máximo 60 caracteres");
            RuleFor(p => p.ObsNFDentroUF).MaximumLength(2000).WithMessage("A observação da NF de dentro da UF deve conter no máximo 2000 caracteres");
            RuleFor(p => p.ObsNFForaUF).MaximumLength(2000).WithMessage("A observação da NF de fora da UF deve conter no máximo 2000 caracteres");
            RuleFor(p => p.EnquadramentoIPI).MaximumLength(4).WithMessage("O enquadramento do IPI deve conter no máximo 4 caracteres");
            RuleFor(p => p.EntradaSaida).NotEqual(p => p.HistoricoEstorno.EntradaSaida).When(p => p.IdHistoricoEstorno != null)
                .WithMessage("O histórico e o histórico de estorno não podem ser ambos de entrada ou ambos de saída");
        }
    }
}