﻿using FluentValidation;
using GAFNETLibrary.Enums;
using System.Collections.Generic;

namespace GAFNETBlazor.Models.Validations
{
    public class PagarReceberModelValidator : AbstractValidator<PagarReceberModel>
    {
        readonly int tamObs = 2000;

        public PagarReceberModelValidator()
        {
            RuleFor(p => p.Pessoa.Id)
                .GreaterThan(0)
                .WithMessage("A pessoa deve ser informada");

            RuleFor(p => p.NDocto)
                .NotNull()
                .NotEmpty()
                .WithMessage("O número do documento deve ser informado");

            RuleFor(p => p.SerieDocto)
                .NotNull()
                .NotEmpty()
                .WithMessage("A série deve ser informada");

            RuleFor(p => p.Itens.Count)
                .GreaterThan(0)
                .WithMessage("Documento sem parcelas");

            RuleFor(p => p.Observacoes)
                .MaximumLength(tamObs)
                .WithMessage($"A observação deve conter no máximo {tamObs} caracteres");

        }
    }
}