﻿using FluentValidation;
using GAFNETLibrary.Enums;
using System.Linq;

namespace GAFNETBlazor.Models.Validations
{
    public class PessoaModelValidator : AbstractValidator<PessoaModel>
    {
        int tamNome = 60;
        int tamNomeFantasia = 60;
        
        public PessoaModelValidator()
        {
            RuleFor(p => p.Nome).NotEmpty().WithMessage("O nome deve ser informado");
            RuleFor(p => p.Nome).MaximumLength(tamNome).WithMessage($"O nome deve conter no máximo {tamNome} caracteres");
            RuleFor(p => p.NomeFantasia).MaximumLength(60).WithMessage($"O nome fantasia deve conter no máximo {tamNomeFantasia} caracteres");
            RuleFor(p => p.TipoRegimeTributacao).NotNull().WithMessage("O regime de tributação deve ser informado");
            RuleFor(p => p.CNPJ).NotEmpty().WithMessage("O CNPJ/CPF deve ser informado");
            RuleFor(p => p.IndicadorIE).NotEmpty().WithMessage("O indicador IE deve ser informado");
            RuleFor(p => p.Enderecos.FirstOrDefault(e => e.EhPrincipal)).NotNull().When(p => p.Enderecos != null).WithMessage("Deve ser informado ao menos um endereço principal");
            RuleFor(p => p.EMails.FirstOrDefault(e => e.EhPadrao)).NotNull().When(p => p.EMails != null).WithMessage("Deve ser informado ao menos um e-mail padrão");
            RuleFor(p => p.Enderecos).NotNull().WithMessage("Deve ser informado ao menos um endereço principal");
            RuleFor(p => p.InscEstadual).NotEmpty().When(p => p.IndicadorIE == IndicadorIE.Contribuinte)
                .WithMessage("Esse indicador IE exige o preenchimento da inscrição estadual.");
            RuleFor(p => p.InscEstadual).NotEqual("ISENTO").When(p => p.IndicadorIE == IndicadorIE.Contribuinte)
                .WithMessage("Esse indicador IE exige o preenchimento da inscrição estadual.");
            RuleFor(p => p.InscEstadual).Empty().When(p => p.IndicadorIE == IndicadorIE.Isento && p.InscEstadual != "ISENTO")
                .WithMessage("Esse Indicador IE não permite o preenchimento da Inscrição estadual.");
            RuleFor(p => p.InscEstadual).Equal("ISENTO").When(p => p.IndicadorIE == IndicadorIE.Isento && !string.IsNullOrEmpty(p.InscEstadual))
                .WithMessage("Esse Indicador IE não permite o preenchimento da Inscrição estadual.");
            RuleFor(p => p).Must(PossuiTipoPessoa).WithMessage("É necessário informar um ou mais tipos de pessoa.");

        }

        private bool PossuiTipoPessoa(PessoaModel pessoa)
        {
            return pessoa.EhCliente || pessoa.EhFornecedor || pessoa.EhTransportadora || pessoa.EhVendedor;
        }

    }
}