﻿using FluentValidation;

namespace GAFNETBlazor.Models.Validations;

public class ProdutoModelValidator : AbstractValidator<ProdutoModel>
{
    int tamNome = 120;
    int tamCodigo = 20;
    int tamCodBarras = 15;
    int tamObs = 100;
    int tamLocalEstoque = 15;
    int tamAcaoServiço = 50;
    int tamCodANP = 9;
    int tamCaminhoImagem = 150;
    int tamItemListaServiço = 15;
    int tamCodTribMunicipio = 15;

    public ProdutoModelValidator()
    {
        RuleFor(p => p.Nome).NotEmpty().WithMessage("O nome deve ser informado");
        RuleFor(p => p.Nome).MaximumLength(tamNome).WithMessage($"O nome deve conter no máximo {tamNome} caracteres");
        RuleFor(p => p.Codigo).NotEmpty().WithMessage("O código deve ser informado");
        RuleFor(p => p.Codigo).MaximumLength(tamCodigo).WithMessage($"O código deve conter no máximo {tamCodigo} caracteres");
        RuleFor(p => p.CodigoDeBarras).MaximumLength(tamCodBarras).WithMessage($"O código de barras deve conter no máximo {tamCodBarras} caracteres");
        RuleFor(p => p.Observacoes).MaximumLength(tamObs).WithMessage($"A observação deve conter no máximo {tamObs} caracteres");
        RuleFor(p => p.LocalEstoque).MaximumLength(tamLocalEstoque).WithMessage($"O local de estoque deve conter no máximo {tamLocalEstoque} caracteres");
        RuleFor(p => p.AcaoServico).MaximumLength(tamAcaoServiço).WithMessage($"Ação serviço deve conter no máximo {tamAcaoServiço} caracteres");
        RuleFor(p => p.CodigoANP).MaximumLength(tamCodANP).WithMessage($"O código ANP deve conter no máximo {tamCodANP} caracteres");
        RuleFor(p => p.CaminhoImagem).MaximumLength(tamCaminhoImagem).WithMessage($"O caminho da imagem deve conter no máximo {tamCaminhoImagem} caracteres");
        RuleFor(p => p.ItemListaServico).MaximumLength(tamItemListaServiço).WithMessage($"O item da lista de serviço deve conter no máximo {tamItemListaServiço} caracteres");
        RuleFor(p => p.CdTributacaoMunicipio).MaximumLength(tamCodTribMunicipio).WithMessage($"O código de tributação no município deve conter no máximo {tamCodTribMunicipio} caracteres");
        RuleFor(p => p.IdAliquota).GreaterThan(0).WithMessage("A alíquota deve ser informada");
        RuleFor(p => p.IdClassificacaoFiscal).GreaterThan(0).WithMessage("O NCM deve ser informada");
        RuleFor(p => p.IdSubgrupo).GreaterThan(0).WithMessage("O subgrupo deve ser informado");
        RuleFor(p => p.IdUnidade).GreaterThan(0).WithMessage("A unidade deve ser informada");
    }
}