﻿using FluentValidation;
using GAFNETBlazor.Models;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Models.Validations
{
    public class UsuarioModelValidator : AbstractValidator<UsuarioModel>
    {
        public UsuarioModelValidator()
        {
            RuleFor(p => p.Nome).NotEmpty().WithMessage("O nome do usuário deve ser informado");
            RuleFor(p => p.Nome).MaximumLength(256).WithMessage("O nome do usuário deve conter no máximo 256 caracteres");
            RuleFor(p => p.Email).NotEmpty().WithMessage("O e-mail do usuário deve ser informado");
            RuleFor(p => p.Email).EmailAddress().When(e => e.Email?.Length > 0).WithMessage("Insira um e-mail válido");
            RuleFor(p => p.Email).MaximumLength(256).WithMessage("O e-mail deve conter no máximo 256 caracteres");
            RuleFor(p => p.PasswordHash).NotEmpty().When(p => string.IsNullOrEmpty(p.Id)).WithMessage("A senha deve ser informada");
            RuleFor(p => p.PasswordHash).MaximumLength(256).WithMessage("A senha deve conter no máximo 256 caracteres");
            RuleFor(p => p.ConfirmPassword).NotEmpty().When(p => string.IsNullOrEmpty(p.Id)).WithMessage("A confirmação da senha deve ser informada");
            RuleFor(p => p.ConfirmPassword).Equal(s => s.PasswordHash).WithMessage("A senha e a confirmação diferem");
            RuleFor(p => p.CNPJCPF).NotEmpty().WithMessage("O CPF/CNPJ do usuário deve ser informado");
            RuleFor(p => p.CNPJCPF).MaximumLength(14).WithMessage("O CPJ/CNPJ do usuário deve conter no máximo 14 caracteres");
        }
    }
}