﻿using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Constantes;
using System;
using System.Net.Http;
using System.Threading.Tasks;
using Newtonsoft.Json;
using GAFNETLibrary.Models;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Stores.Interfaces;

namespace GAFNETBlazor.Services.Administrativo
{
    public class EmpresasService : IEmpresasService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        private readonly ILoginStore _loginStore;
        public EmpresasService(IHttpClientComToken httpClientComToken, ILoginStore loginStore)
        {
            _httpClientComToken = httpClientComToken;
            _loginStore = loginStore;
        }

        public async Task<HttpResponseMessage> CriarEmpresa(EmpresaModel Empresa)
        {
            return await _httpClientComToken.Enviar(HttpMethod.Post, URIsEmpresas.Post, Empresa);
        }

        public async Task<HttpResponseMessage> AlterarEmpresa(EmpresaModel Empresa)
        {
            return await _httpClientComToken.Enviar(HttpMethod.Put, URIsEmpresas.Put, Empresa);
        }

        public async Task<HttpResponseMessage> AlterarEmpresaComoRepresentante(EmpresaModel Empresa)
        {
            return await _httpClientComToken.Enviar(HttpMethod.Put, URIsEmpresas.AlteracaoComoRepresentante, Empresa);
        }

        public Task<HttpResponseMessage> GetEmpresas(StatusAtivo statusAtivo)
        {
            var uri = new Uri(URIsEmpresas.Get + $"?status={(int)statusAtivo}");
            return _httpClientComToken.GetResposta(uri);
        }

        public Task<HttpResponseMessage> GetEmpresasByRepresentanteLogado()
        {
            var uri = new Uri(URIsEmpresas.GetEmpresasRepresentanteLogado + $"status={(int)StatusAtivo.Todos}");
            return _httpClientComToken.GetResposta(uri);
        }

        public async Task<RespostaAPIModel> GetEmpresaByCNPJ(string cnpj)
        {
            var retorno = await _httpClientComToken.GetResposta(new Uri(URIsEmpresas.GetByCNPJ + "cnpj=" + cnpj));
            
            if (!retorno.IsSuccessStatusCode)
            {
                return new RespostaAPIModel { Sucesso = false, MensagemErro = await retorno.Content.ReadAsStringAsync() };
            }
            EmpresaModel empresa = JsonConvert.DeserializeObject<EmpresaModel>(await retorno.Content.ReadAsStringAsync());
            return new RespostaAPIModel { Sucesso = true, Objeto = empresa, Status = retorno.StatusCode };
        }
    }
}
