﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class AliquotasService : IAliquotasService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public AliquotasService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase + "?$Select=id,descricao,ativo,percicms,taba,situacaotributaria,percredicms,percmargem,percmargemred,percredst&$OrderBy=descricao";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }
            select += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            string conteudo = await resposta.Content.ReadAsStringAsync();

            var retornoAPI = new RespostaAPIModel
            {
                MensagemErro = !resposta.IsSuccessStatusCode ? conteudo : string.Empty,
                Sucesso = resposta.IsSuccessStatusCode
            };

            List<AliquotaModel> aliquotas = new();
            if (resposta.IsSuccessStatusCode)
            {
                aliquotas = JsonConvert.DeserializeObject<List<AliquotaModel>>(conteudo);
                retornoAPI.Objeto = aliquotas;
            }
            retornoAPI.Status = resposta.IsSuccessStatusCode ? aliquotas.Count > 0 ? HttpStatusCode.OK : HttpStatusCode.NoContent : resposta.StatusCode;
            return retornoAPI;
        }
        
        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            RespostaAPIModel respostaAPI = new ();
            string select = uriBase + $"?$Count=true&$expand=aliquotasinterestaduais & $Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            if (!resposta.IsSuccessStatusCode)
            {
                respostaAPI.MensagemErro = await resposta.Content.ReadAsStringAsync();
                respostaAPI.Status = resposta.StatusCode;
                return respostaAPI;
            }
            
            var dados = await resposta.Content.ReadAsStringAsync();
            var aliquotas = JsonConvert.DeserializeObject<List<AliquotaModel>>(dados);
            if (aliquotas.Count == 0)
            {
                respostaAPI.MensagemErro = $"Não foi encontrada uma alíquota com o id {id}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }
                
            respostaAPI.Sucesso = true;
            respostaAPI.Objeto = aliquotas.FirstOrDefault();
            

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, string campos, StatusAtivo status)
        {
            RespostaAPIModel respostaAPI = new ();
            string select = uriBase + "?$Count=true";
            select = !string.IsNullOrEmpty(campos) ? select + "&$select=" + campos : select;
            select += $"& $Filter=id eq {id}";
            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            if (!resposta.IsSuccessStatusCode)
            {
                respostaAPI.MensagemErro = await resposta.Content.ReadAsStringAsync();
                respostaAPI.Status = resposta.StatusCode;
            }
            else
            {
                var dados = await resposta.Content.ReadAsStringAsync();
                var aliquotas = JsonConvert.DeserializeObject<List<AliquotaModel>>(dados);
                if (aliquotas.Count == 0)
                {
                    respostaAPI.MensagemErro = $"Não foi encontrada uma alíquota com o id {id}.";
                    respostaAPI.Status = HttpStatusCode.NotFound;
                    return respostaAPI;
                }

                respostaAPI.Sucesso = true;
                respostaAPI.Objeto = aliquotas.FirstOrDefault();
            }

            return respostaAPI;
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            throw new NotImplementedException();
        }

        public async Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsAliquotas.PostAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> PutAsync<T>(T aliquota)
        {

            var retorno = await _httpClientComToken.Enviar(HttpMethod.Put, URIsAliquotas.PutAsync, aliquota);
            return new RespostaAPIModel(retorno);
        }
    }
}
