﻿using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using System.Collections.Generic;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class CFOPCompraService : ICFOPCompraService
    {
        private string _UFEmpresa;
        private bool _operacaoPresencial;
        private HistoricoModel _historico;
        private PessoaModel _pessoa;
        private ItemFaturamentoModel _item;
        private bool _foraUF;
        private int? _cfopST10;
        private int? _cfopST60;
        private int? _cfopProd;
        private ResultadoModel _resultNaoEncontrado;
        public ResultadoModel Get(ParamCFOPCompraModel p)
        {
            InicializarVariaveis(p);
            if (_item.CSTFinal.Length < 2) return new ResultadoModel { Sucesso = false, Retorno = $"Não foi informado o CST para o item de código: {_item.Produto.Codigo}" };

            if (_item.Produto.Tipo == (int)TipoProduto.Servico) return GetCFOPServico();
            if (PegarCFOPProduto()) return new ResultadoModel { Sucesso = true, Retorno = _cfopProd };
            if (_item.CSTFinal == "60" || _item.CSTFinal == "500") return GetCFOP60();
            var listaCFOP10 = new List<string> { "10", "70", "202", "203", "201" };
            if (listaCFOP10.Contains(_item.CSTFinal)) return GetCFOP10();
            if (_item.CSTFinal == "30") return GetCFOPCST30();
            return GetCFOPNormal();
        }

        private void InicializarVariaveis(ParamCFOPCompraModel p)
        {
            _UFEmpresa           = p.UFEmpresa;
            _operacaoPresencial  = p.OperacaoPresencial;
            _historico           = p.Historico;
            _pessoa              = p.Pessoa;
            _item                = p.Item;
            _foraUF              = !_operacaoPresencial && _pessoa.EnderecoPrincipal.Cidade.UF != _UFEmpresa;
            _resultNaoEncontrado = new ResultadoModel { Sucesso = false, Retorno = $"Não foi encontrado o CFOP para o item de código: {_item.Produto.Codigo}" };
            _cfopST10            = _foraUF ? _historico.CFOPForaUFST10 : _historico.CFOPDentroUFST10;
            _cfopST60            = _foraUF ? _historico.CFOPForaUFST60 : _historico.CFOPDentroUFST60;
            _cfopProd            = _foraUF ? _historico.CFOPProdForaUF : _historico.CFOPProdDentroUF;
        }

        private ResultadoModel GetCFOPServico()
        {
            var cfop = _foraUF ? _historico.CFOPForaUFServ : _historico.CFOPDentroUFServ;
            return cfop.HasValue ? new ResultadoModel { Sucesso = true, Retorno = cfop } : _resultNaoEncontrado;
        }

        private ResultadoModel GetCFOP60()
        {
            if (_cfopST60.HasValue) return new ResultadoModel { Sucesso = true, Retorno = _cfopST60 };
            return  _resultNaoEncontrado;
        }

        private ResultadoModel GetCFOP10()
        {
            if (_cfopST10.HasValue) return new ResultadoModel { Sucesso = true, Retorno = _cfopST10 };
            return _resultNaoEncontrado;
        }

        private ResultadoModel GetCFOPNormal()
        {
            var cfop = _foraUF ? _historico.CFOPForaUF : _historico.CFOPDentroUF;
            return cfop.HasValue ? new ResultadoModel { Sucesso = true, Retorno = cfop } : _resultNaoEncontrado;
        }

        private ResultadoModel GetCFOPCST30()
        {
            if (_cfopST10.HasValue) return new ResultadoModel { Sucesso = true, Retorno = _cfopST10 };
            if (_cfopST60.HasValue) return new ResultadoModel { Sucesso = true, Retorno = _cfopST60 };
            return _resultNaoEncontrado;
        }

        private bool PegarCFOPProduto()
        {
            if (_item.Produto.Tipo != (int)TipoProduto.Produto || !_cfopProd.HasValue) return false;
            if ((_item.CSTFinal == "10" || _item.CSTFinal == "202" || _item.CSTFinal == "203") && _cfopST10.HasValue) return false;
            if (_item.CSTFinal == "30" && (_cfopST10.HasValue && _cfopST60.HasValue)) return false;
            if ((_item.CSTFinal == "60" || _item.CSTFinal == "500") && _cfopST60.HasValue) return false;
            return true;
        }
    }
}
