﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Newtonsoft.Json;
using System;
using System.Net.Http;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services
{
    public class CancelamentoFaturamentoService : ICancelamentoFaturamentoService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public CancelamentoFaturamentoService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }
        public async Task<ResultadoModel> EnviarCancelamentoAsync(long idFaturamento, TipoFaturamento tipoFaturamento)
        {
            var uri = URIsFaturamentos.UriCancelamento(tipoFaturamento);
            uri = new Uri(uri + $"?id={idFaturamento}");
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, uri, null);
            var respostaAPI = new RespostaAPIModel(retorno);

            if (!respostaAPI.Sucesso)
            {
                return new ResultadoModel { Sucesso = false, Retorno = respostaAPI.MensagemErro };
            }

            var faturamentoRetorno = JsonConvert.DeserializeObject<FaturamentoModel>(respostaAPI.ConteudoRetorno);
            return new ResultadoModel { Sucesso = true, Retorno = faturamentoRetorno };
        }
    }
}
