﻿using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using GAFNETLibrary.Models;
using System.Net.Http;
using GAFNETBlazor.Constantes;

namespace GAFNETBlazor.Services
{
    public class CentrosDeCustoService : ICentrosDeCustoService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public CentrosDeCustoService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase + "?$Select=id,nome,padrao,ativo&$OrderBy=nome";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }
            select += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            string conteudo = await resposta.Content.ReadAsStringAsync();

            var retornoAPI = new RespostaAPIModel
            {
                MensagemErro = !resposta.IsSuccessStatusCode ? conteudo : string.Empty,
                Sucesso = resposta.IsSuccessStatusCode
            };

            List<CentroDeCustoModel> centrosDeCusto = new ();
            if (resposta.IsSuccessStatusCode)
            {
                centrosDeCusto = JsonConvert.DeserializeObject<List<CentroDeCustoModel>>(conteudo);
                retornoAPI.Objeto = centrosDeCusto;
            }
            retornoAPI.Status = resposta.IsSuccessStatusCode ? centrosDeCusto.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent : resposta.StatusCode;
            return retornoAPI;
        }
        
        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true & $Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            RespostaAPIModel respostaAPI = new(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }
            
            var dados = await resposta.Content.ReadAsStringAsync();
            var centrosDeCusto = JsonConvert.DeserializeObject<List<CentroDeCustoModel>>(dados);
            if (!centrosDeCusto.Any())
            {
                respostaAPI.MensagemErro = $"Não foi encontrado um centro de custo com o id {id}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                respostaAPI.Sucesso = false;
                return respostaAPI;
            }
            respostaAPI.Objeto = centrosDeCusto.FirstOrDefault();

            return respostaAPI;
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            throw new NotImplementedException();
        }

        public async Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsCentrosDeCusto.PostAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> PutAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsCentrosDeCusto.PutAsync, entity);
            return new RespostaAPIModel(retorno);
        }
    }
}
