﻿using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class CidadesService : ICidadesService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public CidadesService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase + "?$Select=id,nome,uf,cdibge,ativo&$orderby=nome";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }
            
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            RespostaAPIModel respostaAPI = new (resposta);


            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }
            
            var cidades = JsonConvert.DeserializeObject<List<CidadeModel>>(respostaAPI.ConteudoRetorno);
            respostaAPI.Objeto = cidades;

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetByCdIBGEAsync(Uri uriBase, int? cdIBGE, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true & $Filter=cdibge eq {cdIBGE}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            RespostaAPIModel respostaAPI = new (resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }

            var dados = JsonConvert.DeserializeObject<List<CidadeModel>>(respostaAPI.ConteudoRetorno);
            if (!dados.Any())
            {
                respostaAPI.MensagemErro = $"Não foi encontrada uma cidade com o código IBGE {cdIBGE}.";
                return respostaAPI;
            }
            
            respostaAPI.Objeto = dados.FirstOrDefault();
            
            return respostaAPI;

        }

        public async Task<RespostaAPIModel> GetByNameAndUFAsync(Uri uriBase, string nome, string uf, StatusAtivo status)
        {
            string select = uriBase + $"?$filter=nome eq '{nome}' and uf eq '{uf}'";

            if (status != StatusAtivo.Todos)
            {
                select += status == StatusAtivo.Ativos ? " and ativo eq true " : " and ativo eq false ";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            RespostaAPIModel respostaAPI = new (resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }
            
            var cidades = JsonConvert.DeserializeObject<List<CidadeModel>>(respostaAPI.ConteudoRetorno);
            respostaAPI.Objeto = cidades[0];

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true & $Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            RespostaAPIModel respostaAPI = new(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }
            
            var cidades = JsonConvert.DeserializeObject<List<CidadeModel>>(respostaAPI.ConteudoRetorno);
            if (!cidades.Any())
            {
                respostaAPI.MensagemErro = $"Não foi encontrada uma cidade com o id {id}.";
                return respostaAPI;
            }
            
            respostaAPI.Objeto = cidades.FirstOrDefault();

            return respostaAPI;
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            throw new NotImplementedException();
        }

        public Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            throw new NotImplementedException();
        }

        public Task<RespostaAPIModel> PutAsync<T>(T entity)
        {
            throw new NotImplementedException();
        }
    }
}
