﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class ClassificacoesFiscaisService : IClassificacoesFiscaisService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public ClassificacoesFiscaisService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase + "?$expand=unidadetributaria($select=id,codigo,descricao)&$OrderBy=numero";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }
            select += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            var retornoAPI = new RespostaAPIModel(resposta);

            List<ClassificacaoFiscalModel> ncms = new ();
            if (resposta.IsSuccessStatusCode)
            {
                ncms = JsonConvert.DeserializeObject<List<ClassificacaoFiscalModel>>(retornoAPI.ConteudoRetorno);
                retornoAPI.Objeto = ncms;
            }
            retornoAPI.Status = resposta.IsSuccessStatusCode ? ncms.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent : resposta.StatusCode;
            return retornoAPI;
        }
        
        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true&$expand=unidadetributaria($select=id,codigo,descricao)&$Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            RespostaAPIModel respostaAPI = new (resposta);

            if (resposta.IsSuccessStatusCode)
            {
                var ncms = JsonConvert.DeserializeObject<List<ClassificacaoFiscalModel>>(respostaAPI.ConteudoRetorno);
                if (!ncms.Any())
                {
                    respostaAPI.Sucesso = false;
                    respostaAPI.MensagemErro = $"Não foi encontrado um NCM com o id {id}.";
                    respostaAPI.Status = HttpStatusCode.NotFound;
                    return respostaAPI;
                }

                respostaAPI.Objeto = ncms.FirstOrDefault();
            }

            return respostaAPI;
        }


        public async Task<RespostaAPIModel> GetByNumeroAsync(Uri uriBase, long numero, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true&$expand=unidadetributaria($select=id,codigo,descricao)&$Filter=numero eq {numero}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            RespostaAPIModel respostaAPI = new(resposta);

            if (resposta.IsSuccessStatusCode)
            {
                var ncms = JsonConvert.DeserializeObject<List<ClassificacaoFiscalModel>>(respostaAPI.ConteudoRetorno);
                if (!ncms.Any())
                {
                    respostaAPI.Sucesso = false;
                    respostaAPI.MensagemErro = $"Não foi encontrado um NCM com o número {numero}.";
                    respostaAPI.Status = HttpStatusCode.NotFound;
                    return respostaAPI;
                }

                respostaAPI.Objeto = ncms.FirstOrDefault();
            }

            return respostaAPI;
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            throw new NotImplementedException();
        }

        public async Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsClassificacoesFiscais.PostAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> PutAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Put, URIsClassificacoesFiscais.PutAsync, entity);
            return new RespostaAPIModel(retorno);
        }
    }
}
