﻿using AutoMapper;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETLibrary.Models;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services;

public class ComprasService : FaturamentoBaseService, IComprasService
{
    const string expandGetById = "&$expand=pessoa($select=id,nome,cnpj;$expand=enderecoprincipal($expand=cidade,pais)),historico($select=id,descricao,gerafinanceiro), itens($expand=produto($select=id,codigo,nome,vlrvenda))" +
                                 ",transportadora($select=id,nome,cnpj),vendedores($expand=vendedor($select=id,nome,cnpj)),centrodecusto($select=id,nome),tipodeoperacao($select=id,descricao)" +
                                 ",condicaodepagamento($select=id,descricao,descricaoparcelas),planodeconta($select=id,descricao),endereco($select=id;$expand=cidade),formulario($select=id,nome)";
   
    private readonly IHttpClientComToken _httpClientComToken;
    private readonly ILoginStore _loginStore;
    private readonly INotaDeXMLService _notaDeXMLService;
    public ComprasService(IHttpClientComToken httpClientComToken, ILoginStore loginStore, IMapper mapper, INotaDeXMLService notaDeXMLService) : base(httpClientComToken, mapper)
    {
        _httpClientComToken = httpClientComToken;
        _loginStore = loginStore;
        _notaDeXMLService = notaDeXMLService;
    }

    public async Task<RespostaAPIModel> GetByNumeroAndSerieAndPessoaAndTipoAsync(int numero, int serie, long idPessoa, int tipo)
    {
        string select = $"?$Filter=numero eq {numero} and serie eq {serie} and IdPessoa eq {idPessoa} and tipo eq {tipo} &$Count=true";
        select += expandGetById;

        var uri = URIsFaturamentos.GetComprasAsync;

        var resposta = await _httpClientComToken.GetResposta(new Uri(uri, select));

        var respostaAPI = new RespostaAPIModel(resposta);

        if (!resposta.IsSuccessStatusCode)
        {
            return respostaAPI;
        }

        var faturamentos = JsonConvert.DeserializeObject<List<FaturamentoModel>>(respostaAPI.ConteudoRetorno);
        if (!faturamentos.Any())
        {
            respostaAPI.Sucesso = false;
            respostaAPI.MensagemErro = $"Não foi encontrado uma compra com o número {numero}.";
            respostaAPI.Status = HttpStatusCode.NotFound;
            return respostaAPI;
        }

        FaturamentoModel faturamento = faturamentos.FirstOrDefault();
        if (faturamento.Transportadora == null) faturamento.Transportadora = new PessoaModel();
        respostaAPI.Objeto = faturamento;

        return respostaAPI;
    }

    public async Task<RespostaAPIModel> ImportaXMLAsync(MemoryStream memoryStream, string fileName, DateTime dtEntrada, long idHistorico)
    {
        var resultNotaDeXML = await _notaDeXMLService.GetFaturamentoAsync(_loginStore.EmpresaLogada.CNPJ, memoryStream, fileName, dtEntrada, idHistorico);
        if (!resultNotaDeXML.Sucesso) return new RespostaAPIModel { Sucesso = false, MensagemErro = (string)resultNotaDeXML.Retorno };
        //var uri = new Uri(URIsFaturamentos.ImportacaoXMLCompra + $"?idHistorico={idHistorico}&dtEntrada={dtEntrada:yyyy/MM/dd}");
        var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsFaturamentos.ImportacaoXMLCompra, (FaturamentoDTO)resultNotaDeXML.Retorno);
        return new RespostaAPIModel(retorno);
    }

    public async Task<RespostaAPIModel> GravarNotaImportadaAsync(FaturamentoModel faturamento)
    {
        var uri = URIsFaturamentos.PostCompraImportadaAsync;
        var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, uri, faturamento);
        return new RespostaAPIModel(retorno);
    }

    public async Task<ResultadoModel> GetNotaDeXMLAsync(string cnpjEmpresa, MemoryStream memoryStream, string fileName, DateTime? dtEntrada, long? idHistorico)
    {
        var retorno = await _notaDeXMLService.GetFaturamentoAsync(_loginStore.EmpresaLogada.CNPJ, memoryStream, fileName, dtEntrada, idHistorico);
        return retorno;
    }
}
