﻿using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using GAFNETLibrary.Models;
using GAFNETBlazor.Constantes;
using System.Net.Http;

namespace GAFNETBlazor.Services
{
    public class CondicoesDePagamentoService : ICondicoesDePagamentoService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public CondicoesDePagamentoService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase + "?$OrderBy=descricao";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }
            select += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            var retornoAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return retornoAPI;
            }
            
            var condicoes = JsonConvert.DeserializeObject<List<CondicaoDePagamentoModel>>(retornoAPI.ConteudoRetorno);
            retornoAPI.Objeto = condicoes;
            retornoAPI.Status = condicoes.Count > 0 ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            return retornoAPI;
        }
        
        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true & $Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            RespostaAPIModel respostaAPI = new(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }
            
            var condicoes = JsonConvert.DeserializeObject<List<CondicaoDePagamentoModel>>(respostaAPI.ConteudoRetorno);
            
            if (!condicoes.Any())
            {
                respostaAPI.MensagemErro = $"Não foi encontrado um histórico com o id {id}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            respostaAPI.Sucesso = true;
            respostaAPI.Objeto = condicoes.FirstOrDefault();

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            string queryString = uriBase + $"?$Select={campos}";
            queryString = !string.IsNullOrEmpty(order) ? queryString += $"&$OrderBy= {order}" : queryString;
            queryString = !string.IsNullOrEmpty(filtros) ? queryString += $"&$Filter={filtros}" : queryString;
            queryString += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(queryString));
            var retornoAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return retornoAPI;
            }
            
            var condicoes = JsonConvert.DeserializeObject<List<CondicaoDePagamentoModel>>(retornoAPI.ConteudoRetorno);
            retornoAPI.Objeto = condicoes;
            retornoAPI.Status = condicoes.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            return retornoAPI;
        }

        public async Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsCondicoesDePagamento.PostAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> PutAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsCondicoesDePagamento.PutAsync, entity);
            return new RespostaAPIModel(retorno);
        }
    }
}
