﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class CorreiosService : ICorreiosService
    {
        private readonly IHttpClientFactory _clientFactory;
        private readonly ICidadesService _cidadesService;
        public CorreiosService(IHttpClientFactory clientFactory, ICidadesService cidadesService)
        {
            _clientFactory  = clientFactory;
            HttpClient      = _clientFactory.CreateClient();
            _cidadesService = cidadesService;
        }
        public HttpClient HttpClient { get; set; }
        public string MensagemErro { get; set; }
        public async Task<RespostaAPIModel> GetEnderecoByCEPAsync(string cep)
        {
            var retorno = await HttpClient.GetAsync(new Uri($"https://viacep.com.br/ws/{cep}/json/"));

            if (!retorno.IsSuccessStatusCode)
            {
                return new RespostaAPIModel { MensagemErro = await retorno.Content.ReadAsStringAsync(), Objeto = null };
            }

            var dados = await retorno.Content.ReadAsStringAsync();
            var retornoCorreio = JsonConvert.DeserializeObject<RetornoCorreioModel>(dados);

            var retornoCidade = await _cidadesService.GetByCdIBGEAsync(URIsPessoas.GetCidadesAsync, (int?)Convert.ToUInt32(retornoCorreio.IBGE), StatusAtivo.Ativos);
            if (!string.IsNullOrEmpty(retornoCidade.MensagemErro)) return new RespostaAPIModel { MensagemErro = retornoCidade.MensagemErro, Objeto = null };

            EnderecoPessoaModel endereco = new()
            {
                CEP         = cep,
                Cidade      = (CidadeModel)retornoCidade.Objeto,
                Logradouro  = retornoCorreio.Logradouro,
                Bairro      = retornoCorreio.Bairro,
                Complemento = retornoCorreio.Complemento,
                IdCidade    = ((CidadeModel)retornoCidade.Objeto).Id
            };

            return new RespostaAPIModel { MensagemErro = "", Sucesso=true,  Objeto = endereco };
        }
    }
}
