﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Newtonsoft.Json;
using System.Net.Http;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services
{
    public class CriaAlteraFaturamentoService : ICriaAlteraFaturamentoService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public CriaAlteraFaturamentoService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }
        public async Task<ResultadoModel> CriarAlterarAsync(FaturamentoModel faturamento)
        {
            faturamento.IdFormulario = faturamento.UsaFormulario ? faturamento.Formulario.Id : null;
            faturamento.Serie = faturamento.Serie == 0 ? faturamento.Formulario.Serie : faturamento.Serie;
            faturamento.IdPessoa = faturamento.Pessoa.Id;
            faturamento.IdTransportadora = faturamento.Transportadora.Id > 0 ? faturamento.Transportadora.Id : null;

            if (faturamento.UsaHistorico) SetarFaturamentoComHistorico(faturamento); else SetarFaturamentoSemHistorico(faturamento);

            foreach (var item in faturamento.Itens)
            {
                item.IdProduto = item.Produto.Id;
                item.IdUnidade = item.IdUnidade == 0 ? item.Produto.IdUnidade : item.IdUnidade;
                item.IdClassificacaoFiscal = item.IdClassificacaoFiscal == 0 ? item.Produto.IdClassificacaoFiscal : item.IdClassificacaoFiscal;
            }

            var retorno = faturamento.Id == 0 ? await PostAsync(faturamento) : await PutAsync(faturamento);

            if (!retorno.Sucesso) return new ResultadoModel { Sucesso = false, Retorno = retorno.MensagemErro };

            FaturamentoModel faturamentoRetorno;
            faturamentoRetorno = JsonConvert.DeserializeObject<FaturamentoModel>(retorno.ConteudoRetorno);
            return new ResultadoModel { Sucesso = retorno.Sucesso, Retorno = faturamentoRetorno };
        }

        private static void SetarFaturamentoSemHistorico(FaturamentoModel faturamento)
        {
            faturamento.IdHistorico = null;
            faturamento.FinNFe = FinNFeHistorico.Normal;
            faturamento.TipoNF = TipoNFHistorico.Normal;
            faturamento.GeraEstoque = false;
            faturamento.GeraFinanceiro = false;
            faturamento.AtualizaCusto = false;
            faturamento.EntradaSaida = (int)EntradaSaidaHistorico.Entrada; // seta como entrada porque é pedido de compra
            faturamento.IdCentroDeCusto = faturamento.CentroDeCusto.Id > 0 ? faturamento.CentroDeCusto.Id : null;
            faturamento.IdCondicaoDePagamento = faturamento.CondicaoDePagamento.Id > 0 ? faturamento.CondicaoDePagamento.Id : null;
            faturamento.IdTipoDeOperacao = faturamento.TipoDeOperacao.Id > 0 ? faturamento.TipoDeOperacao.Id : null;
        }

        private static void SetarFaturamentoComHistorico(FaturamentoModel faturamento)
        {
            faturamento.IdHistorico = faturamento.Historico.Id;
            faturamento.FinNFe = (FinNFeHistorico)faturamento.Historico.FinNFe;
            faturamento.TipoNF = (TipoNFHistorico)faturamento.Historico.TipoNF;
            faturamento.GeraEstoque = faturamento.Historico.GeraEstoque;
            faturamento.GeraFinanceiro = faturamento.Historico.GeraFinanceiro;
            faturamento.AtualizaCusto = faturamento.Historico.AtualizaCusto;
            faturamento.EntradaSaida = faturamento.Historico.EntradaSaida;
            faturamento.IdCentroDeCusto = faturamento.CentroDeCusto.Id > 0 && faturamento.Historico.GeraFinanceiro ? faturamento.CentroDeCusto.Id : null;
            faturamento.IdCondicaoDePagamento = faturamento.CondicaoDePagamento.Id > 0 && faturamento.Historico.GeraFinanceiro ? faturamento.CondicaoDePagamento.Id : null;
            faturamento.IdTipoDeOperacao = faturamento.TipoDeOperacao.Id > 0 && faturamento.Historico.GeraFinanceiro ? faturamento.TipoDeOperacao.Id : null;
        }

        private async Task<RespostaAPIModel> PostAsync(FaturamentoModel faturamento)
        {
            var uri = URIsFaturamentos.UriPost(faturamento);
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, uri, faturamento);
            return new RespostaAPIModel(retorno);
        }

        private async Task<RespostaAPIModel> PutAsync(FaturamentoModel faturamento)
        {
            var uri = URIsFaturamentos.UriPut((TipoFaturamento)faturamento.Tipo);
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Put, uri, faturamento);
            return new RespostaAPIModel(retorno);
        }
    }
}
