﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETBlazor.ViewModels.Usuario.Faturamento;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using System;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services
{
    public class EnvioFaturamentoService : IEnvioFaturamentoService
    {
        private readonly ICriaAlteraFaturamentoService _criaAlteraFaturamentoService;
        private readonly IDuplicacaoFaturamentoService _duplicacaoFaturamentoService;
        private readonly ICancelamentoFaturamentoService _cancelamentoFaturamentoService;
        private readonly IInutilizacaoFaturamentoService _inutilizacaoFaturamentoService;
        private readonly IEstornoFaturamentoService _estornoFaturamentoService;
        private readonly INFeService _nFeService;
        
        public EnvioFaturamentoService(ICriaAlteraFaturamentoService criaAlteraFaturamentoService, 
            IDuplicacaoFaturamentoService duplicacaoFaturamentoService,
            ICancelamentoFaturamentoService cancelamentoFaturamentoService,
            IInutilizacaoFaturamentoService inutilizacaoFaturamentoService,
            IEstornoFaturamentoService estornoFaturamentoService,
            INFeService nFeService)
        {
            _criaAlteraFaturamentoService = criaAlteraFaturamentoService;
            _duplicacaoFaturamentoService = duplicacaoFaturamentoService;
            _cancelamentoFaturamentoService = cancelamentoFaturamentoService;
            _inutilizacaoFaturamentoService = inutilizacaoFaturamentoService;
            _estornoFaturamentoService = estornoFaturamentoService;
            _nFeService = nFeService;
        }
        public async Task<ResultadoModel> CriarAlterarAsync(IFaturamentoBaseVM viewModel)
        {
            viewModel.LogEnvioFaturamento += "Enviando documento para gravação...";
            var faturamento = viewModel.Faturamento;
            var resultEnvio = await _criaAlteraFaturamentoService.CriarAlterarAsync(faturamento);
            if (!resultEnvio.Sucesso)
            {
                viewModel.LogEnvioFaturamento += $"\r\n {resultEnvio.Retorno}";
                return new ResultadoModel { Sucesso = false, Retorno = $"Erro ao enviar o faturamento. Erro: {resultEnvio.Retorno}" };
            }

            var fatRetorno = (FaturamentoModel)resultEnvio.Retorno;
            viewModel.LogEnvioFaturamento += $"\r\n Cadastro realizado com sucesso com o número {fatRetorno.Numero}, tipo {fatRetorno.DescricaoTipoFaturamento}";
            faturamento.Numero = fatRetorno.Numero;
            faturamento.Id = fatRetorno.Id;

            ResultadoModel retornoProcesso = new() { Sucesso = true, Retorno = "OK" };

            if (viewModel is IFaturamentoVM && faturamento.Tipo == (int)TipoFaturamento.NotaFiscal && faturamento.Formulario.NFEletronica)
            {
                try
                {
                    retornoProcesso = await EnvioNFe.Enviar(viewModel, _nFeService, faturamento);
                    viewModel.Enviando = false;
                }
                catch (Exception e)
                {
                    viewModel.LogEnvioFaturamento += $"\r\nOcorreu um erro ao tentar enviar a NFe. Erro: { e.GetBaseException().Message }";
                    return retornoProcesso;
                }
            }
            return retornoProcesso;
        }

        public async Task<ResultadoModel> DuplicarAsync(FaturamentoModel faturamento)
        {
            return await _duplicacaoFaturamentoService.EnviarDuplicacaoAsync(faturamento);
        }

        public async Task<ResultadoModel> CancelarAsync(long idFaturametno, TipoFaturamento tipoFaturamento)
        {
            return await _cancelamentoFaturamentoService.EnviarCancelamentoAsync(idFaturametno, tipoFaturamento);
        }

        public async Task<ResultadoModel> InutilizarAsync(long idFaturamento)
        {
            return await _inutilizacaoFaturamentoService.EnviarInutilizacaoAsync(idFaturamento);
        }

        public async Task<ResultadoModel> EstornarAsync(long idFaturamento, string motivoEstorno)
        {
            return await _estornoFaturamentoService.EnviarEstornoAsync(idFaturamento, motivoEstorno);
        }
    }
}
