﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Hosting;
using System;
using System.IO;

namespace GAFNETBlazor.Services
{
    public class EnvioXMLService : IEnvioXMLService
    {
        private string _cnpjEmpresa;
        private readonly IWebHostEnvironment _env;
        public EnvioXMLService(IWebHostEnvironment env)
        {
            _env = env;
        }
        public ResultadoModel Enviar(string cnpjEmpresa, MemoryStream memoryStream, string fileName)
        {
            if (string.IsNullOrEmpty(cnpjEmpresa)) return new ResultadoModel { Sucesso = false, Retorno = "Não foi possível obter o CNPJ da empresa logada" };
            _cnpjEmpresa = cnpjEmpresa;
            if (Path.GetExtension(fileName).ToLower() != ".xml") return new ResultadoModel { Sucesso = false, Retorno = "arquivo inválido" };

            var resultArquivo = CriarArquivo(memoryStream, fileName);
            if (!resultArquivo.Sucesso) return resultArquivo;
            string nomeArquivo = (string)resultArquivo.Retorno;

            return new ResultadoModel { Sucesso = true, Retorno = nomeArquivo };
        }

        private ResultadoModel CriarArquivo(MemoryStream memoryStream, string fileName)
        {
            var resultDiretorio = CriarDiretorio();
            if (!resultDiretorio.Sucesso) return resultDiretorio;
            string diretorio = (string)resultDiretorio.Retorno;
            var nomeArquivo = $@"{diretorio}\{fileName}";

            try
            {
                using var fs = new FileStream(nomeArquivo, FileMode.Create, FileAccess.Write);
                memoryStream.WriteTo(fs);
                fs.Flush();
            }
            catch (Exception e)
            {
                return new ResultadoModel { Sucesso = false, Retorno = $"Não foi possível criar o arquivo {nomeArquivo}. Erro: {e.GetBaseException().Message}" };
            }

            return new ResultadoModel { Sucesso = true, Retorno = nomeArquivo };
        }

        private ResultadoModel CriarDiretorio()
        {
            var pathRaiz = $@"{_env.WebRootPath}\files\NFe\Compras\{_cnpjEmpresa}";
            try
            {
                if (!Directory.Exists(pathRaiz))
                {
                    Directory.CreateDirectory(pathRaiz);
                }
                return new ResultadoModel { Sucesso = true, Retorno = pathRaiz };
            }
            catch (Exception e)
            {
                return new ResultadoModel { Sucesso = false, Retorno = $"Não foi possível criar o diretório {pathRaiz}. Erro: {e.GetBaseException().Message}" };
            }
        }
    }
}
