﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETLibrary.Models;
using Newtonsoft.Json;
using System;
using System.Net.Http;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services
{
    public class EstornoFaturamentoService : IEstornoFaturamentoService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public EstornoFaturamentoService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }
        public async Task<ResultadoModel> EnviarEstornoAsync(long idFaturamento, string motivoEstorno)
        {
            var uri = URIsFaturamentos.PostEstornoNFeAsync;
            uri = new Uri(uri + $"?idLancamentoOrigem={idFaturamento}&motivoEstorno={motivoEstorno}");
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, uri, null);
            var respostaAPI = new RespostaAPIModel(retorno);

            if (!respostaAPI.Sucesso)
            {
                return new ResultadoModel { Sucesso = false, Retorno = respostaAPI.MensagemErro };
            }

            var faturamentoRetorno = JsonConvert.DeserializeObject<FaturamentoModel>(respostaAPI.ConteudoRetorno);
            return new ResultadoModel { Sucesso = true, Retorno = faturamentoRetorno };
        }
    }
}
