﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Syncfusion.Blazor.Spinner;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services
{
    public class ExibicaoFaturamentoService : IExibicaoFaturamentoService
    {
        private readonly ITimeZoneService _timeZoneService;
        private readonly IFaturamentosService _faturamentosService;

        public ExibicaoFaturamentoService(ITimeZoneService timeZoneService, IFaturamentosService faturamentosService)
        {
            _timeZoneService = timeZoneService;
            _faturamentosService = faturamentosService;
        }
        public async Task ExibirByNumeroAndSerieAndTipo(IFaturamentoBaseVM viewModel)
        {
            if (viewModel.Faturamento.Numero == 0 || viewModel.Faturamento.Serie == 0 || viewModel.Faturamento.Formulario == null)
            {
                await viewModel.LimparTela();
                if (viewModel is IFaturamentoVM vM)
                {
                    await vM.PreencherFormularioPadrao();
                }

                return;
            }

            if (viewModel.Faturamento.Numero == 0) return;

            RespostaAPIModel resposta;

            var spinner = GetSpinner(viewModel);

            try
            {
                viewModel.Localizando = true;
                await spinner.ShowAsync();
                resposta = await _faturamentosService.GetByNumeroAndSerieAndTipoAsync(viewModel.Faturamento.Numero, viewModel.Faturamento.Serie, (TipoFaturamento)viewModel.Faturamento.Tipo);

                if (!resposta.Sucesso)
                {
                    await viewModel.ExibirMensagemToast(resposta.MensagemErro, TipoToast.Erro);
                    return;
                }
                viewModel.Faturamento = (FaturamentoModel)resposta.Objeto;
                Preparar(viewModel);
            }
            finally
            {
                await spinner.HideAsync();
                viewModel.Localizando = false;
            }
        }

        public async Task ExibirById(IFaturamentoBaseVM viewModel, long id, int tipoFaturamento)
        {
            if (id == 0)
            {
                await viewModel.LimparTela();
                if (viewModel is IFaturamentoVM vM)
                {
                    await vM.PreencherFormularioPadrao();
                }

                return;
            }

            RespostaAPIModel resposta;

            var spinner = GetSpinner(viewModel);
            try
            {
                viewModel.Localizando = true;
                if (spinner != null)
                {
                    await spinner.ShowAsync();
                }
                
                resposta = await _faturamentosService.GetByIdAndTipoAsync(id, (TipoFaturamento)tipoFaturamento);

                if (!resposta.Sucesso)
                {
                    await viewModel.ExibirMensagemToast(resposta.MensagemErro, TipoToast.Erro);
                    return;
                }
                viewModel.Faturamento = (FaturamentoModel)resposta.Objeto;
                Preparar(viewModel);
            }
            finally
            {
                if (spinner != null)
                {
                    await spinner.HideAsync();
                }
                viewModel.Localizando = false;
            }
        }

        public async Task ExibirByFaturamento(IFaturamentoBaseVM viewModel, FaturamentoModel faturamento)
        {
            if (faturamento is null || faturamento.Id == 0)
            {
                await viewModel.LimparTela();
                if (viewModel is IFaturamentoVM vM)
                {
                    await vM.PreencherFormularioPadrao();
                }

                return;
            }

            try
            {
                viewModel.Faturamento = faturamento;
                Preparar(viewModel);
            }
            finally
            {
                viewModel.Localizando = false;
            }
        }

        private void Preparar(IFaturamentoBaseVM viewModel)
        {
            viewModel.Faturamento.DtHrLancamento = _timeZoneService.GetLocalDateTime(viewModel.Faturamento.DtHrLancamento);

            viewModel.AoMudarPropriedade("Faturamento");
        }

        private static SfSpinner GetSpinner(IBaseVM viewModel)
        {
            return viewModel switch
            {
                var tipo when tipo is IFaturamentoVM => tipo.SpinnerPrincipal,
                var tipo when tipo is ICompraVM => tipo.SpinnerPrincipal,
                var tipo when tipo is IPesquisaFaturamentoVM => tipo.SpinnerPrincipal,
                _ => new SfSpinner()
            };
        }
    }
}
