﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using AutoMapper;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class FaturamentoBaseService : IFaturamentoBaseService
    {
        const string expandGetById = "&$expand=pessoa($select=id,nome,cnpj,inscestadual;$expand=enderecoprincipal($expand=cidade,pais),emails,telefones),historico($select=id,descricao,gerafinanceiro,finnfe,tiponf,entradasaida,geraestoque,geracomissao)" +
                                     ",itens($expand=produto($select=id,codigo,nome,vlrvenda,tipo,idunidade,idclassificacaofiscal,idaliquota;$expand=unidade,classificacaofiscal,aliquota))" +
                                     ",transportadora($select=id,nome,cnpj),vendedores($expand=vendedor($select=id,nome,cnpj)),centrodecusto($select=id,nome),tipodeoperacao($select=id,descricao)" +
                                     ",condicaodepagamento($select=id,descricao,descricaoparcelas),planodeconta($select=id,descricao),endereco($expand=cidade,pais)," +
                                     "enderecoentrega($expand=cidade,pais),formulario($select=id,nome,Ativo,NFEletronica,EhImpresso)";

        private readonly IHttpClientComToken _httpClientComToken;
        private readonly IMapper _mapper;
        
        public FaturamentoBaseService(IHttpClientComToken httpClientComToken, IMapper mapper)
        {
            _httpClientComToken = httpClientComToken;
            _mapper = mapper;
        }

        public async Task<RespostaAPIModel> CalculaAsync(FaturamentoModel faturamento)
        {
            FaturamentoDTO fatDTO;
            try
            {
                fatDTO = _mapper.Map<FaturamentoDTO>(faturamento);
            }
            catch (Exception e)
            {

                throw;
            }
            
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsFaturamentos.PostCalculoFaturamentoAsync, fatDTO);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> GetByIdAndTipoAsync(long id, TipoFaturamento tipo)
        {
            string select = $"?$Count=true&$Filter=id eq {id} and Tipo eq {(int)tipo}";
            select += expandGetById;

            var uri = URIsFaturamentos.UriGet(tipo);
            var resposta = await _httpClientComToken.GetResposta(new Uri(uri, select));

            var respostaAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }

            var faturamentos = JsonConvert.DeserializeObject<List<FaturamentoModel>>(respostaAPI.ConteudoRetorno);
            if (!faturamentos.Any())
            {
                respostaAPI.Sucesso = false;
                respostaAPI.MensagemErro = $"Não foi encontrado um documento com o id {id}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            FaturamentoModel faturamento = faturamentos.FirstOrDefault();

            if (faturamento.Transportadora == null) faturamento.Transportadora = new PessoaModel();
            if (faturamento.CondicaoDePagamento == null) faturamento.CondicaoDePagamento = new CondicaoDePagamentoModel();
            if (faturamento.TipoDeOperacao == null) faturamento.TipoDeOperacao = new TipoDeOperacaoModel();
            if (faturamento.CentroDeCusto == null) faturamento.CentroDeCusto = new CentroDeCustoModel();
            if (faturamento.PlanoDeConta == null) faturamento.PlanoDeConta = new PlanoDeContaModel();

            respostaAPI.Objeto = faturamento;
            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetOcorrenciasAsync(FaturamentoModel faturamento)
        {
            var uri = URIsFaturamentos.UriOcorrencias((TipoFaturamento)faturamento.Tipo);
            uri = new Uri(uri + $"?idFaturamento={faturamento.Id}");
            var retorno = await _httpClientComToken.GetResposta(uri);
            return new RespostaAPIModel(retorno);
        }
    }
}
