﻿using AutoMapper;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services
{
    public class FaturamentosService : FaturamentoBaseService, IFaturamentosService
    {
        const string expandGetById = "&$expand=pessoa($select=id,nome,cnpj;$expand=enderecos($expand=cidade,pais)),historico($select=id,descricao,gerafinanceiro), itens($expand=produto($select=id,codigo,tipo,nome,vlrvenda,CodigoANP))" +
                                     ",transportadora($select=id,nome,cnpj),vendedores($expand=vendedor($select=id,nome,cnpj)),centrodecusto($select=id,nome),tipodeoperacao($select=id,descricao)" +
                                     ",condicaodepagamento($select=id,descricao,descricaoparcelas),planodeconta($select=id,descricao),endereco($select=id;$expand=cidade),formulario($select=id,nome)";
        private readonly IHttpClientComToken _httpClientComToken;
        private readonly IMapper _mapper;
        public FaturamentosService(IHttpClientComToken httpClientComToken, IMapper mapper) : base(httpClientComToken, mapper)
        {
            _httpClientComToken = httpClientComToken;
            _mapper = mapper;
        }

        public async Task<RespostaAPIModel> GetByNumeroAndSerieAndTipoAsync(int numero, int serie, TipoFaturamento tipo)
        {
            string select = $"?$Filter=numero eq {numero} and serie eq {serie} and tipo eq {(int)tipo} &$Count=true";
            select += expandGetById;

            var uri = tipo switch
            {
                TipoFaturamento.PedidoDeVenda => URIsFaturamentos.GetPedidosAsync,
                TipoFaturamento.OrdemDeServico => URIsFaturamentos.GetOrdensDeServicoAsync,
                TipoFaturamento.Orcamento => URIsFaturamentos.GetOrcamentosAsync,
                TipoFaturamento.NotaFiscal => URIsFaturamentos.GetNotasFiscaisAsync,
                TipoFaturamento.NotaCompra => throw new NotImplementedException(),
                TipoFaturamento.PedidoDeCompra => throw new NotImplementedException(),
                _ => throw new NotImplementedException()
            };

            var resposta = await _httpClientComToken.GetResposta(new Uri(uri, select));

            var respostaAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }

            var faturamentos = JsonConvert.DeserializeObject<List<FaturamentoModel>>(respostaAPI.ConteudoRetorno);
            if (!faturamentos.Any())
            {
                respostaAPI.Sucesso = false;
                respostaAPI.MensagemErro = tipo switch
                {
                    TipoFaturamento.PedidoDeVenda => $"Não foi encontrado um pedido com o número {numero}.",
                    TipoFaturamento.OrdemDeServico => $"Não foi encontrada uma OS com o número {numero}.",
                    TipoFaturamento.Orcamento => $"Não foi encontrado um orçamento com o número {numero}.",
                    TipoFaturamento.NotaFiscal => $"Não foi encontrada uma nota fiscal com o número {numero} e série {serie}.",
                    _ => "Tipo de documento desconhecido"
                };

                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            FaturamentoModel faturamento = faturamentos.FirstOrDefault();
            if (faturamento.Transportadora == null) faturamento.Transportadora = new PessoaModel();
            respostaAPI.Objeto = faturamento;

            return respostaAPI;
        }
    }
}
