﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class FormulariosService : IFormulariosService
    {
        public FormularioModel FormularioPadrao { get; set; } = new FormularioModel();

        private readonly IHttpClientComToken _httpClientComToken;
        public FormulariosService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase + "?$Select=id,nome,serie,tipo,ativo&$OrderBy=nome";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }
            select += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            var retornoAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                retornoAPI.Status = resposta.StatusCode;
                return retornoAPI;
            }

            var formularios = JsonConvert.DeserializeObject<List<FormularioModel>>(retornoAPI.ConteudoRetorno);
            retornoAPI.Objeto = formularios;
            retornoAPI.Status = formularios.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            return retornoAPI;
        }
        
        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true&$Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            RespostaAPIModel respostaAPI = new (resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }
            
            
            var formularios = JsonConvert.DeserializeObject<List<FormularioModel>>(respostaAPI.ConteudoRetorno);
            if (!formularios.Any())
            {
                respostaAPI.Sucesso = false;
                respostaAPI.MensagemErro = $"Não foi encontrado um formulário com o id {id}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            respostaAPI.Objeto = formularios.FirstOrDefault();

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetPadraoAsync(Uri uriBase)
        {
            if (FormularioPadrao.Id > 0)
            {
                return new RespostaAPIModel { Sucesso = true, Objeto = FormularioPadrao };
            }
            
            string select = uriBase + $"?$Count=true&$Filter=ativo eq true and padrao eq true";

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            RespostaAPIModel respostaAPI = new (resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }
                
            var formularios = JsonConvert.DeserializeObject<List<FormularioModel>>(respostaAPI.ConteudoRetorno);
            if (!formularios.Any())
            {
                respostaAPI.Sucesso = false;
                respostaAPI.MensagemErro = $"Não foi encontrado um formulário padrão ativo.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            FormularioPadrao = formularios.FirstOrDefault();
            respostaAPI.Objeto = FormularioPadrao;

            return respostaAPI;
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            throw new NotImplementedException();
        }

        public async Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsFormularios.PostAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> PutAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Put, URIsFormularios.PutAsync, entity);
            return new RespostaAPIModel(retorno);
        }
    }
}
