﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class GenerosProdutoService : IGenerosProdutoService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public GenerosProdutoService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase + "?$Select=id,codigo,descricao,ativo&$OrderBy=descricao";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }
            select += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            var retornoAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return retornoAPI;
            }
            
            var generos = JsonConvert.DeserializeObject<List<GeneroProdutoModel>>(retornoAPI.ConteudoRetorno);
            retornoAPI.Objeto = generos;
            retornoAPI.Status = generos.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            return retornoAPI;
        }
        
        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true&$Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            RespostaAPIModel respostaAPI = new (resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }

            var generos = JsonConvert.DeserializeObject<List<GeneroProdutoModel>>(respostaAPI.ConteudoRetorno);
            if (!generos.Any())
            {
                respostaAPI.Sucesso = false;
                respostaAPI.MensagemErro = $"Não foi encontrado um gênero com o id {id}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            respostaAPI.Objeto = generos.FirstOrDefault();

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetByCodigoAsync(Uri uriBase, int codigo, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true&$Filter=codigo eq {codigo}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            RespostaAPIModel respostaAPI = new (resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }

            var generos = JsonConvert.DeserializeObject<List<GeneroProdutoModel>>(respostaAPI.ConteudoRetorno);
            if (!generos.Any())
            {
                respostaAPI.Sucesso = false;
                respostaAPI.MensagemErro = $"Não foi encontrado um gênero com o código {codigo}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            respostaAPI.Objeto = generos.FirstOrDefault();

            return respostaAPI;
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            throw new NotImplementedException();
        }

        public async Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsGenerosProduto.PostAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> PutAsync<T>(T entity)
        {

            var retorno = await _httpClientComToken.Enviar(HttpMethod.Put, URIsGenerosProduto.PutAsync, entity);
            return new RespostaAPIModel(retorno);
        }
    }
}
