﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class HistoricosService : IHistoricosService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public HistoricosService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase + "?$Select=id,descricao,entradasaida,gerafinanceiro,geraestoque,ativo&$OrderBy=descricao";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }
            select += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            var retornoAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return retornoAPI;
            }
            
            var historicos = JsonConvert.DeserializeObject<List<HistoricoModel>>(retornoAPI.ConteudoRetorno);
            retornoAPI.Objeto = historicos;
            
            retornoAPI.Status = historicos.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            return retornoAPI;
        }

        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true & $Expand=HistoricoEstorno($Select=id,descricao,entradasaida) & $Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            var respostaAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }
            
            var historicos = JsonConvert.DeserializeObject<List<HistoricoModel>>(respostaAPI.ConteudoRetorno);
            if (!historicos.Any())
            {
                respostaAPI.Sucesso = false;
                respostaAPI.MensagemErro = $"Não foi encontrado um histórico com o id {id}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            respostaAPI.Objeto = historicos.FirstOrDefault();

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            string select = uriBase + "?$Count=true";
            select = !string.IsNullOrEmpty(campos) ? select += $"&$select={campos}" : select;
            select = !string.IsNullOrEmpty(filtros) ? select += $"&$filter={filtros}" : select;
            select = !string.IsNullOrEmpty(order) ? select += $"&$orderby={order}" : select;
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            RespostaAPIModel respostaAPI = new(resposta);
            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }
            
            var historicos = JsonConvert.DeserializeObject<List<HistoricoModel>>(respostaAPI.ConteudoRetorno);
            respostaAPI.Objeto = historicos;
            respostaAPI.Status = historicos.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsHistoricos.PostAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> PutAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Put, URIsHistoricos.PutAsync, entity);
            return new RespostaAPIModel(retorno);
        }
    }
}
