﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using Newtonsoft.Json;
using System;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services
{
    public class HttpClientComToken : IHttpClientComToken
    {
        private readonly IHttpClientFactory _clientFactory;

        ILoginStore LoginStore { get; set; }
        public HttpClient HttpClient { get; set; }

        public async Task<HttpResponseMessage> Enviar(HttpMethod HttpMetodo, Uri uri, object objeto)
        {
            return await EnviarComMetodo(HttpMetodo, uri, objeto);
        }

        public async Task<HttpResponseMessage> Enviar(HttpMethod HttpMetodo, Uri uri, object objeto, string mediaType)
        {
            return await EnviarComMetodo(HttpMetodo, uri, objeto, mediaType);
        }

        private async Task<HttpResponseMessage> EnviarComMetodo(HttpMethod HttpMetodo, Uri uri, object objeto, string mediaType = "application/json")
        {
            ConfigurarHeaders(mediaType);
            var conteudo = GetConteudoObjeto(objeto, mediaType);

            return (HttpMetodo.Method.ToUpper()) switch
            {
                "POST" => await HttpClient.PostAsync(uri, conteudo),
                "PUT" => await HttpClient.PutAsync(uri, conteudo),
                _ => null,
            };
        }

        private HttpContent GetConteudoObjeto(object objeto, string mediaType)
        {
            if (mediaType == "multipart/form-data")
            {
                return (HttpContent)objeto;
            }
            string contentObjeto = mediaType == "application/json" ? JsonConvert.SerializeObject(objeto) : (string)objeto;
            return new StringContent(contentObjeto, Encoding.UTF8, mediaType);
        }

        private void ConfigurarHeaders(string mediaType)
        {
            HttpClient.DefaultRequestHeaders.Clear();
            HttpClient.DefaultRequestHeaders.Accept.Clear();
            HttpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue(mediaType));

            PassarDadosLoginStore(); // Feito por causa do ReportViewerController que executa outro escopo e limpa os dados da LoginStore

            HttpClient.DefaultRequestHeaders.Add("Authorization", "bearer " + Controllers.ReportViewerController.Token);

            if (LoginStore.EmpresaLogada != null)
            {
                HttpClient.DefaultRequestHeaders.Add("CNPJEmpresaLogada", LoginStore.EmpresaLogada.CNPJ);
            }

        }

        private void PassarDadosLoginStore()
        {
            if (!string.IsNullOrEmpty(LoginStore.Token))
            {
                Controllers.ReportViewerController.Token = LoginStore.Token;
                Controllers.ReportViewerController.EmpresaLogada = LoginStore.EmpresaLogada;
                Controllers.ReportViewerController.EmpresasUsuario.Clear();
                Controllers.ReportViewerController.EmpresasUsuario.AddRange(LoginStore.EmpresasUsuario);
            }
            else
            {
                LoginStore.Token = Controllers.ReportViewerController.Token;
                LoginStore.EmpresaLogada = Controllers.ReportViewerController.EmpresaLogada;
                LoginStore.EmpresasUsuario.Clear();
                LoginStore.EmpresasUsuario.AddRange(Controllers.ReportViewerController.EmpresasUsuario);
            }
        }

        public async Task<HttpResponseMessage> GetResposta(Uri uri)
        {
            ConfigurarHeaders("application/json");
            return await HttpClient.GetAsync(uri);
        }

        public HttpClientComToken(IHttpClientFactory clientFactory, ILoginStore loginStore)
        {
            _clientFactory = clientFactory;
            LoginStore = loginStore;
            HttpClient = _clientFactory.CreateClient();
        }


    }
}
