﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Services.Interfaces;
using GAFNETLibrary.Models;
using Newtonsoft.Json;
using System;
using System.IO;
using System.Net.Http;
using System.Threading.Tasks;
using System.Xml;

namespace GAFNETBlazor.Services
{
    public class NotaDeXMLService : INotaDeXMLService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        private readonly IEnvioXMLService _envioXMLService;
        public NotaDeXMLService(IHttpClientComToken httpClientComToken, IEnvioXMLService envioXMLService)
        {
            _httpClientComToken = httpClientComToken;
            _envioXMLService = envioXMLService;
        }
        public async Task<ResultadoModel> GetFaturamentoAsync(string cnpjEmpresa, MemoryStream memoryStream, string fileName, DateTime? dtEntrada, long? idHistorico)
        {
            var resultEnviaXML = _envioXMLService.Enviar(cnpjEmpresa, memoryStream, fileName);
            if (!resultEnviaXML.Sucesso) return new ResultadoModel { Sucesso = false, Retorno = (string)resultEnviaXML.Retorno };

            var doc = new XmlDocument();
            var nomeArquivo = (string)resultEnviaXML.Retorno;
            doc.Load(nomeArquivo);
            var uri = URIsNFe.NotaDeXML;
            string dadosAdicionais = idHistorico.HasValue ? $"?idHistorico={idHistorico}" : "";
            if (dtEntrada.HasValue)
            {
                dadosAdicionais += !string.IsNullOrEmpty(dadosAdicionais) ? $"&dtEntrada={dtEntrada:yyyy/MM/dd}" : $"?dtEntrada={dtEntrada:yyyy/MM/dd}";
            }
            uri = new Uri(uri + dadosAdicionais);

            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, uri, doc.OuterXml, "application/xml");
            var resposta = new RespostaAPIModel(retorno);
            if (!resposta.Sucesso)
            {
                return new ResultadoModel { Sucesso = false, Retorno = resposta.MensagemErro };
            }
            var nota = JsonConvert.DeserializeObject<FaturamentoDTO>(resposta.ConteudoRetorno);
            return new ResultadoModel { Sucesso = true, Retorno = nota };
        }
    }
}
