﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services
{
    public class PagarReceberService : IPagarReceberService
    {
        const string expandDetalhamentos = "$expand=centrodecusto($select=id,nome),tipodeoperacao($select=id,descricao),planodeconta($select=id,descricao)";
        const string expandPessoa = "$expand=pessoa($select = id, nome, cnpj)";
        const string expandItens = $"itens($expand = detalhamentospagarreceber({expandDetalhamentos}), baixasPagarReceber($expand=itensMovCaixa))";

        const string expandGetById = $"&{expandPessoa},{expandItens}";
        private readonly IHttpClientComToken _httpClientComToken;
        public PagarReceberService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            throw new NotImplementedException();
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            throw new NotImplementedException();
        }

        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            string select = $"?$Filter=id eq {id} &$Count=true";
            select += expandGetById;



            var uri = URIsPagarReceber.GetAsync;

            var resposta = await _httpClientComToken.GetResposta(new Uri(uri, select));

            var respostaAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }

            var contasPagarReceber = JsonConvert.DeserializeObject<List<PagarReceberModel>>(respostaAPI.ConteudoRetorno);
            if (contasPagarReceber.Any())
            {
                respostaAPI.Sucesso = false;
                respostaAPI.MensagemErro = $"Não foi encontrado um documento com o Id {id}";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            PagarReceberModel pagarReceber = contasPagarReceber.FirstOrDefault();
            respostaAPI.Objeto = pagarReceber;

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetDocumento(Uri uriBase, int tipo, long idPessoa, string nDocto, string serieDocto)
        {
            string select = $"?$Filter=tipo eq {tipo} and idpessoa={idPessoa} and serie eq {serieDocto} &$Count=true";
            select += expandGetById;

            var resposta = await _httpClientComToken.GetResposta(new Uri(uriBase, select));

            var respostaAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }

            var contasPagarReceber = JsonConvert.DeserializeObject<List<PagarReceberModel>>(respostaAPI.ConteudoRetorno);
            if (contasPagarReceber.Any())
            {
                respostaAPI.Sucesso = false;
                respostaAPI.MensagemErro = $"Não foi encontrado um documento com o número {nDocto}, série {serieDocto} para a pessoa de código {idPessoa}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            PagarReceberModel pagarReceber = contasPagarReceber.FirstOrDefault();
            respostaAPI.Objeto = pagarReceber;

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsPagarReceber.PostAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> PutAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Put, URIsPagarReceber.PutAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> CriarAlterarAsync<T>(T entity)
        {
            if ((entity as PagarReceberModel).Id == 0)
            {
                return await PostAsync(entity);
            }

            return await PutAsync(entity);

        }

        public async Task<RespostaAPIModel> GerarItensAsync(PagarReceberModel pagarReceber)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsPagarReceber.GeracaoItensAsync, pagarReceber);
            return new RespostaAPIModel(retorno);
        }
    }
}
