﻿using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class PaisesService : IPaisesService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public PaisesService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase + "?$Select=id,nome,codigo,ativo&$OrderBy=nome";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            var retornoAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return retornoAPI;
            }

            var paises = JsonConvert.DeserializeObject<List<PaisModel>>(retornoAPI.ConteudoRetorno);
            retornoAPI.Objeto = paises;
            retornoAPI.Status = paises.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            return retornoAPI;
        }

        public async Task<RespostaAPIModel> GetByCodigoAsync(Uri uriBase, int codigo, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true & $Filter=codigo eq {codigo}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            RespostaAPIModel respostaAPI = new(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }
            
            var paises = JsonConvert.DeserializeObject<List<PaisModel>>(respostaAPI.ConteudoRetorno);
            if (!paises.Any())
            {
                respostaAPI.MensagemErro = $"Não foi encontrado um país com o código {codigo}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            respostaAPI.Sucesso = true;
            respostaAPI.Objeto = paises.FirstOrDefault();

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetPadraoAsync(Uri uriBase)
        {
            string select = uriBase + $"?$Count=true & $Filter=codigo eq {1058}";

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            RespostaAPIModel respostaAPI = new(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                respostaAPI.MensagemErro = await resposta.Content.ReadAsStringAsync();
                return respostaAPI;
            }
            
            var paises = JsonConvert.DeserializeObject<List<PaisModel>>(respostaAPI.ConteudoRetorno);
            
            if (!paises.Any())
            {
                respostaAPI.MensagemErro = $"Não foi encontrado um país com o código {1058}.";
                return respostaAPI;
            }

            respostaAPI.Sucesso = true;
            respostaAPI.Objeto = paises.FirstOrDefault();

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true & $Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            RespostaAPIModel respostaAPI = new(resposta);


            if (!resposta.IsSuccessStatusCode)
            {
                respostaAPI.MensagemErro = await resposta.Content.ReadAsStringAsync();
                respostaAPI.Status = resposta.StatusCode;
                return respostaAPI;
            }
            
            var paises = JsonConvert.DeserializeObject<List<PaisModel>>(respostaAPI.ConteudoRetorno);
            
            if (!paises.Any())
            {
                respostaAPI.MensagemErro = $"Não foi encontrado um país com o id {id}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            respostaAPI.Sucesso = true;
            respostaAPI.Objeto = paises.FirstOrDefault();
            return respostaAPI;
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            throw new NotImplementedException();
        }

        public Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            throw new NotImplementedException();
        }

        public Task<RespostaAPIModel> PutAsync<T>(T entity)
        {
            throw new NotImplementedException();
        }
    }
}
