﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Net;
using System.Threading.Tasks;
using GAFNETLibrary.Models;
using System.Linq;

namespace GAFNETBlazor.Services
{
    public class PesquisasFaturamentoService : IPesquisasFaturamentoService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public PesquisasFaturamentoService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(PesquisaFaturamentoModel parametros)
        {
            string select = "?$Select=Id,Numero,DtHrLancamento,Valor,Tipo,Serie,DtCancelamento,ChaveEletronica, IdEndereco, IdEnderecoEntrega, &$Count=true";

            string filter = $"&$filter=numero ge 0";
            if (parametros.DtHrLancamentoIni.HasValue) filter += $" and DtHrLancamento ge {parametros.DtHrLancamentoIni.Value:yyyy-MM-dd}";
            if (parametros.DtHrLancamentoFin.HasValue) filter += $" and DtHrLancamento le {parametros.DtHrLancamentoFin.Value:yyyy-MM-dd}";
            if (parametros.NumeroIni.HasValue) filter += $" and numero ge {parametros.NumeroIni.Value}";
            if (parametros.NumeroFin.HasValue) filter += $" and numero le {parametros.NumeroFin.Value}";
            if (parametros.Endereco.IdCidade > 0) filter += $" and enderecos/any(e: e/idcidade eq {parametros.Endereco.IdCidade})";
            if (parametros.Historico.Id > 0) filter += $" and IdHistorico eq {parametros.Historico.Id}";

            string strFiltroPessoa = "";
            if (!string.IsNullOrEmpty(parametros.NomePessoa))
            {
                strFiltroPessoa = (TipoPesquisaString)parametros.TipoPesquisaNomePessoa switch
                {
                    TipoPesquisaString.Exato => $" and pessoa/nome eq '{parametros.NomePessoa}')",
                    TipoPesquisaString.ComecaCom => $" and startswith(pessoa/nome, '{parametros.NomePessoa}')",
                    TipoPesquisaString.TerminaCom => $" and endswith(pessoa/nome, '{parametros.NomePessoa}')",
                    TipoPesquisaString.Contem => $" and substringof(pessoa/nome, '{parametros.NomePessoa}')",
                    _ => ""
                };
            }

            filter += strFiltroPessoa;

            string expand = $"&$expand=pessoa($select=id,cnpj,nome),endereco($expand=cidade),itens($select=qtdlancada,qtddevolvida,qtdretornada,qtdfaturada),formulario";
            expand += parametros.Tipo == (int)TipoFaturamento.NotaFiscal ? ",RetornosNFe($Select=CStat,NProt)" : "";

            select += expand;
            select += filter;

            if (parametros.QtdRegistros > 0) select += $"&$top={parametros.QtdRegistros}";

            select += String.IsNullOrEmpty(parametros.Ordem) ? "&$OrderBy=DtHrLancamento" : $"&$OrderBy={parametros.Ordem}";

            Uri uri = (TipoFaturamento)parametros.Tipo switch
            {
                TipoFaturamento.NotaCompra => URIsFaturamentos.GetComprasAsync,
                TipoFaturamento.PedidoDeVenda => URIsFaturamentos.GetPedidosAsync,
                TipoFaturamento.OrdemDeServico => URIsFaturamentos.GetOrdensDeServicoAsync,
                TipoFaturamento.Orcamento => URIsFaturamentos.GetOrcamentosAsync,
                TipoFaturamento.NotaFiscal => URIsFaturamentos.GetNotasFiscaisAsync,
                TipoFaturamento.PedidoDeCompra => URIsFaturamentos.GetPedidosDeCompraAsync,
                _ => throw new NotImplementedException()
            };

            var resposta = await _httpClientComToken.GetResposta(new Uri(uri, select));
            var respostaAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }

            var faturamentos = JsonConvert.DeserializeObject<List<FaturamentoModel>>(respostaAPI.ConteudoRetorno);
            respostaAPI.Objeto = faturamentos;
            respostaAPI.Status = faturamentos.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            return respostaAPI;
        }
    }
}
