﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Net;
using System.Threading.Tasks;
using GAFNETLibrary.Models;
using System.Linq;

namespace GAFNETBlazor.Services
{
    public class PesquisasPagarReceberService : IPesquisasPagarReceberService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public PesquisasPagarReceberService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(PesquisaPagarReceberModel parametros)
        {
            string select = "?$Select=Id,NDocto,DtEmissao,Valor,Tipo,SerieDocto, &$Count=true";

            string filter = $"&$filter=tipo eq {parametros.Tipo}";
            if (parametros.DtEmissaoIni.HasValue) filter += $" and DtEmissao ge {parametros.DtEmissaoIni.Value:yyyy-MM-dd}";
            if (parametros.DtEmissaoFin.HasValue) filter += $" and DtEmissao le {parametros.DtEmissaoFin.Value:yyyy-MM-dd}";
            if (!string.IsNullOrEmpty(parametros.NDocto)) filter += $" and numero eq {parametros.NDocto}";

            string strFiltroPessoa = "";
            if (!string.IsNullOrEmpty(parametros.NomePessoa))
            {
                strFiltroPessoa = (TipoPesquisaString)parametros.TipoPesquisaNomePessoa switch
                {
                    TipoPesquisaString.Exato => $" and pessoa/nome eq '{parametros.NomePessoa}')",
                    TipoPesquisaString.ComecaCom => $" and startswith(pessoa/nome, '{parametros.NomePessoa}')",
                    TipoPesquisaString.TerminaCom => $" and endswith(pessoa/nome, '{parametros.NomePessoa}')",
                    TipoPesquisaString.Contem => $" and substringof(pessoa/nome, '{parametros.NomePessoa}')",
                    _ => ""
                };
            }

            filter += strFiltroPessoa;

            string expand = $"&$expand=pessoa($select=id,cnpj,nome),endereco($expand=cidade)";
            select += expand;
            select += filter;

            if (parametros.QtdRegistros > 0) select += $"&$top={parametros.QtdRegistros}";

            select += string.IsNullOrEmpty(parametros.Ordem) ? "&$OrderBy=DtEmissao desc" : $"&$OrderBy={parametros.Ordem}";

            Uri uri = URIsPagarReceber.GetAsync;

            var resposta = await _httpClientComToken.GetResposta(new Uri(uri, select));
            var respostaAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }

            var contasPagarReceber = JsonConvert.DeserializeObject<List<PagarReceberModel>>(respostaAPI.ConteudoRetorno);
            respostaAPI.Objeto = contasPagarReceber;
            respostaAPI.Status = contasPagarReceber.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            return respostaAPI;
        }
    }
}
