﻿using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using GAFNETLibrary.Models;
using GAFNETBlazor.Constantes;
using System.Net.Http;

namespace GAFNETBlazor.Services
{
    public class PlanosDeContaService : IPlanosDeContaService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public PlanosDeContaService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase + "?$Select=id,descricao,tipo,debitocredito,classificacao1,classificacao2,classificacao3,ativo&$OrderBy=classificacao1,classificacao2,classificacao3,descricao";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }

            select += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            var retornoAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return retornoAPI;
            }
            
            
            var planos = JsonConvert.DeserializeObject<List<PlanoDeContaModel>>(retornoAPI.ConteudoRetorno);
            retornoAPI.Objeto = planos;
            retornoAPI.Status = planos.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            return retornoAPI;
        }
        
        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            string select = uriBase + $"?$Expand=ContaContrapartida & $Count=true & $Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            RespostaAPIModel respostaAPI = new(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                respostaAPI.MensagemErro = await resposta.Content.ReadAsStringAsync();
                respostaAPI.Status = resposta.StatusCode;
                return respostaAPI;
            }

            var planos = JsonConvert.DeserializeObject<List<PlanoDeContaModel>>(respostaAPI.ConteudoRetorno);
            if (!planos.Any())
            {
                respostaAPI.Sucesso = false;
                respostaAPI.MensagemErro = $"Não foi encontrado um plano de contas com o código {id}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            respostaAPI.Sucesso = true;
            respostaAPI.Objeto = planos.FirstOrDefault();

            return respostaAPI;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            string select = uriBase + $"?$Select={campos}";
            select = !string.IsNullOrEmpty(order) ? select += $"&$OrderBy= {order}" : select;
            select = !string.IsNullOrEmpty(filtros) ? select += $"&$Filter={filtros}" : select;
            select += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            var retornoAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return retornoAPI;
            }

            var planos = JsonConvert.DeserializeObject<List<PlanoDeContaModel>>(retornoAPI.ConteudoRetorno);
            retornoAPI.Objeto = planos;
            retornoAPI.Status = planos.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            return retornoAPI;
        }

        public async Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsPlanosDeConta.PostAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> PutAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Put, URIsPlanosDeConta.PutAsync, entity);
            return new RespostaAPIModel(retorno);
        }
    }
}
