﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;

namespace GAFNETBlazor.Services
{
    public class PreenchimentoCSTCFOPCompraService: IPreenchimentoCSTCFOPCompraService
    {
        private readonly ICFOPCompraService _cfopCompraService;
        private readonly ILoginStore _loginStore;
        public PreenchimentoCSTCFOPCompraService(ICFOPCompraService cfopCompraService, ILoginStore loginStore)
        {
            _cfopCompraService = cfopCompraService;
            _loginStore = loginStore;
        }
        public void Preencher(ICompraVM viewModel)
        {
            var itemFaturamento = viewModel.ItemFaturamento;
            if (itemFaturamento.Produto.Codigo is null) return;
            itemFaturamento.CST = itemFaturamento.Produto.Aliquota.CST;
            var resultCFOP = _cfopCompraService.Get(GetParamCFOP(viewModel, _loginStore));
            if (!resultCFOP.Sucesso) return;
            itemFaturamento.CFOP = (int)resultCFOP.Retorno;
        }

        private ParamCFOPCompraModel GetParamCFOP(ICompraVM viewModel, ILoginStore loginStore)
        {
            return new ParamCFOPCompraModel
            {
                Historico = viewModel.Faturamento.Historico,
                Item = viewModel.ItemFaturamento,
                OperacaoPresencial = viewModel.Faturamento.OperacaoPresencial,
                Pessoa = viewModel.Faturamento.Pessoa,
                UFEmpresa = loginStore.EmpresaLogada.Cidade.UF
            };
        }
    }
}
