﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class SubgruposProdutoService : ISubgruposProdutoService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public SubgruposProdutoService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase + "?$Select=id,nome,ativo&$expand=grupo($select=id,nome)&$OrderBy=nome";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }
            select += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            var retornoAPI = new RespostaAPIModel(resposta);

            List<SubgrupoProdutoModel> subgrupos = new ();
            if (resposta.IsSuccessStatusCode)
            {
                subgrupos = JsonConvert.DeserializeObject<List<SubgrupoProdutoModel>>(retornoAPI.ConteudoRetorno);
                retornoAPI.Objeto = subgrupos;
            }
            retornoAPI.Status = resposta.IsSuccessStatusCode ? subgrupos.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent : resposta.StatusCode;
            return retornoAPI;
        }
        
        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true&$expand=grupo($select=id,nome)&$Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            RespostaAPIModel respostaAPI = new (resposta);

            if (resposta.IsSuccessStatusCode)
            {
                var subgrupos = JsonConvert.DeserializeObject<List<SubgrupoProdutoModel>>(respostaAPI.ConteudoRetorno);
                if (subgrupos.Count == 0)
                {
                    respostaAPI.Sucesso = false;
                    respostaAPI.MensagemErro = $"Não foi encontrado um subgrupo de produto com o id {id}.";
                    respostaAPI.Status = HttpStatusCode.NotFound;
                    return respostaAPI;
                }

                respostaAPI.Objeto = subgrupos.FirstOrDefault();
            }

            return respostaAPI;
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            throw new NotImplementedException();
        }

        public async Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsSubgruposProduto.PostAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> PutAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Put, URIsSubgruposProduto.PutAsync, entity);
            return new RespostaAPIModel(retorno);
        }
    }
}
