﻿using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class TiposDeOperacaoService : ITiposDeOperacaoService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public TiposDeOperacaoService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            string select = uriBase + "?$OrderBy=descricao";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += "&$Filter=Ativo eq true" : select += "&$Filter=Ativo eq false";
            }
            select += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            var retornoAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return retornoAPI;
            }

            var tipos = JsonConvert.DeserializeObject<List<TipoDeOperacaoModel>>(retornoAPI.ConteudoRetorno);
            retornoAPI.Objeto = tipos;
            retornoAPI.Status = tipos.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            return retornoAPI;
        }
        
        public async Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            string select = uriBase + $"?$Count=true & $Filter=id eq {id}";

            if (status != StatusAtivo.Todos)
            {
                select = status == StatusAtivo.Ativos ? select += " and Ativo eq true" : select += " and Ativo eq false";
            }

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            RespostaAPIModel respostaAPI = new(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                respostaAPI.MensagemErro = await resposta.Content.ReadAsStringAsync();
                respostaAPI.Status = resposta.StatusCode;
                return respostaAPI;
            }
            
            var tipos = JsonConvert.DeserializeObject<List<TipoDeOperacaoModel>>(respostaAPI.ConteudoRetorno);
            
            if (!tipos.Any())
            {
                respostaAPI.Sucesso = false;
                respostaAPI.MensagemErro = $"Não foi encontrado um tipo de operação com o código {id}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }

            respostaAPI.Sucesso = true;
            respostaAPI.Objeto = tipos.FirstOrDefault();

            return respostaAPI;
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            throw new NotImplementedException();
        }

        public Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            throw new NotImplementedException();
        }

        public Task<RespostaAPIModel> PutAsync<T>(T entity)
        {
            throw new NotImplementedException();
        }
    }
}
