﻿using BlazorDownloadFile;
using GAFNETBlazor.Services.Interfaces;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Syncfusion.Blazor.Inputs;
using System;
using System.IO;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services;

public class TrocaArquivosService : ControllerBase, ITrocaArquivosService
{
    private readonly IWebHostEnvironment _env;
    private readonly IBlazorDownloadFileService _blazorDownloadFileService;
    private readonly IHttpClientComToken _httpClientComToken;

    public TrocaArquivosService(IWebHostEnvironment env, IBlazorDownloadFileService blazorDownloadFileService, IHttpClientComToken httpClientComToken)
    {
        _env = env;
        _blazorDownloadFileService = blazorDownloadFileService;
        _httpClientComToken = httpClientComToken;
    }
    public ResultadoModel Upload(UploadChangeEventArgs args)
    {
        foreach (var file in args.Files)
        {
            var resultArquivo = UploadArquivo(file);
            if (!resultArquivo.Sucesso) return resultArquivo;
            string nomeArquivo = (string)resultArquivo.Retorno;

            return new ResultadoModel { Sucesso = true, Retorno = nomeArquivo };
        };

        return new ResultadoModel { Sucesso = false, Retorno = "Não foram processados arquivos." };
    }

    private ResultadoModel UploadArquivo(UploadFiles file)
    {
        var resultDiretorio = PastaDestinoUpload();
        if (!resultDiretorio.Sucesso) return resultDiretorio;
        string diretorio = (string)resultDiretorio.Retorno;
        var nomeArquivo = $@"{diretorio}\{file.FileInfo.Name}";

        try
        {
            using var filestream = new FileStream(nomeArquivo, FileMode.Create, FileAccess.Write);
            file.Stream.WriteTo(filestream);
        }
        catch (Exception e)
        {
            return new ResultadoModel { Sucesso = false, Retorno = $"Não foi possível fazer o upload do arquivo {nomeArquivo}. Erro: {e.GetBaseException().Message}" };
        }

        return new ResultadoModel { Sucesso = true, Retorno = nomeArquivo };
    }

    private ResultadoModel PastaDestinoUpload()
    {
        var pathRaiz = $@"{_env.WebRootPath}\files\Uploads";
        try
        {
            if (!Directory.Exists(pathRaiz))
            {
                Directory.CreateDirectory(pathRaiz);
            }
            return new ResultadoModel { Sucesso = true, Retorno = pathRaiz };
        }
        catch (Exception e)
        {
            return new ResultadoModel { Sucesso = false, Retorno = $"Não foi possível criar o diretório {pathRaiz}. Erro: {e.GetBaseException().Message}" };
        }
    }

    public async Task<ResultadoModel> Download(Uri uri, string nomeArquivo)
    {
        var retorno = await _httpClientComToken.GetResposta(uri);
        if (!retorno.IsSuccessStatusCode)
        {
            var conteudoRetorno = await retorno.Content.ReadAsStringAsync();
            return new ResultadoModel { Sucesso = false, Retorno = conteudoRetorno };
        }

        try
        {
            string fileName = Path.GetFileName(nomeArquivo);
            byte[] ms = await retorno.Content.ReadAsByteArrayAsync();
            TimeSpan time = TimeSpan.FromSeconds(30);
            await _blazorDownloadFileService.DownloadFile(fileName, ms, 5000, "application/octet-stream");
            return new ResultadoModel { Sucesso = true, Retorno = fileName };

        }
        catch (Exception ex)
        {

            return new ResultadoModel { Sucesso = false, Retorno = ex.GetBaseException().Message };
        }
    }

    public async Task<ResultadoModel> Download(string caminhoArquivo)
    {
        try
        {
            if (!System.IO.File.Exists(caminhoArquivo))
            {
                return new ResultadoModel { Sucesso = false, Retorno = "Não foi encontrado o arquivo para download. Arquivo " + caminhoArquivo };
            }

            var bytes = System.IO.File.ReadAllBytes(caminhoArquivo);

            var nomeArquivo = Path.GetFileName(caminhoArquivo);
            await _blazorDownloadFileService.DownloadFile(nomeArquivo, bytes, "application/octet-stream");

            return new ResultadoModel { Sucesso = true, Retorno = caminhoArquivo };
        }
        catch (System.Exception e)
        {
            return new ResultadoModel { Sucesso = false, Retorno = "Ocorreu um erro ao tentar baixar o arquivo.Erro: " + e.GetBaseException().Message };
        }
    }
}
