﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Services
{
    public class UFICMSsService : IUFICMSsService
    {
        private readonly IHttpClientComToken _httpClientComToken;
        public UFICMSsService(IHttpClientComToken httpClientComToken)
        {
            _httpClientComToken = httpClientComToken;
        }

        public async Task<RespostaAPIModel> GetAsync(Uri uriBase)
        {
            string select = uriBase + "?$Select=id,uf,percentual&$OrderBy=uf";

            select += "&$count=true";
            var resposta = await _httpClientComToken.GetResposta(new Uri(select));

            var retornoAPI = new RespostaAPIModel(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return retornoAPI;
            }

            var ufs = JsonConvert.DeserializeObject<List<UFICMSModel>>(retornoAPI.ConteudoRetorno);
            retornoAPI.Objeto = ufs;
            retornoAPI.Status = ufs.Any() ? HttpStatusCode.OK : HttpStatusCode.NoContent;
            return retornoAPI;
        }
        
        public async Task<RespostaAPIModel> GetByUFAsync(Uri uriBase, string uf)
        {
            string select = uriBase + $"?$Count=true & $Filter=uf eq '{uf}'";

            var resposta = await _httpClientComToken.GetResposta(new Uri(select));
            RespostaAPIModel respostaAPI = new(resposta);

            if (!resposta.IsSuccessStatusCode)
            {
                return respostaAPI;
            }   
            
            var ufs = JsonConvert.DeserializeObject<List<UFICMSModel>>(respostaAPI.ConteudoRetorno);
            
            if (!ufs.Any())
            {
                respostaAPI.MensagemErro = $"Não foi encontrada uma alíquota com a UF {uf}.";
                respostaAPI.Status = HttpStatusCode.NotFound;
                return respostaAPI;
            }
                
            respostaAPI.Objeto = ufs.FirstOrDefault();

            return respostaAPI;
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, string campos, string filtros, string order, string top)
        {
            throw new NotImplementedException();
        }

        public async Task<RespostaAPIModel> PostAsync<T>(T entity)
        {
            var retorno = await _httpClientComToken.Enviar(HttpMethod.Post, URIsUFICMSs.PostAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public async Task<RespostaAPIModel> PutAsync<T>(T entity)
        {

            var retorno = await _httpClientComToken.Enviar(HttpMethod.Put, URIsUFICMSs.PutAsync, entity);
            return new RespostaAPIModel(retorno);
        }

        public Task<RespostaAPIModel> GetAsync(Uri uriBase, StatusAtivo status)
        {
            throw new NotImplementedException();
        }

        public Task<RespostaAPIModel> GetByIdAsync(Uri uriBase, long id, StatusAtivo status)
        {
            throw new NotImplementedException();
        }
    }
}
