﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Hosting;
using Syncfusion.Blazor.Inputs;
using System;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Threading.Tasks;

namespace GAFNETBlazor.Services
{
    public class UploadLogoDANFENFeService : IUploadLogoDANFENFeService
    {
        private readonly ILoginStore _loginStore;
        private readonly IWebHostEnvironment _env;
        private readonly IHttpClientComToken _httpClientComToken;

        public UploadLogoDANFENFeService(IWebHostEnvironment env, ILoginStore loginStore, IHttpClientComToken httpClientComToken)
        {
            _env = env;
            _loginStore = loginStore;
            _httpClientComToken = httpClientComToken;
        }

        public async Task<ResultadoModel> Upload(UploadChangeEventArgs args)
        {
            foreach (var file in args.Files)
            {
                string extArq = Path.GetExtension(file.FileInfo.Name).ToLower();

                if (!new string[] { ".jpeg", ".jpg", ".bmp", ".png"}.Contains(extArq) ) return new ResultadoModel { Sucesso = false, Retorno = "arquivo inválido" };

                var resultArquivo = await UploadArquivoLogo(file);
                if (!resultArquivo.Sucesso) return resultArquivo;
                string nomeArquivo = (string)resultArquivo.Retorno;

                return new ResultadoModel { Sucesso = true, Retorno = nomeArquivo };
            };

            return new ResultadoModel { Sucesso = false, Retorno = "Não foram processados arquivos." };
        }

        private async Task<ResultadoModel> UploadArquivoLogo(UploadFiles file)
        {
            var resultDiretorio = CriarDiretorioLogoDANFE();
            if (!resultDiretorio.Sucesso) return resultDiretorio;
            string diretorio = (string)resultDiretorio.Retorno;
            var nomeArquivo = $@"{diretorio}\{file.FileInfo.Name}";

            try
            {
                using var filestream = new FileStream(nomeArquivo, FileMode.Create, FileAccess.Write);
                file.Stream.WriteTo(filestream);
            }
            catch (Exception e)
            {
                return new ResultadoModel { Sucesso = false, Retorno = $"Não foi possível faser o upload do arquivo {nomeArquivo}. Erro: {e.GetBaseException().Message}" };
            }

            //Manda o arquivo para a API
            try
            {
                using var form = new MultipartFormDataContent();
                using var fs = File.OpenRead(nomeArquivo);
                using var streamContent = new StreamContent(fs);
                using var fileContent = new ByteArrayContent(await streamContent.ReadAsByteArrayAsync());
                form.Add(fileContent, "arquivo", Path.GetFileName(nomeArquivo));
                HttpResponseMessage response = await _httpClientComToken.Enviar(HttpMethod.Post, URIsNFe.PostLogoDANFE, form, "multipart/form-data");
            }
            catch (Exception e)
            {
                return new ResultadoModel { Sucesso = false, Retorno = $"Não foi possível fazer o upload do arquivo para a API {nomeArquivo}. Erro: {e.GetBaseException().Message}" };
            }

            return new ResultadoModel { Sucesso = true, Retorno = nomeArquivo };
        }

        private ResultadoModel CriarDiretorioLogoDANFE()
        {
            var pathRaiz = $@"{_env.WebRootPath}\files\LogosDANFENFe\{_loginStore.EmpresaLogada.CNPJ}";
            try
            {
                if (!Directory.Exists(pathRaiz))
                {
                    Directory.CreateDirectory(pathRaiz);
                }
                return new ResultadoModel { Sucesso = true, Retorno = pathRaiz };
            }
            catch (Exception e)
            {
                return new ResultadoModel { Sucesso = false, Retorno = $"Não foi possível criar o diretório {pathRaiz}. Erro: {e.GetBaseException().Message}" };
            }
        }
    }
}
