using BlazorDownloadFile;
using GAFNETBlazor.Controllers;
using GAFNETBlazor.Services;
using GAFNETBlazor.Services.Administrativo;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Services.Relatorios;
using GAFNETBlazor.Shared;
using GAFNETBlazor.Stores;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels;
using GAFNETBlazor.ViewModels.Administrativo;
using GAFNETBlazor.ViewModels.Administrativo.CadastroClaim;
using GAFNETBlazor.ViewModels.Administrativo.CadastroEmpresa;
using GAFNETBlazor.ViewModels.Administrativo.CadastroRole;
using GAFNETBlazor.ViewModels.Administrativo.CadastroUsuario;
using GAFNETBlazor.ViewModels.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Administrativo;
using GAFNETBlazor.ViewModels.Interfaces.Representante;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETBlazor.ViewModels.Interfaces.Usuario.Relatorios;
using GAFNETBlazor.ViewModels.Representante;
using GAFNETBlazor.ViewModels.Usuario;
using GAFNETBlazor.ViewModels.Usuario.CadastroAliquota;
using GAFNETBlazor.ViewModels.Usuario.CadastroCentroDeCusto;
using GAFNETBlazor.ViewModels.Usuario.CadastroClassificacaoFiscal;
using GAFNETBlazor.ViewModels.Usuario.CadastroCondicaoDePagamento;
using GAFNETBlazor.ViewModels.Usuario.CadastroEmbalagem;
using GAFNETBlazor.ViewModels.Usuario.CadastroFormulario;
using GAFNETBlazor.ViewModels.Usuario.CadastroGeneroProduto;
using GAFNETBlazor.ViewModels.Usuario.CadastroGrupoProduto;
using GAFNETBlazor.ViewModels.Usuario.CadastroHistorico;
using GAFNETBlazor.ViewModels.Usuario.CadastroPessoa;
using GAFNETBlazor.ViewModels.Usuario.CadastroPlanoDeConta;
using GAFNETBlazor.ViewModels.Usuario.CadastroProduto;
using GAFNETBlazor.ViewModels.Usuario.CadastroSubgrupoProduto;
using GAFNETBlazor.ViewModels.Usuario.CadastroUFICMS;
using GAFNETBlazor.ViewModels.Usuario.CadastroUnidadeProduto;
using GAFNETBlazor.ViewModels.Usuario.Compra;
using GAFNETBlazor.ViewModels.Usuario.Faturamento;
using GAFNETBlazor.ViewModels.Usuario.PagarReceber;
using GAFNETBlazor.ViewModels.Usuario.Parametrizacao;
using GAFNETBlazor.ViewModels.Usuario.PesquisaFaturamento;
using GAFNETBlazor.ViewModels.Usuario.PesquisaPagarReceber;
using GAFNETBlazor.ViewModels.Usuario.PesquisaPessoa;
using GAFNETBlazor.ViewModels.Usuario.Relatorios.Pessoas;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Localization;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.FileProviders;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Options;
using Syncfusion.Blazor;
using System.Collections.Generic;
using System.Globalization;
using System.IO;

namespace GAFNETBlazor
{
    public class Startup
    {
        // This method gets called by the runtime. Use this method to add services to the container.
        // For more information on how to configure your application, visit https://go.microsoft.com/fwlink/?LinkID=398940
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddControllers().AddNewtonsoftJson();
            services.AddAutoMapper(typeof(Startup));
            services.AddRazorPages();
            services.AddServerSideBlazor();
            services.AddHttpClient();
            services.AddServerSideBlazor().AddCircuitOptions(options => { options.DetailedErrors = true; });

            services.AddScoped<ReportViewerController>();

            services.AddScoped<BrowserService>(); // scoped service
            services.AddScoped<ICorreiosService, CorreiosService>();
            services.AddScoped<ICidadesService, CidadesService>();
            services.AddTransient<IHttpClientComToken, HttpClientComToken>();

            services.AddScoped<ISideMenuService, SideMenuService>();
            services.AddScoped<ILoginStore, LoginStore>();
            services.AddScoped<IUsuariosService, UsuariosService>();
            services.AddScoped<IClaimsService, ClaimsService>();
            services.AddScoped<IEmpresasService, EmpresasService>();
            services.AddScoped<IRepresentantesService, RepresentantesService>();
            services.AddScoped<IRolesService, RolesService>();
            services.AddScoped<IPaisesService, PaisesService>();
            services.AddScoped<ITiposPessoaService, TiposPessoaService>();
            services.AddSingleton<ITiposFaturamentoService, TiposFaturamentoService>();
            services.AddSingleton<ITiposPesquisaStringService, TiposPesquisaStringService>();
            services.AddSingleton<ITiposFreteService, TiposFreteService>();
            services.AddSingleton<ITiposPagarReceberService, TiposPagarReceberService>();
            services.AddScoped<IAcoesFaturamentoService, AcoesFaturamentoService>();
            services.AddScoped<IPessoasService, PessoasService>();
            services.AddScoped<IConsultaPessoaAPIService, ConsultaPessoaAPIService>();
            services.AddScoped<IProdutosService, ProdutosService>();
            services.AddScoped<IHistoricosService, HistoricosService>();
            services.AddScoped<IPlanosDeContaService, PlanosDeContaService>();
            services.AddScoped<ICentrosDeCustoService, CentrosDeCustoService>();
            services.AddScoped<ICondicoesDePagamentoService, CondicoesDePagamentoService>();
            services.AddScoped<ITiposDeOperacaoService, TiposDeOperacaoService>();
            services.AddScoped<IAliquotasService, AliquotasService>();
            services.AddScoped<IUFICMSsService, UFICMSsService>();
            services.AddScoped<IUnidadesProdutoService, UnidadesProdutoService>();
            services.AddScoped<IGruposProdutoService, GruposProdutoService>();
            services.AddScoped<ISubgruposProdutoService, SubgruposProdutoService>();
            services.AddScoped<IEmbalagensService, EmbalagensService>();
            services.AddScoped<IClassificacoesFiscaisService, ClassificacoesFiscaisService>();
            services.AddScoped<IGenerosProdutoService, GenerosProdutoService>();
            services.AddScoped<IProdutosService, ProdutosService>();
            services.AddScoped<IFormulariosService, FormulariosService>();
            services.AddScoped<IFaturamentoBaseService, FaturamentoBaseService>();
            services.AddScoped<IFaturamentosService, FaturamentosService>();
            services.AddScoped<INFeService, NFeService>();
            services.AddScoped<IComprasService, ComprasService>();
            services.AddScoped<IParametrizacoesService, ParametrizacoesService>();
            services.AddScoped<ICFOPCompraService, CFOPCompraService>();
            services.AddScoped<IPesquisasFaturamentoService, PesquisasFaturamentoService>();
            services.AddScoped<IDadosRelatService, DadosRelatService>();
            services.AddScoped<ITimeZoneService, TimeZoneService>();
            services.AddScoped<ILeituraRetornoNFeService, LeituraRetornoNFeService>();
            services.AddTransient<ILoginViewModel, LoginViewModel>();
            services.AddScoped<ICadastroUsuarioVM, CadastroUsuarioVM>();
            services.AddScoped<ISetaSenhaUsuarioVM, SetaSenhaUsuarioVM>();
            services.AddScoped<ICadastroRepresentanteVM, CadastroRepresentanteVM>();
            services.AddScoped<ICadastroEmpresaVM, CadastroEmpresaVM>();
            services.AddScoped<ICadastroClaimVM, CadastroClaimVM>();
            services.AddScoped<ICadastroRoleVM, CadastroRoleVM>();
            services.AddScoped<ICadastroPessoaVM, CadastroPessoaVM>();
            services.AddScoped<ICadastroHistoricoVM, CadastroHistoricoVM>();
            services.AddScoped<ICadastroAliquotaVM, CadastroAliquotaVM>();
            services.AddScoped<ICadastroUFICMSVM, CadastroUFICMSVM>();
            services.AddScoped<ICadastroUnidadeProdutoVM, CadastroUnidadeProdutoVM>();
            services.AddScoped<ICadastroGrupoProdutoVM, CadastroGrupoProdutoVM>();
            services.AddScoped<ICadastroSubgrupoProdutoVM, CadastroSubgrupoProdutoVM>();
            services.AddScoped<ICadastroEmbalagemVM, CadastroEmbalagemVM>();
            services.AddScoped<ICadastroClassificacaoFiscalVM, CadastroClassificacaoFiscalVM>();
            services.AddScoped<ICadastroGeneroProdutoVM, CadastroGeneroProdutoVM>();
            services.AddScoped<ICadastroProdutoVM, CadastroProdutoVM>();
            services.AddScoped<ICadastroCondicaoDePagamentoVM, CadastroCondicaoDePagamentoVM>();
            services.AddScoped<ICadastroCentroDeCustoVM, CadastroCentroDeCustoVM>();
            services.AddScoped<ICadastroFormularioVM, CadastroFormularioVM>();
            services.AddScoped<ICadastroPlanoDeContaVM, CadastroPlanoDeContaVM>();
            services.AddScoped<IParametrizacaoVM, ParametrizacaoVM>();
            services.AddScoped<IManutencaoEmpresaVM, ManutencaoEmpresaVM>();
            services.AddScoped<ITrocaSenhaUsuarioVM, TrocaSenhaUsuarioVM>();
            services.AddScoped<IFaturamentoVM, FaturamentoVM>();
            services.AddScoped<ICompraVM, CompraVM>();
            services.AddScoped<IDevolucaoVM, DevolucaoVM>();
            services.AddScoped<IDeParaCompraImportacaoXMLVM, DeParaCompraImportacaoXMLVM>();
            services.AddScoped<IPesquisaFaturamentoVM, PesquisaFaturamentoVM>();
            services.AddScoped<IPagarReceberVM, PagarReceberVM>();
            services.AddScoped<IPesquisaPagarReceberVM, PesquisaPagarReceberVM>();
            services.AddScoped<IPesquisaPessoaVM, PesquisaPessoaVM>();
            services.AddScoped<ITabelaIBPTVM, TabelaIBPTVM>();
            services.AddScoped<IParamRelatPessoasVM, ParamRelatPessoasVM>();
            services.AddScoped<IUploadsVM, UploadsVM>();
            services.AddScoped<IDownloadsVM, DownloadsVM>(); 
            services.AddSingleton<ITiposRegimeTributacaoService, TiposRegimeTributacaoService>();
            services.AddSingleton<IIndicadoresIEService, IndicadoresIEService>();
            services.AddSingleton<ITiposCTEHistoricoService, TiposCTEHistoricoService>();
            services.AddSingleton<IEntradaSaidaService, EntradaSaidaService>();
            services.AddSingleton<ITiposNFHistoricoService, TiposNFHistoricoService>();
            services.AddSingleton<IFinNFeHistoricoService, FinNFeHistoricoService>();
            services.AddSingleton<ISitTribService, SitTribService>();
            services.AddSingleton<ISitTribPISCOFINSService, SitTribPISCOFINSService>();
            services.AddSingleton<ISitTribPISCOFINSEntradaService, SitTribPISCOFINSEntradaService>();
            services.AddSingleton<ISitTribIPIService, SitTribIPIService>();
            services.AddSingleton<ITabelaAService, TabelaAService>();
            services.AddSingleton<ITiposReducaoICMSService, TiposReducaoICMSService>();
            services.AddSingleton<ITiposReducaoICMSSTService, TiposReducaoICMSSTService>();
            services.AddSingleton<ITiposProdutoService, TiposProdutoService>();
            services.AddSingleton<ITiposFormularioService, TiposFormularioService>();
            services.AddScoped<IEnderecoPeloCEPService, EnderecoPeloCEPService>();
            services.AddScoped<IUploadCertificadoDigitalService, UploadCertificadoDigitalService>();
            services.AddScoped<IUploadLogoDANFENFeService, UploadLogoDANFENFeService>();
            services.AddScoped<IImportacaoTabelaIBPTService, ImportacaoTabelaIBPTService>();
            services.AddScoped<IEnvioFaturamentoService, EnvioFaturamentoService>();
            services.AddScoped<ICriaAlteraFaturamentoService, CriaAlteraFaturamentoService>();
            services.AddScoped<ICancelamentoFaturamentoService, CancelamentoFaturamentoService>();
            services.AddScoped<IEstornoFaturamentoService, EstornoFaturamentoService>();
            services.AddScoped<IDuplicacaoFaturamentoService, DuplicacaoFaturamentoService>();
            services.AddScoped<IInutilizacaoFaturamentoService, InutilizacaoFaturamentoService>();
            services.AddScoped<IDadosDevolucaoService, DadosDevolucaoService>();
            services.AddScoped<INotaDeXMLService, NotaDeXMLService>();
            services.AddScoped<IEnvioXMLService, EnvioXMLService>();
            services.AddScoped<ITrocaArquivosService, TrocaArquivosService>();
            services.AddScoped<IStatusAtivoService, StatusAtivoService>();
            services.AddScoped<INomesRelatoriosService, NomesRelatoriosService>();
            services.AddScoped<IParametrosRelatorioService, ParametrosRelatorioService>();
            services.AddScoped<ITipoOrdenacaoService, TipoOrdenacaoService>();
            services.AddScoped<ITiposDebitoCreditoContaService, TiposDebitoCreditoContaService>();
            services.AddScoped<ITiposDespesaPlanoDeContaService, TiposDespesaPlanoDeContaService>();
            services.AddScoped<ITiposPlanoDeContaService, TiposPlanoDeContaService>();
            services.AddScoped<ILogoutService, LogoutService>();
            services.AddScoped<IExibicaoFaturamentoService, ExibicaoFaturamentoService>();
            services.AddScoped<IExibicaoProdutoFaturamentoService, ExibicaoProdutoFaturamentoService>();
            services.AddScoped<IPreenchimentoCSTCFOPCompraService, PreenchimentoCSTCFOPCompraService>();
            services.AddScoped<IPesquisasPagarReceberService, PesquisasPagarReceberService>();
            services.AddScoped<IPagarReceberService, PagarReceberService>();
            services.AddLocalization(options => options.ResourcesPath = "Resources");
            services.AddSyncfusionBlazor();
            services.AddSingleton(typeof(ISyncfusionStringLocalizer), typeof(SyncfusionLocalizer));
            services.AddBlazorDownloadFile(ServiceLifetime.Scoped);
            services.Configure<RequestLocalizationOptions>(options =>
            {
                // Define the list of cultures your app will support
                var supportedCultures = new List<CultureInfo>()
            {
                new CultureInfo("pt")
            };
                // Set the default culture
                options.DefaultRequestCulture = new RequestCulture("pt");
                options.SupportedCultures = supportedCultures;

                options.SupportedUICultures = supportedCultures;
                options.RequestCultureProviders = new List<IRequestCultureProvider>() {
                 new QueryStringRequestCultureProvider() // Here, You can also use other localization provider
                };
            });

            IFileProvider physicalProvider = new PhysicalFileProvider(Directory.GetCurrentDirectory() + @"\wwwroot\Files");
            services.AddSingleton<IFileProvider>(physicalProvider);

        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            //Syncfusion.Licensing.SyncfusionLicenseProvider.RegisterLicense("NTYxMjMxQDMxMzkyZTM0MmUzMFR2ZjI2eXl5dDIyNHp4QzVTQWJZbVRpRFliN0JTdmhGRkkrY1lzMTNkcWs9");
            Syncfusion.Licensing.SyncfusionLicenseProvider.RegisterLicense("Njc5MDk3QDMyMzAyZTMyMmUzMEdjbkI4V0ROaXprdmhmNm1KbVowMGZ1eVdYNlZCTFMyWTlQVmIzZHROamM9");
            Bold.Licensing.BoldLicenseProvider.RegisterLicense("OmD063V/wsnLrPnDqiBTzv3/a3ag0M1VnthDbe5La00=");

            app.UseRequestLocalization(app.ApplicationServices.GetService<IOptions<RequestLocalizationOptions>>().Value);

            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            else
            {
                // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
                app.UseHsts();
            }

            app.UseHttpsRedirection();

            app.UseStaticFiles();
            app.UseRouting();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
                endpoints.MapBlazorHub();
                endpoints.MapFallbackToPage("/_Host");
            });
        }
    }
}
