﻿using GAFNETLibrary.Enums;
using GAFNETBlazor.Stores.Interfaces;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;
using GAFNETBlazor.Controllers;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.Stores
{
    public class LoginStore : ILoginStore
    {
        public event Action OnChange;
        public string Token { get; set; }
        public string ErroLogin { get; set; }
        public List<Claim> Claims { get; set; } = new List<Claim>();
        public bool EhAdmin => Claims?.FirstOrDefault(c => c.Type == "Admin") != null;
        public bool EhRepresentante => Claims?.FirstOrDefault(c => c.Type == "Representante") != null;

        public List<EmpresaModel> EmpresasUsuario { get; } = new List<EmpresaModel>();
        public EmpresaModel EmpresaLogada { get; set; } = new EmpresaModel();
        public string EMailLogado { get; set; }
        public long? IdRepresentante { get; set; }
        public bool LogadoEmEmpresa => !string.IsNullOrEmpty(EmpresaLogada.CNPJ);

        public async Task<HttpResponseMessage> RealizarLogin(LoginUserModel loginUserModel)
        {
            string uri = "https://www.autenticacao.gafnetapi.com.br/api/token/requesttoken";
            //string uri = "http://localhost:64246/api/token/requesttoken";
            HttpResponseMessage consulta;
            using var client = new HttpClient();
            client.DefaultRequestHeaders.Accept.Clear();
            client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));

            var json = JsonConvert.SerializeObject(loginUserModel);

            consulta = await client.PostAsync(uri, new StringContent(json, Encoding.UTF8, "application/json"));

            if (consulta.StatusCode != HttpStatusCode.OK)
            {
                ErroLogin = await consulta.Content.ReadAsStringAsync();
                Token = null;
                return consulta;
            }
            
            Token = await consulta.Content.ReadAsStringAsync();
            Token = Token.Substring(10, Token.Length - 12); // retira os caracteres do json que podem invalidar o token
            var jwtEncodedString = Token;
            var token = new JwtSecurityToken(jwtEncodedString);
            Claims.AddRange(token.Claims);
            //Os dados das empresas vem como cnpj || nome
            var empresas = Claims.Where(c => c.Type == "EmpresasUsuario").ToList();
            foreach (var empresa in empresas)
            {
                var dadosEmpresa = empresa.Value.Split("||");
                var empresaObj = new EmpresaModel { CNPJ = dadosEmpresa[0], Nome = $"{dadosEmpresa[1]}", Cidade = new CidadeModel { UF = dadosEmpresa[2] }, TipoRegimeTributacao = (TipoRegimeTributacao) Convert.ToUInt32(dadosEmpresa[3]) };
                EmpresasUsuario.Add(empresaObj);
            }

            if (EmpresasUsuario?.Count == 1)
            {
                EmpresaLogada = EmpresasUsuario[0];
            }
            EMailLogado = loginUserModel.Email;

            if (EhRepresentante)
            {
                var idRepres = Claims?.FirstOrDefault(c => c.Type == "IdRepresentante");
                IdRepresentante = idRepres != null ? Convert.ToInt64(idRepres.Value) : null ;
            }

            ReportViewerController.Token = Token;
            ReportViewerController.EmpresaLogada = EmpresaLogada;
            ReportViewerController.EmpresasUsuario.Clear();
            ReportViewerController.EmpresasUsuario.AddRange(EmpresasUsuario);
            

            OnChange?.Invoke();
            return consulta;
        }
        public void RealizarLogout()
        {
            Token = "";
            Claims.Clear();
            EmpresaLogada = new EmpresaModel();
            EMailLogado = string.Empty;
            EmpresasUsuario.Clear();

            ReportViewerController.EmpresaLogada = new();
            ReportViewerController.EmpresasUsuario?.Clear();
            ReportViewerController.Token = "";

            OnChange?.Invoke();
            
        }
    }
}
