﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Administrativo;
using Microsoft.AspNetCore.Components.Web;
using Newtonsoft.Json;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Administrativo.CadastroClaim
{
    public class CadastroClaimVM : CadastroBaseVM, ICadastroClaimVM
    {
        private readonly IClaimsService _claimsService;

        private ClaimModel _claim;
        private bool _dlgClaimsVisible;

        public SfTextBox NomeTextBox { get; set; }
        public List<ClaimModel> Claims { get; set; } = new List<ClaimModel>();
        public ClaimModel ClaimSelecionada { get; set; }
        public bool DlgClaimsVisible
        {
            get { return _dlgClaimsVisible; }
            set
            {
                _dlgClaimsVisible = value;
                AoMudarPropriedade(nameof(DlgClaimsVisible));
            }
        }

        public ClaimModel Claim
        {
            get { return _claim; }
            set
            {
                _claim = value;
                AoMudarPropriedade(nameof(Claim));
            }
        }

        public SfDialog Dlg { get; set; }

        public CadastroClaimVM(IClaimsService claimsService)
        {
            _claim = new ClaimModel();
            _claimsService = claimsService;
            _dlgClaimsVisible = false;
            MensagemResultadoEnvio = " ";
        }
        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }
            await EnvioDadosCadastroClaimVM.Enviar(this, _claimsService);
        }

        public void AoSelecionarClaim(RowSelectEventArgs<ClaimModel> e)
        {
            ClaimSelecionada = e?.Data;
        }

        public async void AoFecharDlgClaims(object e)
        {
            DlgClaimsVisible = false;
            await NomeTextBox.FocusAsync();
        }
        public void AoClicarOkBtnDlgClaims(MouseEventArgs e)
        {
            Claim.CopiarDados(ClaimSelecionada);
            DlgClaimsVisible = false;
        }

        public async void AoCriarNomeTextBox(object e)
        {
            await NomeTextBox.FocusAsync();
        }

        public async void AoFecharDlg(object e)
        {
            DlgVisible = false;
            await NomeTextBox.FocusAsync();
        }

        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            await LocalizaCadastroClaimVM.Localizar(this, _claimsService);
        }

        public bool DadosValidos()
        {
            return ValidaCadastroClaimVM.Validar(this);
        }

        public void LimparTela()
        {
            LimpaTelaCadastroClaimVM.Limpar(this);
        }
    }
}
