﻿using FluentValidation;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Administrativo;
using Microsoft.AspNetCore.Components.Web;
using Newtonsoft.Json;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using Syncfusion.Blazor.Spinner;
using System;
using System.Collections.Generic;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Administrativo.CadastroEmpresa
{
    public class CadastroEmpresaVM : CadastroBaseVM, ICadastroEmpresaVM
    {
        private readonly IEmpresasService _empresasService;
        private readonly IRepresentantesService _representantesService;
        private readonly ICidadesService _cidadesService;
        private readonly IEnderecoPeloCEPService _enderecoPeloCEPService;

        private bool _dlgEmpresasVisible;
        private bool _dlgRepresentantesVisible;
        private bool _dlgCidadesVisible;
        private int? _tipoRegimeTributacaoSelecionado;

        private EmpresaModel _empresa;

        public SfTextBox NomeTextBox { get; set; }
        public SfMaskedTextBox CEPTextBox { get; set; }
        public RepresentanteModel Representante { get; set; }
        public RepresentanteModel RepresentanteSelecionado { get; set; }
        public EmpresaModel EmpresaSelecionada { get; set; }
        public List<ListaModel> TiposRegimeTributacao { get; }
        public EmpresaModel Empresa
        {
            get { return _empresa; }
            set
            {
                _empresa = value;
                AoMudarPropriedade(nameof(Empresa));
            }
        }

        public List<RepresentanteModel> Representantes { get; set; } = new List<RepresentanteModel>();
        public List<EmpresaModel> Empresas { get; set; } = new List<EmpresaModel>();

        public int? TipoRegimeTributacaoSelecionado
        {
            get
            {
                _tipoRegimeTributacaoSelecionado = (int?)Empresa.TipoRegimeTributacao;
                return _tipoRegimeTributacaoSelecionado;
            }
            set
            {
                _tipoRegimeTributacaoSelecionado = value;
                Empresa.TipoRegimeTributacao = (TipoRegimeTributacao)_tipoRegimeTributacaoSelecionado;
            }
        }
        public bool DlgEmpresasVisible
        {
            get { return _dlgEmpresasVisible; }
            set
            {
                _dlgEmpresasVisible = value;
                AoMudarPropriedade(nameof(DlgEmpresasVisible));
            }
        }
        public bool DlgRepresentantesVisible
        {
            get { return _dlgRepresentantesVisible; }
            set
            {
                _dlgRepresentantesVisible = value;
                AoMudarPropriedade(nameof(DlgRepresentantesVisible));
            }
        }
        public bool DlgCidadesVisible
        {
            get { return _dlgCidadesVisible; }
            set
            {
                _dlgCidadesVisible = value;
                AoMudarPropriedade(nameof(DlgCidadesVisible));
            }
        }

        public bool EmpresasBtnEnabled { get; set; } = true;
        public bool RepresentantesBtnEnabled { get; set; } = true;
        public SfDialog Dlg { get; set; }
        public List<CidadeModel> Cidades { get; set; } = new List<CidadeModel>();
        public CidadeModel CidadeSelecionada { get; set; } = new CidadeModel();
        public bool CidadesBtnEnabled { get; set; } = true;
        public bool PesquisandoCidades { get; set; }

        public CadastroEmpresaVM(IEmpresasService empresasService, 
            IRepresentantesService representantesService, 
            ICidadesService CidadesService, 
            IEnderecoPeloCEPService enderecoPeloCEPService, 
            ITiposRegimeTributacaoService tiposRegimeTributacaoService)
        {
            Representante            = new RepresentanteModel();
            RepresentanteSelecionado = new RepresentanteModel();
            Empresa                  = new EmpresaModel { Cidade = new CidadeModel() };
            Empresa.DataCadastro     = DateTime.Today;
            EmpresaSelecionada       = new EmpresaModel();
            TiposRegimeTributacao    = tiposRegimeTributacaoService?.Itens(); 
            _empresasService         = empresasService;
            _representantesService   = representantesService;
            _cidadesService          = CidadesService;
            _enderecoPeloCEPService  = enderecoPeloCEPService;
            MensagemResultadoEnvio   = " ";
        }
        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            Enviando = true;
            string mensagemSucesso;

            Empresa.Representante = null;

            if (Empresa.DataCadastro?.Hour == 0)
            {
                Empresa.DataCadastro = Empresa.DataCadastro.Value.AddHours(12);
            }
            if (Empresa.DtVctoLicenca?.Hour == 0)
            {
                Empresa.DtVctoLicenca = Empresa.DtVctoLicenca.Value.AddHours(12);
            }

            HttpResponseMessage retorno;
            if (Empresa.Id == 0)
            {
                mensagemSucesso = "Empresa criada com sucesso";
                retorno = await _empresasService.CriarEmpresa(Empresa);
            }
            else
            {
                mensagemSucesso = "Empresa alterada com sucesso";
                retorno = await _empresasService.AlterarEmpresa(Empresa);
            }


            if (!retorno.IsSuccessStatusCode)
            {
                Mensagem = Task.Run(() => retorno.Content.ReadAsStringAsync()).GetAwaiter().GetResult();
                MensagemResultadoEnvio = Mensagem;
            }
            else
            {
                Mensagem = string.Empty;
                MensagemResultadoEnvio = mensagemSucesso;
            }

            Enviando = false;
            DlgVisible = true;
            LimparTela();
        }

        public void AoSelecionarCidade(RowSelectEventArgs<CidadeModel> e)
        {
            CidadeSelecionada = e?.Data;
        }

        public async void AoClicarCidadesBtnAsync(MouseEventArgs e)
        {
            await LocalizaCidadeCadastroEmpresa.LocalizarAsync(this, _cidadesService);
        }

        public void AoFecharDialogoCidades()
        {
            CidadesBtnEnabled = true;
            DlgCidadesVisible = false;
        }
        public void AoClicarOkBtnDlgCidades(MouseEventArgs e)
        {
            Empresa.Cidade = CidadeSelecionada;
            Empresa.IdCidade = CidadeSelecionada.Id;
            DlgCidadesVisible = false;
        }

        public void AoFecharDlgCidades(object e)
        {
            CidadesBtnEnabled = true;
            DlgCidadesVisible = false;
        }

        public async Task<List<EmpresaModel>> GetEmpresas()
        {
            var retorno = await _empresasService.GetEmpresas(StatusAtivo.Todos);

            if (!retorno.IsSuccessStatusCode)
            {
                Mensagem = await retorno.Content.ReadAsStringAsync();
                return null;
            }
            
            var dados = await retorno.Content.ReadAsStringAsync();
            Empresas = JsonConvert.DeserializeObject<List<EmpresaModel>>(dados);
            Mensagem = string.Empty;
            return Empresas;
        }

        public async Task<List<RepresentanteModel>> GetRepresentantes()
        {
            var retorno = await _representantesService.GetRepresentantes(StatusAtivo.Todos);

            if (!retorno.IsSuccessStatusCode)
            {
                Mensagem = await retorno.Content.ReadAsStringAsync();
                return null;
            }
            
            var dados = await retorno.Content.ReadAsStringAsync();
            Representantes = JsonConvert.DeserializeObject<List<RepresentanteModel>>(dados);
            Mensagem = string.Empty;
            return Representantes;
        }

        public void AoSelecionarRepresentante(RowSelectEventArgs<RepresentanteModel> e)
        {
            RepresentanteSelecionado = e.Data;
        }

        public void AoSelecionarEmpresa(RowSelectEventArgs<EmpresaModel> e)
        {
            EmpresaSelecionada = e.Data;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            await GetEmpresas();
            Localizando = false;
            DlgEmpresasVisible = true;
        }
        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }
        public void LimparTela()
        {
            ErrosValidacaoFormulario.Clear();
            Representante = new RepresentanteModel();
            RepresentanteSelecionado = new RepresentanteModel();
            EmpresaSelecionada = new EmpresaModel();
            Empresa = new EmpresaModel { Cidade = new CidadeModel() };
            Empresa.DataCadastro = DateTime.Today;
        }

        public void AoFecharDlgRepresentantes(object e)
        {
            Localizando = false;
            DlgRepresentantesVisible = false;
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
            NomeTextBox.FocusAsync();
        }

        public async void AoClicarEmpresasBtn()
        {
            EmpresasBtnEnabled = false;
            await GetEmpresas();
            EmpresasBtnEnabled = true;
            DlgEmpresasVisible = true;
        }

        public void AoFecharDlgEmpresas(object e)
        {
            EmpresasBtnEnabled = true;
            DlgEmpresasVisible = false;
        }
        public void AoClicarOkBtnDlgEmpresas(MouseEventArgs e)
        {
            if (EmpresaSelecionada == null)
            {
                return;
            }
            Empresa = EmpresaSelecionada;
            Representante = EmpresaSelecionada.Representante;
            ErrosValidacaoFormulario.Clear();
            DlgEmpresasVisible = false;

        }

        public void AoCriarNomeTextBox(object e)
        {
            NomeTextBox.FocusAsync();
        }

        public void AoClicarOkBtnDlgRepresentantes(MouseEventArgs e)
        {
            if (RepresentanteSelecionado == null)
            {
                return;
            }
            Representante = RepresentanteSelecionado;
            Empresa.IdRepresentante = Representante.Id;
            DlgRepresentantesVisible = false;
        }

        public async void AoClicarRepresentantesBtn(MouseEventArgs e)
        {
            RepresentantesBtnEnabled = false;
            await GetRepresentantes();
            RepresentantesBtnEnabled = true;
            DlgRepresentantesVisible = true;
        }

        public bool DadosValidos()
        {
            EmpresaModelValidator validator = new();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(Empresa);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }

            return true;
        }

        public async void CEPAoSairAsync(MaskBlurEventArgs args)
        {
            if (string.IsNullOrEmpty(Empresa.CEP)) return;
            await ExibirEnderecoPeloCEP(args.Value);
            AoMudarPropriedade(nameof(Empresa));
        }

        private async Task ExibirEnderecoPeloCEP(string cep)
        {
            CidadesBtnEnabled = false;
            await EnderecoPeloCEPCadastroEmpresa.Endereco(this, cep, _enderecoPeloCEPService);
            CidadesBtnEnabled = true;
        }

    }
}
