﻿using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Administrativo;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;
using Newtonsoft.Json;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using System.Collections.Generic;
using System.Net.Http;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Administrativo
{
    public class CadastroRepresentanteVM : CadastroBaseVM, ICadastroRepresentanteVM
    {
        private readonly IUsuariosService _usuariosService;
        private readonly IRepresentantesService _representantesService;

        private bool _dlgUsuariosVisible;
        private bool _dlgRepresentantesVisible;

        public SfTextBox NomeTextBox { get; set; }
        public RepresentanteModel Representante { get; set; }
        public RepresentanteModel RepresentanteSelecionado { get; set; }
        public UsuarioModel UsuarioSelecionado { get; set; }
        public UsuarioModel Usuario { get; set; }
        public List<RepresentanteModel> Representantes { get; set; } = new List<RepresentanteModel>();
        public List<UsuarioModel> Usuarios { get; set; } = new List<UsuarioModel>();
        public bool DlgUsuariosVisible
        {
            get { return _dlgUsuariosVisible; }
            set
            {
                _dlgUsuariosVisible = value;
                AoMudarPropriedade(nameof(DlgUsuariosVisible));
            }
        }
        public bool DlgRepresentantesVisible
        {
            get { return _dlgRepresentantesVisible; }
            set
            {
                _dlgRepresentantesVisible = value;
                AoMudarPropriedade(nameof(DlgRepresentantesVisible));
            }
        }
        public SfDialog Dlg { get; set; }
        public bool UsuariosBtnEnabled { get; set; } = true;
        public CadastroRepresentanteVM(IUsuariosService usuariosService, IRepresentantesService representantesService)
        {
            Representante = new RepresentanteModel();
            //RepresentanteSelecionado = new RepresentanteModel();
            Usuario = new UsuarioModel();
            UsuarioSelecionado = new UsuarioModel();
            _usuariosService = usuariosService;
            _representantesService = representantesService;
            MensagemResultadoEnvio = " ";
        }
        public async void Enviar()
        {
            Enviando = true;
            string mensagemSucesso;

            Representante.Usuario = null;

            HttpResponseMessage retorno;
            if (Representante.Id == 0)
            {
                mensagemSucesso = "Representante criado com sucesso";
                retorno = await _representantesService.CriarRepresentante(Representante);
            }
            else
            {
                mensagemSucesso = "Representante alterado com sucesso";
                retorno = await _representantesService.AlterarRepresentante(Representante);
            }


            if (!retorno.IsSuccessStatusCode)
            {
                Mensagem = Task.Run(() => retorno.Content.ReadAsStringAsync()).GetAwaiter().GetResult();
                MensagemResultadoEnvio = Mensagem;
            }
            else
            {
                Mensagem = string.Empty;
                MensagemResultadoEnvio = mensagemSucesso;
                LimparTela();
            }

            Enviando = false;
            DlgVisible = true;
        }

        public async Task<List<UsuarioModel>> GetUsuarios()
        {
            var retorno = await _usuariosService.GetUsuarios();

            if (!retorno.IsSuccessStatusCode)
            {
                Mensagem = await retorno.Content.ReadAsStringAsync();
                return null;
            }
            else
            {
                var dados = await retorno.Content.ReadAsStringAsync();
                Usuarios = JsonConvert.DeserializeObject<List<UsuarioModel>>(dados);
                Mensagem = string.Empty;
                return Usuarios;
            }
        }

        public async Task<List<RepresentanteModel>> GetRepresentantes()
        {
            var retorno = await _representantesService.GetRepresentantes(StatusAtivo.Todos);

            if (!retorno.IsSuccessStatusCode)
            {
                Mensagem = await retorno.Content.ReadAsStringAsync();
                return null;
            }
            else
            {
                var dados = await retorno.Content.ReadAsStringAsync();
                Representantes = JsonConvert.DeserializeObject<List<RepresentanteModel>>(dados);
                Mensagem = string.Empty;
                return Representantes;
            }
        }
        public async Task<RepresentanteModel> GetRepresentanteById()
        {
            var retorno = await _representantesService.GetRepresentanteById(RepresentanteSelecionado.Id, StatusAtivo.Todos);

            if (!retorno.IsSuccessStatusCode)
            {
                Mensagem = await retorno.Content.ReadAsStringAsync();
                return null;
            }
            else
            {
                var representante = JsonConvert.DeserializeObject<RepresentanteModel>(await retorno.Content.ReadAsStringAsync());
                Mensagem = string.Empty;
                return representante;
            }
        }

        public void AoSelecionarRepresentante(RowSelectEventArgs<RepresentanteModel> e)
        {
            RepresentanteSelecionado = e?.Data;
        }

        public void AoSelecionarUsuario(RowSelectEventArgs<UsuarioModel> e)
        {
            UsuarioSelecionado = e.Data;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            RepresentanteSelecionado = null;
            Localizando              = true;
            await GetRepresentantes();
            Localizando              = false;
            DlgRepresentantesVisible = true;
        }
        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }
        public void LimparTela()
        {
            Representante = new RepresentanteModel();
            UsuarioSelecionado = new UsuarioModel();
            Usuario = new UsuarioModel();
        }

        public void AoFecharDlgGridRepresentantes()
        {
            Localizando = false;
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
            NomeTextBox.FocusAsync();
        }

        public async void AoClicarUsuariosBtn(MouseEventArgs e)
        {
            UsuariosBtnEnabled = false;
            await GetUsuarios();
            UsuariosBtnEnabled = true;
            DlgUsuariosVisible = true;
        }

        public void AoFecharDlgUsuarios(object e)
        {
            UsuariosBtnEnabled = true;
            DlgUsuariosVisible = false;
        }
        public void AoClicarOkBtnDlgUsuarios(MouseEventArgs e)
        {
            if (UsuarioSelecionado == null)
            {
                return;
            }
            Usuario = UsuarioSelecionado;
            Representante.IdUsuario = Usuario.Id;
            DlgUsuariosVisible = false;

        }

        public void AoCriarNomeTextBox(object e)
        {
            NomeTextBox.FocusAsync();
        }

        public void AoClicarOkBtnDlgRepresentantes(MouseEventArgs e)
        {
            if (RepresentanteSelecionado == null)
            {
                return;
            }
            Localizando = false;
            Representante = RepresentanteSelecionado;
            Usuario = RepresentanteSelecionado.Usuario;
            DlgRepresentantesVisible = false;
        }

        public void AoFecharDlgRepresentantes(object e)
        {
            Localizando = false;
            DlgRepresentantesVisible = false;
        }

        public bool DadosValidos()
        {
            throw new System.NotImplementedException();
        }
    }
}
