﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Administrativo;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Administrativo.CadastroRole
{
    public class CadastroRoleVM : CadastroBaseVM, ICadastroRoleVM
    {
        private readonly IRolesService _RolesService;

        private RoleModel _Role;
        private bool _dlgRolesVisible;

        public SfTextBox NomeTextBox { get; set; }
        public List<RoleModel> Roles { get; set; } = new List<RoleModel>();
        public RoleModel RoleSelecionada { get; set; }
        public bool DlgRolesVisible
        {
            get { return _dlgRolesVisible; }
            set
            {
                _dlgRolesVisible = value;
                AoMudarPropriedade(nameof(DlgRolesVisible));
            }
        }

        public RoleModel Role
        {
            get { return _Role; }
            set
            {
                _Role = value;
                AoMudarPropriedade(nameof(Role));
            }
        }

        public SfDialog Dlg { get; set; }

        public CadastroRoleVM(IRolesService RolesService)
        {
            _Role = new RoleModel();
            _RolesService = RolesService;
            _dlgRolesVisible = false;
            MensagemResultadoEnvio = " ";
        }
        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }
            await EnvioDadosCadastroRoleVM.Enviar(this, _RolesService);
        }

        public void AoSelecionarRole(RowSelectEventArgs<RoleModel> e)
        {
            RoleSelecionada = e.Data;
        }

        public void AoFecharDlgRoles(object e)
        {
            DlgRolesVisible = false;
            NomeTextBox.FocusAsync();
        }
        public void AoClicarOkBtnDlgRoles(MouseEventArgs e)
        {
            Role.CopiarDados(RoleSelecionada);
            DlgRolesVisible = false;
        }

        public void AoCriarNomeTextBox(object e)
        {
            NomeTextBox.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
            NomeTextBox.FocusAsync();
        }

        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            await LocalizaCadastroRoleVM.Localizar(this, _RolesService);
        }

        public bool DadosValidos()
        {
            return ValidaCadastroRoleVM.Validar(this);
        }

        public void LimparTela()
        {
            LimpaTelaCadastroRoleVM.Limpar(this);
        }
    }
}
