﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Administrativo;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Administrativo.CadastroUsuario
{
    public class CadastroUsuarioVM : UsuarioBaseVM, ICadastroUsuarioVM
    {
        private readonly IClaimsService _claimsService;
        private readonly IRolesService _rolesService;
        private readonly IEmpresasService _empresasService;
        
        private bool _dlgClaimsVisible;
        private bool _dlgEmpresasVisible;
        private bool _dlgRolesVisible;

        //private List<ClaimModel> _claims;

        public SfTextBox NomeTextBox { get; set; }
        public List<EmpresaModel> Empresas { get; set; } = new List<EmpresaModel>();
        public List<RoleModel> Roles { get; set; } = new List<RoleModel>();
        public List<ClaimModel> Claims { get; set; } 
        public string ClaimUsuarioSelecionada { get; set; }
        public string ClaimSelecionada { get; set; }
        public EmpresaModel EmpresaSelecionada { get; set; }
        public RoleModel RoleSelecionada { get; set; }
        public RoleModel RoleUsuarioSelecionada { get; set; }
        public UsuarioEmpresaModel EmpresaUsuarioSelecionada { get; set; }

        public bool DlgClaimsVisible
        {
            get { return _dlgClaimsVisible; }
            set
            {
                _dlgClaimsVisible = value;
                AoMudarPropriedade(nameof(DlgClaimsVisible));
            }
        }

        public bool DlgEmpresasVisible
        {
            get { return _dlgEmpresasVisible; }
            set
            {
                _dlgEmpresasVisible = value;
                AoMudarPropriedade(nameof(DlgEmpresasVisible));
            }
        }

        public bool DlgRolesVisible
        {
            get { return _dlgRolesVisible; }
            set
            {
                _dlgRolesVisible = value;
                AoMudarPropriedade(nameof(DlgRolesVisible));
            }
        }

        public bool ClaimsBtnEnabled { get; set; } = true;
        public bool EmpresasBtnEnabled { get; set; } = true;
        public bool RolesBtnEnabled { get; set; } = true;
        public CadastroUsuarioVM(IUsuariosService usuariosService,
            IClaimsService claimsService,
            IRolesService rolesService,
            IEmpresasService empresasService) : base(usuariosService)
        {
            UsuarioModel.Empresas = new List<UsuarioEmpresaModel>();
            UsuarioModel.Claims = new List<UsuarioClaimModel>();
            UsuarioModel.Roles = new List<UsuarioRoleModel>();

            _claimsService = claimsService;
            _rolesService = rolesService;
            _empresasService = empresasService;
            _dlgClaimsVisible = false;
            EmpresaUsuarioSelecionada = new UsuarioEmpresaModel { Empresa = new EmpresaModel() };
            RoleUsuarioSelecionada = new RoleModel();
            MensagemResultadoEnvio = " ";
        }
        public override async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroUsuarioVM.Enviar(this, _usuariosService);
        }

        public void AoSelecionarRoleUsuario(RowSelectEventArgs<UsuarioRoleModel> e)
        {
            RoleUsuarioSelecionada = e.Data.Role;
        }

        public void AoSelecionarClaimUsuario(RowSelectEventArgs<UsuarioClaimModel> e)
        {
            ClaimUsuarioSelecionada = e.Data.ClaimType;
        }

        public void AoSelecionarEmpresaUsuario(RowSelectEventArgs<UsuarioEmpresaModel> e)
        {
            EmpresaUsuarioSelecionada = e.Data;
            EmpresaSelecionada = EmpresaUsuarioSelecionada.Empresa;
        }

        public async void AoClicarClaimsBtn(MouseEventArgs e)
        {
            await LocalizaClaimsCadastroUsuarioVM.Localizar(this, _claimsService);
        }
        public void AoClicarRetirarClaimBtn(MouseEventArgs e)
        {
            RetiraClaimCadastroUsuarioVM.Retirar(this);
        }

        public void AoClicarInserirClaimBtn(MouseEventArgs e)
        {
            AddClaimCadastroUsuarioVM.Add(this);
        }

        public void AoClicarRetirarEmpresaBtn(MouseEventArgs e)
        {
            RetiraEmpresaCadastroUsuarioVM.Retirar(this);
        }
        public void AoClicarRetirarRoleBtn(MouseEventArgs e)
        {
            RetiraRoleCadastroUsuarioVM.Retirar(this);
        }

        public void AoClicarEmpresasBtn(MouseEventArgs e)
        {
            LocalizaEmpresasCadastroUsuarioVM.Localizar(this, _empresasService);
        }

        public void AoClicarRolesBtn(MouseEventArgs e)
        {
            LocalizaRolesCadastroUsuarioVM.Localizar(this, _rolesService);
        }

        public void AoSelecionarClaim(Syncfusion.Blazor.Grids.RowSelectEventArgs<ClaimModel> e)
        {
            ClaimSelecionada = e.Data.Nome;
        }

        public void AoFecharDialogoClaims(object e)
        {
            ClaimsBtnEnabled = true;
            DlgClaimsVisible = false;
        }
        public void AoClicarOkBtnDialogoClaims(MouseEventArgs e)
        {
            ClaimUsuarioSelecionada = ClaimSelecionada;
            DlgClaimsVisible = false;
        }

        public void AoCriarNomeTextBox(object e)
        {
            NomeTextBox.FocusAsync();
        }

        public void AoSelecionarEmpresa(Syncfusion.Blazor.Grids.RowSelectEventArgs<EmpresaModel> e)
        {
            EmpresaSelecionada = e.Data;
        }

        public void AoSelecionarRole(Syncfusion.Blazor.Grids.RowSelectEventArgs<RoleModel> e)
        {
            RoleSelecionada = e.Data;
        }

        public void AoFecharDialogoEmpresas(object e)
        {
            EmpresasBtnEnabled = true;
            DlgEmpresasVisible = false;
        }

        public void AoFecharDialogoRoles(object e)
        {
            RolesBtnEnabled = true;
            DlgRolesVisible = false;
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
            NomeTextBox.FocusAsync();

        }

        public void AoClicarOkBtnDialogoEmpresas(MouseEventArgs e)
        {
            EmpresaUsuarioSelecionada.Empresa = EmpresaSelecionada;
            EmpresaUsuarioSelecionada.IdEmpresa = EmpresaSelecionada.Id;
            DlgEmpresasVisible = false;
        }

        public void AoClicarOkBtnDialogoRoles(MouseEventArgs e)
        {
            RoleUsuarioSelecionada = RoleSelecionada;
            DlgRolesVisible = false;
        }

        public void AoClicarInserirEmpresaBtn(MouseEventArgs e)
        {
            AddEmpresaCadastroUsuarioVM.Add(this);
            EmpresaUsuarioSelecionada = new UsuarioEmpresaModel { Empresa = new EmpresaModel() };
            EmpresaSelecionada = new EmpresaModel();
        }

        public void AoClicarInserirRoleBtn(MouseEventArgs e)
        {
            AddRoleCadastroUsuarioVM.Add(this);
            RoleUsuarioSelecionada = new RoleModel();
        }

        public void AoSetarUsaGafMobile(Microsoft.AspNetCore.Components.ChangeEventArgs e)
        {
            if (!(bool)e.Value)
            {
                EmpresaUsuarioSelecionada.ValorRepasseGafmobile = 0;
            }
        }

        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }

        public void LimparTela()
        {
            LimpaTelaCadastroUsuarioVM.Limpar(this);
        }
    }
}
