﻿using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Administrativo;
using GAFNETLibrary.Enums;
using Microsoft.AspNetCore.Hosting;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Administrativo.CadastroEmpresa
{
    public class TabelaIBPTVM : CadastroBaseVM, ITabelaIBPTVM
    {
        private readonly IImportacaoTabelaIBPTService _importTabIBPTService;
        private readonly ILoginStore _loginStore;
        private string _logImportacaoTabelasIBPT;

        public TabelaIBPTVM(IImportacaoTabelaIBPTService importTabIBPTService, ILoginStore loginStore, IWebHostEnvironment env)
        {
            _importTabIBPTService = importTabIBPTService;
            _loginStore = loginStore;
            _importTabIBPTService.OnProcessarTabelaIBPT += AoProcessarTabelaIBPT;
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }

        public bool PodeAcessar
        {
            get
            {
                return (_loginStore.EhAdmin);
            }
        }
        public string LogImportacaoTabelasIBPT
        {
            get { return _logImportacaoTabelasIBPT; }
            set
            {
                _logImportacaoTabelasIBPT = value;
                AoMudarPropriedade(nameof(LogImportacaoTabelasIBPT));
            }
        }

        public void AoFecharDlg(object e)
        {
            throw new System.NotImplementedException();
        }

        public async Task AoClicarImportarTabelasIBPTBtn()
        {
            LogImportacaoTabelasIBPT = "";
            await SpinnerPrincipal.ShowAsync();
            var resultImport = await _importTabIBPTService.Importar();
            await SpinnerPrincipal.HideAsync();
            if (!resultImport.Sucesso)
            {
                await ExibirMensagemToast((string)resultImport.Retorno, TipoToast.Erro);
                return;
            }
            await ExibirMensagemToast("Tabelas IBPT importadas com sucesso", TipoToast.Sucesso);
        }

        public bool DadosValidos()
        {
            throw new System.NotImplementedException();
        }

        public void Enviar()
        {
            throw new System.NotImplementedException();
        }

        public void LimparTela()
        {
            throw new System.NotImplementedException();
        }
        private void AoProcessarTabelaIBPT(string args)
        {
            LogImportacaoTabelasIBPT += $"\r\n {args}";
        }

    }
}
