﻿using FluentValidation;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Models.Validations;
using Microsoft.AspNetCore.Components.Web;
using Newtonsoft.Json;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Popups;
using System.Collections.Generic;
using System.Threading.Tasks;
using GAFNETLibrary.Enums;
using FluentValidation.Results;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Administrativo
{
    public abstract class UsuarioBaseVM : CadastroBaseVM
    {
        protected readonly IUsuariosService _usuariosService;

        private UsuarioModel _usuarioModel;
        private bool _dialogoUsuariosVisible;
        public UsuarioModel UsuarioModel
        {
            get { return _usuarioModel; }
            set
            {
                _usuarioModel = value;
                AoMudarPropriedade(nameof(UsuarioModel));
            }
        }

        public UsuarioModel UsuarioSelecionado { get; set; }
        protected string EMailSelecionado { get; set; }
        public List<UsuarioModel> Usuarios { get; set; } = new List<UsuarioModel>();
        public SfDialog Dialogo { get; set; }
        public bool DialogoUsuariosVisible
        {
            get { return _dialogoUsuariosVisible; }
            set
            {
                _dialogoUsuariosVisible = value;
                AoMudarPropriedade(nameof(DialogoUsuariosVisible));
            }
        }

        public UsuarioBaseVM(IUsuariosService usuariosService)
        {
            _usuarioModel           = new UsuarioModel();
            _usuariosService        = usuariosService;
            _dialogoUsuariosVisible = false;
            DlgVisible              = false;
            MensagemResultadoEnvio  = " ";
        }
        public abstract void Enviar();

        public async Task<List<UsuarioModel>> GetUsuarios()
        {
            var retorno = await _usuariosService.GetUsuarios();

            if (!retorno.IsSuccessStatusCode)
            {
                Mensagem = await retorno.Content.ReadAsStringAsync();
                return null;
            }
            else
            {
                var dados = await retorno.Content.ReadAsStringAsync();
                Usuarios = JsonConvert.DeserializeObject<List<UsuarioModel>>(dados);
                Mensagem = string.Empty;
                return Usuarios;
            }
        }

        public async Task<UsuarioModel> GetUsuarioByEMail()
        {
            var retorno = await _usuariosService.GetUsuarioByEMail(EMailSelecionado, StatusAtivo.Todos);

            if (!retorno.IsSuccessStatusCode)
            {
                Mensagem = await retorno.Content.ReadAsStringAsync();
                return null;
            }
            else
            {
                var usuario = JsonConvert.DeserializeObject<UsuarioModel>(await retorno.Content.ReadAsStringAsync());
                Mensagem = string.Empty;
                UsuarioModel = usuario;
                return usuario;
            }
        }

        public void AoSelecionarUsuario(RowSelectEventArgs<UsuarioModel> e)
        {
            EMailSelecionado = e.Data.Email;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            EMailSelecionado = string.Empty;
            Localizando = true;
            await GetUsuarios();
            Localizando = false;
            DialogoUsuariosVisible = true;
        }
        public void AoFecharDialogoGridUsuarios(object e)
        {
            Localizando = false;
            DialogoUsuariosVisible = false;
        }
        public async void AoClicarOkBtnDialogoGridUsuarios(MouseEventArgs e)
        {
            ErrosValidacaoFormulario.Clear();
            if (string.IsNullOrEmpty(EMailSelecionado))
            {
                return;
            }
            Localizando = false;
            UsuarioSelecionado = await GetUsuarioByEMail();
            UsuarioModel.CopiarDados(UsuarioSelecionado);
            DialogoUsuariosVisible = false;
        }

        public void AoFecharDialogo(object e)
        {
            DlgVisible = false;
        }

        public bool DadosValidos()
        {
            ValidationResult results;
            if (this is SetaSenhaUsuarioVM)
            {
                var validator = new AlteraSenhaUsuarioModelValidator();
                results = validator.Validate(UsuarioModel);
            }
            else
            {
                var validator = new UsuarioModelValidator();
                results = validator.Validate(UsuarioModel);
            }
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new List<string>();

            
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }
    }
}
