﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Components;
using GAFNETLibrary.Enums;
using Syncfusion.Blazor.Spinner;
using System.ComponentModel;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels;

public partial class BaseVM : ObservableObject
{
    [ObservableProperty]
    private int tempoToast;
    partial void OnTempoToastChanged(int value) => AoMudarPropriedade(nameof(TempoToast));

    [ObservableProperty]
    private TipoToast tipoToast;
    partial void OnTipoToastChanged(TipoToast value) => AoMudarPropriedade(nameof(TipoToast));

    [ObservableProperty]
    private string mensagem;
    partial void OnMensagemChanged(string value) => AoMudarPropriedade(nameof(Mensagem));

    public SfSpinner SpinnerPrincipal { get; set; }
    public MeuToast MeuToast { get; set; }
    public event PropertyChangedEventHandler PropriedadeMudou;
    public void AoMudarPropriedade(string propertyName)
    {
        PropriedadeMudou?.Invoke(this, new PropertyChangedEventArgs(propertyName));
    }

    public async Task ExibirMensagemToast(string mensagem, TipoToast tipoToast)
    {
        await ExibirMensagemToastGeral(mensagem, tipoToast, 5000);
    }

    public async Task ExibirMensagemToast(string mensagem, TipoToast tipoToast, int tempoToast)
    {
        await ExibirMensagemToastGeral(mensagem, tipoToast, tempoToast);
    }

    private async Task ExibirMensagemToastGeral(string mensagem, TipoToast tipoToast, int tempoToast)
    {
        Mensagem = mensagem;
        TipoToast = tipoToast;
        TempoToast = tempoToast;
        await Task.Delay(100);
        AoMudarPropriedade(nameof(TipoToast));
        await MeuToast.Show();
    }

    public async Task HideAllToasts()
    {
        await MeuToast.HideAll();
    }
}
