﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces;
using GAFNETLibrary.Models;
using System.Collections.Generic;
using System.ComponentModel;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels
{
    public class LoginViewModel : ILoginViewModel
    {
        public event PropertyChangedEventHandler PropertyChanged;

        private bool _dialogoVisible;
        public LoginUserModel LoginUserModel { get; set; } = new LoginUserModel();

        private ILoginStore _loginStore { get; set; }
        public bool EnviarBtnDisabled { get; set; } = false;
        public string MensagemErro { get; set; }

        public bool DialogoVisible
        {
            get { return _dialogoVisible; }
            set
            {
                _dialogoVisible = value;
                PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(DialogoVisible)));
            }
        }

        public List<string> ErrosValidacaoFormulario { get; } = new List<string>();


        public LoginViewModel(ILoginStore loginStore)
        {
            _loginStore = loginStore;
        }

        public async Task Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            EnviarBtnDisabled = true;

            var retornoLogin = await _loginStore.RealizarLogin(LoginUserModel);

            if (string.IsNullOrEmpty(_loginStore.Token))
            {
                MensagemErro = await retornoLogin.Content.ReadAsStringAsync();
                DialogoVisible = true;
            }
            else
            {
                MensagemErro = string.Empty;
                DialogoVisible = false;
            }
            EnviarBtnDisabled = false;
        }

        public void AoFecharDialogo(object e)
        {
            DialogoVisible = false;
            MensagemErro = string.Empty;
        }

        public bool DadosValidos()
        {
            LoginUserModelValidator validator = new();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(LoginUserModel);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }

            return true;
        }

    }
}
