﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Services;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Representante;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.FileProviders;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Representante;

public partial class DownloadsVM : CadastroBaseVM, IDownloadsVM
{
    private readonly IFileProvider _fileProvider;
    private readonly ILoginStore _loginStore;
    private readonly IWebHostEnvironment _env;
    private readonly ITrocaArquivosService _trocaArquivoService;

    public DownloadsVM(ITrocaArquivosService trocaArquivosService, ILoginStore loginStore, IWebHostEnvironment env, IFileProvider fileProvider, ITrocaArquivosService trocaArquivoService)
    {
        _fileProvider = fileProvider;
        _loginStore = loginStore;
        _env = env;
        _trocaArquivoService = trocaArquivoService;
    }

    public string UrlRedirecionamento
    {
        get
        {
            if (!PodeAcessar)
            {
                return "/Login";
            }
            if (!_loginStore.LogadoEmEmpresa)
            {
                return "/SelecionaEmpresaLogada";
            }
            return string.Empty;
        }
    }

    public bool PodeAcessar
    {
        get
        {
            return (_loginStore.EhAdmin || _loginStore.EhRepresentante);
        }
    }

    public IFileInfo DownloadSelecionado { get; set; }

    [ObservableProperty]
    private List<IFileInfo> arquivosParaDownload;
    partial void OnArquivosParaDownloadChanged(List<IFileInfo> value) => AoMudarPropriedade(nameof(ArquivosParaDownload));

    [ObservableProperty]
    private string logUpload;
    partial void OnLogUploadChanged(string value) => AoMudarPropriedade(nameof(LogUpload));
    [ObservableProperty]
    private string logDownload;
    partial void OnLogDownloadChanged(string value) => AoMudarPropriedade(nameof(LogDownload));

    public void GetArquivosParaDownload()
    {
        List<IFileInfo> arquivos = new();
        var cnpjRepresentante = _loginStore.Claims.FirstOrDefault(c => c.Type == "CnpjRepresentante");
        var pasta = $@"downloads\{cnpjRepresentante.Value}";
        var contents = _fileProvider.GetDirectoryContents(pasta);
        foreach (var item in contents)
        {
            arquivos.Add(item);
        }
        ArquivosParaDownload = arquivos;
    }

    public bool DadosValidos()
    {
        throw new System.NotImplementedException();
    }

    public void Enviar()
    {
        throw new System.NotImplementedException();
    }

    public void LimparTela()
    {
        throw new System.NotImplementedException();
    }
    public Task AoClicarDownloadBtn()
    {
        throw new System.NotImplementedException();
    }

    public void AoFecharDlg(object e)
    {
        throw new System.NotImplementedException();
    }

    public void AoSelecionarArquivoDownload(RowSelectEventArgs<IFileInfo> e)
    {
        DownloadSelecionado = e.Data;
    }

    public async Task<ResultadoModel> AoClicarSelecionarBtnGridDownloads(CommandClickEventArgs<IFileInfo> e)
    {
        try
        {
            await SpinnerPrincipal.ShowAsync();

            string caminhoArquivo = e.RowData.PhysicalPath;
            var result = await _trocaArquivoService.Download(caminhoArquivo);
            if (!result.Sucesso)
            {
                await ExibirMensagemToast((string)result.Retorno, TipoToast.Erro);
                return result;
            }

            return new ResultadoModel { Sucesso = true, Retorno = "OK" };
        }
        finally
        {
            await SpinnerPrincipal.HideAsync();
        }
    }
}
