﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Representante;
using GAFNETBlazor.ViewModels.Usuario.CadastroAliquota;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Components.Web;
using Newtonsoft.Json;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using System.Collections.Generic;
using System.Net.Http;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Administrativo
{
    public class ManutencaoEmpresaVM : CadastroBaseVM, IManutencaoEmpresaVM
    {
        private readonly IEmpresasService _empresasService;

        private bool _dlgEmpresasVisible;
        private EmpresaModel _empresa;

        public SfTextBox NomeTextBox { get; set; }
        public RepresentanteModel Representante { get; set; }
        public EmpresaModel EmpresaSelecionada { get; set; }
        public EmpresaModel Empresa
        {
            get { return _empresa; }
            set
            {
                _empresa = value;
                AoMudarPropriedade(nameof(Empresa));
            }
        }

        public List<EmpresaModel> Empresas { get; set; } = new List<EmpresaModel>();
        public bool DlgEmpresasVisible
        {
            get { return _dlgEmpresasVisible; }
            set
            {
                _dlgEmpresasVisible = value;
                AoMudarPropriedade(nameof(DlgEmpresasVisible));
            }
        }
        public SfDialog Dlg { get; set; }

        public ManutencaoEmpresaVM(IEmpresasService empresasService)
        {
            Representante = new RepresentanteModel();
            Empresa = new EmpresaModel();
            EmpresaSelecionada = new EmpresaModel();
            _empresasService = empresasService;
            MensagemResultadoEnvio = " ";
        }
        public async void Enviar()
        {
            Enviando = true;
            string mensagemSucesso;

            Empresa.Representante = null;

            HttpResponseMessage retorno;
            mensagemSucesso = "Empresa alterada com sucesso";
            retorno = await _empresasService.AlterarEmpresaComoRepresentante(Empresa);



            if (!retorno.IsSuccessStatusCode)
            {
                Mensagem = Task.Run(() => retorno.Content.ReadAsStringAsync()).GetAwaiter().GetResult();
                MensagemResultadoEnvio = Mensagem;
            }
            else
            {
                Mensagem = string.Empty;
                MensagemResultadoEnvio = mensagemSucesso;
                LimparTela();
            }

            Enviando = false;
            DlgVisible = true;
        }

        public async Task<List<EmpresaModel>> GetEmpresas()
        {
            var retorno = await _empresasService.GetEmpresasByRepresentanteLogado();

            if (!retorno.IsSuccessStatusCode)
            {
                Mensagem = await retorno.Content.ReadAsStringAsync();
                return null;
            }
            else
            {
                var dados = await retorno.Content.ReadAsStringAsync();
                Empresas = JsonConvert.DeserializeObject<List<EmpresaModel>>(dados);
                Mensagem = string.Empty;
                return Empresas;
            }
        }

        public void AoSelecionarEmpresa(RowSelectEventArgs<EmpresaModel> e)
        {
            EmpresaSelecionada = e.Data;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            await GetEmpresas();
            Localizando = false;
            DlgEmpresasVisible = true;
        }
        public void LimparTela()
        {
            Representante = new RepresentanteModel();
            EmpresaSelecionada = new EmpresaModel();
            Empresa = new EmpresaModel();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public void AoFecharDlgEmpresas(object e)
        {
            DlgEmpresasVisible = false;
        }
        public void AoClicarOkBtnDlgEmpresas(MouseEventArgs e)
        {
            if (EmpresaSelecionada == null)
            {
                return;
            }
            ExibirAliquotaSelecionada();

        }

        public bool DadosValidos()
        {
            throw new System.NotImplementedException();
        }

        public void AoClicarSelecionarBtnGridEmpresas(CommandClickEventArgs<EmpresaModel> e)
        {
            EmpresaSelecionada = e.RowData;
            ExibirAliquotaSelecionada();
        }

        private void ExibirAliquotaSelecionada()
        {
            ErrosValidacaoFormulario.Clear();
            
            Empresa = EmpresaSelecionada;
            Representante = EmpresaSelecionada.Representante;
            DlgEmpresasVisible = false;
        }


    }
}
