﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Representante;
using GAFNETLibrary.Enums;
using Microsoft.AspNetCore.Hosting;
using Syncfusion.Blazor.Inputs;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Representante;

public partial class UploadsVM : CadastroBaseVM, IUploadsVM
{
    private readonly ITrocaArquivosService _trocaArquivosService;
    private readonly ILoginStore _loginStore;
    private readonly IWebHostEnvironment _env;

    public UploadsVM(ITrocaArquivosService trocaArquivosService, ILoginStore loginStore, IWebHostEnvironment env)
    {
        _trocaArquivosService = trocaArquivosService;
        _loginStore = loginStore;
        _env = env;
    }

    public string UrlRedirecionamento
    {
        get
        {
            if (!PodeAcessar)
            {
                return "/Login";
            }
            if (!_loginStore.LogadoEmEmpresa)
            {
                return "/SelecionaEmpresaLogada";
            }
            return string.Empty;
        }
    }

    public bool PodeAcessar
    {
        get
        {
            return (_loginStore.EhAdmin || _loginStore.EhRepresentante);
        }
    }

    [ObservableProperty]
    private string logUpload;
    partial void OnLogUploadChanged(string value) => AoMudarPropriedade(nameof(LogUpload));
    public async Task AoSelecionarUpload(UploadChangeEventArgs args)
    {
        LogUpload = "";
        await SpinnerPrincipal.ShowAsync();
        var resultImport = _trocaArquivosService.Upload(args);
        await SpinnerPrincipal.HideAsync();
        if (!resultImport.Sucesso)
        {
            await ExibirMensagemToast((string)resultImport.Retorno, TipoToast.Erro);
            return;
        }
        await ExibirMensagemToast("Upload realizado com sucesso.", TipoToast.Sucesso);
    }

    public bool DadosValidos()
    {
        throw new System.NotImplementedException();
    }

    public void Enviar()
    {
        throw new System.NotImplementedException();
    }

    public void LimparTela()
    {
        throw new System.NotImplementedException();
    }
    private void AoProcessarUpload(string args)
    {
        LogUpload += $"\r\n {args}";
    }

    public Task AoClicarDownloadBtn()
    {
        throw new System.NotImplementedException();
    }

    public void AoFecharDlg(object e)
    {
        throw new System.NotImplementedException();
    }
}
