﻿using FluentValidation;
using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.DropDowns;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using Syncfusion.Blazor.Spinner;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroAliquota
{
    public class CadastroAliquotaVM : CadastroBaseVM, ICadastroAliquotaVM
    {
        private readonly IAliquotasService _aliquotasService;
        private readonly ILoginStore _loginStore;
        private readonly ISitTribService _sitTribService;
        private readonly ISitTribPISCOFINSService _sitTribPISCOFINSService;
        private readonly ISitTribPISCOFINSEntradaService _sitTribPISCOFINSEntradaService;
        private readonly ISitTribIPIService _sitTribIPIService;
        private readonly ITabelaAService _tabelaAService;
        private readonly ITiposReducaoICMSService _tiposReducaoICMS;
        private readonly ITiposReducaoICMSSTService _tiposReducaoICMSST;

        private bool _dlgAliquotasVisible;
        private AliquotaModel _aliquota;

        public AliquotaModel Aliquota
        {
            get { return _aliquota; }
            set
            {
                _aliquota = value;
                AoMudarPropriedade(nameof(Aliquota));
            }
        }

        public bool PodeAcessar
        {
            get
            {
                var claimAliquota = _loginStore.Claims.FirstOrDefault(c => c.Type == "Aliquota");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimAliquota != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }
        public bool DlgAliquotasVisible
        {
            get { return _dlgAliquotasVisible; }
            set
            {
                _dlgAliquotasVisible = value;
                AoMudarPropriedade(nameof(DlgAliquotasVisible));
            }
        }

        public AliquotaInterestadualModel AliquotaInterestadual { get; set; } = new AliquotaInterestadualModel();
        public List<ListaModel> ListaTabelaA { get; }
        public List<ListaModel> ListaSitTrib { get; }
        public SfDialog Dlg { get; set; }
        public SfTextBox DescricaoTextBox { get; set; }
        public SfDropDownList<string, string> UFDropDownList { get; set; }

        public AliquotaModel AliquotaSelecionada { get; set; } = new AliquotaModel();
        public List<AliquotaModel> Aliquotas { get; set; } = new List<AliquotaModel>();
        public List<ListaModel> ListaCSTPISCOFINS { get; }
        public List<ListaModel> ListaCSTPISCOFINSEntrada { get; }
        public List<ListaModel> ListaSitTribIPI { get; }
        public List<ListaModel> ListaTipoReducaoICMS { get; }

        public List<ListaModel> ListaTipoReducaoICMSST { get; }

        public List<string> ListaUFs => UFsService.UFs();

        public CadastroAliquotaVM(IAliquotasService aliquotasService,
            ISitTribService sitTribService,
            ILoginStore loginStore,
            ISitTribPISCOFINSService sitTribPISCOFINSService,
            ISitTribPISCOFINSEntradaService sitTribPISCOFINSEntradaService,
            ISitTribIPIService sitTribIPIService,
            ITabelaAService tabelaAService,
            ITiposReducaoICMSService tiposReducaoICMS,
            ITiposReducaoICMSSTService tiposReducaoICMSST)
        {
            _loginStore                     = loginStore;
            _aliquotasService               = aliquotasService;
            _sitTribService                 = sitTribService;
            _sitTribIPIService              = sitTribIPIService;
            _sitTribPISCOFINSService        = sitTribPISCOFINSService;
            _sitTribPISCOFINSEntradaService = sitTribPISCOFINSEntradaService;
            _tabelaAService                 = tabelaAService;
            _tiposReducaoICMS               = tiposReducaoICMS;
            _tiposReducaoICMSST             = tiposReducaoICMSST;
            Aliquota                        = new AliquotaModel();
            MensagemResultadoEnvio          = " ";
            ListaTabelaA                    = _tabelaAService.Itens();
            ListaSitTrib                    = _sitTribService.Itens();
            ListaCSTPISCOFINS = _sitTribPISCOFINSService.Itens();
            ListaCSTPISCOFINSEntrada        = _sitTribPISCOFINSEntradaService.Itens();
            ListaSitTribIPI                 = _sitTribIPIService.Itens();
            ListaTipoReducaoICMS            = _tiposReducaoICMS.Itens();
            ListaTipoReducaoICMSST          = _tiposReducaoICMSST.Itens();
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroAliquota.Enviar(this, _aliquotasService);
        }

        public void AoCriarDescricaoTextBox(object e)
        {
            DescricaoTextBox.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }

        public async void AoClicarRetirarAliqInterestBtnAsync(MouseEventArgs e)
        {
            await RetiraAliqInterestCadastroAliquota.Retirar(this);
        }

        public async void AoClicarInserirAliqInterestBtnAsync(MouseEventArgs e)
        {
            await InsereAliqInterestCadastroAliquota.Add(this);
        }

        public bool DadosValidos()
        {
            var validator = new AliquotaModelValidator();
            ErrosValidacaoFormulario.Clear();
            var erros = new List<string>();

            var results = validator.Validate(Aliquota);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }

        public void AoFecharDlgAliquotas(object e)
        {
            DlgAliquotasVisible = false;
        }

        public async void AoClicarOkBtnDlgAliquotas(MouseEventArgs e)
        {
            if (AliquotaSelecionada == null)
            {
                return;
            }
            ErrosValidacaoFormulario.Clear();
            DlgAliquotasVisible = false;
            Aliquota = await AliquotaByIdCadastroAliquota.Get(this, _aliquotasService, AliquotaSelecionada.Id, StatusAtivo.Todos);
        }

        public void AoSelecionarAliquota(RowSelectEventArgs<AliquotaModel> e)
        {
            AliquotaSelecionada = e.Data;
        }

        public async Task<List<AliquotaModel>> GetAliquotas()
        {
            var retorno = await _aliquotasService.GetAsync(URIsAliquotas.GetAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<AliquotaModel>();
            }
            return (List<AliquotaModel>)retorno.Objeto;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            Aliquotas = await GetAliquotas();
            Localizando = false;
            DlgAliquotasVisible = true;
        }

        public async void IdAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            if (Aliquota.Id == 0) return;
            Aliquota = await AliquotaByIdCadastroAliquota.Get(this, _aliquotasService, Aliquota.Id, StatusAtivo.Todos);
        }

        public void AoSelecionarAliquotaInterestadual(RowSelectEventArgs<AliquotaInterestadualModel> e)
        {
            AliquotaInterestadual = e.Data;
        }

        public void AoSelecionarTabelaAEstadual()
        {
            AoMudarPropriedade(nameof(Aliquota));
        }

        public void AoSelecionarSitTribEstadual()
        {
            AoMudarPropriedade(nameof(Aliquota));
        }

        public void AoSelecionarUFAliqInterest()
        {
            var aliqInterest = Aliquota.AliquotasInterestaduais.FirstOrDefault(a => a.UF == AliquotaInterestadual.UF);
            if (aliqInterest != null)
            {
                AliquotaInterestadual = aliqInterest.Clone();
            }
        }
        public void LimparTela()
        {
            LimpaTelaCadastroAliquota.Limpar(this, true);
        }

        public async void AoClicarSelecionarBtnGridAliquotas(CommandClickEventArgs<AliquotaModel> e)
        {
            AliquotaSelecionada = e.RowData;
            await ExibirAliquotaSelecionada();
        }

        private async Task ExibirAliquotaSelecionada()
        {
            ErrosValidacaoFormulario.Clear();
            DlgAliquotasVisible = false;
            Aliquota = await AliquotaByIdCadastroAliquota.Get(this, _aliquotasService, AliquotaSelecionada.Id, StatusAtivo.Todos);
        }
    }
}
