﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroCentroDeCusto
{
    public partial class CadastroCentroDeCustoVM : CadastroBaseVM, ICadastroCentroDeCustoVM
    {
        private readonly ICentrosDeCustoService _centrosDeCustoService;
        private readonly ILoginStore _loginStore;

        [ObservableProperty]
        private CentroDeCustoModel centroDeCusto;
        partial void OnCentroDeCustoChanged(CentroDeCustoModel value) => AoMudarPropriedade(nameof(CentroDeCusto));

        [ObservableProperty]
        private bool dlgCentrosDeCustoVisible;
        partial void OnDlgCentrosDeCustoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgCentrosDeCustoVisible));

        public bool PodeAcessar
        {
            get
            {
                var claimCentroDeCusto = _loginStore.Claims.FirstOrDefault(c => c.Type == "Centro de Custo");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimCentroDeCusto != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }
        

        public SfNumericTextBox<long> IdTextBox { get; set; }
        public SfTextBox NomeTextBox { get; set; }
        public CentroDeCustoModel CentroDeCustoSelecionado { get; set; } = new CentroDeCustoModel();
        public List<CentroDeCustoModel> CentrosDeCusto { get; set; } = new List<CentroDeCustoModel>();
        public CadastroCentroDeCustoVM(ICentrosDeCustoService centrosDeCustoService, ILoginStore loginStore)
        {
            _loginStore = loginStore;
            _centrosDeCustoService = centrosDeCustoService;
            CentroDeCusto = new CentroDeCustoModel { Ativo = true };
            MensagemResultadoEnvio = " ";
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroCentroDeCusto.Enviar(this, _centrosDeCustoService);
        }

        public async void AoCriarDescricaoTextBox(object e)
        {
            await NomeTextBox.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public async void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
            await Task.Delay(100);
            await NomeTextBox.FocusAsync();
        }

        public bool DadosValidos()
        {
            var validator = new CentroDeCustoModelValidator();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(CentroDeCusto);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }

        public void AoFecharDlgCentrosDeCusto(object e)
        {
            DlgCentrosDeCustoVisible = false;
        }

        public async void AoClicarOkBtnDlgCentrosDeCusto(MouseEventArgs e)
        {
            if (CentroDeCustoSelecionado?.Id == 0)
            {
                return;
            }
            ErrosValidacaoFormulario.Clear();
            DlgCentrosDeCustoVisible = false;
            CentroDeCusto = await CentroDeCustoByIdCadastroCentroDeCusto.Get(this, _centrosDeCustoService, CentroDeCustoSelecionado.Id, StatusAtivo.Todos);
        }

        public void AoSelecionarCentroDeCusto(RowSelectEventArgs<CentroDeCustoModel> e)
        {
            CentroDeCustoSelecionado = e.Data;
        }

        public async Task<List<CentroDeCustoModel>> GetCentrosDeCusto()
        {
            var retorno = await _centrosDeCustoService.GetAsync(URIsCentrosDeCusto.GetAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<CentroDeCustoModel>();
            }
            return (List<CentroDeCustoModel>)retorno.Objeto;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            CentrosDeCusto = await GetCentrosDeCusto();
            Localizando = false;
            DlgCentrosDeCustoVisible = true;
        }

        public async void IdAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            if (CentroDeCusto.Id == 0) return;
            CentroDeCusto = await CentroDeCustoByIdCadastroCentroDeCusto.Get(this, _centrosDeCustoService, CentroDeCusto.Id, StatusAtivo.Todos);
        }

        public void LimparTela()
        {
            LimpaTelaCadastroCentroDeCusto.Limpar(this);
        }

        public async void AoClicarSelecionarBtnGridCentrosDeCusto(CommandClickEventArgs<CentroDeCustoModel> e)
        {
            CentroDeCustoSelecionado = e.RowData;
            await ExibirCentroDeCustoSelecionado();
        }

        private async Task ExibirCentroDeCustoSelecionado()
        {
            ErrosValidacaoFormulario.Clear();
            DlgCentrosDeCustoVisible = false;
            CentroDeCusto = await CentroDeCustoByIdCadastroCentroDeCusto.Get(this, _centrosDeCustoService, CentroDeCustoSelecionado.Id, StatusAtivo.Todos);
        }

        public async Task AoPressionarTeclaCentrosDeCustoGrid(KeyboardEventArgs args)
        {
            if (args.Key == "Enter" && CentroDeCustoSelecionado?.Id > 0)
            {
                await ExibirCentroDeCustoSelecionado();
                await NomeTextBox.FocusAsync();
            }
        }

    }
}
