﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroClassificacaoFiscal
{
    public partial class CadastroClassificacaoFiscalVM : CadastroBaseVM, ICadastroClassificacaoFiscalVM
    {
        private readonly IUnidadesProdutoService _unidadesService;
        private readonly IClassificacoesFiscaisService _classfiscaisService;
        private readonly ILoginStore _loginStore;

        private bool _dlgUnidadesTribVisible;
        private bool _dlgClassFiscaisVisible;
        //private ClassificacaoFiscalModel _classFiscal;

        [ObservableProperty]
        private ClassificacaoFiscalModel classFiscal;
        partial void OnClassFiscalChanged(ClassificacaoFiscalModel value) => AoMudarPropriedade(nameof(ClassFiscal));

        public bool PodeAcessar
        {
            get
            {
                var claimClassFiscal = _loginStore.Claims.FirstOrDefault(c => c.Type == "Classificacao Fiscal");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimClassFiscal != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }
        public bool DlgUnidadesTribVisible
        {
            get { return _dlgUnidadesTribVisible; }
            set
            {
                _dlgUnidadesTribVisible = value;
                AoMudarPropriedade(nameof(DlgUnidadesTribVisible));
            }
        }

        public bool DlgClassFiscaisVisible
        {
            get { return _dlgClassFiscaisVisible; }
            set
            {
                _dlgClassFiscaisVisible = value;
                AoMudarPropriedade(nameof(DlgClassFiscaisVisible));
            }
        }

        public SfDialog Dlg { get; set; }
        public SfNumericTextBox<int> NumeroTextBox { get; set; }
        public SfTextBox DescricaoTextBox { get; set; }
        public UnidadeProdutoModel UnidadeTribSelecionada { get; set; } = new UnidadeProdutoModel();
        public List<UnidadeProdutoModel> UnidadesTrib { get; set; } = new List<UnidadeProdutoModel>();
        public ClassificacaoFiscalModel ClassFiscalSelecionada { get; set; } = new ClassificacaoFiscalModel();
        public List<ClassificacaoFiscalModel> ClassFiscais { get; set; } = new List<ClassificacaoFiscalModel>();
        public bool UnidadesTribBtnEnabled { get; set; } = true;
        public bool PesquisandoUnidadesTrib { get; set; }

        public CadastroClassificacaoFiscalVM(IUnidadesProdutoService unidadesService, IClassificacoesFiscaisService classFiscaisService, ILoginStore loginStore)
        {
            _loginStore = loginStore;
            _unidadesService = unidadesService;
            _classfiscaisService = classFiscaisService;
            ClassFiscal = new ClassificacaoFiscalModel { Ativo = true, UnidadeTributaria = new UnidadeProdutoModel() };
            MensagemResultadoEnvio = " ";
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroClassificacaoFiscal.Enviar(this, _classfiscaisService);
        }

        public void AoCriarNumeroTextBox(object e)
        {
            NumeroTextBox.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }

        public bool DadosValidos()
        {
            var validator = new ClassificacaoFiscalModelValidator();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(ClassFiscal);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }

        public void AoFecharDlgUnidadesTrib(object e)
        {
            DlgUnidadesTribVisible = false;
        }

        public async void AoClicarOkBtnDlgUnidadesTrib(MouseEventArgs e)
        {
            if (UnidadeTribSelecionada == null)
            {
                return;
            }
            ClassFiscal.UnidadeTributaria = await UnidadeTribByIdCadastroClassificacaoFiscal.Get(this, _unidadesService, UnidadeTribSelecionada.Id, StatusAtivo.Todos);
            ClassFiscal.IdUnidadeTributaria = ClassFiscal.UnidadeTributaria.Id;
            AoMudarPropriedade(nameof(ClassFiscal));
            DlgUnidadesTribVisible = false;

        }

        public void AoSelecionarUnidadeTrib(RowSelectEventArgs<UnidadeProdutoModel> e)
        {
            UnidadeTribSelecionada = e.Data;
        }

        public async Task<List<UnidadeProdutoModel>> GetUnidadesTrib()
        {
            var retorno = await _unidadesService.GetAsync(URIsClassificacoesFiscais.GetUnidadesAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<UnidadeProdutoModel>();
            }
            return (List<UnidadeProdutoModel>)retorno.Objeto;
        }

        public async Task<List<ClassificacaoFiscalModel>> GetClassFiscais()
        {
            var retorno = await _classfiscaisService.GetAsync(URIsClassificacoesFiscais.GetAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<ClassificacaoFiscalModel>();
            }
            return (List<ClassificacaoFiscalModel>)retorno.Objeto;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            ClassFiscais = await GetClassFiscais();
            Localizando = false;
            DlgClassFiscaisVisible = true;
        }

        public async void IdAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            if (ClassFiscal.Id == 0) return;
            ClassFiscal = await ClassFiscalByIdCadastroClassificacaoFiscal.Get(this, _classfiscaisService, ClassFiscal.Id, StatusAtivo.Todos);
        }

        public void AoSelecionarClassFiscal(RowSelectEventArgs<ClassificacaoFiscalModel> e)
        {
            ClassFiscalSelecionada = e.Data;
        }

        public async void IdUnidadeTribAoSairAsync()
        {
            if (!ClassFiscal.IdUnidadeTributaria.HasValue)
            {
                ClassFiscal.UnidadeTributaria = new UnidadeProdutoModel();
                return;
            }

            ClassFiscal.UnidadeTributaria = await UnidadeTribByIdCadastroClassificacaoFiscal.Get(this, _unidadesService, ClassFiscal.IdUnidadeTributaria.Value, StatusAtivo.Ativos);

            AoMudarPropriedade(nameof(ClassFiscal.UnidadeTributaria));
        }

        public void AoFecharDlgClassFiscais(object e)
        {
            DlgClassFiscaisVisible = false;
        }

        public async void AoClicarOkBtnDlgClassFiscais(MouseEventArgs e)
        {
            if (ClassFiscalSelecionada == null)
            {
                return;
            }
            await ExibirClassificacaoFiscalSelecionada();
        }

        private async Task ExibirClassificacaoFiscalSelecionada()
        {
            ErrosValidacaoFormulario.Clear();
            DlgClassFiscaisVisible = false;
            ClassFiscal = await ClassFiscalByIdCadastroClassificacaoFiscal.Get(this, _classfiscaisService, ClassFiscalSelecionada.Id, StatusAtivo.Todos);
            await Task.Delay(300);
            await DescricaoTextBox.FocusAsync();
        }

        public async void AoClicarUnidadesTribBtnAsync(MouseEventArgs e)
        {
            await LocalizaUnidadeTribCadastroClassFiscal.LocalizarAsync(this, _unidadesService);
        }

        public void LimparTela()
        {
            LimpaTelaCadastroClassificacaoFiscal.Limpar(this);
        }

        public async void AoClicarSelecionarBtnGridClassificacoesFiscais(CommandClickEventArgs<ClassificacaoFiscalModel> e)
        {
            ClassFiscalSelecionada = e.RowData;
            await ExibirClassificacaoFiscalSelecionada();
        }

        public async Task AoPressionarTeclaClassificacoesFiscaisGrid(KeyboardEventArgs args)
        {
            if (args.Key == "Enter" && ClassFiscalSelecionada?.Id > 0)
            {
                await ExibirClassificacaoFiscalSelecionada();
            }
        }
    }
}
