﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroCondicaoDePagamento
{
    public partial class CadastroCondicaoDePagamentoVM : CadastroBaseVM, ICadastroCondicaoDePagamentoVM
    {
        private readonly ICondicoesDePagamentoService _condicoesDePagamentoService;
        private readonly ILoginStore _loginStore;

        [ObservableProperty]
        private CondicaoDePagamentoModel condicaoDePagamento;
        partial void OnCondicaoDePagamentoChanged(CondicaoDePagamentoModel value) => AoMudarPropriedade(nameof(CondicaoDePagamento));

        [ObservableProperty]
        private bool dlgCondicoesDePagamentoVisible;
        partial void OnDlgCondicoesDePagamentoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgCondicoesDePagamentoVisible));

        public bool PodeAcessar
        {
            get
            {
                var claimCondicaoDePagamento = _loginStore.Claims.FirstOrDefault(c => c.Type == "CondicaoDePagamento");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimCondicaoDePagamento != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }
        

        public SfNumericTextBox<long> CodigoTextBox { get; set; }
        public SfTextBox DescricaoParcelasTextBox { get; set; }
        public CondicaoDePagamentoModel CondicaoDePagamentoSelecionada { get; set; } = new CondicaoDePagamentoModel();
        public List<CondicaoDePagamentoModel> CondicoesDePagamento { get; set; } = new List<CondicaoDePagamentoModel>();
        public CadastroCondicaoDePagamentoVM(ICondicoesDePagamentoService CondicoesDePagamentoService, ILoginStore loginStore)
        {
            _loginStore = loginStore;
            _condicoesDePagamentoService = CondicoesDePagamentoService;
            CondicaoDePagamento = new CondicaoDePagamentoModel { Ativo = true };
            MensagemResultadoEnvio = " ";
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroCondicaoDePagamento.Enviar(this, _condicoesDePagamentoService);
        }

        public async void AoCriarDescricaoTextBox(object e)
        {
            await DescricaoParcelasTextBox.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public async void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
            await Task.Delay(100);
            await DescricaoParcelasTextBox.FocusAsync();
        }

        public bool DadosValidos()
        {
            var validator = new CondicaoDePagamentoModelValidator();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(CondicaoDePagamento);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }

            var parcelas = CondicaoDePagamento.DescricaoParcelas.Split("-");
            foreach (var dias in parcelas)
            {
                try
                {
                    int teste = Convert.ToInt32(dias);
                }
                catch (System.Exception)
                {
                    ErrosValidacaoFormulario.Add("Existem caracteres inválidos na descrição das parcelas. Use somente números e traço (-) para separar as parcelas.");
                    return false;
                }
            }

            return true;
        }

        public void AoFecharDlgCondicoesDePagamento(object e)
        {
            DlgCondicoesDePagamentoVisible = false;
        }

        public async void AoClicarOkBtnDlgCondicoesDePagamento(MouseEventArgs e)
        {
            if (CondicaoDePagamentoSelecionada?.Id == 0)
            {
                return;
            }
            ErrosValidacaoFormulario.Clear();
            DlgCondicoesDePagamentoVisible = false;
            CondicaoDePagamento = await CondPgtoByIdCadastroCondPgto.Get(this, _condicoesDePagamentoService, CondicaoDePagamentoSelecionada.Id, StatusAtivo.Todos);
        }

        public void AoSelecionarCondicaoDePagamento(RowSelectEventArgs<CondicaoDePagamentoModel> e)
        {
            CondicaoDePagamentoSelecionada = e.Data;
        }

        public async Task<List<CondicaoDePagamentoModel>> GetCondicoesDePagamento()
        {
            var retorno = await _condicoesDePagamentoService.GetAsync(URIsCondicoesDePagamento.GetAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<CondicaoDePagamentoModel>();
            }
            return (List<CondicaoDePagamentoModel>)retorno.Objeto;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            CondicoesDePagamento = await GetCondicoesDePagamento();
            Localizando = false;
            DlgCondicoesDePagamentoVisible = true;
        }

        public async void IdAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            if (CondicaoDePagamento.Id == 0) return;
            CondicaoDePagamento = await CondPgtoByIdCadastroCondPgto.Get(this, _condicoesDePagamentoService, CondicaoDePagamento.Id, StatusAtivo.Todos);
        }

        public void LimparTela()
        {
            LimpaTelaCadastroCondicaoDePagamento.Limpar(this);
        }

        public async void AoClicarSelecionarBtnGridCondicoesDePagamento(CommandClickEventArgs<CondicaoDePagamentoModel> e)
        {
            CondicaoDePagamentoSelecionada = e.RowData;
            await ExibirCondicaoDePagamentoSelecionada();
        }

        private async Task ExibirCondicaoDePagamentoSelecionada()
        {
            ErrosValidacaoFormulario.Clear();
            dlgCondicoesDePagamentoVisible = false;
            CondicaoDePagamento = await CondPgtoByIdCadastroCondPgto.Get(this, _condicoesDePagamentoService, CondicaoDePagamentoSelecionada.Id, StatusAtivo.Todos);
        }

        public async Task AoPressionarTeclaCondicoesDePagamentoGrid(KeyboardEventArgs args)
        {
            if (args.Key == "Enter" && CondicaoDePagamentoSelecionada?.Id > 0)
            {
                await ExibirCondicaoDePagamentoSelecionada();
                await DescricaoParcelasTextBox.FocusAsync();
            }
        }

    }
}
