﻿using FluentValidation;
using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using Syncfusion.Blazor.Spinner;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroEmbalagem
{
    public class CadastroEmbalagemVM : CadastroBaseVM, ICadastroEmbalagemVM
    {
        private readonly IEmbalagensService _embalagensService;
        private readonly ILoginStore _loginStore;

        private bool _dlgEmbalagensVisible;
        private EmbalagemModel _embalagem;

        public EmbalagemModel Embalagem
        {
            get { return _embalagem; }
            set
            {
                _embalagem = value;
                AoMudarPropriedade(nameof(Embalagem));
            }
        }

        public bool PodeAcessar
        {
            get
            {
                var claimEmbalagem = _loginStore.Claims.FirstOrDefault(c => c.Type == "Embalagem");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimEmbalagem != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }
        public bool DlgEmbalagensVisible
        {
            get { return _dlgEmbalagensVisible; }
            set
            {
                _dlgEmbalagensVisible = value;
                AoMudarPropriedade(nameof(DlgEmbalagensVisible));
            }
        }

        public SfDialog Dlg { get; set; }
        public SfTextBox DescricaoTextBox { get; set; }
        public EmbalagemModel EmbalagemSelecionada { get; set; } = new EmbalagemModel();
        public List<EmbalagemModel> Embalagens { get; set; } = new List<EmbalagemModel>();
        public CadastroEmbalagemVM(IEmbalagensService embalagensService, ILoginStore loginStore)
        {
            _loginStore = loginStore;
            _embalagensService = embalagensService;
            Embalagem = new EmbalagemModel { Ativo = true };
            MensagemResultadoEnvio = " ";
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroEmbalagem.Enviar(this, _embalagensService);
        }

        public void AoCriarDescricaoTextBox(object e)
        {
            DescricaoTextBox.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }

        public bool DadosValidos()
        {
            var validator = new EmbalagemModelValidator();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(Embalagem);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }

        public void AoFecharDlgEmbalagens(object e)
        {
            DlgEmbalagensVisible = false;
        }

        public async void AoClicarOkBtnDlgEmbalagens(MouseEventArgs e)
        {
            if (EmbalagemSelecionada == null)
            {
                return;
            }
            ErrosValidacaoFormulario.Clear();
            DlgEmbalagensVisible = false;
            Embalagem = await EmbalagemByIdCadastroEmbalagem.Get(this, _embalagensService, EmbalagemSelecionada.Id, StatusAtivo.Todos);
        }

        public void AoSelecionarEmbalagem(RowSelectEventArgs<EmbalagemModel> e)
        {
            EmbalagemSelecionada = e.Data;
        }

        public async Task<List<EmbalagemModel>> GetEmbalagens()
        {
            var retorno = await _embalagensService.GetAsync(URIsEmbalagens.GetAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<EmbalagemModel>();
            }
            return (List<EmbalagemModel>)retorno.Objeto;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            Embalagens = await GetEmbalagens();
            Localizando = false;
            DlgEmbalagensVisible = true;
        }

        public async void IdAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            if (Embalagem.Id == 0) return;
            Embalagem = await EmbalagemByIdCadastroEmbalagem.Get(this, _embalagensService, Embalagem.Id, StatusAtivo.Todos);
        }

        public void LimparTela()
        {
            LimpaTelaCadastroEmbalagem.Limpar(this);
        }
    }
}
