﻿using FluentValidation;
using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using Syncfusion.Blazor.Spinner;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroGeneroProduto
{
    public class CadastroGeneroProdutoVM : CadastroBaseVM, ICadastroGeneroProdutoVM
    {
        private readonly IGenerosProdutoService _generosService;
        private readonly ILoginStore _loginStore;

        private bool _dlgGenerosVisible;
        private GeneroProdutoModel _unidade;

        public GeneroProdutoModel Genero
        {
            get { return _unidade; }
            set
            {
                _unidade = value;
                AoMudarPropriedade(nameof(Genero));
            }
        }

        public bool PodeAcessar
        {
            get
            {
                var claimGenero = _loginStore.Claims.FirstOrDefault(c => c.Type == "GeneroProduto");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimGenero != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }
        public bool DlgGenerosVisible
        {
            get { return _dlgGenerosVisible; }
            set
            {
                _dlgGenerosVisible = value;
                AoMudarPropriedade(nameof(DlgGenerosVisible));
            }
        }

        public SfDialog Dlg { get; set; }
        public SfNumericTextBox<int> CodigoTextBox { get; set; }
        public GeneroProdutoModel GeneroSelecionada { get; set; } = new GeneroProdutoModel();
        public List<GeneroProdutoModel> Generos { get; set; } = new List<GeneroProdutoModel>();
        public CadastroGeneroProdutoVM(IGenerosProdutoService generosService, ILoginStore loginStore)
        {
            _loginStore = loginStore;
            _generosService = generosService;
            Genero = new GeneroProdutoModel();
            MensagemResultadoEnvio = " ";
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroGeneroProduto.Enviar(this, _generosService);
        }

        public void AoCriarCodigoTextBox(object e)
        {
            CodigoTextBox.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }

        public bool DadosValidos()
        {
            var validator = new GeneroProdutoModelValidator();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(Genero);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }

        public void AoFecharDlgGeneros(object e)
        {
            DlgGenerosVisible = false;
        }

        public async void AoClicarOkBtnDlgGeneros(MouseEventArgs e)
        {
            if (GeneroSelecionada == null)
            {
                return;
            }
            ErrosValidacaoFormulario.Clear();
            DlgGenerosVisible = false;
            Genero = await GeneroByIdCadastroGeneroProduto.Get(this, _generosService, GeneroSelecionada.Id, StatusAtivo.Todos);
        }

        public void AoSelecionarGenero(RowSelectEventArgs<GeneroProdutoModel> e)
        {
            GeneroSelecionada = e.Data;
        }

        public async Task<List<GeneroProdutoModel>> GetGeneros()
        {
            var retorno = await _generosService.GetAsync(URIsGenerosProduto.GetAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<GeneroProdutoModel>();
            }
            return (List<GeneroProdutoModel>)retorno.Objeto;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            Generos = await GetGeneros();
            Localizando = false;
            DlgGenerosVisible = true;
        }

        public async void IdAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            if (Genero.Id == 0) return;
            Genero = await GeneroByIdCadastroGeneroProduto.Get(this, _generosService, Genero.Id, StatusAtivo.Todos);
        }

        public void LimparTela()
        {
            LimpaTelaCadastroGeneroProduto.Limpar(this);
        }
    }
}
