﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroGrupoProduto
{
    public class CadastroGrupoProdutoVM : CadastroBaseVM, ICadastroGrupoProdutoVM
    {
        private readonly IGruposProdutoService _gruposService;
        private readonly ILoginStore _loginStore;

        private bool _dlgGruposVisible;
        private GrupoProdutoModel _grupo;

        public GrupoProdutoModel Grupo
        {
            get { return _grupo; }
            set
            {
                _grupo = value;
                AoMudarPropriedade(nameof(Grupo));
            }
        }

        public bool PodeAcessar
        {
            get
            {
                var claimGrupo = _loginStore.Claims.FirstOrDefault(c => c.Type == "GrupoProduto");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimGrupo != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }
        public bool DlgGruposVisible
        {
            get { return _dlgGruposVisible; }
            set
            {
                _dlgGruposVisible = value;
                AoMudarPropriedade(nameof(DlgGruposVisible));
            }
        }

        public SfDialog Dlg { get; set; }
        public SfTextBox NomeTextBox { get; set; }
        public GrupoProdutoModel GrupoSelecionado { get; set; } = new GrupoProdutoModel();
        public List<GrupoProdutoModel> Grupos { get; set; } = new List<GrupoProdutoModel>();
        public CadastroGrupoProdutoVM(IGruposProdutoService GruposService, ILoginStore loginStore)
        {
            _loginStore = loginStore;
            _gruposService = GruposService;
            Grupo = new GrupoProdutoModel { Ativo = true };
            MensagemResultadoEnvio = " ";
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroGrupoProduto.Enviar(this, _gruposService);
        }

        public void AoCriarNomeTextBox(object e)
        {
            NomeTextBox.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }

        public bool DadosValidos()
        {
            var validator = new GrupoProdutoModelValidator();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(Grupo);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }

        public void AoFecharDlgGrupos(object e)
        {
            DlgGruposVisible = false;
        }

        public async void AoClicarOkBtnDlgGrupos(MouseEventArgs e)
        {
            if (GrupoSelecionado == null)
            {
                return;
            }
            ErrosValidacaoFormulario.Clear();
            DlgGruposVisible = false;
            Grupo = await GrupoByIdCadastroGrupoProduto.Get(this, _gruposService, GrupoSelecionado.Id, StatusAtivo.Todos);
        }

        public void AoSelecionarGrupo(RowSelectEventArgs<GrupoProdutoModel> e)
        {
            GrupoSelecionado = e.Data;
        }

        public async Task<List<GrupoProdutoModel>> GetGrupos()
        {
            var retorno = await _gruposService.GetAsync(URIsGruposProduto.GetAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<GrupoProdutoModel>();
            }
            return (List<GrupoProdutoModel>)retorno.Objeto;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            Grupos = await GetGrupos();
            Localizando = false;
            DlgGruposVisible = true;
        }

        public async void IdAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            if (Grupo.Id == 0) return;
            Grupo = await GrupoByIdCadastroGrupoProduto.Get(this, _gruposService, Grupo.Id, StatusAtivo.Todos);
        }

        public void LimparTela()
        {
            LimpaTelaCadastroGrupoProduto.Limpar(this);
        }
    }
}
