﻿using FluentValidation;
using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using Syncfusion.Blazor.Spinner;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroHistorico
{
    public class CadastroHistoricoVM : CadastroBaseVM, ICadastroHistoricoVM
    {
        private readonly IHistoricosService _historicosService;
        private readonly IPlanosDeContaService _planosDeContaService;
        private readonly ILoginStore _loginStore;
        private readonly IEntradaSaidaService _entradaSaidaService;
        private readonly ITiposCTEHistoricoService _tiposCTEHistoricoService;
        private readonly ITiposNFHistoricoService _tiposNFHistoricoService;
        private readonly IFinNFeHistoricoService _finalidadesNFeHistoricoService;
        private readonly ISitTribService _sitTribService;
        private readonly ISitTribPISCOFINSService _sitTribPISCOFINSService;
        private readonly ISitTribIPIService _sitTribIPIService;
        private readonly ITabelaAService _tabelaAService;
        private bool _dlgHistoricosVisible;
        private bool _dlgHistoricosEstornoVisible;
        private HistoricoModel _historico;


        public HistoricoModel Historico
        {
            get { return _historico; }
            set
            {
                _historico = value;
                AoMudarPropriedade(nameof(Historico));
            }
        }

        public bool PodeAcessar
        {
            get
            {
                var claimPessoa = _loginStore.Claims.FirstOrDefault(c => c.Type == "Historico");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimPessoa != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }

        public bool DlgHistoricosVisible
        {
            get { return _dlgHistoricosVisible; }
            set
            {
                _dlgHistoricosVisible = value;
                AoMudarPropriedade(nameof(DlgHistoricosVisible));
            }
        }

        public bool DlgHistoricosEstornoVisible
        {
            get { return _dlgHistoricosEstornoVisible; }
            set
            {
                _dlgHistoricosEstornoVisible = value;
                AoMudarPropriedade(nameof(DlgHistoricosEstornoVisible));
            }
        }

        public SfDialog Dlg { get; set; }
        public SfTextBox DescricaoTextBox { get; set; }
        public HistoricoModel HistoricoSelecionado { get; set; } = new HistoricoModel();
        public HistoricoModel HistoricoEstornoSelecionado { get; set; }
        public List<HistoricoModel> Historicos { get; set; } = new List<HistoricoModel>();
        public SfSpinner SpinnerVendedor { get; set; }
        public List<ListaModel> ListaEntradaSaida { get; }
        public List<ListaModel> ListaTipoCTE { get; }
        public List<ListaModel> ListaTipoNF { get; }
        public List<ListaModel> ListaFinNFe { get; }

        public List<ListaModel> ListaTabelaA { get; }

        public List<ListaModel> ListaSitTrib { get; }

        public List<ListaModel> ListaSitTribPISCOFINS { get; }

        public List<ListaModel> ListaSitTribIPI { get; }

        public bool HistoricosEstornoBtnEnabled { get; set; } = true;
        public bool PesquisandoHistoricosEstorno { get; set; }
        public SfNumericTextBox<long?> IdHistoricoEstornoInput { get; set; }
        public PlanoDeContaModel PlanoDeContaSelecionado { get; set; } = new ();
        public List<PlanoDeContaModel> PlanosDeConta { get; set; } = new ();
        public bool PlanosDeContaBtnEnabled { get; set; }
        public bool PesquisandoPlanosDeConta { get; set; }
        public SfNumericTextBox<long?> IdPlanoDeContaInput { get; set; }
        public bool DlgPlanosDeContaVisible { get; set; } = true;

        public CadastroHistoricoVM(IHistoricosService historicosService,
            ILoginStore loginStore,
            IEntradaSaidaService entradaSaidaService,
            ITiposCTEHistoricoService tiposCTEHistoricoService,
            ITiposNFHistoricoService tiposNFHistoricoService,
            IFinNFeHistoricoService finalidadesNFeHistoricoService,
            ISitTribService sitTribService,
            ISitTribPISCOFINSService sitTribPISCOFINSService,
            ISitTribIPIService sitTribIPIService,
            ITabelaAService tabelaAService,
            IPlanosDeContaService planosDeContaService)
        {
            _loginStore = loginStore;
            _historicosService = historicosService;
            _entradaSaidaService = entradaSaidaService;
            _tiposCTEHistoricoService = tiposCTEHistoricoService;
            _tiposNFHistoricoService = tiposNFHistoricoService;
            _finalidadesNFeHistoricoService = finalidadesNFeHistoricoService;
            _sitTribService = sitTribService;
            _sitTribPISCOFINSService = sitTribPISCOFINSService;
            _sitTribIPIService = sitTribIPIService;
            _tabelaAService = tabelaAService;
            _planosDeContaService = planosDeContaService;

            Historico = new HistoricoModel { HistoricoEstorno = new HistoricoModel() };
            ListaEntradaSaida = _entradaSaidaService.Itens();
            ListaTipoCTE = _tiposCTEHistoricoService.Itens();
            ListaTipoNF = _tiposNFHistoricoService.Itens();
            ListaFinNFe = _finalidadesNFeHistoricoService.Itens();
            ListaSitTrib = _sitTribService.Itens();
            ListaTabelaA = _tabelaAService.Itens();
            ListaSitTribPISCOFINS = _sitTribPISCOFINSService.Itens();
            ListaSitTribIPI = _sitTribIPIService.Itens();
            MensagemResultadoEnvio = " ";
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroHistorico.Enviar(this, _historicosService);
            AoMudarPropriedade(nameof(Historico));
        }

        public async void AoCriarDescricaoTextBox(object e)
        {
            await DescricaoTextBox.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public async void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
            await DescricaoTextBox.FocusAsync();
        }

        public bool DadosValidos()
        {
            var validator = new HistoricoModelValidator();
            ErrosValidacaoFormulario.Clear();
            var erros = new List<string>();

            var results = validator.Validate(Historico);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }

        public async void AoClicarOkBtnDlgHistoricos(MouseEventArgs e)
        {
            if (HistoricoSelecionado == null)
            {
                return;
            }
            ErrosValidacaoFormulario.Clear();
            DlgHistoricosVisible = false;
            Historico = await HistoricoByIdCadastroHistorico.Get(this, _historicosService, HistoricoSelecionado.Id, StatusAtivo.Todos);
        }

        public async Task<List<HistoricoModel>> GetHistoricos()
        {
            var retorno = await _historicosService.GetAsync(URIsHistoricos.GetAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<HistoricoModel>();
            }
            return (List<HistoricoModel>)retorno.Objeto;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            Historicos = await GetHistoricos();
            Localizando = false;
            DlgHistoricosVisible = true;
        }

        public async void IdAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            if (Historico.Id == 0) return;
            Historico = await HistoricoByIdCadastroHistorico.Get(this, _historicosService, Historico.Id, StatusAtivo.Todos);
        }

        public void AoSelecionarHistorico(RowSelectEventArgs<HistoricoModel> e)
        {
            HistoricoSelecionado = e.Data;
        }

        public void AoFecharDlgHistoricos(object e)
        {
            DlgHistoricosVisible = false;
        }

        public void AoSelecionarHistoricoEstorno(RowSelectEventArgs<HistoricoModel> e)
        {
            HistoricoEstornoSelecionado = e.Data;
        }

        public async void AoClicarHistoricosEstornoBtnAsync(MouseEventArgs e)
        {
            await LocalizaHistoricoEstornoCadastroHistorico.LocalizarAsync(this, _historicosService);
        }

        public async void AoClicarOkBtnDlgHistoricosEstorno(MouseEventArgs e)
        {
            DlgHistoricosEstornoVisible = false;
            if (HistoricoEstornoSelecionado == null)
            {
                return;
            }
            Historico.HistoricoEstorno = await HistoricoByIdCadastroHistorico.Get(this, _historicosService, HistoricoEstornoSelecionado.Id, StatusAtivo.Ativos);
            Historico.IdHistoricoEstorno = Historico.HistoricoEstorno.Id;
            AoMudarPropriedade(nameof(Historico.HistoricoEstorno));
        }

        public void AoFecharDlgHistoricosEstorno(object e)
        {
            DlgHistoricosEstornoVisible = false;
        }

        public async void IdHistoricoEstornoAoSairAsync()
        {
            if (!Historico.IdHistoricoEstorno.HasValue)
            {
                LimpaHistoricoEstorno();
                return;
            }

            Historico.HistoricoEstorno = await HistoricoByIdCadastroHistorico.Get(this, _historicosService, Historico.IdHistoricoEstorno.Value, StatusAtivo.Ativos);

            if (Historico.HistoricoEstorno.Id == 0) LimpaHistoricoEstorno();

            AoMudarPropriedade(nameof(Historico.HistoricoEstorno));
        }

        private void LimpaHistoricoEstorno()
        {
            Historico.HistoricoEstorno = new HistoricoModel();
            Historico.IdHistoricoEstorno = (long?)null;
        }

        public void LimparTela()
        {
            LimpaTelaCadastroHistorico.Limpar(this);
        }

        public async Task AoClicarPlanosDeContaBtnAsync(MouseEventArgs e)
        {
            PlanosDeContaBtnEnabled = false;
            PesquisandoPlanosDeConta = true;
            await SpinnerPrincipal.ShowAsync();
            var retorno = await _planosDeContaService?.GetAsync(URIsPlanosDeConta.GetAsync, StatusAtivo.Ativos);
            PesquisandoHistoricosEstorno = false;
            HistoricosEstornoBtnEnabled = true;

            if (!retorno.Sucesso)
            {
                Mensagem = retorno.MensagemErro;
                DlgVisible = true;
            }
            else
            {
                PlanosDeConta = (List<PlanoDeContaModel>)retorno.Objeto;
                DlgPlanosDeContaVisible = true;
            }
        }

        public void AoClicarOkBtnDlgPlanosDeConta(MouseEventArgs e)
        {
            throw new System.NotImplementedException();
        }

        public void AoFecharDlgPlanosDeContaEstorno(object e)
        {
            throw new System.NotImplementedException();
        }

        public void IdPlanoDeContaAoSairAsync()
        {
            throw new System.NotImplementedException();
        }

        public async Task AoClicarSelecionarBtnGridHistoricos(CommandClickEventArgs<HistoricoModel> e)
        {
            DlgHistoricosVisible = false;
            Historico = await HistoricoByIdCadastroHistorico.Get(this, _historicosService, e.RowData.Id, StatusAtivo.Todos);
            AoMudarPropriedade(nameof(Historico));
        }

        public void AoClicarSelecionarBtnGridHistoricosEstorno(CommandClickEventArgs<HistoricoModel> e)
        {
            DlgHistoricosEstornoVisible = false;
            Historico.IdHistoricoEstorno = e.RowData.Id;
            Historico.HistoricoEstorno = e.RowData;
            AoMudarPropriedade(nameof(Historico));
        }

        public async Task AoPressionarTecla(KeyboardEventArgs args)
        {
            if (args.Key == "Enter" && HistoricoSelecionado?.Id > 0)
            {
                DlgHistoricosVisible = false;
                Historico = await HistoricoByIdCadastroHistorico.Get(this, _historicosService, HistoricoSelecionado.Id, StatusAtivo.Todos);
                AoMudarPropriedade(nameof(Historico));
            }
            
        }
    }
}
