﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.DropDowns;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using Syncfusion.Blazor.Spinner;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using FocusEventArgs = Microsoft.AspNetCore.Components.Web.FocusEventArgs;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroPessoa
{
    public partial class CadastroPessoaVM : CadastroBaseVM, ICadastroPessoaVM
    {
        private readonly ICidadesService _cidadesService;
        private readonly IPaisesService _paisesService;
        private readonly IPessoasService _pessoasService;
        private readonly ITiposPessoaService _tiposPessoaService;
        private readonly ILoginStore _loginStore;
        private readonly IEnderecoPeloCEPService _enderecoPeloCEPService;
        private readonly IProdutosService _produtosService;
        private readonly IHistoricosService _historicosService;
        private readonly ITiposDeOperacaoService _tiposDeOperacaoService;
        private readonly ICondicoesDePagamentoService _condicoesDePagamentoService;
        private readonly IPlanosDeContaService _planosDeContaService;
        private readonly ICentrosDeCustoService _centrosDeCustoService;
        private readonly NavigationManager _navigation;

        private int? _tipoRegimeTributacaoSelecionado;
        private int? _indicadorIESelecionado;

        [ObservableProperty]
        private PessoaModel pessoa;
        partial void OnPessoaChanged(PessoaModel value)
        {
            TipoPessoaSelecionado = value.Id == 0 ? (int?)TipoPessoa.Juridica : value.CNPJ.Length > 11 ? (int?)TipoPessoa.Juridica : (int?)TipoPessoa.Fisica;
            AoMudarPropriedade(nameof(Pessoa));
        }

        [ObservableProperty]
        private bool vendedoresBtnEnabled;
        partial void OnVendedoresBtnEnabledChanged(bool value) => AoMudarPropriedade(nameof(VendedoresBtnEnabled));

        [ObservableProperty]
        private EnderecoPessoaModel endereco;
        partial void OnEnderecoChanged(EnderecoPessoaModel value) => AoMudarPropriedade(nameof(Endereco));

        [ObservableProperty]
        public bool dlgCidadesVisible;
        partial void OnDlgCidadesVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgCidadesVisible));

        [ObservableProperty]
        public bool dlgPaisesVisible;
        partial void OnDlgPaisesVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgPaisesVisible));

        [ObservableProperty]
        public bool dlgPessoasVisible;
        partial void OnDlgPessoasVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgPessoasVisible));

        public int? TipoRegimeTributacaoSelecionado
        {
            get
            {
                _tipoRegimeTributacaoSelecionado = (int?)Pessoa.TipoRegimeTributacao;
                return _tipoRegimeTributacaoSelecionado;
            }
            set
            {
                _tipoRegimeTributacaoSelecionado = value;
                Pessoa.TipoRegimeTributacao = (TipoRegimeTributacao)_tipoRegimeTributacaoSelecionado;
            }
        }

        public int? IndicadorIESelecionado
        {
            get
            {
                _indicadorIESelecionado = (int?)Pessoa.IndicadorIE;
                return _indicadorIESelecionado;
            }
            set
            {
                _indicadorIESelecionado = value;
                Pessoa.IndicadorIE = (IndicadorIE)_indicadorIESelecionado;
            }
        }
        [ObservableProperty]
        private PessoaModel transportadora;
        partial void OnTransportadoraChanged(PessoaModel value) => AoMudarPropriedade(nameof(Transportadora));

        [ObservableProperty]
        private bool dlgVendedoresVisible;
        partial void OnDlgVendedoresVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgVendedoresVisible));
        [ObservableProperty]
        private bool dlgTransportadorasVisible;
        partial void OnDlgTransportadorasVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgTransportadorasVisible));

        [ObservableProperty]
        private ProdutoPessoaModel produtoPessoa;
        partial void OnProdutoPessoaChanged(ProdutoPessoaModel value) => AoMudarPropriedade(nameof(ProdutoPessoa));

        [ObservableProperty]
        public bool dlgProdutosVisible;
        partial void OnDlgProdutosVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgProdutosVisible));

        [ObservableProperty]
        public bool dlgHistoricosVisible;
        partial void OnDlgHistoricosVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgHistoricosVisible));

        [ObservableProperty]
        public bool dlgTiposDeOperacaoVisible;
        partial void OnDlgTiposDeOperacaoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgTiposDeOperacaoVisible));

        [ObservableProperty]
        public bool dlgCondicoesDePagamentoVisible;
        partial void OnDlgCondicoesDePagamentoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgCondicoesDePagamentoVisible));

        [ObservableProperty]
        public bool dlgCentrosDeCustoVisible;
        partial void OnDlgCentrosDeCustoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgCentrosDeCustoVisible));

        [ObservableProperty]
        public bool dlgPlanosDeContaVisible;
        partial void OnDlgPlanosDeContaVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgPlanosDeContaVisible));

        [ObservableProperty]
        public List<ListaModel> tiposPessoa;
        partial void OnTiposPessoaChanged(List<ListaModel> value) => AoMudarPropriedade(nameof(TiposPessoa));

        public bool PodeAcessar
        {
            get
            {
                var claimPessoa = _loginStore.Claims.FirstOrDefault(c => c.Type == "Pessoa");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimPessoa != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }
        public VendedorPessoaModel VendedorPessoa { get; set; } = new VendedorPessoaModel { Vendedor = new PessoaModel() };
        public int? TipoPessoaSelecionado { get; set; } = (int?)TipoPessoa.Juridica;
        public EMailPessoaModel EMail { get; set; } = new EMailPessoaModel();
        public List<CidadeModel> Cidades { get; set; } = new List<CidadeModel>();
        public List<PaisModel> Paises { get; set; } = new List<PaisModel>();
        public CidadeModel CidadeSelecionada { get; set; } = new CidadeModel();
        public PaisModel PaisSelecionado { get; set; } = new PaisModel();
        public SfDialog Dlg { get; set; }
        public bool CidadesBtnEnabled { get; set; } = true;
        public bool PaisesBtnEnabled { get; set; } = true;
        public SfMaskedTextBox CNPJTextBox { get; set; }
        public SfDropDownList<int?, ListaModel> TipoPessoaDropDownList { get; set; }
        public SfMaskedTextBox TelefoneTextBox { get; set; }
        public SfMaskedTextBox CEPTextBox { get; set; }
        public SfTextBox NomeTextBox { get; set; }
        public SfTextBox EMailTextBox { get; set; }
        public string MascaraCNPJCPF => TipoPessoaSelecionado == (int?)TipoPessoa.Juridica ? "00.000.000/0000-00" : "000.000.000-00";
        public string CNPJCPFLbl => TipoPessoaSelecionado == (int?)TipoPessoa.Juridica ? "CNPJ" : "CPF";
        public List<ListaModel> TiposRegimeTributacao { get; }
        public List<ListaModel> IndicadoresIE { get; }
        public SfSpinner SpinnerEndereco { get; set; }
        public SfSpinner SpinnerVendedor { get; set; }
        public List<PessoaModel> Vendedores { get; set; } = new List<PessoaModel>();
        public PessoaModel VendedorSelecionado { get; set; }
        public List<ProdutoModel> Produtos { get; set; } = new List<ProdutoModel>();
        public SfSpinner SpinnerProduto { get; set; }
        public SfSpinner SpinnerFinanceiro { get; set; }
        public bool ProdutosBtnEnabled { get; set; } = true;
        public ProdutoModel ProdutoSelecionado { get; set; }
        public bool PesquisandoCidades { get; set; }
        public bool PesquisandoProdutos { get; set; }
        public bool PesquisandoVendedores { get; set; }
        public bool PesquisandoPaises { get; set; }
        public bool HistoricosBtnEnabled { get; set; } = true;
        public HistoricoModel HistoricoSelecionado { get; set; }
        public List<HistoricoModel> Historicos { get; set; } = new List<HistoricoModel>();
        public bool PesquisandoHistoricos { get; set; }
        public bool PlanosDeContaBtnEnabled { get; set; } = true;
        public PlanoDeContaModel PlanoDeContaSelecionado { get; set; }
        public List<PlanoDeContaModel> PlanosDeContas { get; set; } = new List<PlanoDeContaModel>();
        public SfSpinner SpinnerPlanoDeConta { get; set; }
        public bool PesquisandoPlanosDeConta { get; set; }
        public bool CentrosDeCustoBtnEnabled { get; set; } = true;
        public CentroDeCustoModel CentroDeCustoSelecionado { get; set; }
        public List<CentroDeCustoModel> CentrosDeCusto { get; set; } = new List<CentroDeCustoModel>();
        public bool PesquisandoCentrosDeCusto { get; set; }
        public bool CondicoesDePagamentoBtnEnabled { get; set; } = true;
        public CondicaoDePagamentoModel CondicaoDePagamentoSelecionada { get; set; }
        public List<CondicaoDePagamentoModel> CondicoesDePagamento { get; set; } = new List<CondicaoDePagamentoModel>();
        public bool PesquisandoCondicoesDePagamento { get; set; }
        public bool TiposDeOperacaoBtnEnabled { get; set; } = true;
        public TipoDeOperacaoModel TipoDeOperacaoSelecionado { get; set; }
        public List<TipoDeOperacaoModel> TiposDeOperacao { get; set; } = new List<TipoDeOperacaoModel>();
        public bool PesquisandoTiposDeOperacao { get; set; }
        public bool TransportadorasBtnEnabled { get; set; } = true;
        public TransportadoraPessoaModel TransportadoraPessoaSelecionada { get; set; }
        public PessoaModel TransportadoraSelecionada { get; set; }
        public List<PessoaModel> Transportadoras { get; set; } = new List<PessoaModel>();
        public SfSpinner SpinnerTransportadora { get; set; }
        public bool PesquisandoTransportadoras { get; set; }
        public SfNumericTextBox<long?> IdPlanoContaInput { get; set; }
        public SfNumericTextBox<long?> IdCondPgtoInput { get; set; }
        public SfNumericTextBox<long?> IdTipoDeOperacaoInput { get; set; }
        public SfNumericTextBox<long?> IdCentroCustoInput { get; set; }
        public SfNumericTextBox<long?> IdHistoricoInput { get; set; }
        public SfNumericTextBox<decimal> ComissaoVendedorInput { get; set; }
        public SfTextBox CodigoProdutoInput { get; set; }
        public TelefonePessoaModel Telefone { get; set; }
        public string NomePessoaLbl { get; set; } = "Razão Social";

        public CadastroPessoaVM(IPessoasService pessoasService,
            ICidadesService CidadesService,
            IPaisesService paisesService,
            ITiposPessoaService tiposPessoaService,
            ILoginStore loginStore,
            ITiposRegimeTributacaoService tiposRegimeTributacaoService,
            IIndicadoresIEService indicadoresIEService,
            IEnderecoPeloCEPService enderecoPeloCEPService,
            IProdutosService produtosService,
            IHistoricosService historicosService,
            IPlanosDeContaService planosDeContaService,
            ICondicoesDePagamentoService condicoesDePagamentoService,
            ICentrosDeCustoService centrosDeCustoService,
            ITiposDeOperacaoService tiposDeOperacaoService,
            NavigationManager navigation)
        {
            _loginStore = loginStore;
            _pessoasService = pessoasService;
            _cidadesService = CidadesService;
            _paisesService = paisesService;
            _tiposPessoaService = tiposPessoaService;
            _produtosService = produtosService;
            _historicosService = historicosService;
            _planosDeContaService = planosDeContaService;
            _condicoesDePagamentoService = condicoesDePagamentoService;
            _centrosDeCustoService = centrosDeCustoService;
            _tiposDeOperacaoService = tiposDeOperacaoService;
            TiposRegimeTributacao = tiposRegimeTributacaoService?.Itens();
            IndicadoresIE = indicadoresIEService?.Itens();
            _enderecoPeloCEPService = enderecoPeloCEPService;
            endereco = new EnderecoPessoaModel { Cidade = new CidadeModel(), Pais = new PaisModel() };
            TiposPessoa = _tiposPessoaService.Itens();
            Pessoa = _pessoasService?.GetPessoaVazia();
            Transportadora = new PessoaModel();
            ProdutoPessoa = new ProdutoPessoaModel { Produto = new ProdutoModel() };
            Telefone = new TelefonePessoaModel();
            VendedorPessoa = new VendedorPessoaModel { Vendedor = new PessoaModel() };
            IndicadorIESelecionado = (int?)Pessoa.IndicadorIE;
            MensagemResultadoEnvio = " ";
            VendedoresBtnEnabled = true;
            _navigation = navigation;
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }
            Pessoa.DtCadastro = Pessoa.DtCadastro.HasValue ? Pessoa.DtCadastro.Value.ToUniversalTime() : Pessoa.DtCadastro;
            Pessoa.DtNascimento = Pessoa.DtNascimento.HasValue ? Pessoa.DtNascimento.Value.ToUniversalTime() : Pessoa.DtNascimento;
            await EnvioDadosCadastroPessoa.Enviar(this, _pessoasService);
            await CNPJTextBox.FocusAsync();
        }

        public void AoSelecionarCidade(RowSelectEventArgs<CidadeModel> e)
        {
            CidadeSelecionada = e?.Data;
        }

        public void AoSelecionarPais(RowSelectEventArgs<PaisModel> e)
        {
            PaisSelecionado = e?.Data;
        }

        public async void AoClicarCidadesBtnAsync(MouseEventArgs e)
        {
            await LocalizaCidadeCadastroPessoa.LocalizarAsync(this, _cidadesService);
        }

        public void AoFecharDialogoCidades()
        {
            CidadesBtnEnabled = true;
            DlgCidadesVisible = false;
        }
        public void AoClicarOkBtnDlgCidades(MouseEventArgs e)
        {
            Endereco.Cidade = CidadeSelecionada;
            DlgCidadesVisible = false;
        }

        public void AoFecharDlgPaises(object e)
        {
            PaisesBtnEnabled = true;
            DlgPaisesVisible = false;
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public async Task AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            await LimparTelaAsync();
            await Task.Delay(300);
            if (CNPJTextBox != null)
            {
                await CNPJTextBox.FocusAsync();
            }
        }

        public async void AoClicarPaisesBtnAsync(MouseEventArgs e)
        {
            await LocalizaPaisCadastroPessoa.LocalizarAsync(this, _paisesService);
        }

        public async void AoClicarRetirarEnderecoBtnAsync(MouseEventArgs e)
        {
            await RetiraEnderecoCadastroPessoa.Retirar(this);
        }

        public async Task AoClicarInserirEnderecoBtnAsync(MouseEventArgs e)
        {
            await InsereEnderecoCadastroPessoa.Add(this);
        }

        public void AoFecharDlgCidades(object e)
        {
            CidadesBtnEnabled = true;
            DlgCidadesVisible = false;
        }

        public void AoClicarOkBtnDlgPaises(MouseEventArgs e)
        {
            Endereco.Pais = PaisSelecionado;
            DlgPaisesVisible = false;
        }

        public bool DadosValidos()
        {
            var validator = new PessoaModelValidator();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(Pessoa);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }

            return true;
        }

        public async Task AoFecharDlgPessoas(object e)
        {
            DlgPessoasVisible = false;
            await Task.Delay(100);
            if (Pessoa.Id == 0)
            {
                //await TipoPessoaDropDownList.FocusAsync();
                await CNPJTextBox.FocusAsync();
            }
        }

        public async Task ExibirPessoa(long id)
        {
            ErrosValidacaoFormulario.Clear();
            DlgPessoasVisible = false;
            Pessoa = await PessoaByIdCadastroPessoa.Get(this, _pessoasService, id, StatusAtivo.Todos);
            var endPessoa = Pessoa.EnderecoPrincipal;
            endPessoa = endPessoa?.Id > 0 ? endPessoa : Pessoa.Enderecos.FirstOrDefault();
            Endereco = endPessoa;
            NomePessoaLbl = (TipoPessoa)TipoPessoaSelecionado == TipoPessoa.Juridica ? "Razão Social" : "Nome";
        }

        public async Task<List<PessoaModel>> GetPessoas()
        {
            var retorno = await _pessoasService.GetAsync(URIsPessoas.GetAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<PessoaModel>();
            }
            var pessoas = (List<PessoaModel>)retorno.Objeto;
            return pessoas;
        }

        public void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            _navigation.NavigateTo("/Usuario/Cadastros/PesquisaPessoa/false");
        }

        public async void CEPAoSairAsync(MaskBlurEventArgs args)
        {
            if (string.IsNullOrEmpty(Endereco.CEP)) return;
            await ExibirEnderecoPeloCEP(args.Value);
            AoMudarPropriedade(nameof(Endereco));
        }

        private async Task ExibirEnderecoPeloCEP(string cep)
        {
            CidadesBtnEnabled = false;
            await EnderecoPeloCEPCadastroPessoa.Endereco(this, cep, _enderecoPeloCEPService);
            CidadesBtnEnabled = true;
        }

        public async void IdAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            if (Pessoa.Id == 0) return;
            await ExibirPessoa(Pessoa.Id);
            await Task.Delay(100);
            await NomeTextBox.FocusAsync();
        }

        public async void CNPJAoSairAsync(FocusEventArgs args)
        {
            ErrosValidacaoFormulario.Clear();
            if (string.IsNullOrEmpty(Pessoa.CNPJ)) return;
            var consulta = await PessoaByCNPJCadastroPessoa.Get(this, _pessoasService, Pessoa.CNPJ, StatusAtivo.Todos);

            if (string.IsNullOrEmpty(consulta?.Nome)) return;
                
            Pessoa = consulta;
            Endereco = consulta.EnderecoPrincipal;
            await NomeTextBox.FocusAsync();
        }

        public void AoSelecionarEndereco(RowSelectEventArgs<EnderecoPessoaModel> e)
        {
            Endereco = e.Data;
        }

        public async Task<EnderecoPessoaModel> GetEnderecoVazio()
        {
            var pais = (PaisModel)(await _paisesService.GetPadraoAsync(URIsPessoas.GetPaisesAsync)).Objeto;
            return new EnderecoPessoaModel { Cidade = new CidadeModel(), Pais = pais ?? new PaisModel() };
        }

        public async void AoClicarRetirarEMailBtnAsync(MouseEventArgs e)
        {
            await RetiraEMailCadastroPessoa.Retirar(this);
        }

        public async void AoClicarInserirEMailBtnAsync(MouseEventArgs e)
        {
            await InsereEMailCadastroPessoa.Add(this);
        }

        public void AoSelecionarEMail(RowSelectEventArgs<EMailPessoaModel> e)
        {
            EMail = e.Data;
        }

        public async void IdVendedorAoSairAsync()
        {
            if (VendedorPessoa.Vendedor.Id == 0) return;
            VendedorPessoa.Vendedor = await VendedorByIdCadastroPessoa.Get(this, _pessoasService, VendedorPessoa.Vendedor.Id, StatusAtivo.Ativos);
        }

        public void AoSelecionarVendedorPessoa(RowSelectEventArgs<VendedorPessoaModel> e)
        {
            VendedorPessoa = e.Data;
        }

        public async void AoClicarVendedoresBtnAsync(MouseEventArgs e)
        {
            await LocalizaVendedorCadastroPessoa.LocalizarAsync(this, _pessoasService);
        }

        public void AoSelecionarVendedor(RowSelectEventArgs<PessoaModel> e)
        {
            VendedorSelecionado = e.Data;
        }

        public void AoFecharDlgVendedores(object e)
        {
            DlgVendedoresVisible = false;
        }

        public async void AoClicarOkBtnDlgVendedores(MouseEventArgs e)
        {
            DlgVendedoresVisible = false;
            if (VendedorSelecionado == null)
            {
                return;
            }
            VendedorPessoa.Vendedor = await VendedorByIdCadastroPessoa.Get(this, _pessoasService, VendedorSelecionado.Id, StatusAtivo.Ativos);
            AoMudarPropriedade(nameof(VendedorPessoa));
        }

        public async void AoClicarRetirarVendedorBtnAsync(MouseEventArgs e)
        {
            await RetiraVendedorCadastroPessoa.Retirar(this);
        }

        public async Task AoClicarInserirVendedorBtnAsync(MouseEventArgs e)
        {
            await InsereVendedorCadastroPessoa.Add(this);
        }

        public void AoClicarRetirarProdutoBtnAsync(MouseEventArgs e)
        {
            RetiraProdutoCadastroPessoa.Retirar(this);
        }

        public void AoClicarInserirProdutoBtnAsync(MouseEventArgs e)
        {
            InsereProdutoCadastroPessoa.Add(this);
        }

        public void AoFecharDlgProdutos(object e)
        {
            DlgProdutosVisible = false;
        }

        public async void AoClicarOkBtnDlgProdutos(MouseEventArgs e)
        {
            DlgProdutosVisible = false;
            if (ProdutoSelecionado == null) return;
            var produto = await ProdutoByCodigoCadastroPessoa.Get(this, _produtosService, ProdutoSelecionado.Codigo, StatusAtivo.Ativos);
            ProdutoPessoa = new ProdutoPessoaModel { Produto = produto, IdProduto = produto.Id };
        }

        public async void AoClicarProdutosBtnAsync(MouseEventArgs e)
        {
            await LocalizaProdutoCadastroPessoa.LocalizarAsync(this, _produtosService);
        }

        public async void CodigoProdutoAoSairAsync()
        {
            var codigo = ProdutoPessoa.Produto.Codigo;
            if (string.IsNullOrEmpty(codigo)) return;
            var produto = await ProdutoByCodigoCadastroPessoa.Get(this, _produtosService, codigo, StatusAtivo.Ativos);
            ProdutoPessoa = new ProdutoPessoaModel { Produto = produto, IdProduto = produto.Id };
        }

        public void AoSelecionarProduto(RowSelectEventArgs<ProdutoModel> e)
        {
            ProdutoSelecionado = e.Data;
        }

        public void AoSelecionarProdutoPessoa(RowSelectEventArgs<ProdutoPessoaModel> e)
        {
            ProdutoPessoa = e.Data;
        }

        public async void AoClicarHistoricosBtnAsync(MouseEventArgs e)
        {
            await LocalizaHistoricoCadastroPessoa.LocalizarAsync(this, _historicosService);
        }

        public async void AoClicarOkBtnDlgHistoricos(MouseEventArgs e)
        {
            DlgHistoricosVisible = false;
            if (HistoricoSelecionado == null)
            {
                return;
            }
            Pessoa.Historico = await HistoricoByIdCadastroPessoa.Get(this, _historicosService, HistoricoSelecionado.Id, StatusAtivo.Ativos);
            Pessoa.IdHistorico = Pessoa.Historico.Id;
            AoMudarPropriedade(nameof(Pessoa.Historico));
        }

        public void AoFecharDlgHistoricos(object e)
        {
            DlgHistoricosVisible = false;
        }

        public async void IdHistoricoAoSairAsync()
        {
            if (!Pessoa.IdHistorico.HasValue)
            {
                LimpaHistoricoPessoa();
                return;
            }

            Pessoa.Historico = await HistoricoByIdCadastroPessoa.Get(this, _historicosService, Pessoa.IdHistorico.Value, StatusAtivo.Ativos);

            if (Pessoa.Historico.Id == 0) LimpaHistoricoPessoa();

            AoMudarPropriedade(nameof(Pessoa.Historico));
        }

        private void LimpaHistoricoPessoa()
        {
            Pessoa.Historico = new HistoricoModel();
            Pessoa.IdHistorico = (long?)null;
        }

        public void AoSelecionarHistorico(RowSelectEventArgs<HistoricoModel> e)
        {
            HistoricoSelecionado = e.Data;
        }

        public void AoSelecionarPlanoDeConta(RowSelectEventArgs<PlanoDeContaModel> e)
        {
            PlanoDeContaSelecionado = e.Data;
        }

        public async void AoClicarPlanosDeContaBtnAsync(MouseEventArgs e)
        {
            await LocalizaPlanoDeContaCadastroPessoa.LocalizarAsync(this, _planosDeContaService);
        }

        public async void AoClicarOkBtnDlgPlanosDeConta(MouseEventArgs e)
        {
            DlgPlanosDeContaVisible = false;
            if (PlanoDeContaSelecionado == null)
            {
                return;
            }
            Pessoa.PlanoDeConta = await PlanoDeContaByIdCadastroPessoa.Get(this, _planosDeContaService, PlanoDeContaSelecionado.Id, StatusAtivo.Ativos);
            Pessoa.IdPlanoDeConta = Pessoa.PlanoDeConta.Id;
            AoMudarPropriedade(nameof(Pessoa.PlanoDeConta));
        }

        public void AoFecharDlgPlanosDeConta(object e)
        {
            DlgPlanosDeContaVisible = false;
        }

        public async void IdPlanoDeContaAoSairAsync()
        {
            if (!Pessoa.IdPlanoDeConta.HasValue)
            {
                LimpaPlanoDeContaPessoa();
                return;
            }
            Pessoa.PlanoDeConta = await PlanoDeContaByIdCadastroPessoa.Get(this, _planosDeContaService, Pessoa.IdPlanoDeConta.Value, StatusAtivo.Ativos);

            if (Pessoa.PlanoDeConta.Id == 0) LimpaPlanoDeContaPessoa();

            AoMudarPropriedade(nameof(Pessoa.PlanoDeConta));
        }

        private void LimpaPlanoDeContaPessoa()
        {
            Pessoa.PlanoDeConta = new PlanoDeContaModel();
            Pessoa.IdPlanoDeConta = (long?)null;
        }

        public void AoSelecionarCentroDeCusto(RowSelectEventArgs<CentroDeCustoModel> e)
        {
            CentroDeCustoSelecionado = e.Data;
        }

        public async void AoClicarCentrosDeCustoBtnAsync(MouseEventArgs e)
        {
            await LocalizaCentroDeCustoCadastroPessoa.LocalizarAsync(this, _centrosDeCustoService);
        }

        public async void AoClicarOkBtnDlgCentrosDeCusto(MouseEventArgs e)
        {
            DlgCentrosDeCustoVisible = false;
            if (CentroDeCustoSelecionado == null)
            {
                return;
            }
            Pessoa.CentroDeCusto = await CentroDeCustoByIdCadastroPessoa.Get(this, _centrosDeCustoService, CentroDeCustoSelecionado.Id, StatusAtivo.Ativos);
            Pessoa.IdCentroDeCusto = Pessoa.CentroDeCusto.Id;
            AoMudarPropriedade(nameof(Pessoa.CentroDeCusto));
        }

        public void AoFecharDlgCentrosDeCusto(object e)
        {
            DlgCentrosDeCustoVisible = false;
        }

        public async void IdCentroDeCustoAoSairAsync()
        {
            if (!Pessoa.IdCentroDeCusto.HasValue)
            {
                LimpaCentroDeCustoPessoa();
                return;
            }
            Pessoa.CentroDeCusto = await CentroDeCustoByIdCadastroPessoa.Get(this, _centrosDeCustoService, Pessoa.IdCentroDeCusto.Value, StatusAtivo.Ativos);

            if (Pessoa.CentroDeCusto.Id == 0) LimpaCentroDeCustoPessoa();

            AoMudarPropriedade(nameof(Pessoa.CentroDeCusto));
        }

        private void LimpaCentroDeCustoPessoa()
        {
            Pessoa.CentroDeCusto = new CentroDeCustoModel();
            Pessoa.IdCentroDeCusto = (long?)null;
        }

        public void AoSelecionarCondicaoDePagamento(RowSelectEventArgs<CondicaoDePagamentoModel> e)
        {
            CondicaoDePagamentoSelecionada = e.Data;
        }

        public async void AoClicarCondicoesDePagamentoBtnAsync(MouseEventArgs e)
        {
            await LocalizaCondPgtoCadastroPessoa.LocalizarAsync(this, _condicoesDePagamentoService);
        }

        public async void AoClicarOkBtnDlgCondicoesDePagamento(MouseEventArgs e)
        {
            DlgCondicoesDePagamentoVisible = false;
            if (CondicaoDePagamentoSelecionada == null)
            {
                return;
            }
            Pessoa.CondicaoDePagamento = await CondPgtoByIdCadastroPessoa.Get(this, _condicoesDePagamentoService, CondicaoDePagamentoSelecionada.Id, StatusAtivo.Ativos);
            Pessoa.IdCondicaoDePagamento = Pessoa.CondicaoDePagamento.Id;
            AoMudarPropriedade(nameof(Pessoa.CondicaoDePagamento));
        }

        public void AoFecharDlgCondicoesDePagamento(object e)
        {
            DlgCondicoesDePagamentoVisible = false;
        }

        public async void IdCondicaoDePagamentoAoSairAsync()
        {
            if (!Pessoa.IdCondicaoDePagamento.HasValue)
            {
                LimpaCondicaoDePagamentoPessoa();
                return;
            }

            Pessoa.CondicaoDePagamento = await CondPgtoByIdCadastroPessoa.Get(this, _condicoesDePagamentoService, Pessoa.IdCondicaoDePagamento.Value, StatusAtivo.Ativos);

            if (Pessoa.CondicaoDePagamento.Id == 0) LimpaCondicaoDePagamentoPessoa();

            AoMudarPropriedade(nameof(Pessoa.CondicaoDePagamento));
        }

        private void LimpaCondicaoDePagamentoPessoa()
        {
            Pessoa.CondicaoDePagamento = new CondicaoDePagamentoModel();
            Pessoa.IdCondicaoDePagamento = (long?)null;
        }

        public void AoSelecionarTipoDeOperacao(RowSelectEventArgs<TipoDeOperacaoModel> e)
        {
            TipoDeOperacaoSelecionado = e.Data;
        }

        public async void AoClicarTiposDeOperacaoBtnAsync(MouseEventArgs e)
        {
            await LocalizaTipoDeOperacaoCadastroPessoa.LocalizarAsync(this, _tiposDeOperacaoService);
        }

        public async void AoClicarOkBtnDlgTiposDeOperacao(MouseEventArgs e)
        {
            DlgTiposDeOperacaoVisible = false;
            if (TipoDeOperacaoSelecionado == null)
            {
                return;
            }
            Pessoa.TipoDeOperacao = await TipoDeOperacaoByIdCadastroPessoa.Get(this, _tiposDeOperacaoService, TipoDeOperacaoSelecionado.Id, StatusAtivo.Ativos);
            Pessoa.IdTipoDeOperacao = Pessoa.TipoDeOperacao.Id;
            AoMudarPropriedade(nameof(Pessoa.TipoDeOperacao));
        }

        public void AoFecharDlgTiposDeOperacao(object e)
        {
            DlgTiposDeOperacaoVisible = false;
        }

        public async void IdTipoDeOperacaoAoSairAsync()
        {
            if (!Pessoa.IdTipoDeOperacao.HasValue)
            {
                LimpaTipoDeOperacaoPessoa();
                return;
            }
            Pessoa.TipoDeOperacao = await TipoDeOperacaoByIdCadastroPessoa.Get(this, _tiposDeOperacaoService, Pessoa.IdTipoDeOperacao.Value, StatusAtivo.Ativos);

            if (Pessoa.TipoDeOperacao.Id == 0) LimpaTipoDeOperacaoPessoa();

            AoMudarPropriedade(nameof(Pessoa.TipoDeOperacao));
        }
        private void LimpaTipoDeOperacaoPessoa()
        {
            Pessoa.TipoDeOperacao = new TipoDeOperacaoModel();
            Pessoa.IdTipoDeOperacao = (long?)null;
        }

        public void AoSelecionarTransportadoraPessoa(RowSelectEventArgs<TransportadoraPessoaModel> e)
        {
            TransportadoraPessoaSelecionada = e.Data;
            Transportadora = e.Data.Transportadora;
        }

        public void AoSelecionarTransportadora(RowSelectEventArgs<PessoaModel> e)
        {
            TransportadoraSelecionada = e.Data;
        }

        public async void AoClicarTransportadorasBtnAsync(MouseEventArgs e)
        {
            PesquisandoTransportadoras = true;
            var retorno = await _pessoasService.GetAsync(URIsPessoas.GetAsync, "Id,Nome,CNPJ,Ativo,DtCadastro", "ehtransportadora eq true and ativo eq true", "nome", "");
            Transportadoras = (List<PessoaModel>)retorno.Objeto;
            PesquisandoTransportadoras = false;
            DlgTransportadorasVisible = true;
        }

        public async void AoClicarRetirarTransportadoraBtnAsync(MouseEventArgs e)
        {
            await RetiraTransportadoraCadastroPessoa.Retirar(this);
        }

        public async Task AoClicarInserirTransportadoraBtnAsync(MouseEventArgs e)
        {
            await InsereTransportadoraCadastroPessoa.Add(this);
        }

        public void AoFecharDlgTransportadoras(object e)
        {
            DlgTransportadorasVisible = false;
        }

        public async void AoClicarOkBtnDlgTransportadoras(MouseEventArgs e)
        {
            DlgTransportadorasVisible = false;
            if (TransportadoraSelecionada == null)
            {
                return;
            }
            Transportadora = await TransportadoraByIdCadastroPessoa.Get(this, _pessoasService, TransportadoraSelecionada.Id);
        }

        public async void IdTransportadoraAoSairAsync()
        {
            if (Transportadora.Id == 0) return;
            Transportadora = await TransportadoraByIdCadastroPessoa.Get(this, _pessoasService, Transportadora.Id);
        }

        public void AoSelecionarTelefone(RowSelectEventArgs<TelefonePessoaModel> e)
        {
            Telefone = e.Data;
        }

        public async void AoClicarRetirarTelefoneBtnAsync(MouseEventArgs e)
        {
            await RetiraTelefoneCadastroPessoa.Retirar(this);
        }

        public async void AoClicarInserirTelefoneBtnAsync(MouseEventArgs e)
        {
            await InsereTelefoneCadastroPessoa.Add(this);
        }

        public async Task LimparTelaAsync()
        {
            await LimpaTelaCadastroPessoa.Limpar(this, _pessoasService, true);
        }

        void ICadastroBaseVM.LimparTela()
        {
            throw new NotImplementedException();
        }
    }
}
