﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroPessoa
{
    public static class HistoricoByIdCadastroPessoa
    {
        public static async Task<HistoricoModel> Get(CadastroPessoaVM viewModel, IHistoricosService historicosService, long id, StatusAtivo statusAtivo)
        {
            RespostaAPIModel resposta = new();
            try
            {
                viewModel.HistoricosBtnEnabled = false;
                await viewModel.SpinnerFinanceiro.ShowAsync();
                resposta = await historicosService.GetByIdAsync(URIsPessoas.GetHistoricosAsync, id, statusAtivo);
            }
            finally
            {
                await viewModel.SpinnerFinanceiro.HideAsync();
                viewModel.HistoricosBtnEnabled = true;
            }



            if (resposta.Sucesso)
            {
                return (HistoricoModel)resposta.Objeto;
            }
            
            viewModel.Mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um histórico ativo com o Id {id}" :
                $"Ocorreu um erro ao consultar o histórico com o id {id}. Erro {resposta.MensagemErro}";

            viewModel.TipoToast = TipoToast.Aviso;
            viewModel.AoMudarPropriedade("TipoToast");
            await viewModel.MeuToast.Show();
            await viewModel.IdHistoricoInput.FocusAsync();
            return new HistoricoModel();
        }
    }
}
