﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroPessoa
{
    public static class PlanoDeContaByIdCadastroPessoa
    {
        public static async Task<PlanoDeContaModel> Get(CadastroPessoaVM viewModel, IPlanosDeContaService planosDeContaService, long id, StatusAtivo statusAtivo)
        {
            RespostaAPIModel resposta = new();
            try
            {
                viewModel.PlanosDeContaBtnEnabled = false;
                await viewModel.SpinnerFinanceiro.ShowAsync();
                resposta = await planosDeContaService.GetByIdAsync(URIsPessoas.GetPlanosDeContaAsync, id, statusAtivo);
            }
            finally
            {
                viewModel.PlanosDeContaBtnEnabled = true;
                await viewModel.SpinnerFinanceiro.HideAsync();
            }


            if (resposta.Sucesso)
            {
               return (PlanoDeContaModel)resposta.Objeto;
            }
            else
            {
                viewModel.Mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um plano de contas ativo com o Id {id}" :
                    $"Ocorreu um erro ao consultar o plano de contas com o id {id}. Erro {resposta.MensagemErro}";

                viewModel.TipoToast = TipoToast.Aviso;
                viewModel.AoMudarPropriedade("TipoToast");
                await viewModel.MeuToast.Show();
                await viewModel.IdPlanoContaInput.FocusAsync();

                return new PlanoDeContaModel();
            }
        }
    }
}
