﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroPessoa
{
    public static class ProdutoByCodigoCadastroPessoa
    {
        public static async Task<ProdutoModel> Get(CadastroPessoaVM viewModel, IProdutosService produtosService, string codigo, StatusAtivo statusAtividade)
        {
            await viewModel.SpinnerProduto.ShowAsync();
            viewModel.ProdutosBtnEnabled = false;

            var resposta = await produtosService?.GetByCodigoAsync(URIsPessoas.GetProdutosAsync, codigo, statusAtividade);
            await viewModel.SpinnerProduto.HideAsync();
            viewModel.ProdutosBtnEnabled = true;

            viewModel.Mensagem = resposta.MensagemErro;

            if (resposta.Sucesso)
            {
                return (ProdutoModel)resposta.Objeto;
            }
            else
            {
                viewModel.Mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um item com o código {codigo}" :
                    $"Ocorreu um erro ao consultar o item com o código {codigo}. Erro {resposta.MensagemErro}";

                viewModel.TipoToast = TipoToast.Aviso;
                viewModel.AoMudarPropriedade("TipoToast");
                await viewModel.MeuToast.Show();
                await viewModel.CodigoProdutoInput.FocusAsync();
                return new ProdutoModel();
            }
        }
    }
}
