﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroPessoa
{
    public static class VendedorByIdCadastroPessoa
    {
        public static async Task<PessoaModel> Get(CadastroPessoaVM viewModel, IPessoasService service, long id, StatusAtivo statusAtivo)
        {
            viewModel.VendedoresBtnEnabled = false;
            await viewModel.SpinnerVendedor.ShowAsync();
            viewModel.PesquisandoVendedores = true;
            var resposta = await service.GetByIdAsync(URIsPessoas.GetVendedoresAsync, id, statusAtivo);
            
            await viewModel.SpinnerVendedor.HideAsync();
            viewModel.VendedoresBtnEnabled = true;
            viewModel.PesquisandoVendedores = false;

            if (resposta.Sucesso)
            {
                return (PessoaModel)resposta.Objeto;
            }
            else
            {
                viewModel.Mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um vendedor ativo com o Id {id}" :
                    $"Ocorreu um erro ao consultar o vendedor com o id {id}. Erro {resposta.MensagemErro}";

                viewModel.TipoToast = TipoToast.Aviso;
                viewModel.AoMudarPropriedade("TipoToast");
                await viewModel.MeuToast.Show();

                return new PessoaModel();
            }
        }
    }
}
