﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroProduto
{
    public static class AliquotaByIdCadastroProduto
    {
        public static async Task<AliquotaModel> Get(CadastroProdutoVM viewModel, IAliquotasService aliquotasService, long id, StatusAtivo ativos)
        {
            if (id == 0)
            {
                return new AliquotaModel();
            }

            RespostaAPIModel resposta = new();
            try
            {
                viewModel.AliquotasBtnEnabled = false;
                await viewModel.SpinnerPrincipal.ShowAsync();
                resposta = await aliquotasService.GetByIdAsync(URIsProdutos.GetAliquotasAsync, id, "id,descricao", ativos);
            }
            finally
            {
                viewModel.AliquotasBtnEnabled = true;
                await viewModel.SpinnerPrincipal.HideAsync();
            }

            if (!resposta.Sucesso)
            {
                var mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrada uma alíquota ativa com o Id {id}" :
                    $"Ocorreu um erro ao consultar a alíquota com o id {id}. Erro {resposta.MensagemErro}";

                await viewModel.ExibirMensagemToast(mensagem, TipoToast.Aviso);
                await viewModel.IdAliquotaInput.FocusAsync();
                return new AliquotaModel();
            }
            return (AliquotaModel)resposta.Objeto;
        }
    }
}
