﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroProduto;

public partial class CadastroProdutoVM : CadastroBaseVM, ICadastroProdutoVM
{
    private readonly IAliquotasService _aliquotasService;
    private readonly IClassificacoesFiscaisService _classFiscaisService;
    private readonly IEmbalagensService _embalagensService;
    private readonly IGenerosProdutoService _generosService;
    private readonly ILoginStore _loginStore;
    private readonly IProdutosService _produtosService;
    private readonly ISubgruposProdutoService _subgruposService;
    private readonly IUnidadesProdutoService _unidadesService;
    private readonly IParametrizacoesService _parametrizacoesService;

    [ObservableProperty]
    private ParametrizacaoModel parametrizacao;
    partial void OnParametrizacaoChanged(ParametrizacaoModel value) => AoMudarPropriedade(nameof(Parametrizacao));

    [ObservableProperty]
    private ProdutoModel produto;
    partial void OnProdutoChanged(ProdutoModel value) => AoMudarPropriedade(nameof(Produto));

    [ObservableProperty]
    private bool dlgAliquotasVisible;
    partial void OnDlgAliquotasVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgAliquotasVisible));

    [ObservableProperty]
    public bool dlgClassFiscaisVisible;
    partial void OnDlgClassFiscaisVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgClassFiscaisVisible));

    [ObservableProperty]
    private bool dlgEmbalagensVisible;
    partial void OnDlgEmbalagensVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgEmbalagensVisible));

    [ObservableProperty]
    public bool dlgGenerosVisible;
    partial void OnDlgGenerosVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgGenerosVisible));

    [ObservableProperty]
    public bool dlgProdutosVisible;
    partial void OnDlgProdutosVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgProdutosVisible));

    [ObservableProperty]
    public bool dlgSubgruposVisible;
    partial void OnDlgSubgruposVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgSubgruposVisible));

    [ObservableProperty]
    public bool dlgUnidadesVisible;
    partial void OnDlgUnidadesVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgUnidadesVisible));


    [ObservableProperty]
    public List<ListaModel> tiposProduto;
    partial void OnTiposProdutoChanged(List<ListaModel> value) => AoMudarPropriedade(nameof(TiposProduto));

    public bool PodeAcessar
    {
        get
        {
            var claimProduto = _loginStore.Claims.FirstOrDefault(c => c.Type == "Produto");
            return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimProduto != null);
        }
    }

    public string UrlRedirecionamento
    {
        get
        {
            if (!PodeAcessar)
            {
                return "/Login";
            }
            if (!_loginStore.LogadoEmEmpresa)
            {
                return "/SelecionaEmpresaLogada";
            }
            return string.Empty;
        }
    }

    public AliquotaModel AliquotaSelecionada { get; set; } = new AliquotaModel();
    public ClassificacaoFiscalModel ClassFiscalSelecionada { get; set; } = new ClassificacaoFiscalModel();
    public ProdutoModel ProdutoSelecionado { get; set; } = new ProdutoModel();
    public EmbalagemModel EmbalagemSelecionada { get; set; }
    public GeneroProdutoModel GeneroSelecionado { get; set; }
    public GrupoProdutoModel GrupoSelecionado { get; set; }
    public SubgrupoProdutoModel SubgrupoSelecionado { get; set; }
    public UnidadeProdutoModel UnidadeSelecionada { get; set; }
    public GrupoProdutoModel Grupo { get; set; } = new GrupoProdutoModel();
    public SfDialog Dlg { get; set; }
    public bool ClassFiscaisBtnEnabled { get; set; } = true;
    public bool AliquotasBtnEnabled { get; set; } = true;
    public bool GruposBtnEnabled { get; set; } = true;
    public bool SubgruposBtnEnabled { get; set; } = true;
    public bool UnidadesBtnEnabled { get; set; } = true;
    public bool GenerosBtnEnabled { get; set; } = true;
    public bool EmbalagensBtnEnabled { get; set; } = true;
    public SfTextBox CodigoProdutoTextBox { get; set; }
    public SfTextBox NomeTextBox { get; set; }
    public List<ProdutoModel> Produtos { get; set; } = new List<ProdutoModel>();
    public List<EmbalagemModel> Embalagens { get; set; } = new List<EmbalagemModel>();
    public List<AliquotaModel> Aliquotas { get; set; } = new List<AliquotaModel>();
    public List<ClassificacaoFiscalModel> ClassFiscais { get; set; } = new List<ClassificacaoFiscalModel>();
    public List<GrupoProdutoModel> Grupos { get; set; } = new List<GrupoProdutoModel>();
    public List<SubgrupoProdutoModel> Subgrupos { get; set; } = new List<SubgrupoProdutoModel>();
    public List<UnidadeProdutoModel> Unidades { get; set; } = new List<UnidadeProdutoModel>();
    public bool PesquisandoAliquotas { get; set; }
    public bool PesquisandoClassFiscais { get; set; }
    public bool PesquisandoEmbalagens { get; set; }
    public bool PesquisandoGeneros { get; set; }
    public bool PesquisandoGrupos { get; set; }
    public bool PesquisandoSubgrupos { get; set; }
    public bool PesquisandoUnidades { get; set; }

    public SfNumericTextBox<long> IdAliquotaInput { get; set; }
    public SfNumericTextBox<long> IdGrupoInput { get; set; }
    public SfNumericTextBox<long> IdSubgrupoInput { get; set; }
    public SfNumericTextBox<long> IdClassFiscalInput { get; set; }
    public SfNumericTextBox<long> IdUnidadeInput { get; set; }
    public SfNumericTextBox<long?> IdGeneroInput { get; set; }
    public SfNumericTextBox<decimal> VlrVendaInput { get; set; }
    public SfNumericTextBox<long?> IdEmbalagemInput { get; set; }
    public SfNumericTextBox<decimal> ComissaoVendedorInput { get; set; }
    public TelefonePessoaModel Telefone { get; set; }
    public List<GeneroProdutoModel> Generos { get; set; } = new List<GeneroProdutoModel>();
    public List<SCANCNFModel> ListaSCANCNF => ListaSCANCService.ItensNF;
    public List<SCANCNFModel> ListaSCANCEstoque => ListaSCANCService.ItensEstoque;

    public CadastroProdutoVM(IAliquotasService aliquotasService,
        IClassificacoesFiscaisService classFiscaisService,
        IEmbalagensService embalagensService,
        ILoginStore loginStore,
        IGenerosProdutoService generosService,
        IProdutosService produtosService,
        ISubgruposProdutoService subgruposService,
        ITiposProdutoService tiposProdutoService,
        IUnidadesProdutoService unidadesService,
        IParametrizacoesService parametrizacoesService)
    {
        Produto = new ProdutoModel
        {
            Aliquota = new AliquotaModel(),
            ClassificacaoFiscal = new ClassificacaoFiscalModel(),
            Subgrupo = new SubgrupoProdutoModel { Grupo = new GrupoProdutoModel() },
            Unidade = new UnidadeProdutoModel()
        };
        _loginStore = loginStore;
        _embalagensService = embalagensService;
        _aliquotasService = aliquotasService;
        _classFiscaisService = classFiscaisService;
        _generosService = generosService;
        _produtosService = produtosService;
        _subgruposService = subgruposService;
        _unidadesService = unidadesService;
        TiposProduto = tiposProdutoService?.Itens();
        _parametrizacoesService = parametrizacoesService;
    }

    public async void Enviar()
    {
        if (!DadosValidos())
        {
            return;
        }

        await EnvioDadosCadastroProduto.Enviar(this, _produtosService);
        await Task.Delay(500);
        if (Parametrizacao.CodigoAutomaticoNoProduto)
        {
            await NomeTextBox.FocusAsync();
        }
        else
        {
            await CodigoProdutoTextBox.FocusAsync();
        }
    }

    public void AoSelecionarAliquota(RowSelectEventArgs<AliquotaModel> e)
    {
        AliquotaSelecionada = e?.Data;
    }

    public void AoCriarCodigoProdutoTextBox(object e)
    {
        if (Produto.Id == 0) CodigoProdutoTextBox.FocusAsync();
    }

    public void AoCriarNomeTextBox(object e)
    {
        if (Produto.Id > 0) NomeTextBox.FocusAsync();
    }

    public void AoFecharDlgAliquotas(object e)
    {
        AliquotasBtnEnabled = true;
        DlgAliquotasVisible = false;
    }

    public void AoFecharDlg(object e)
    {
        DlgVisible = false;
    }

    public async void AoClicarNovoRegistroBtn(MouseEventArgs e)
    {
        LimparTela();
        if (Parametrizacao.CodigoAutomaticoNoProduto)
        {
            await Task.Delay(500);
            await NomeTextBox.FocusAsync();
        }
    }

    public void AoFecharDlgClassFiscais(object e)
    {
        ClassFiscaisBtnEnabled = true;
        DlgClassFiscaisVisible = false;
    }

    public bool DadosValidos()
    {
        var validator = new ProdutoModelValidator();
        ErrosValidacaoFormulario.Clear();
        List<string> erros = new();

        var results = validator.Validate(Produto);
        if (!results.IsValid)
        {
            foreach (var erro in results.Errors)
            {
                erros.Add(erro.ErrorMessage);
            }
            ErrosValidacaoFormulario.AddRange(erros);
            return false;
        }
        return true;
    }

    public void AoFecharDlgEmbalagens(object e)
    {
        DlgEmbalagensVisible = false;
    }

    public async void AoClicarOkBtnDlgProdutos(MouseEventArgs e)
    {
        if (ProdutoSelecionado == null)
        {
            return;
        }
        await ExibirProdutoSelecionado();
    }

    private async Task ExibirProdutoSelecionado()
    {
        ErrosValidacaoFormulario.Clear();
        DlgProdutosVisible = false;
        Produto = await ProdutoByIdCadastroProduto.Get(this, _produtosService, ProdutoSelecionado.Id, StatusAtivo.Todos);
    }

    public void AoSelecionarProduto(RowSelectEventArgs<ProdutoModel> e)
    {
        ProdutoSelecionado = e.Data;
    }

    public async Task<List<ProdutoModel>> GetProdutos()
    {
        var retorno = await _produtosService.GetAsync(URIsProdutos.GetAsync, StatusAtivo.Todos);

        if (!string.IsNullOrEmpty(retorno.MensagemErro))
        {
            Mensagem = retorno.MensagemErro;
            return new List<ProdutoModel>();
        }
        return (List<ProdutoModel>)retorno.Objeto;
    }

    public async void AoClicarLocalizarBtn(MouseEventArgs e)
    {
        Localizando = true;
        Produtos = await GetProdutos();
        Localizando = false;
        DlgProdutosVisible = true;
    }

    public async void IdAoSairAsync()
    {
        ErrosValidacaoFormulario.Clear();
        if (Produto.Id == 0) return;
        Produto = await ProdutoByIdCadastroProduto.Get(this, _produtosService, Produto.Id, StatusAtivo.Todos);
    }

    public void AoFecharDlgGeneros(object e)
    {
        DlgGenerosVisible = false;
    }

    public void AoFecharDlgProdutos(object e)
    {
        DlgProdutosVisible = false;
    }

    public async void AoClicarProdutosBtnAsync()
    {
        await LocalizaProdutoCadastroProduto.LocalizarAsync(this, _produtosService);
    }

    public async void CodigoProdutoAoSairAsync()
    {
        ErrosValidacaoFormulario.Clear();
        if (string.IsNullOrEmpty(Produto.Codigo)) return;
        var consulta = await ProdutoByCodigoCadastroProduto.Get(this, _produtosService, Produto.Codigo, StatusAtivo.Todos);

        if (consulta?.Id > 0)
        {
            Produto = consulta;
        }
    }

    public void AoFecharDlgSubgrupos(object e)
    {
        DlgSubgruposVisible = false;
    }

    public void AoSelecionarEmbalagem(RowSelectEventArgs<EmbalagemModel> e)
    {
        EmbalagemSelecionada = e.Data;
    }

    public void AoSelecionarGenero(RowSelectEventArgs<GeneroProdutoModel> e)
    {
        GeneroSelecionado = e.Data;
    }

    public async void AoClicarAliquotasBtnAsync(MouseEventArgs e)
    {
        await LocalizaAliquotaCadastroProduto.LocalizarAsync(this, _aliquotasService);
    }

    public async void AoClicarOkBtnDlgAliquotas(MouseEventArgs e)
    {
        await ExibirAliquota();
    }

    private async Task ExibirAliquota()
    {
        DlgAliquotasVisible = false;
        if (AliquotaSelecionada == null)
        {
            return;
        }
        Produto.Aliquota = await AliquotaByIdCadastroProduto.Get(this, _aliquotasService, AliquotaSelecionada.Id, StatusAtivo.Ativos);
        Produto.IdAliquota = Produto.Aliquota.Id;
        AoMudarPropriedade(nameof(Produto));
        await IdClassFiscalInput.FocusAsync();
    }

    public async void IdSubgrupoAoSairAsync()
    {
        if (Produto.IdSubgrupo == 0)
        {
            Produto.Subgrupo = new SubgrupoProdutoModel { Grupo = new GrupoProdutoModel() };
            return;
        }
        Produto.Subgrupo = await SubgrupoByIdCadastroProduto.Get(this, _subgruposService, Produto.IdSubgrupo, StatusAtivo.Ativos);

        Grupo = Produto.Subgrupo.Grupo;

        AoMudarPropriedade(nameof(Produto));
        await IdUnidadeInput.FocusAsync();
    }

    public void AoSelecionarSubgrupo(RowSelectEventArgs<SubgrupoProdutoModel> e)
    {
        SubgrupoSelecionado = e.Data;
    }

    public async void AoClicarClassFiscaisBtnAsync(MouseEventArgs e)
    {
        await LocalizaClassFiscaisCadastroProduto.LocalizarAsync(this, _classFiscaisService);
    }

    public async void AoClicarOkBtnDlgClassFiscais(MouseEventArgs e)
    {
        await ExibirClassificacaoFiscal();
    }

    private async Task ExibirClassificacaoFiscal()
    {
        DlgClassFiscaisVisible = false;
        if (ClassFiscalSelecionada == null)
        {
            return;
        }
        Produto.ClassificacaoFiscal = await ClassFiscalByIdCadastroProduto.Get(this, _classFiscaisService, ClassFiscalSelecionada.Id, StatusAtivo.Ativos);
        Produto.IdClassificacaoFiscal = Produto.ClassificacaoFiscal.Id;
        AoMudarPropriedade(nameof(Produto));
    }

    public void AoFecharDlgUnidades(object e)
    {
        DlgUnidadesVisible = false;
    }

    public async void IdUnidadeAoSairAsync()
    {
        if (Produto.IdUnidade == 0)
        {
            Produto.Unidade = new UnidadeProdutoModel();
            return;
        }
        Produto.Unidade.Codigo = ""; // feito para não executar CodigoUnidadeAoSairAsync()
        Produto.Unidade = await UnidadeByIdCadastroProduto.Get(this, _unidadesService, Produto.IdUnidade, StatusAtivo.Ativos);
        Produto.IdUnidade = Produto.Unidade.Id;
        AoMudarPropriedade(nameof(Produto));
    }

    public async void CodigoUnidadeAoSairAsync()
    {
        if (Produto.Unidade.Codigo == "")
        {
            Produto.Unidade = new UnidadeProdutoModel();
            return;
        }

        Produto.Unidade = await UnidadeByCodigoCadastroProduto.Get(this, _unidadesService, Produto.Unidade.Codigo, StatusAtivo.Ativos);
        Produto.IdUnidade = Produto.Unidade.Id;
        AoMudarPropriedade(nameof(Produto));
        await VlrVendaInput.FocusAsync();
    }

    public void AoSelecionarUnidade(RowSelectEventArgs<UnidadeProdutoModel> e)
    {
        UnidadeSelecionada = e.Data;
    }

    public async void AoClicarUnidadesBtnAsync(MouseEventArgs e)
    {
        await LocalizaUnidadeCadastroProduto.LocalizarAsync(this, _unidadesService);
    }

    public async void AoClicarOkBtnDlgUnidades(MouseEventArgs e)
    {
        await ExibirUnidadeSelecionada();
    }

    private async Task ExibirUnidadeSelecionada()
    {
        DlgUnidadesVisible = false;
        if (UnidadeSelecionada == null)
        {
            return;
        }
        Produto.Unidade = await UnidadeByIdCadastroProduto.Get(this, _unidadesService, UnidadeSelecionada.Id, StatusAtivo.Ativos);
        Produto.IdUnidade = Produto.Unidade.Id;
        AoMudarPropriedade(nameof(Produto));
    }

    public async void IdEmbalagemAoSairAsync()
    {
        if (!Produto.IdEmbalagem.HasValue)
        {
            Produto.Embalagem = new EmbalagemModel();
            return;
        }
        Produto.Embalagem = await EmbalagemByIdCadastroProduto.Get(this, _embalagensService, Produto.IdEmbalagem.Value, StatusAtivo.Ativos);

        AoMudarPropriedade(nameof(Produto));
    }

    public void AoSelecionarClassFiscal(RowSelectEventArgs<ClassificacaoFiscalModel> e)
    {
        ClassFiscalSelecionada = e.Data;
    }

    public async void AoClicarEmbalagensBtnAsync(MouseEventArgs e)
    {
        await LocalizaEmbalagemCadastroProduto.LocalizarAsync(this, _embalagensService);
    }

    public async void AoClicarGenerosBtnAsync(MouseEventArgs e)
    {
        await LocalizaGeneroCadastroProduto.LocalizarAsync(this, _generosService);
    }

    public async void AoClicarSubgruposBtnAsync(MouseEventArgs e)
    {
        await LocalizaSubgrupoCadastroProduto.LocalizarAsync(this, _subgruposService);
    }

    public void AoClicarRetirarMateriaPrimaBtnAsync(MouseEventArgs e)
    {
        throw new System.NotImplementedException();
    }

    public Task AoClicarInserirMateriaPrimaBtnAsync(MouseEventArgs e)
    {
        throw new System.NotImplementedException();
    }

    public async void AoClicarOkBtnDlgEmbalagens(MouseEventArgs e)
    {
        DlgEmbalagensVisible = false;
        if (EmbalagemSelecionada == null)
        {
            return;
        }
        Produto.Embalagem = await EmbalagemByIdCadastroProduto.Get(this, _embalagensService, EmbalagemSelecionada.Id, StatusAtivo.Ativos);
        Produto.IdEmbalagem = Produto.Embalagem.Id;
        AoMudarPropriedade(nameof(Produto));
    }

    public async void AoClicarOkBtnDlgGeneros(MouseEventArgs e)
    {
        DlgGenerosVisible = false;
        if (GeneroSelecionado == null)
        {
            return;
        }
        Produto.Genero = await GeneroByIdCadastroProduto.Get(this, _generosService, GeneroSelecionado.Id, StatusAtivo.Ativos);
        Produto.IdGenero = Produto.Genero.Id;
        AoMudarPropriedade(nameof(Produto));
    }

    public async void AoClicarOkBtnDlgSubgrupos(MouseEventArgs e)
    {
        await ExibirSubgrupo();
    }

    private async Task ExibirSubgrupo()
    {
        DlgSubgruposVisible = false;
        if (SubgrupoSelecionado == null)
        {
            return;
        }
        Produto.Subgrupo = await SubgrupoByIdCadastroProduto.Get(this, _subgruposService, SubgrupoSelecionado.Id, StatusAtivo.Ativos);
        Produto.IdSubgrupo = Produto.Subgrupo.Id;
        AoMudarPropriedade(nameof(Produto));
    }

    public async void IdAliquotaAoSairAsync()
    {
        Produto.Aliquota = await AliquotaByIdCadastroProduto.Get(this, _aliquotasService, Produto.IdAliquota, StatusAtivo.Ativos);

        AoMudarPropriedade(nameof(Produto));
    }

    public async void IdClassFiscalAoSairAsync()
    {
        if (Produto.IdClassificacaoFiscal == 0)
        {
            Produto.ClassificacaoFiscal = new ClassificacaoFiscalModel();
            return;
        }
        Produto.ClassificacaoFiscal.Numero = 0; // feito isso para não executar NumeroClassFiscalAoSairAsync()
        Produto.ClassificacaoFiscal = await ClassFiscalByIdCadastroProduto.Get(this, _classFiscaisService, Produto.IdClassificacaoFiscal, StatusAtivo.Ativos);
        Produto.IdClassificacaoFiscal = Produto.ClassificacaoFiscal.Id;

        AoMudarPropriedade(nameof(Produto));
    }

    public async void NumeroClassFiscalAoSairAsync()
    {
        Produto.ClassificacaoFiscal = await ClassFiscalByNumeroCadastroProduto.Get(this, _classFiscaisService, Produto.ClassificacaoFiscal.Numero, StatusAtivo.Ativos);
        Produto.IdClassificacaoFiscal = Produto.ClassificacaoFiscal.Id;
        AoMudarPropriedade(nameof(Produto));
    }

    public async void IdGeneroAoSairAsync()
    {
        if (!Produto.IdGenero.HasValue)
        {
            Produto.Genero = new GeneroProdutoModel();
            return;
        }
        Produto.Genero = await GeneroByIdCadastroProduto.Get(this, _generosService, Produto.IdGenero.Value, StatusAtivo.Ativos);

        AoMudarPropriedade(nameof(Produto));
    }

    public async void AoClicarSelecionarBtnGridProdutos(CommandClickEventArgs<ProdutoModel> e)
    {
        ProdutoSelecionado = e.RowData;
        await ExibirProdutoSelecionado();
    }

    public async void AoClicarSelecionarBtnGridNCMs(CommandClickEventArgs<ClassificacaoFiscalModel> e)
    {
        ClassFiscalSelecionada = e.RowData;
        await ExibirClassificacaoFiscal();
        await IdSubgrupoInput.FocusAsync();
    }

    public async void AoClicarSelecionarBtnGridUnidades(CommandClickEventArgs<UnidadeProdutoModel> e)
    {
        UnidadeSelecionada = e.RowData;
        await ExibirUnidadeSelecionada();
        await VlrVendaInput.FocusAsync();
    }

    public async Task GetParametrizacoes()
    {
        await _parametrizacoesService.GetAsync(URIsParametrizacoes.GetAsync);
        Parametrizacao = _parametrizacoesService.Parametrizacao;
    }

    public void LimparTela()
    {
        Produto = new ProdutoModel
        {
            Codigo = Parametrizacao != null ? Parametrizacao.CodigoAutomaticoNoProduto ? "0" : "" : "",
            Aliquota = new AliquotaModel(),
            Subgrupo = new SubgrupoProdutoModel { Grupo = new GrupoProdutoModel() },
            Embalagem = new EmbalagemModel(),
            ClassificacaoFiscal = new ClassificacaoFiscalModel(),
            Genero = new GeneroProdutoModel(),
            Unidade = new UnidadeProdutoModel()
        };
    }

    public async Task AoPressionarTeclaAliquotasGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && AliquotaSelecionada?.Id > 0)
        {
            await ExibirAliquota();
        }
    }

    public async void AoClicarSelecionarBtnGridAliquotas(CommandClickEventArgs<AliquotaModel> e)
    {
        AliquotaSelecionada = e.RowData;
        await ExibirAliquota();
        await IdClassFiscalInput.FocusAsync();

    }

    public async Task AoPressionarTeclaClassificacoesFiscaisGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && ClassFiscalSelecionada?.Id > 0)
        {
            await ExibirClassificacaoFiscal();
            await IdSubgrupoInput.FocusAsync();
        }
    }

    public async void AoClicarSelecionarBtnGridSubgrupos(CommandClickEventArgs<SubgrupoProdutoModel> e)
    {
        SubgrupoSelecionado = e.RowData;
        await ExibirSubgrupo();
        await IdUnidadeInput.FocusAsync();

    }

    public async Task AoPressionarTeclaSubgruposGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && SubgrupoSelecionado?.Id > 0)
        {
            await ExibirSubgrupo();
            await IdUnidadeInput.FocusAsync();
        }
    }

    public async Task AoPressionarTeclaUnidadesGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && UnidadeSelecionada?.Id > 0)
        {
            await ExibirUnidadeSelecionada();
            await VlrVendaInput.FocusAsync();
        }
    }

    public async Task AoPressionarTeclaProdutosGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && ProdutoSelecionado?.Id > 0)
        {
            await ExibirProdutoSelecionado();
            await NomeTextBox.FocusAsync();
        }
    }
}
