﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroProduto
{
    public static class ClassFiscalByNumeroCadastroProduto
    {
        public static async Task<ClassificacaoFiscalModel> Get(ICadastroProdutoVM viewModel, IClassificacoesFiscaisService classFiscaisService, long numero, StatusAtivo ativos)
        {
            if (numero == 0)
            {
                return new ClassificacaoFiscalModel();
            }

            RespostaAPIModel resposta = new();
            try
            {
                viewModel.ClassFiscaisBtnEnabled = false;
                await viewModel.SpinnerPrincipal.ShowAsync();
                resposta = await classFiscaisService.GetByNumeroAsync(URIsProdutos.GetClassFiscaisAsync, numero, ativos);
            }
            finally
            {
                viewModel.ClassFiscaisBtnEnabled = true;
                await viewModel.SpinnerPrincipal.HideAsync();
            }

            if (!resposta.Sucesso)
            {
                var mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um NCM ativo com o número {numero}" :
                    $"Ocorreu um erro ao consultar o NCM com o número {numero}. Erro {resposta.MensagemErro}";
                await viewModel.ExibirMensagemToast(mensagem, TipoToast.Aviso);
                await viewModel.IdClassFiscalInput.FocusAsync();
                return new ClassificacaoFiscalModel();
            }
            await viewModel.IdSubgrupoInput.FocusAsync();
            return (ClassificacaoFiscalModel)resposta.Objeto;
            
        }
    }
}
