﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroProduto;

public static class SubgrupoByIdCadastroProduto
{
    public static async Task<SubgrupoProdutoModel> Get(ICadastroProdutoVM viewModel, ISubgruposProdutoService subgruposService, long id, StatusAtivo ativos)
    {
        RespostaAPIModel resposta = new();
        try
        {
            viewModel.SubgruposBtnEnabled = false;
            await viewModel.SpinnerPrincipal.ShowAsync();
            resposta = await subgruposService.GetByIdAsync(URIsProdutos.GetSubgruposAsync, id, ativos);
        }
        finally
        {
            viewModel.SubgruposBtnEnabled = true;
            await viewModel.SpinnerPrincipal.HideAsync();
        }

        if (!resposta.Sucesso)
        {
            var mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um subgrupo ativo com o Id {id}" :
                $"Ocorreu um erro ao consultar o subgrupo com o id {id}. Erro {resposta.MensagemErro}";

            await viewModel.ExibirMensagemToast(mensagem, TipoToast.Aviso);
            await viewModel.IdSubgrupoInput.FocusAsync();
            return new SubgrupoProdutoModel { Grupo = new GrupoProdutoModel() };
        }

        return (SubgrupoProdutoModel)resposta.Objeto;

    }
}
