﻿using FluentValidation;
using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using Syncfusion.Blazor.Spinner;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroSubgrupoProduto
{
    public class CadastroSubgrupoProdutoVM : CadastroBaseVM, ICadastroSubgrupoProdutoVM
    {
        private readonly IGruposProdutoService _gruposService;
        private readonly ISubgruposProdutoService _subgruposService;
        private readonly ILoginStore _loginStore;

        private bool _dlgGruposVisible;
        private bool _dlgSubgruposVisible;
        private SubgrupoProdutoModel _subgrupo;

        public SubgrupoProdutoModel Subgrupo
        {
            get { return _subgrupo; }
            set
            {
                _subgrupo = value;
                AoMudarPropriedade(nameof(Subgrupo));
            }
        }

        public bool PodeAcessar
        {
            get
            {
                var claimGrupo = _loginStore.Claims.FirstOrDefault(c => c.Type == "SubgrupoProduto");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimGrupo != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }
        public bool DlgGruposVisible
        {
            get { return _dlgGruposVisible; }
            set
            {
                _dlgGruposVisible = value;
                AoMudarPropriedade(nameof(DlgGruposVisible));
            }
        }

        public bool DlgSubgruposVisible
        {
            get { return _dlgSubgruposVisible; }
            set
            {
                _dlgSubgruposVisible = value;
                AoMudarPropriedade(nameof(DlgSubgruposVisible));
            }
        }

        public SfDialog Dlg { get; set; }
        public SfTextBox NomeTextBox { get; set; }
        public GrupoProdutoModel GrupoSelecionado { get; set; } = new GrupoProdutoModel();
        public List<GrupoProdutoModel> Grupos { get; set; } = new List<GrupoProdutoModel>();
        public SubgrupoProdutoModel SubgrupoSelecionado { get; set; } = new SubgrupoProdutoModel();
        public List<SubgrupoProdutoModel> Subgrupos { get; set; } = new List<SubgrupoProdutoModel>();
        public bool GruposBtnEnabled { get; set; } = true;
        public bool PesquisandoGrupos { get; set; }

        public CadastroSubgrupoProdutoVM(IGruposProdutoService gruposService, ISubgruposProdutoService subgruposService, ILoginStore loginStore)
        {
            _loginStore = loginStore;
            _gruposService = gruposService;
            _subgruposService = subgruposService;
            Subgrupo = new SubgrupoProdutoModel { Ativo = true, Grupo = new GrupoProdutoModel() };
            MensagemResultadoEnvio = " ";
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroSubgrupoProduto.Enviar(this, _subgruposService);
        }

        public void AoCriarNomeTextBox(object e)
        {
            NomeTextBox.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }

        public bool DadosValidos()
        {
            var validator = new SubgrupoProdutoModelValidator();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(Subgrupo);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }

        public void AoFecharDlgGrupos(object e)
        {
            DlgGruposVisible = false;
        }

        public async void AoClicarOkBtnDlgGrupos(MouseEventArgs e)
        {
            if (GrupoSelecionado == null)
            {
                return;
            }
            DlgGruposVisible = false;
            Subgrupo.Grupo = await GrupoByIdCadastroSubgrupoProduto.Get(this, _gruposService, GrupoSelecionado.Id, StatusAtivo.Todos);
            Subgrupo.IdGrupo = Subgrupo.Grupo.Id;
            AoMudarPropriedade(nameof(Subgrupo));
        }

        public void AoSelecionarGrupo(RowSelectEventArgs<GrupoProdutoModel> e)
        {
            GrupoSelecionado = e.Data;
        }

        public async Task<List<GrupoProdutoModel>> GetGrupos()
        {
            var retorno = await _gruposService.GetAsync(URIsSubgruposProduto.GetGruposAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<GrupoProdutoModel>();
            }
            return (List<GrupoProdutoModel>)retorno.Objeto;
        }

        public async Task<List<SubgrupoProdutoModel>> GetSubgrupos()
        {
            var retorno = await _subgruposService.GetAsync(URIsSubgruposProduto.GetAsync, StatusAtivo.Todos);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<SubgrupoProdutoModel>();
            }
            return (List<SubgrupoProdutoModel>)retorno.Objeto;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            Subgrupos = await GetSubgrupos();
            Localizando = false;
            DlgSubgruposVisible = true;
        }

        public async void IdAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            if (Subgrupo.Id == 0) return;
            Subgrupo = await SubgrupoByIdCadastroSubgrupoProduto.Get(this, _subgruposService, Subgrupo.Id, StatusAtivo.Todos);
        }

        public void AoSelecionarSubgrupo(RowSelectEventArgs<SubgrupoProdutoModel> e)
        {
            SubgrupoSelecionado = e.Data;
        }

        public async void IdGrupoAoSairAsync()
        {
            if (Subgrupo.IdGrupo <= 0)
            {
                Subgrupo.Grupo = new GrupoProdutoModel();
                return;
            }

            Subgrupo.Grupo = await GrupoByIdCadastroSubgrupoProduto.Get(this, _gruposService, Subgrupo.IdGrupo, StatusAtivo.Ativos);

            AoMudarPropriedade(nameof(Subgrupo.Grupo));
        }

        public void AoFecharDlgSubgrupos(object e)
        {
            DlgSubgruposVisible = false;
        }

        public async void AoClicarOkBtnDlgSubgrupos(MouseEventArgs e)
        {
            if (SubgrupoSelecionado == null)
            {
                return;
            }
            ErrosValidacaoFormulario.Clear();
            DlgSubgruposVisible = false;
            Subgrupo = await SubgrupoByIdCadastroSubgrupoProduto.Get(this, _subgruposService, SubgrupoSelecionado.Id, StatusAtivo.Todos);
        }

        public async void AoClicarGruposBtnAsync(MouseEventArgs e)
        {
            await LocalizaGrupoCadastroSubgrupo.LocalizarAsync(this, _gruposService);
        }

        public void LimparTela()
        {
            LimpaTelaCadastroSubgrupoProduto.Limpar(this);
        }
    }
}
