﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.DropDowns;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Popups;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.CadastroUFICMS
{
    public class CadastroUFICMSVM : CadastroBaseVM, ICadastroUFICMSVM
    {
        private readonly IUFICMSsService _ufICMSsService;
        private readonly ILoginStore _loginStore;

        private bool _dlgUFICMSsVisible;
        private UFICMSModel _ufICMS;

        public UFICMSModel UFICMS
        {
            get { return _ufICMS; }
            set
            {
                _ufICMS = value;
                AoMudarPropriedade(nameof(UFICMS));
            }
        }

        public bool PodeAcessar
        {
            get
            {
                var claimUFICMS = _loginStore.Claims.FirstOrDefault(c => c.Type == "UFICMS");
                return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimUFICMS != null);
            }
        }

        public string UrlRedirecionamento
        {
            get
            {
                if (!PodeAcessar)
                {
                    return "/Login";
                }
                if (!_loginStore.LogadoEmEmpresa)
                {
                    return "/SelecionaEmpresaLogada";
                }
                return string.Empty;
            }
        }
        public bool DlgUFICMSsVisible
        {
            get { return _dlgUFICMSsVisible; }
            set
            {
                _dlgUFICMSsVisible = value;
                AoMudarPropriedade(nameof(DlgUFICMSsVisible));
            }
        }

        public SfDialog Dlg { get; set; }
        public SfDropDownList<string, string> UFDropDownList { get; set; }

        public UFICMSModel UFICMSSelecionada { get; set; } = new UFICMSModel();
        public List<UFICMSModel> UFICMSs { get; set; } = new List<UFICMSModel>();
        public List<string> ListaUFs => UFsService.UFs();

        public CadastroUFICMSVM(IUFICMSsService ufICMSsService, ILoginStore loginStore)
        {
            _loginStore = loginStore;
            _ufICMSsService = ufICMSsService;
            UFICMS = new UFICMSModel();
            MensagemResultadoEnvio = " ";
        }

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            await EnvioDadosCadastroUFICMS.Enviar(this, _ufICMSsService);
        }

        public void AoCriarUFDropDownList(object e)
        {
            UFDropDownList.FocusAsync();
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
        }

        public void AoClicarNovoRegistroBtn(MouseEventArgs e)
        {
            LimparTela();
        }

        public bool DadosValidos()
        {
            var validator = new UFICMSModelValidator();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(UFICMS);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }

        public void AoFecharDlgUFICMSs(object e)
        {
            DlgUFICMSsVisible = false;
        }

        public async void AoClicarOkBtnDlgUFICMSs(MouseEventArgs e)
        {
            if (UFICMSSelecionada == null)
            {
                return;
            }
            ErrosValidacaoFormulario.Clear();
            DlgUFICMSsVisible = false;
            UFICMS = await UFICMSByUFCadastroUFICMS.Get(this, _ufICMSsService, UFICMSSelecionada.UF);
        }

        public void AoSelecionarUFICMS(RowSelectEventArgs<UFICMSModel> e)
        {
            UFICMSSelecionada = e.Data;
        }

        public async Task<List<UFICMSModel>> GetUFICMSs()
        {
            var retorno = await _ufICMSsService.GetAsync(URIsUFICMSs.GetAsync);

            if (!string.IsNullOrEmpty(retorno.MensagemErro))
            {
                Mensagem = retorno.MensagemErro;
                return new List<UFICMSModel>();
            }
            return (List<UFICMSModel>)retorno.Objeto;
        }

        public async void AoClicarLocalizarBtn(MouseEventArgs e)
        {
            Localizando = true;
            UFICMSs = await GetUFICMSs();
            Localizando = false;
            DlgUFICMSsVisible = true;
        }

        public void LimparTela()
        {
            LimpaTelaCadastroUFICMS.Limpar(this, true);
        }

        public async void AoSelecionarUF()
        {
            ErrosValidacaoFormulario.Clear();
            if (string.IsNullOrEmpty(UFICMS.UF)) return;
            UFICMS = await UFICMSByUFCadastroUFICMS.Get(this, _ufICMSsService, UFICMS.UF);
        }
    }
}
