﻿using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.Compra
{
    public static class CompraByNumeroAndSerieAndPessoaAndTipo
    {
        public static async Task<FaturamentoModel> Get(ICompraVM viewModel, IComprasService comprasService)
        {
            var faturamento = new FaturamentoModel();
            if (viewModel.Faturamento.Numero == 0) return faturamento;

            RespostaAPIModel resposta;

            try
            {
                viewModel.Localizando = true;
                await viewModel.SpinnerPrincipal.ShowAsync();
                resposta = await comprasService.GetByNumeroAndSerieAndPessoaAndTipoAsync(
                        viewModel.Faturamento.Numero, 
                        viewModel.Faturamento.Serie, 
                        viewModel.Faturamento.IdPessoa, 
                        viewModel.Faturamento.Tipo
                    );

                if (!resposta.Sucesso)
                {
                    viewModel.Mensagem = resposta.MensagemErro;
                    viewModel.TipoToast = TipoToast.Aviso;
                    viewModel.AoMudarPropriedade("TipoToast");
                    await viewModel.MeuToast.Show();
                }
                else
                {
                    faturamento = (FaturamentoModel)resposta.Objeto;
                }
            }
            finally
            {
                await viewModel.SpinnerPrincipal.HideAsync();
                viewModel.Localizando = false;
            }
            return faturamento;
        }
    }
}
