﻿using AutoMapper;
using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETBlazor.ViewModels.Usuario.FaturamentoBase;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Inputs;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.Compra;

public partial class CompraVM : FaturamentoBaseVM, ICompraVM
{
    private readonly IComprasService _comprasService;
    private readonly ILoginStore _loginStore;
    private readonly IHistoricosService _historicosService;
    private readonly ISitTribService _sitTribService;
    private readonly ITabelaAService _tabelaAService;
    private readonly IDadosDevolucaoService _dadosDevolucaoService;
    private readonly IExibicaoFaturamentoService _exibicaoFaturamentoService;
    private readonly IPreenchimentoCSTCFOPCompraService _preenchimentoCSTCFOPCompraService;
    private readonly IMapper _mapper;
    private bool _dlgImportacaoXMLVisible;
    private bool _dlgDeParaImportacaoXMLVisible;

    private FaturamentoModel _faturamentoEmImportacao;

    [ObservableProperty]
    private FaturamentoModel faturamentoEmDevolucao;
    partial void OnFaturamentoEmDevolucaoChanged(FaturamentoModel value) => AoMudarPropriedade(nameof(FaturamentoEmDevolucao));
    public string UrlUploadXML
    {
        get
        {
            return URIsFaturamentos.ImportacaoXMLCompra.ToString();
        }
    }

    public FaturamentoModel FaturamentoEmImportacao
    {
        get { return _faturamentoEmImportacao; }
        set
        {
            _faturamentoEmImportacao = value;
            AoMudarPropriedade(nameof(FaturamentoEmImportacao));
        }
    }

    public bool DlgImportacaoXMLVisible
    {
        get { return _dlgImportacaoXMLVisible; }
        set
        {
            _dlgImportacaoXMLVisible = value;
            AoMudarPropriedade(nameof(DlgImportacaoXMLVisible));
        }
    }

    public bool DlgDeParaImportacaoXMLVisible
    {
        get { return _dlgDeParaImportacaoXMLVisible; }
        set
        {
            _dlgDeParaImportacaoXMLVisible = value;
            AoMudarPropriedade(nameof(DlgDeParaImportacaoXMLVisible));
        }
    }

    public SfNumericTextBox<int> SerieInput { get; set; }
    public SfNumericTextBox<long> IdHistoricoImportacaoXMLInput { get; set; }
    public List<ListaModel> ListaTabelaA { get; }
    public List<ListaModel> ListaSitTrib { get; }
    public SolicitanteProdutosCompra SolicitanteProdutos { get; set; }
    public IDeParaCompraImportacaoXMLVM DeParaCompraImportacaoXMLVM { get; set; }
    public bool HistoricosDlgImportacaoXMLBtnEnabled { get; set; } = true;
    public bool PesquisandoHistoricosDlgImportacaoXML { get; set; }
    public string NPedCompDlgImportXML { get; set; }
    public SolicitanteHistoricosCompra SolicitanteHistoricos { get; set; }
    public UploadChangeEventArgs ArgsUpload { get; set; }


    public CompraVM(IComprasService comprasService,
        IPessoasService pessoasService,
        ILoginStore loginStore,
        IProdutosService produtosService,
        IHistoricosService historicosService,
        ICondicoesDePagamentoService condicoesDePagamentoService,
        ICentrosDeCustoService centrosDeCustoService,
        ITiposDeOperacaoService tiposDeOperacaoService,
        ITiposFaturamentoService tiposFaturamentoService,
        ITiposFreteService tiposFreteService,
        IPesquisaFaturamentoVM pesquisaFaturamentoVM,
        ITimeZoneService timeZoneService,
        IDeParaCompraImportacaoXMLVM deParaCompraImportacaoXMLVM,
        ISitTribService sitTribIPIService,
        ITabelaAService tabelaAService,
        IAcoesFaturamentoService acoesFaturamentoService,
        IEnvioFaturamentoService envioFaturamentoService,
        INFeService nFeService,
        IDadosDevolucaoService dadosDevolucaoService,
        IExibicaoFaturamentoService exibicaoFaturamentoService,
        IPreenchimentoCSTCFOPCompraService preenchimentoCSTCFOPCompraService,
        IExibicaoProdutoFaturamentoService exibicaoProdutoFaturamentoService,
        IMapper mapper,
        NavigationManager navigation,
        IParametrizacoesService parametrizacoesService) : base(comprasService,
            pessoasService,
            loginStore,
            produtosService,
            historicosService,
            condicoesDePagamentoService,
            centrosDeCustoService,
            tiposDeOperacaoService,
            tiposFreteService,
            pesquisaFaturamentoVM,
            timeZoneService,
            acoesFaturamentoService,
            tiposFaturamentoService,
            envioFaturamentoService,
            nFeService,
            dadosDevolucaoService,
            exibicaoFaturamentoService,
            exibicaoProdutoFaturamentoService,
            parametrizacoesService,
            navigation)
    {
        _loginStore = loginStore;
        _comprasService = comprasService;
        _historicosService = historicosService;
        _tabelaAService = tabelaAService;
        _sitTribService = sitTribIPIService;
        _dadosDevolucaoService = dadosDevolucaoService;
        _exibicaoFaturamentoService = exibicaoFaturamentoService;
        _preenchimentoCSTCFOPCompraService = preenchimentoCSTCFOPCompraService;
        _mapper = mapper;
        ListaTabelaA = _tabelaAService.Itens();
        ListaSitTrib = _sitTribService.Itens();
        DeParaCompraImportacaoXMLVM = deParaCompraImportacaoXMLVM;
        DeParaCompraImportacaoXMLVM.CompraVM = this;

        Faturamento = new FaturamentoModel { Tipo = (int)TipoFaturamento.NotaCompra };
        FaturamentoEmImportacao = new FaturamentoModel { Tipo = (int)TipoFaturamento.NotaCompra, ProdutosFornecedor = new ObservableCollection<ProdutoFornecedorModel>() };
        Produto = new ProdutoModel();
        MensagemResultadoEnvio = " ";
        TiposFaturamento = tiposFaturamentoService.Itens().Where(t => t.Valor == (int)TipoFaturamento.NotaCompra || t.Valor == (int)TipoFaturamento.PedidoDeCompra).ToList();
    }

    public override async void NumeroAoSairAsync()
    {
        await ExibirCompraPelasChavesPrimarias();
    }

    public async void SerieAoSairAsync()
    {
        await ExibirCompraPelasChavesPrimarias();
    }

    private async Task ExibirCompraPelasChavesPrimarias()
    {
        ErrosValidacaoFormulario.Clear();
        if (Faturamento.Serie == 0 || Faturamento.Pessoa.Id == 0 || Faturamento.Numero == 0) return;
        Faturamento = await CompraByNumeroAndSerieAndPessoaAndTipo.Get(this, _comprasService);
        if (Faturamento.Numero > 0) await IdHistoricoInput.FocusAsync();
        await _exibicaoFaturamentoService.ExibirByFaturamento(this, Faturamento);
    }

    public async void AoSelecionarFileUpload(UploadChangeEventArgs args)
    {
        FaturamentoEmImportacao = new FaturamentoModel { Tipo = (int)TipoFaturamento.NotaCompra, ProdutosFornecedor = new ObservableCollection<ProdutoFornecedorModel>() };

        await SpinnerPrincipal.ShowAsync();

        var resultGetNota = await _comprasService.GetNotaDeXMLAsync(_loginStore.EmpresaLogada.CNPJ,
            args.Files[0].Stream,
            args.Files[0].FileInfo.Name,
            FaturamentoEmImportacao.DtRecebimento,
            FaturamentoEmImportacao.IdHistorico);

        await SpinnerPrincipal.HideAsync();

        if (!resultGetNota.Sucesso)
        {
            await ExibirMensagemToast((string)resultGetNota.Retorno, TipoToast.Erro);
            return;
        }

        FaturamentoEmImportacao = _mapper.Map<FaturamentoModel>((FaturamentoDTO)resultGetNota.Retorno);

        DlgImportacaoXMLVisible = true;
        ArgsUpload = args;
    }

    private async Task ImportarNota(UploadChangeEventArgs args)
    {
        await SpinnerPrincipal.ShowAsync();
        var resultGetNota = await _comprasService.GetNotaDeXMLAsync(_loginStore.EmpresaLogada.CNPJ, args.Files[0].Stream,
            args.Files[0].FileInfo.Name,
            FaturamentoEmImportacao.DtRecebimento.Value,
            FaturamentoEmImportacao.IdHistorico.Value);

        await SpinnerPrincipal.HideAsync();

        if (!resultGetNota.Sucesso)
        {
            await ExibirMensagemToast((string)resultGetNota.Retorno, TipoToast.Erro);
            return;
        }

        FaturamentoEmImportacao = _mapper.Map<FaturamentoModel>((FaturamentoDTO)resultGetNota.Retorno);

        var deveFazerDePara = FaturamentoEmImportacao.ProdutosFornecedor.Any(p => p.Produto.Id == 0 || (p.CdUnidadeFornAnt != "" && p.CdUnidadeFornAnt != null && p.CdUnidadeFornAnt != p.CdUnidadeForn));

        DlgDeParaImportacaoXMLVisible = deveFazerDePara;

        if (!deveFazerDePara)
        {
            try
            {
                await SpinnerPrincipal.ShowAsync();
                await GravarNotaImportada();
            }
            finally
            {
                await SpinnerPrincipal.HideAsync();
            }
        }
        else
        {
            await Task.Delay(100);
            await DeParaCompraImportacaoXMLVM.ProdutoFornecedorGrid.EditCellAsync(0, "Produto.Codigo");
            AoMudarPropriedade("FaturamentoEmImportacao.ProdutosFornecedor");
        }
    }

    public void AoFecharDlgDeParaImportacaoXML(object e)
    {
        DlgDeParaImportacaoXMLVisible = false;
        DeParaCompraImportacaoXMLVM.EditarAoEntrar = true;
    }

    public async void AoClicarOkBtnDlgDeParaImportacaoXML(MouseEventArgs e)
    {
        var resultDePara = DeParaCompraImportacaoXMLVM.DeParaCompleto();
        if (!resultDePara.Sucesso)
        {
            await ExibirMensagemToast((string)resultDePara.Retorno, TipoToast.Erro);
            return;
        }
        try
        {
            await DeParaCompraImportacaoXMLVM.SpinnerGridProdutoFornecedor.ShowAsync();
            await GravarNotaImportada();
        }
        finally
        {
            await DeParaCompraImportacaoXMLVM.SpinnerGridProdutoFornecedor.HideAsync();
        }
    }

    private async Task GravarNotaImportada()
    {
        FaturamentoEmImportacao.Tipo = (int)TipoFaturamento.NotaCompra;
        var resultGravacao = await _comprasService.GravarNotaImportadaAsync(FaturamentoEmImportacao);
        DlgDeParaImportacaoXMLVisible = false;
        DeParaCompraImportacaoXMLVM.EditarAoEntrar = true;
        AoMudarPropriedade(nameof(DlgDeParaImportacaoXMLVisible));

        if (!resultGravacao.Sucesso)
        {
            await ExibirMensagemToast($"Ocorreu um erro ao tentar gravar a nota de compra de número: {FaturamentoEmImportacao.Numero}. Erro: {resultGravacao.ConteudoRetorno}", TipoToast.Erro);
            return;
        }

        await ExibirMensagemToast("Compra importada com sucesso.", TipoToast.Sucesso);

    }

    public async void AoClicarOkBtnDlgImportacaoXML(MouseEventArgs e)
    {
        bool tudoOK = FaturamentoEmImportacao.DtRecebimento.HasValue && FaturamentoEmImportacao.IdHistorico > 0;

        if (!tudoOK)
        {
            if (!FaturamentoEmImportacao.DtRecebimento.HasValue) await ExibirMensagemToast("A Data de recebimento deve ser informada.", TipoToast.Aviso);
            if (FaturamentoEmImportacao.IdHistorico == 0) await ExibirMensagemToast("O histórico deve ser informado.", TipoToast.Aviso);
            return;
        }

        DlgImportacaoXMLVisible = false;
        try
        {
            await ImportarNota(ArgsUpload);

        }
        catch (Exception ex)
        {
            await ExibirMensagemToast($"Ocorreu um erro ao tentar importar a nota. Erro: {ex.GetBaseException().Message}", TipoToast.Erro); ;
        }
    }

    public async void IdHistoricoImportacaoXMLAoSairAsync()
    {
        var resultHistorico = await HistoricoById.Get(this, _historicosService, FaturamentoEmImportacao.Historico.Id);
        if (resultHistorico.Sucesso)
        {
            FaturamentoEmImportacao.Historico = (HistoricoModel)resultHistorico.Retorno;
        }
        else
        {
            await IdHistoricoImportacaoXMLInput.FocusAsync();
        }
        AoMudarPropriedade(nameof(FaturamentoEmImportacao.Historico));
    }

    public void AoFecharDlgImportacaoXML(object e)
    {
        DlgImportacaoXMLVisible = false;
    }

    public void NPedCompDlgImportXMLAoSairAsync()
    {
        throw new NotImplementedException();
    }

    public async void AoClicarHistoricosDlgImportacaoXMLBtnAsync(MouseEventArgs e)
    {
        SolicitanteHistoricos = SolicitanteHistoricosCompra.DlgImportacaoXML;
        HistoricosDlgImportacaoXMLBtnEnabled = false;
        PesquisandoHistoricosDlgImportacaoXML = true;
        await LocalizaHistorico.LocalizarAsync(this, _historicosService);
        DlgHistoricosVisible = true;
        HistoricosDlgImportacaoXMLBtnEnabled = true;
        PesquisandoHistoricosDlgImportacaoXML = false;
    }

    public void AoAlterarSituacaoTributaria()
    {
        _preenchimentoCSTCFOPCompraService.Preencher(this);
    }

    public void AoAlterarTabelaA()
    {
        _preenchimentoCSTCFOPCompraService.Preencher(this);
    }

    public void AoFecharDlgDevolucao(object e)
    {
        DlgDevolucaoVisible = false; ;
    }

    public async void AoClicarOkBtnDlgDevolucao(MouseEventArgs e)
    {
        _dadosDevolucaoService.FaturamentoADevolver = Faturamento;
        var resultado = await _dadosDevolucaoService.DevolucaoVM.PrepararFaturamentoParaDevolver();

        if (!resultado.Sucesso)
        {
            await ExibirMensagemToast((string)resultado.Retorno, TipoToast.Aviso);
            return;
        }
        Faturamento = (FaturamentoModel)resultado.Retorno;
        Navigation.NavigateTo("/Usuario/Movimentacoes/Faturamento");

    }
}
