﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Spinner;
using Syncfusion.Presentation;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.Compra
{
    public class DeParaCompraImportacaoXMLVM : IDeParaCompraImportacaoXMLVM
    {
        private readonly IProdutosService _produtosService;
        private bool _dlgProdutosVisible;
        
        public bool TabularParaFatorConversao { get; set; }
        public bool TabularParaCodigoProduto { get; set; }
        public double IndiceLinha { get; set; }
        public bool PodeSelecionarLinha { get; set; } = true;
        public ProdutoFornecedorModel ProdutoFornecedorAtivo { get; set; }

        public ICompraVM CompraVM { get; set; }

        public SfGrid<ProdutoFornecedorModel> ProdutoFornecedorGrid { get; set; }
        public SfSpinner SpinnerGridProdutoFornecedor { get; set; }
        public bool EditarAoEntrar { get; set; } = true;

        public event PropertyChangedEventHandler PropertyChanged;
        public void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        public bool DlgProdutosVisible
        {
            get { return _dlgProdutosVisible; }
            set
            {
                _dlgProdutosVisible = value;
                OnPropertyChanged(nameof(DlgProdutosVisible));
            }
        }

        public DeParaCompraImportacaoXMLVM(IProdutosService produtosService)
        {
            _produtosService = produtosService;
        }

        public async void AoClicarCommandBtnProdFornGrid(CommandClickEventArgs<ProdutoFornecedorModel> args)
        {
            await SpinnerGridProdutoFornecedor.ShowAsync();
            ProdutoFornecedorAtivo = args.RowData;

            var retorno = await _produtosService?.GetAsync(URIsFaturamentos.GetProdutosAsync, StatusAtivo.Ativos);
            if (!retorno.Sucesso)
            {
                await CompraVM.ExibirMensagemToast(retorno.MensagemErro, TipoToast.Erro);
            }
            else
            {
                CompraVM.Produtos = (List<ProdutoModel>)retorno.Objeto;
                DlgProdutosVisible = true;
            }

            await SpinnerGridProdutoFornecedor.HideAsync();
        }

        public async Task<bool> PodeSelecionarProdutoDePara(string codigo, string cdProdForn)
        {
            if (CompraVM.FaturamentoEmImportacao.ProdutosFornecedor.Any(p => p.Produto.Codigo == codigo && p.CdProdForn != cdProdForn))
            {
                await CompraVM.ExibirMensagemToast("Já existe um outro item do fornecedor associado ao código digitado.", TipoToast.Aviso);
                return false;
            }
            return true;
        }

        public ResultadoModel DeParaCompleto()
        {
            if (CompraVM.FaturamentoEmImportacao.ProdutosFornecedor.Any(p => p.Produto.Id == 0 || p.FatorConversao <= 0))
            {
                return new ResultadoModel { Sucesso = false, Retorno = "Existem dados incompletos no de => para. Favor revisar." };
            }
            return new ResultadoModel { Sucesso = true, Retorno = "OK" };
        }

        public async Task AoClicarOkBtnDlgProdutos(MouseEventArgs e)
        {
            DlgProdutosVisible = false;
            var produto = CompraVM.ProdutoSelecionado;
            if (produto == null) return;

            if (!await PodeSelecionarProdutoDePara(produto.Codigo, ProdutoFornecedorAtivo.CdProdForn)) return;
            ProdutoFornecedorAtivo.Produto = produto;
            ProdutoFornecedorAtivo.IdProduto = ProdutoFornecedorAtivo.Produto.Id;
        }

        public void AoFecharDlgProdutos(object e)
        {
            DlgProdutosVisible = false;
        }

        public async void AoClicarSelecionarBtnGridProdutos(CommandClickEventArgs<ProdutoModel> e)
        {
            DlgProdutosVisible = false;
            var produto = e.RowData;
            if (!await PodeSelecionarProdutoDePara(produto.Codigo, ProdutoFornecedorAtivo.CdProdForn)) return;
            ProdutoFornecedorAtivo.Produto = new ProdutoModel { Id = produto.Id, Codigo = produto.Codigo, Nome = produto.Nome };
            ProdutoFornecedorAtivo.Produto = produto;
            ProdutoFornecedorAtivo.IdProduto = ProdutoFornecedorAtivo.Produto.Id;
        }

        public async Task CodigoProdutoGridAoSairAsync(FocusOutEventArgs args)
        {
            if (args.Value == "" || args.Value == null)
            {
                ProdutoFornecedorAtivo.Produto = new ProdutoModel();
                return;
            }


            if (!await PodeSelecionarProdutoDePara(args.Value, ProdutoFornecedorAtivo.CdProdForn))
            {
                ProdutoFornecedorAtivo.Produto = new ProdutoModel();
                return;
            }

            var resposta = await _produtosService?.GetByCodigoAsync(URIsFaturamentos.GetProdutosAsync, args.Value, "Id,Codigo,Nome,IdClassificacaoFiscal,IdUnidade,IdAliquota", StatusAtivo.Ativos);

            if (!resposta.Sucesso)
            {
                string mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um item ativo com o código {args.Value}" : $"Ocorreu um erro ao consultar o item com o código {args.Value}. Erro {resposta.MensagemErro}";
                await CompraVM.ExibirMensagemToast(mensagem, TipoToast.Aviso);
                ProdutoFornecedorAtivo.Produto = new ProdutoModel();
            }

            ProdutoFornecedorAtivo.Produto = (ProdutoModel)resposta.Objeto;
            ProdutoFornecedorAtivo.IdProduto = ProdutoFornecedorAtivo.Produto.Id;
        }

        public void AoSelecionarLinhaGrid(RowSelectEventArgs<ProdutoFornecedorModel> args)
        {
            ProdutoFornecedorAtivo = args.Data;
            IndiceLinha = args.RowIndex;
        }

        public async void AoComecarAcaoDeParaGrid(ActionEventArgs<ProdutoFornecedorModel> args)
        {
            if (args.Action == "Edit")
            {
                args.Data.Produto = ProdutoFornecedorAtivo.Produto;
                args.Data.IdProduto = ProdutoFornecedorAtivo.Produto.Id;
            }

            if (args.RequestType == Syncfusion.Blazor.Grids.Action.Save)
            {
                await ProdutoFornecedorGrid.SelectRowAsync(IndiceLinha + 1);
            }
        }

        public async void AoSairFatorConversao(NumericBlurEventArgs<decimal> args)
        {
            await ProdutoFornecedorGrid.EndEditAsync();
            await ProdutoFornecedorGrid.SelectRowAsync(IndiceLinha + 1);
        }
    }
}
