﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETBlazor.ViewModels.Usuario.PesquisaFaturamento;
using GAFNETLibrary.Enums;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.Faturamento;

public static class EstornoNF
{
    public static async Task Estornar(IBaseVM viewModel, IEnvioFaturamentoService envioFaturamentoService, INFeService nFeService, FaturamentoModel faturamento)
    {
        try
        {
            await viewModel.SpinnerPrincipal.ShowAsync();
            await ExecutarCodigo(viewModel, envioFaturamentoService, nFeService, faturamento);
        }
        finally
        {
            await viewModel.SpinnerPrincipal.HideAsync();
        }
        

    }

    private static async Task ExecutarCodigo(IBaseVM viewModel, IEnvioFaturamentoService envioFaturamentoService, INFeService nFeService, FaturamentoModel faturamento)
    {
        ResetarLog(viewModel);
        ExibirDlgLog(viewModel);

        SetarLog(viewModel, $"\n\rEstornando a nota no banco...");

        var resultado = await envioFaturamentoService.EstornarAsync(faturamento.Id, faturamento.MotivoEstorno);

        if (!resultado.Sucesso)
        {
            SetarLog(viewModel, $"\n\r{(string)resultado.Retorno}");
            return;
        }

        var retornoFat =  (FaturamentoModel)resultado.Retorno;
        faturamento.DtEstorno = retornoFat.DtEstorno;

        SetarLog(viewModel, $"\n\r documento estornado com sucesso no banco");

        var resultNFe = await EnvioNFe.Enviar(viewModel, nFeService, faturamento);

        if (!resultNFe.Sucesso)
        {
            await viewModel.ExibirMensagemToast((string)resultNFe.Retorno, TipoToast.Erro);
        }

        if (viewModel is FaturamentoVM vM)
        {
            await vM.PreencherFormularioPadrao();
            vM.AoMudarPropriedade(nameof(Faturamento));
        }
    }

    private static void SetarLog(IBaseVM viewModel, string mensagem)
    {
        switch (viewModel)
        {
            case var fatVM when fatVM is IFaturamentoVM:
                ((IFaturamentoVM)fatVM).LogEnvioEstorno += mensagem;
                break;
            case var pesqVM when pesqVM is PesquisaFaturamentoVM:
                ((IPesquisaFaturamentoVM)pesqVM).LogEnvioEstorno += mensagem;
                break;
            default:
                break;
        }

    }

    private static void ResetarLog(IBaseVM viewModel)
    {
        switch (viewModel)
        {
            case var fatVM when fatVM is IFaturamentoVM:
                ((IFaturamentoVM)fatVM).LogEnvioEstorno = "";
                ((IFaturamentoVM)fatVM).LogEnvioFaturamento = "";
                break;
            case var pesqVM when pesqVM is IPesquisaFaturamentoVM:
                ((IPesquisaFaturamentoVM)pesqVM).LogEnvioEstorno = "";
                ((IPesquisaFaturamentoVM)pesqVM).LogEnvioFaturamento = "";
                break;
            default:
                break;
        }

    }

    private static void ExibirDlgLog(IBaseVM viewModel)
    {
        switch (viewModel)
        {
            case var fatVM when fatVM is IFaturamentoVM:
                ((IFaturamentoVM)fatVM).DlgLogEnvioEstornoVisible = true;
                break;
            case var pesqVM when pesqVM is PesquisaFaturamentoVM:
                ((IPesquisaFaturamentoVM)pesqVM).DlgLogEnvioEstornoVisible = true;
                break;
            default:
                break;
        }
    }

}
