﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.Faturamento
{
    public static class ExibicaoFormulario
    {
        public static async Task PreencherPadrao(IFaturamentoVM vM, IFormulariosService formulariosService)
        {
            var resposta = await formulariosService.GetPadraoAsync(URIsFaturamentos.GetFormulariosAsync);

            if (! resposta.Sucesso)
            {
                await vM.ExibirMensagemToast(resposta.MensagemErro, TipoToast.Erro);
                return;
            }
            
            vM.Faturamento.Formulario = string.IsNullOrEmpty(resposta.MensagemErro) && resposta.Sucesso ? (FormularioModel)resposta.Objeto : new FormularioModel();
            vM.Faturamento.Serie = vM.Faturamento.Formulario.Serie;
            vM.Faturamento.IdFormulario = vM.Faturamento.Formulario.Id;
        }

        public static async Task PreencherById(IFaturamentoVM vM, IFormulariosService formulariosService, long id)
        {
            vM.DlgFormulariosVisible = false;

            RespostaAPIModel resposta = new();
            try
            {
                vM.FormulariosBtnEnabled = false;
                await vM.SpinnerPrincipal.ShowAsync();
                resposta = await formulariosService.GetByIdAsync(URIsFaturamentos.GetFormulariosAsync, id, StatusAtivo.Ativos);
            }
            finally
            {
                await vM.SpinnerPrincipal.HideAsync();
                vM.FormulariosBtnEnabled = true;
            }

            if (!resposta.Sucesso)
            {
                vM.Faturamento.Formulario = new FormularioModel();
                vM.Faturamento.IdFormulario = 0;
                vM.Faturamento.Serie = 0;

                string msg = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um formulário ativo com o Id {id}" :
                    $"Ocorreu um erro ao consultar o formulário com o id {id}. Erro {resposta.MensagemErro}";
                await vM.ExibirMensagemToast(resposta.MensagemErro, TipoToast.Erro);
                return;
            }
            var formulario = (FormularioModel)resposta.Objeto;

            vM.Faturamento.Formulario = formulario;
            vM.Faturamento.IdFormulario = vM.Faturamento.Formulario.Id;
            vM.Faturamento.Serie = vM.Faturamento.Formulario.Serie;
        }
    }
}
