﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETBlazor.ViewModels.Usuario.FaturamentoBase;
using GAFNETLibrary.Enums;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.Faturamento
{
    public partial class FaturamentoVM : FaturamentoBaseVM, IFaturamentoVM
    {
        private readonly IPessoasService _pessoasService;
        private readonly IFormulariosService _formulariosService;
        private readonly INFeService _nFeService;
        private readonly IPesquisaFaturamentoVM _pesquisaFaturamentoVM;
        private readonly IEnvioFaturamentoService _envioFaturamentoService;
        private readonly IDadosDevolucaoService _dadosDevolucaoService;
        private readonly IExibicaoFaturamentoService _exibicaoFaturamentoService;
        
        [ObservableProperty]
        private bool dlgLogEnvioInutilizacaoVisible;
        partial void OnDlgLogEnvioInutilizacaoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgLogEnvioInutilizacaoVisible));

        [ObservableProperty]
        private string logEnvioInutilizacao;
        partial void OnLogEnvioInutilizacaoChanged(string value) => AoMudarPropriedade(nameof(LogEnvioInutilizacao));

        [ObservableProperty]
        private bool dlgLogEnvioCancelamentoVisible;
        partial void OnDlgLogEnvioCancelamentoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgLogEnvioCancelamentoVisible));

        [ObservableProperty]
        private string logEnvioCancelamento;
        partial void OnLogEnvioCancelamentoChanged(string value) => AoMudarPropriedade(nameof(LogEnvioCancelamento));

        [ObservableProperty]
        private bool dlgLogEnvioEstornoVisible;
        partial void OnDlgLogEnvioEstornoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgLogEnvioEstornoVisible));

        [ObservableProperty]
        private string logEnvioEstorno;
        partial void OnLogEnvioEstornoChanged(string value) => AoMudarPropriedade(nameof(LogEnvioEstorno));

        [ObservableProperty]
        private bool dlgMotivoCancelamentoVisible;
        partial void OnDlgMotivoCancelamentoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgMotivoCancelamentoVisible));

        [ObservableProperty]
        private bool dlgMotivoInutilizacaoVisible;
        partial void OnDlgMotivoInutilizacaoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgMotivoInutilizacaoVisible));

        [ObservableProperty]
        private bool dlgMotivoEstornoVisible;
        partial void OnDlgMotivoEstornoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgMotivoEstornoVisible));

        [ObservableProperty]
        private bool dlgConfirmacaoEnvioEMailNFeVisible;
        partial void OnDlgConfirmacaoEnvioEMailNFeVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgConfirmacaoEnvioEMailNFeVisible));

        [ObservableProperty]
        private bool vendedoresBtnEnabled;
        partial void OnVendedoresBtnEnabledChanged(bool value) => AoMudarPropriedade(nameof(VendedoresBtnEnabled));

        [ObservableProperty]
        private FaturamentoModel faturamentoEmDevolucao;
        partial void OnFaturamentoEmDevolucaoChanged(FaturamentoModel value) => AoMudarPropriedade(nameof(FaturamentoEmDevolucao));

        [ObservableProperty]
        private bool dlgVendedoresVisible;
        partial void OnDlgVendedoresVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgVendedoresVisible));

        [ObservableProperty]
        private bool dlgInsercaoVendedoresVisible;
        partial void OnDlgInsercaoVendedoresVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgInsercaoVendedoresVisible));


        [ObservableProperty]
        private bool dlgFormulariosVisible;
        partial void OnDlgFormulariosVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgFormulariosVisible));
        [ObservableProperty]
        public bool pesquisandoVendedores;
        partial void OnPesquisandoVendedoresChanged(bool value) => AoMudarPropriedade(nameof(PesquisandoVendedores));


        [ObservableProperty]
        public PessoaModel vendedor;
        partial void OnVendedorChanged(PessoaModel value) => AoMudarPropriedade(nameof(Vendedor));
        [ObservableProperty]
        public VendedorFaturamentoModel vendedorFaturamento;
        partial void OnVendedorFaturamentoChanged(VendedorFaturamentoModel value) => AoMudarPropriedade(nameof(VendedorFaturamento));
        public List<PessoaModel> Vendedores { get; set; } = new List<PessoaModel>();
        public PessoaModel VendedorSelecionado { get; set; }
        public bool PesquisandoPaises { get; set; }
        public bool FormulariosBtnEnabled { get; set; } = true;
        public FormularioModel FormularioSelecionado { get; set; }
        public List<FormularioModel> Formularios { get; set; } = new List<FormularioModel>();
        public bool PesquisandoFormularios { get; set; }
        public SfNumericTextBox<long> IdVendedorInput { get; set; }
        public SfNumericTextBox<decimal> ComissaoVendedorInput { get; set; }
        public SfNumericTextBox<long> IdFormularioInput { get; set; }

        public FaturamentoVM(IFaturamentosService faturamentosService,
            IPessoasService pessoasService,
            ILoginStore loginStore,
            IProdutosService produtosService,
            IHistoricosService historicosService,
            ICondicoesDePagamentoService condicoesDePagamentoService,
            ICentrosDeCustoService centrosDeCustoService,
            ITiposDeOperacaoService tiposDeOperacaoService,
            IFormulariosService formulariosService,
            ITiposFreteService tiposFreteService,
            IPesquisaFaturamentoVM pesquisaFaturamentoVM,
            ITimeZoneService timeZoneService,
            IAcoesFaturamentoService acoesFaturamentoService,
            IDadosDevolucaoService dadosDevolucaoService,
            INFeService nFeService,
            ITiposFaturamentoService tiposFaturamentoService,
            IExibicaoFaturamentoService exibicaoFaturamentoService,
            IEnvioFaturamentoService envioFaturamentoService,
            IExibicaoProdutoFaturamentoService exibicaoProdutoFaturamentoService,
            IParametrizacoesService parametrizacoesService,
            NavigationManager navigation) : base(faturamentosService,
                pessoasService,
                loginStore,
                produtosService,
                historicosService,
                condicoesDePagamentoService,
                centrosDeCustoService,
                tiposDeOperacaoService,
                tiposFreteService,
                pesquisaFaturamentoVM,
                timeZoneService,
                acoesFaturamentoService,
                tiposFaturamentoService,
                envioFaturamentoService,
                nFeService,
                dadosDevolucaoService,
                exibicaoFaturamentoService,
                exibicaoProdutoFaturamentoService,
                parametrizacoesService,
                navigation)
        {
            FaturamentoEmDevolucao = new FaturamentoModel();
            _pessoasService = pessoasService;
            _formulariosService = formulariosService;
            _nFeService = nFeService;
            Faturamento = new FaturamentoModel { Vendedores = new ObservableCollection<VendedorFaturamentoModel>() };
            Produto = new ProdutoModel();
            Vendedor = new PessoaModel();
            VendedorFaturamento = new VendedorFaturamentoModel { Vendedor = new PessoaModel() };
            MensagemResultadoEnvio = " ";
            _dadosDevolucaoService = dadosDevolucaoService;
            _exibicaoFaturamentoService = exibicaoFaturamentoService;
            _dadosDevolucaoService.FaturamentoVM = this;
            VendedoresBtnEnabled = true;
            _envioFaturamentoService = envioFaturamentoService;
            _pesquisaFaturamentoVM = pesquisaFaturamentoVM;
            _pesquisaFaturamentoVM.TiposFaturamento = TiposFaturamento;
            _nFeService.OnEtapaEnvioNFe += AoEtapaEnvioNFe;
            _nFeService.OnEtapaCancelamentoNFe += AoEtapaCancelamentoNFe;
            _nFeService.OnEtapaInutilizacaoNFe += AoEtapaInutilizacaoNFe;
        }

        private void AoEtapaInutilizacaoNFe(string mensagem)
        {
            LogEnvioInutilizacao += $"\n\r{mensagem}";
        }
        private void AoEtapaCancelamentoNFe(string mensagem)
        {
            LogEnvioCancelamento += $"\n\r{mensagem}";
        }

        private void AoEtapaEnvioNFe(string mensagem)
        {
            LogEnvioFaturamento += $"\n\r{mensagem}";
            if (AcaoFaturamento == TipoAcaoFaturamento.Estornar)
            {
                LogEnvioEstorno += $"\n\r{mensagem}";
            }
        }

        public async Task AoClicarRetirarVendedorBtnAsync(MouseEventArgs e)
        {
            await RetiraVendedorFaturamento.Retirar(this);
        }

        public async Task AoClicarInserirVendedorBtnAsync(MouseEventArgs e)
        {
            await InsereVendedorFaturamento.Add(this);
        }

        public override async void NumeroAoSairAsync()
        {
            ErrosValidacaoFormulario.Clear();
            await _exibicaoFaturamentoService.ExibirByNumeroAndSerieAndTipo(this);
            AoMudarPropriedade(nameof(Faturamento));
        }

        public async void IdVendedorAoSairAsync()
        {
            await ExibicaoVendedor.ExibirPorId(this, _pessoasService, VendedorFaturamento.Vendedor.Id);
            AoMudarPropriedade(nameof(VendedorFaturamento));
        }

        public void AoSelecionarVendedorFaturamento(RowSelectEventArgs<VendedorFaturamentoModel> e)
        {
            VendedorFaturamento = e.Data;
        }

        public async void AoClicarOKBtnDlgFormularios(MouseEventArgs e)
        {
            await ExibicaoFormulario.PreencherById(this, _formulariosService, FormularioSelecionado.Id);
            AoMudarPropriedade(nameof(Faturamento));
        }

        public void AoFecharDlgFormularios(object e)
        {
            DlgFormulariosVisible = false;
        }

        public async void IdFormularioAoSairAsync()
        {
            await ExibicaoFormulario.PreencherById(this, _formulariosService, Faturamento.Formulario.Id);
            AoMudarPropriedade(nameof(Faturamento.Formulario));
        }

        public void AoSelecionarFormulario(RowSelectEventArgs<FormularioModel> e)
        {
            FormularioSelecionado = e.Data;
        }

        public async void AoClicarFormulariosBtnAsync(MouseEventArgs e)
        {
            await LocalizaFormulario.LocalizarAsync(this, _formulariosService);
        }

        public void AoSelecionarVendedor(RowSelectEventArgs<PessoaModel> e)
        {
            VendedorSelecionado = e.Data;
        }

        public async void AoClicarVendedoresBtnAsync(MouseEventArgs e)
        {
            await ExibicaoPesquisaVendedores.Exibir(this, _pessoasService);
        }

        public void AoFecharDlgVendedores(object e)
        {
            DlgVendedoresVisible = false;
        }

        public async void AoClicarOKBtnDlgVendedores(MouseEventArgs e)
        {
            await ExibirVendedor();
        }

        private async Task ExibirVendedor()
        {
            DlgVendedoresVisible = false;
            if (VendedorSelecionado == null) return;
            await ExibicaoVendedor.ExibirPorId(this, _pessoasService, VendedorSelecionado.Id);
            AoMudarPropriedade("Faturamento.VendedorFaturamento");
        }

        public async void AoClicarSelecionarBtnGridVendedores(CommandClickEventArgs<PessoaModel> e)
        {
            VendedorSelecionado = e.RowData;
            await ExibirVendedor();
        }

        public async Task AoPressionarTeclaVendedoresGrid(KeyboardEventArgs args)
        {
            if (args.Key == "Enter" && VendedorSelecionado?.Id > 0)
            {
                await ExibirVendedor();
            }
        }

        public async void AoFecharDlgInsercaoVendedores(object e)
        {
            DlgInsercaoVendedoresVisible = false;

            if (Parametrizacao.UsaVendedor && !Vendedores.Any())
            {
                await ExibirMensagemToast("É necessário inserir ao menos um vendedor.", TipoToast.Aviso);
                return;
            }
            
        }

        public void AoClicarVendedoresBtn()
        {
            DlgInsercaoVendedoresVisible = true;
        }

        public void AoClicarRetirarVendedorBtn()
        {
            if (VendedorSelecionado == null) return;

            var vendExcluir = Faturamento.Vendedores.FirstOrDefault(v => v.IdVendedor == VendedorSelecionado.Id);
            Faturamento.Vendedores.Remove(vendExcluir);
        }

        public async Task AoClicarSimBtnDlgConfirmacaoEnvioEMailNFe(MouseEventArgs e)
        {
            DlgConfirmacaoEnvioEMailNFeVisible = false;
            await EnvioEMailNFe.EnviarXMLEPDF(this, _nFeService, Faturamento.ChaveEletronica);
        }

        public void AoClicarNaoBtnDlgConfirmacaoEnvioEMailNFe(MouseEventArgs e)
        {
            DlgConfirmacaoEnvioEMailNFeVisible = false;
        }

        public void AoClicarSimBtnDlgConfirmacaoCancelamento(MouseEventArgs e)
        {
            DlgConfirmacaoCancelamentoVisible = false;
            DlgMotivoCancelamentoVisible = true;
        }

        public void AoClicarSimBtnDlgConfirmacaoInutilizacao(MouseEventArgs e)
        {
            DlgConfirmacaoInutilizacaoVisible = false;
            DlgMotivoInutilizacaoVisible = true;
        }

        public void AoClicarNaoBtnDlgConfirmacaoCancelamento(MouseEventArgs e)
        {
            DlgConfirmacaoCancelamentoVisible = false;
        }

        public void AoClicarNaoBtnDlgConfirmacaoInutilizacao(MouseEventArgs e)
        {
            DlgConfirmacaoInutilizacaoVisible = false;
        }
        public async void AoClicarOKBtnDlgMotivoCancelamento(MouseEventArgs e)
        {
            DlgMotivoCancelamentoVisible = false;
            await CancelamentoDocumento.Cancelar(this, Faturamento, _nFeService, _envioFaturamentoService);
            AoMudarPropriedade(nameof(Faturamento));
        }

        public async void AoClicarOKBtnDlgMotivoInutilizacao(MouseEventArgs e)
        {
            DlgMotivoInutilizacaoVisible = false;
            await InutilizacaoDocumento.Inutilizar(this, Faturamento, _nFeService, _envioFaturamentoService);
        }

        public void AoFecharDlgDevolucao(object e)
        {
            DlgDevolucaoVisible = false;
        }

        public async void AoClicarOKBtnDlgDevolucao(MouseEventArgs e)
        {
            _dadosDevolucaoService.FaturamentoADevolver = Faturamento;
            var resultado = await _dadosDevolucaoService.DevolucaoVM.PrepararFaturamentoParaDevolver();
            
            if (!resultado.Sucesso)
            {
                await ExibirMensagemToast((string)resultado.Retorno, TipoToast.Aviso);
                return;
            }
            Faturamento = (FaturamentoModel)resultado.Retorno;
        }

        public async void AoClicarOKBtnDlgMotivoEstorno(MouseEventArgs e)
        {
            DlgMotivoEstornoVisible = false;
            await EstornoNF.Estornar(this, _envioFaturamentoService, _nFeService, Faturamento);
        }

        public async Task PreencherFormularioPadrao()
        {
            await ExibicaoFormulario.PreencherPadrao(this, _formulariosService);
        }
    }
}
