﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase
{
    public static class CentroDeCustoById
    {
        public static async Task<CentroDeCustoModel> Get(IFaturamentoBaseVM viewModel, ICentrosDeCustoService centrosDeCustoService, long id)
        {
            if (id == 0) return new CentroDeCustoModel();
            var resposta = new RespostaAPIModel();
            try
            {
                viewModel.CentrosDeCustoBtnEnabled = false;
                await viewModel.SpinnerPrincipal.ShowAsync();
                resposta = await centrosDeCustoService.GetByIdAsync(URIsFaturamentos.GetCentrosDeCustoAsync, id, StatusAtivo.Ativos);
            }
            finally
            {
                viewModel.CentrosDeCustoBtnEnabled = true;
                await viewModel.SpinnerPrincipal.HideAsync();
            }

            if (resposta.Sucesso)
            {
                return (CentroDeCustoModel)resposta.Objeto;
            }
            else
            {
                viewModel.Mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um centro de custo ativo com o Id {id}" :
                    $"Ocorreu um erro ao consultar o centro de custo com o id {id}. Erro {resposta.MensagemErro}";

                viewModel.TipoToast = TipoToast.Aviso;
                viewModel.AoMudarPropriedade("TipoToast");
                await viewModel.MeuToast.Show();
                await viewModel.IdCentroDeCustoInput.FocusAsync();
                return new CentroDeCustoModel();
            }
        }
    }
}
