﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase
{
    public static class CondicaoDePagamentoById
    {
        public static async Task<CondicaoDePagamentoModel> Get(IFaturamentoBaseVM viewModel, ICondicoesDePagamentoService condPgtosService, long id)
        {
            if (id == 0) return new CondicaoDePagamentoModel();

            var resposta = new RespostaAPIModel();
            try
            {
                await viewModel.SpinnerPrincipal.ShowAsync();
                viewModel.CondicoesDePagamentoBtnEnabled = false;
                resposta = await condPgtosService?.GetByIdAsync(URIsFaturamentos.GetCondicoesDePagamentoAsync, id, StatusAtivo.Ativos);
            }
            finally
            {
                viewModel.CondicoesDePagamentoBtnEnabled = true;
                await viewModel.SpinnerPrincipal.HideAsync();
            }

            if (resposta.Sucesso)
            {
                return (CondicaoDePagamentoModel)resposta.Objeto;
            }
            else
            {
                viewModel.Mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado uma condição de pagamento ativa com o Id {id}" :
                    $"Ocorreu um erro ao consultar a condição de pagamento com o id {id}. Erro {resposta.MensagemErro}";

                viewModel.TipoToast = TipoToast.Aviso;
                viewModel.AoMudarPropriedade("TipoToast");
                await viewModel.MeuToast.Show();
                await viewModel.IdCondPgtoInput.FocusAsync();

                return new CondicaoDePagamentoModel();
            }
        }
    }
}
