﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETBlazor.ViewModels.Usuario.FaturamentoBase;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.Compra
{
    public partial class DevolucaoVM : BaseVM, IDevolucaoVM
    {
        private readonly IHistoricosService _historicosService;
        private readonly IFormulariosService _formulariosService;
        private readonly IFaturamentosService _faturamentosService;
        private readonly IDadosDevolucaoService _dadosDevolucaoService;
        public double IndiceLinha { get; set; }
        public FormularioModel FormularioSelecionado { get; set; }
        public SfGrid<ItemFaturamentoModel> ItensDevolucaoGrid { get; set; }
        public bool EditarAoEntrar { get; set; }
        public bool PesquisandoHistoricos { get; set; }
        public bool PesquisandoFormularios { get; set; }
        public SfNumericTextBox<long> IdFormularioInput { get; set; }
        public SfNumericTextBox<long> IdHistoricoInput { get; set; }
        public bool HistoricosBtnEnabled { get; set; } = true;
        public bool FormulariosBtnEnabled { get; set; } = true;
        public bool DlgFormulariosVisible { get; set; } = false;
        public bool DlgHistoricosVisible { get; set; } = false;
        public List<FormularioModel> Formularios { get; set; } = new List<FormularioModel>();
        public bool TabularParaQuantidade { get; set; }
        [ObservableProperty]
        private TipoAcaoFaturamento tipoAcaoFaturamento;
        partial void OnTipoAcaoFaturamentoChanged(TipoAcaoFaturamento value)
        {
            DescricaoOperacao = value == TipoAcaoFaturamento.Devolver ? "DEVOLUÇÃO" : "RETORNO";
            AoMudarPropriedade(nameof(TipoAcaoFaturamento));
            AoMudarPropriedade(nameof(DescricaoOperacao));
        }
        [ObservableProperty]
        private bool dlgDevolucaoVisible;
        [ObservableProperty]
        private string descricaoOperacao;
        partial void OnDescricaoOperacaoChanged(string value) => AoMudarPropriedade(nameof(DescricaoOperacao));

        [ObservableProperty]
        private FaturamentoModel faturamento;
        partial void OnFaturamentoChanged(FaturamentoModel value)
        {
            SetFaturamentoEmDevolucao();
            AoMudarPropriedade(nameof(Faturamento));
            AoMudarPropriedade(nameof(FaturamentoEmDevolucao));
        }
        [ObservableProperty]
        private FaturamentoModel faturamentoEmDevolucao;
        partial void OnFaturamentoEmDevolucaoChanged(FaturamentoModel value) => AoMudarPropriedade(nameof(FaturamentoEmDevolucao));
        [ObservableProperty]
        private HistoricoModel historicoSelecionado;
        partial void OnHistoricoSelecionadoChanged(HistoricoModel value) => AoMudarPropriedade(nameof(HistoricoSelecionado));
        public List<HistoricoModel> Historicos { get; set; } = new List<HistoricoModel>();

        public DevolucaoVM(IHistoricosService historicosService, 
            IFormulariosService formulariosService, 
            IFaturamentosService faturamentosService,
            IDadosDevolucaoService dadosDevolucaoService)
        {
            _formulariosService = formulariosService;
            _historicosService = historicosService;
            _faturamentosService = faturamentosService;
            _dadosDevolucaoService = dadosDevolucaoService;
            _dadosDevolucaoService.DevolucaoVM = this;
            Faturamento = _dadosDevolucaoService.FaturamentoADevolver;
            TipoAcaoFaturamento = _dadosDevolucaoService.TipoAcaoFaturamento;
        }
        public async Task CelulaSalva(CellSaveArgs<ItemFaturamentoModel> args)
        {
            if (args.ColumnName != "QtdLancada") return;

            IndiceLinha = await ItensDevolucaoGrid.GetRowIndexByPrimaryKeyAsync(args.Data.IdProduto);
            TabularParaQuantidade = true;

            if (args.Data.QtdSaldo < 0)
            {
                args.Data.QtdLancada = (decimal)args.PreviousValue;
                await ExibirMensagemToast("Quantidade lançada é maior que o saldo do item. Não é possível lançar.", TipoToast.Aviso);
            }
            else
            {
                args.Data = AcertarFreteSeguroOutDespAcess(args.Data);
            }



            await ItensDevolucaoGrid.EndEditAsync();
        }

        public async Task DataBound()
        {
            if (EditarAoEntrar && ItensDevolucaoGrid.DataSource.Any())
            {
                EditarAoEntrar = false;
                await Task.Delay(100);
                await ItensDevolucaoGrid.EditCellAsync(0, "QtdLancada");
                AoMudarPropriedade(nameof(FaturamentoEmDevolucao));

            }
            if (TabularParaQuantidade)
            {
                await ItensDevolucaoGrid.EditCellAsync(IndiceLinha + 1, "QtdLancada");

            }
            TabularParaQuantidade = false;
        }

        public ResultadoModel DevolucaoCompleta()
        {
            if (!FaturamentoEmDevolucao.IdFormulario.HasValue)
                return new ResultadoModel { Sucesso = false, Retorno = "É necessário selecionar um formulário." };

            if (!FaturamentoEmDevolucao.IdHistorico.HasValue)
                return new ResultadoModel { Sucesso = false, Retorno = "É necessário selecionar um histórico." };

            if (TipoAcaoFaturamento == TipoAcaoFaturamento.Devolver && !(FaturamentoEmDevolucao.Historico?.FinNFe == (int)FinNFeHistorico.Devolucao))
                return new ResultadoModel { Sucesso = false, Retorno = "É necessário selecionar um historico de devolução." };

            if (TipoAcaoFaturamento == TipoAcaoFaturamento.Retornar && !(FaturamentoEmDevolucao.Historico?.TipoNF == (int)TipoNFHistorico.Retorno))
                return new ResultadoModel { Sucesso = false, Retorno = "É necessário selecionar um historico de retorno." };

            if (!FaturamentoEmDevolucao.Itens.Any(p => p.QtdLancada > 0))
                return new ResultadoModel { Sucesso = false, Retorno = "Não foi lançada nenhuma quantidade para devolver." };

            return new ResultadoModel { Sucesso = true, Retorno = "OK" };
        }

        public async void IdFormularioAoSairAsync()
        {
            await GetFormularioById(FaturamentoEmDevolucao.Formulario.Id);
        }

        public async void IdHistoricoAoSairAsync()
        {
            await GetHistoricoById(FaturamentoEmDevolucao.Historico.Id);
        }

        private async Task GetHistoricoById(long id)
        {
            RespostaAPIModel resposta;
            if (id == 0)
            {
                return;
            }

            try
            {
                HistoricosBtnEnabled = false;
                await SpinnerPrincipal.ShowAsync();
                resposta = await _historicosService.GetByIdAsync(URIsFaturamentos.GetHistoricosAsync, id, StatusAtivo.Ativos);
            }
            finally
            {
                await SpinnerPrincipal.HideAsync();
                HistoricosBtnEnabled = true;
            }

            if (!await ValidarHistorico(id, resposta)) return;

            FaturamentoEmDevolucao.Historico = (HistoricoModel)resposta.Objeto;
            FaturamentoEmDevolucao.IdHistorico = FaturamentoEmDevolucao.Historico.Id;
            OnPropertyChanged(nameof(FaturamentoEmDevolucao.Historico));
        }

        private async Task<bool> ValidarHistorico(long id, RespostaAPIModel resposta)
        {
            if (!resposta.Sucesso)
            {
                string mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um histórico ativo com o Id {id}" :
                    $"Ocorreu um erro ao consultar o histórico com o id {id}. Erro {resposta.MensagemErro}";

                _ = new HistoricoModel();
                await ExibirMensagemToast(mensagem, TipoToast.Aviso);
                return false;
            }

            var historico = (HistoricoModel)resposta.Objeto;
            if (
                    (TipoAcaoFaturamento == TipoAcaoFaturamento.Devolver && historico.FinNFe != (int)FinNFeHistorico.Devolucao) ||
                    (TipoAcaoFaturamento == TipoAcaoFaturamento.Retornar && historico.TipoNF != (int)TipoNFHistorico.Retorno)
               )
            {
                FaturamentoEmDevolucao.Historico = new HistoricoModel();
                string msgTipoAcao = TipoAcaoFaturamento == TipoAcaoFaturamento.Devolver ? "devolução" : "retorno";
                await ExibirMensagemToast($"Não foi encontrado um histórico de {msgTipoAcao} com esse código.", TipoToast.Aviso);
                OnPropertyChanged(nameof(FaturamentoEmDevolucao.Historico));
                return false;
            }

            if (Faturamento.EntradaSaida == historico.EntradaSaida)
            {
                FaturamentoEmDevolucao.Historico = new HistoricoModel();
                string msgEntradaSaida = Faturamento.EntradaSaida == (int)EntradaSaidaHistorico.Entrada ? "saída" : "entrada";
                await ExibirMensagemToast($"Não foi encontrado um histórico de devolução de {msgEntradaSaida} com esse código.", TipoToast.Aviso);
                OnPropertyChanged(nameof(FaturamentoEmDevolucao.Historico));
                return false;
            }
            return true;
        }

        private async Task GetFormularioById(long id)
        {
            RespostaAPIModel resposta;
            if (id == 0)
            {
                return;
            }

            try
            {
                FormulariosBtnEnabled = false;
                await SpinnerPrincipal.ShowAsync();
                resposta = await _formulariosService.GetByIdAsync(URIsFaturamentos.GetFormulariosAsync, id, StatusAtivo.Ativos);
            }
            finally
            {
                await SpinnerPrincipal.HideAsync();
                FormulariosBtnEnabled = true;
            }

            if (!resposta.Sucesso)
            {
                string mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um formulário ativo com o Id {id}" :
                    $"Ocorreu um erro ao consultar o formulário com o id {id}. Erro {resposta.MensagemErro}";

                _ = new FormularioModel();
                await ExibirMensagemToast(mensagem, TipoToast.Aviso);
                return;
            }

            FaturamentoEmDevolucao.Formulario = (FormularioModel)resposta.Objeto;
            FaturamentoEmDevolucao.IdFormulario = FaturamentoEmDevolucao.Formulario.Id;
            OnPropertyChanged(nameof(FaturamentoEmDevolucao.Formulario));
        }

        public async void AoClicarOkBtnDlgHistoricos(MouseEventArgs e)
        {
            DlgHistoricosVisible = false;
            if (HistoricoSelecionado == null) return;
            await GetHistoricoById(HistoricoSelecionado.Id);
        }

        public void AoFecharDlgHistoricos(object e)
        {
            DlgHistoricosVisible = false;
        }

        public void AoSelecionarFormulario(RowSelectEventArgs<FormularioModel> e)
        {
            FormularioSelecionado = e.Data;
        }

        public async void AoClicarOkBtnDlgFormularios(MouseEventArgs e)
        {
            DlgFormulariosVisible = false;
            if (FormularioSelecionado == null) return;
            await GetFormularioById(FormularioSelecionado.Id);
            OnPropertyChanged(nameof(Faturamento));
        }

        public void AoFecharDlgFormularios(object e)
        {
            DlgFormulariosVisible = false;
        }

        public async void AoClicarFormulariosBtnAsync(MouseEventArgs e)
        {
            FormulariosBtnEnabled = false;
            PesquisandoFormularios = true;
            var retorno = await _formulariosService.GetAsync(URIsFaturamentos.GetFormulariosAsync, StatusAtivo.Ativos);
            PesquisandoFormularios = false;
            FormulariosBtnEnabled = true;

            if (!retorno.Sucesso)
            {
                await ExibirMensagemToast(retorno.MensagemErro, TipoToast.Aviso);
                return;
            }

            Formularios = (List<FormularioModel>)retorno.Objeto;
            DlgFormulariosVisible = true;
        }

        public async void AoClicarHistoricosBtnAsync(MouseEventArgs e)
        {
            string campos = "id,descricao,entradasaida,gerafinanceiro,geraestoque,ativo";
            string filtros = "Ativo eq true";
            filtros += TipoAcaoFaturamento == TipoAcaoFaturamento.Devolver ? $" and FinNFe eq {(int)FinNFeHistorico.Devolucao}" : $" and TipoNF eq {(int)TipoNFHistorico.Retorno}";
            filtros += Faturamento.EntradaSaida == (int)EntradaSaidaHistorico.Saida ?
                $" and EntradaSaida eq {(int)EntradaSaidaHistorico.Entrada}" : $" and EntradaSaida eq {(int)EntradaSaidaHistorico.Saida}";

            string order = "descricao";

            PesquisandoHistoricos = true;
            HistoricosBtnEnabled = false;
            var retorno = await _historicosService.GetAsync(URIsFaturamentos.GetHistoricosAsync, campos, filtros, order, "");
            HistoricosBtnEnabled = true;
            PesquisandoHistoricos = false;

            if (!retorno.Sucesso)
            {
                await ExibirMensagemToast(retorno.MensagemErro, TipoToast.Erro);
                return;
            }

            Historicos = (List<HistoricoModel>)retorno.Objeto;
            DlgHistoricosVisible = true;
            OnPropertyChanged(nameof(DlgHistoricosVisible));
        }

        public void AoClicarDevolverCompletoBtnAsync(MouseEventArgs e)
        {
            FaturamentoEmDevolucao.Itens.Clear();
            foreach (var item in Faturamento.Itens)
            {
                if (item.QtdLancada - (item.QtdDevolvida + item.QtdRetornada) <= 0) continue;
                var novoItem = item;
                novoItem.QtdLancadaOrigem = item.QtdLancada;
                novoItem.QtdRetornadaAnterior = item.QtdRetornada;
                novoItem.QtdDevolvidaAnterior = item.QtdDevolvida;
                novoItem.QtdLancada = item.QtdLancada - (item.QtdDevolvida + item.QtdRetornada);
                novoItem = AcertarFreteSeguroOutDespAcess(novoItem);
                FaturamentoEmDevolucao.Itens.Add(novoItem);
            }
        }

        private ItemFaturamentoModel AcertarFreteSeguroOutDespAcess(ItemFaturamentoModel itemDevolvido)
        {

            if (itemDevolvido.QtdLancada == 0) return itemDevolvido;

            var itemOriginal = Faturamento.Itens.FirstOrDefault(i => i.IdProduto == itemDevolvido.IdProduto);
            itemDevolvido.VlrFrete = itemOriginal.VlrFrete / itemOriginal.QtdLancada * itemDevolvido.QtdLancada;
            itemDevolvido.VlrSeguro = itemOriginal.VlrSeguro / itemOriginal.QtdLancada * itemDevolvido.QtdLancada;
            itemDevolvido.VlrOutDespAcess = itemOriginal.VlrOutDespAcess / itemOriginal.QtdLancada * itemDevolvido.QtdLancada;
            return itemDevolvido;
        }

        public async Task AoClicarSelecionarBtnGridHistoricos(CommandClickEventArgs<HistoricoModel> e)
        {
            DlgHistoricosVisible = false;
            await GetHistoricoById(e.RowData.Id);
        }

        public void AoSelecionarHistorico(RowSelectEventArgs<HistoricoModel> e)
        {
            HistoricoSelecionado = e.Data;
        }

        private void SetFaturamentoEmDevolucao()
        {
            FaturamentoEmDevolucao = new FaturamentoModel();

            if (Faturamento == null) return;

            FaturamentoEmDevolucao.IdLancamentoOrigem = faturamento.Id;
            FaturamentoEmDevolucao.IdPessoa = faturamento.IdPessoa;
            FaturamentoEmDevolucao.Pessoa = faturamento.Pessoa;
            FaturamentoEmDevolucao.Itens = new ObservableCollection<ItemFaturamentoModel>();
            FaturamentoEmDevolucao.Tipo = (int)TipoFaturamento.NotaFiscal;
            FaturamentoEmDevolucao.FinNFe = TipoAcaoFaturamento == TipoAcaoFaturamento.Devolver ? FinNFeHistorico.Devolucao : FinNFeHistorico.Normal;
            FaturamentoEmDevolucao.TipoNF = TipoAcaoFaturamento == TipoAcaoFaturamento.Devolver ? TipoNFHistorico.Normal : TipoNFHistorico.Retorno;
            FaturamentoEmDevolucao.Serie = faturamento.Serie;

            var itens = FaturamentoEmDevolucao.Itens;
            foreach (var item in faturamento.Itens)
            {
                itens.Add(new ItemFaturamentoModel
                {
                    IdProduto = item.IdProduto,
                    Produto = item.Produto,
                    QtdLancadaOrigem = item.QtdLancada,
                    QtdRetornadaAnterior = item.QtdRetornada,
                    QtdDevolvidaAnterior = item.QtdDevolvida,
                    CST = item.CST
                });
            }
        }

        public async Task<ResultadoModel> PrepararFaturamentoParaDevolver()
        {
            var devolucaoCompleta = DevolucaoCompleta();
            if (!devolucaoCompleta.Sucesso)
            {
                await ExibirMensagemToast((string)devolucaoCompleta.Retorno, TipoToast.Aviso);
                return devolucaoCompleta;
            }
            DlgDevolucaoVisible = false;

            Faturamento.IdLancamentoOrigem = Faturamento.Id; // porque a origem da nota de devolução é a nota que está sendo devolvida
            Faturamento.Id = 0;
            Faturamento.Numero = 0;
            Faturamento.ChaveEletronica = "";
            Faturamento.Formulario = FaturamentoEmDevolucao.Formulario;
            Faturamento.IdFormulario = Faturamento.Formulario.Id;
            Faturamento.Historico = FaturamentoEmDevolucao.Historico;
            Faturamento.IdHistorico = Faturamento.Historico.Id;

            if (!Faturamento.Historico.GeraFinanceiro)
            {
                Faturamento.LimparDadosFinanceiros();
            }

            Faturamento.Tipo = (int)TipoFaturamento.NotaFiscal;
            Faturamento.Itens.Clear();
            Faturamento.VlrFrete = 0;
            Faturamento.VlrOutDespAcess = 0;
            Faturamento.VlrSeguro = 0;
            foreach (var item in FaturamentoEmDevolucao.Itens)
            {
                if (item.QtdLancada == 0) continue;

                Faturamento.VlrOutDespAcess += item.VlrOutDespAcess;
                Faturamento.VlrSeguro += item.VlrSeguro;
                Faturamento.VlrFrete += item.VlrFrete;
                Faturamento.Itens.Add(item);
            }

            
            var resposta = await CalculoFaturamento.Calcular(Faturamento, _faturamentosService); // seta os valores calculados no faturamento
            if (resposta.Sucesso)
            {
                _dadosDevolucaoService.FaturamentoVM.Faturamento = Faturamento;
            }
            return new ResultadoModel { Sucesso = resposta.Sucesso, Retorno = resposta.Sucesso ? Faturamento : resposta.MensagemErro };
                
        }
    }
}
