﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Usuario.Compra;
using GAFNETBlazor.ViewModels.Usuario.Faturamento;
using GAFNETBlazor.ViewModels.Usuario.PesquisaFaturamento;
using GAFNETLibrary.Enums;
using Syncfusion.Blazor.Spinner;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase;

public static class DuplicacaoDocumento
{
    public static void PedirConfirmacao(BaseVM viewModel)
    {
        switch (viewModel)
        {
            case var tipo when tipo is FaturamentoBaseVM:
                ((FaturamentoVM)tipo).DlgConfirmacaoDuplicacaoVisible = true;
                break;

            case var tipo when tipo is PesquisaFaturamentoVM:
                ((PesquisaFaturamentoVM)tipo).DlgConfirmacaoDuplicacaoVisible = true;
                break;
        };
    }

    public static async Task Duplicar(BaseVM viewModel, IEnvioFaturamentoService envioFaturamentoService, FaturamentoModel faturamento)
    {
        SfSpinner spinner = GetSpinner(viewModel);

        try
        {
            await spinner.ShowAsync();
            await ExecutarCodigo(viewModel, envioFaturamentoService, faturamento);
        }
        finally
        {
            await spinner.HideAsync();
        }
    }

    private static async Task ExecutarCodigo(BaseVM viewModel, IEnvioFaturamentoService envioFaturamentoService, FaturamentoModel faturamento)
    {
        var resultado = await envioFaturamentoService.DuplicarAsync(faturamento);

        if (!resultado.Sucesso)
        {
            await viewModel.ExibirMensagemToast((string)resultado.Retorno, TipoToast.Erro);
            return;
        }

        var docDuplicado = (FaturamentoModel)resultado.Retorno;

        if (viewModel is FaturamentoVM vM)
        {
            await vM.PreencherFormularioPadrao();
            vM.AoMudarPropriedade(nameof(Faturamento));
        }
        await viewModel.ExibirMensagemToast($"Duplicação realizada com sucesso com o número { docDuplicado.Numero }, tipo {docDuplicado.DescricaoTipoFaturamento}", TipoToast.Sucesso);
    }

    private static SfSpinner GetSpinner(BaseVM viewModel)
    {
        return viewModel switch
        {
            var tipo when tipo is FaturamentoVM => tipo.SpinnerPrincipal,
            var tipo when tipo is CompraVM => tipo.SpinnerPrincipal,
            var tipo when tipo is PesquisaFaturamentoVM => tipo.SpinnerPrincipal,
            _ => new SfSpinner()
        };
    }
}
