﻿using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Usuario.Compra;
using GAFNETBlazor.ViewModels.Usuario.Faturamento;
using GAFNETBlazor.ViewModels.Usuario.PesquisaFaturamento;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Syncfusion.Blazor.Spinner;
using System;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase;

public static class EnvioEMailNFe
{
    public static async Task<bool> EnviarXMLEPDF(BaseVM vM, INFeService nFeService, string chaveEletronica)
    {
        SfSpinner spinner = GetSpinner(vM);

        try
        {
            await spinner.ShowAsync();
            return await ExecutarCodigo(vM, nFeService, chaveEletronica);
        }
        finally
        {
            await spinner.HideAsync();
        }
    }

    private static async Task<bool> ExecutarCodigo(BaseVM vM, INFeService nFeService, string chaveEletronica)
    {
        ResultadoModel resultDANFE;
        try
        {
            resultDANFE = await nFeService.EnviarEMail(chaveEletronica, TipoAnexoEMailNFe.XMLEPDF);
            if (!resultDANFE.Sucesso)
            {
                await vM.ExibirMensagemToast((string)resultDANFE.Retorno, TipoToast.Erro);
                return false;
            }
        }
        catch (Exception e)
        {
            await vM.ExibirMensagemToast($"Ocorreu um erro ao tentar enviar o E-Mail. Erro: {e.GetBaseException().Message}", TipoToast.Erro);
            return false;
        }
        await vM.ExibirMensagemToast("E-Mail enviado com sucesso.", TipoToast.Sucesso);
        return true;
    }

    private static SfSpinner GetSpinner(BaseVM vM)
    {
        return vM switch
        {
            var tipo when tipo is FaturamentoVM => tipo.SpinnerPrincipal,
            var tipo when tipo is CompraVM => tipo.SpinnerPrincipal,
            var tipo when tipo is PesquisaFaturamentoVM => tipo.SpinnerPrincipal,
            _ => new SfSpinner()
        };
    }

    public static void PedirConfirmacao(BaseVM vM)
    {
        switch(vM)
        {
            case var tipo when tipo is FaturamentoVM: 
                ((FaturamentoVM)tipo).DlgConfirmacaoEnvioEMailNFeVisible = true;
                break;

            case var tipo when tipo is PesquisaFaturamentoVM:
                ((PesquisaFaturamentoVM)tipo).DlgConfirmacaoEnvioEMailNFeVisible = true;
                break;
            
        };
    }

}
