﻿using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Newtonsoft.Json;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase
{
    public static class ExibicaoOcorrenciasFaturamento
    {
        public static async Task Exibir(IFaturamentoBaseVM viewModel, IFaturamentoBaseService faturamentosService)
        {
            viewModel.Ocorrencias = new List<OcorrenciaFaturamentoModel>();
            if (viewModel.SpinnerPrincipal != null)
            {
                await viewModel.SpinnerPrincipal.ShowAsync();
            }

            var retOcorrencias = await faturamentosService.GetOcorrenciasAsync(viewModel.Faturamento);

            if (viewModel.SpinnerPrincipal != null)
            {
                await viewModel.SpinnerPrincipal.HideAsync();
            }

            if (!retOcorrencias.Sucesso)
            {
                await viewModel.ExibirMensagemToast($"Ocorreu um erro ao tentar pesquisar as ocorrências. Erro: {retOcorrencias.MensagemErro}", TipoToast.Erro);
                return;
            }

            viewModel.Ocorrencias = JsonConvert.DeserializeObject<List<OcorrenciaFaturamentoModel>>(retOcorrencias.ConteudoRetorno);
            if (viewModel.Ocorrencias.Count == 0)
            {
                await viewModel.ExibirMensagemToast("Não há ocorrências para este documento.", TipoToast.Informacao);
                return;
            }
            viewModel.DlgOcorrenciasVisible = true;
        }
    }
}
