﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase;

public static class ExibicaoPessoaFaturamento
{
    public static async Task<ResultadoModel> Exibir(IFaturamentoBaseVM viewModel, IPessoasService pessoasService, IFaturamentoBaseService faturamentosService, long id)
    {
        if (id == 0)
        {
            return new ResultadoModel { Sucesso = false, Retorno = "Não foi informado o id da pessoa." };
        }
        var faturamento = viewModel.Faturamento;
        await viewModel.SpinnerPrincipal.ShowAsync();
        viewModel.PesquisandoPessoas = true;
        TipoCadastroPessoa tipoCadastro = (viewModel is IFaturamentoVM) ? TipoCadastroPessoa.Cliente : TipoCadastroPessoa.Fornecedor;
        var resultPesquisa = await PessoaById.Get(viewModel, pessoasService, id, tipoCadastro);
        await viewModel.SpinnerPrincipal.HideAsync();
        viewModel.PesquisandoPessoas = false;

        PessoaModel pessoa;
        if (resultPesquisa.Sucesso)
        {
            pessoa = (PessoaModel)resultPesquisa.Retorno;
        }
        else
        {
            pessoa = new PessoaModel();
        }

        if (!resultPesquisa.Sucesso)
        {
            return new ResultadoModel { Sucesso = false, Retorno = (string)resultPesquisa.Retorno };
        }

        faturamento.Pessoa = pessoa;
        faturamento.IdPessoa = pessoa.Id;

        faturamento.Historico = faturamento.UsaHistorico && faturamento.Pessoa.Historico?.Ativo == true ? faturamento.Pessoa.Historico : new HistoricoModel();
        faturamento.IdHistorico = faturamento.UsaHistorico && faturamento.Historico.Id > 0 ? faturamento.Historico.Id : null;
        faturamento.CondicaoDePagamento = faturamento.Pessoa.CondicaoDePagamento?.Ativo == true ? faturamento.Pessoa.CondicaoDePagamento : new CondicaoDePagamentoModel();
        faturamento.IdCondicaoDePagamento = faturamento.CondicaoDePagamento.Id > 0 ? faturamento.CondicaoDePagamento.Id : null;
        faturamento.TipoDeOperacao = faturamento.Pessoa.TipoDeOperacao?.Ativo == true ? faturamento.Pessoa.TipoDeOperacao : new TipoDeOperacaoModel();
        faturamento.IdTipoDeOperacao = faturamento.TipoDeOperacao.Id > 0 ? faturamento.TipoDeOperacao.Id : null;
        faturamento.CentroDeCusto = faturamento.Pessoa.CentroDeCusto?.Ativo == true ? faturamento.Pessoa.CentroDeCusto : new CentroDeCustoModel();
        faturamento.IdCentroDeCusto = faturamento.CentroDeCusto.Id > 0 ? faturamento.CentroDeCusto.Id : null;
        faturamento.PlanoDeConta = faturamento.UsaPlanoDeConta && faturamento.Pessoa.PlanoDeConta?.Ativo == true ? faturamento.Pessoa.PlanoDeConta : new PlanoDeContaModel();
        faturamento.IdPlanoDeConta = faturamento.UsaPlanoDeConta && faturamento.PlanoDeConta.Id > 0 ? faturamento.PlanoDeConta.Id : null;
        faturamento.Historico = faturamento.Pessoa.Historico?.Ativo == true ? faturamento.Pessoa.Historico : new HistoricoModel();

        RespostaAPIModel respostaCalculo = new() { Sucesso = true };

        if (faturamento.Itens.Count > 0)
        {
            await viewModel.SpinnerPrincipal.ShowAsync();
            respostaCalculo = await CalculoFaturamento.Calcular(faturamento, faturamentosService);
            await viewModel.SpinnerPrincipal.HideAsync();
        }
        viewModel.AoMudarPropriedade(nameof(Faturamento));

        if (!respostaCalculo.Sucesso)
        {
            await viewModel.ExibirMensagemToast(respostaCalculo.MensagemErro, TipoToast.Erro);
        }

        return new ResultadoModel { Sucesso = respostaCalculo.Sucesso, Retorno = pessoa };
    }
}
