﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.DropDowns;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using Syncfusion.Blazor.SplitButtons;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase;

public abstract partial class FaturamentoBaseVM : CadastroBaseVM, IFaturamentoBaseVM
{
    private readonly IEnvioFaturamentoService _envioFaturamentoService;
    private readonly IPessoasService _pessoasService;
    private readonly IFaturamentoBaseService _faturamentosService;
    private readonly ILoginStore _loginStore;
    private readonly IProdutosService _produtosService;
    private readonly ITiposDeOperacaoService _tiposDeOperacaoService;
    private readonly ICondicoesDePagamentoService _condicoesDePagamentoService;
    private readonly ICentrosDeCustoService _centrosDeCustoService;
    private readonly IPesquisaFaturamentoVM _pesquisaFaturamentoVM;
    private readonly IHistoricosService _historicosService;
    private readonly IAcoesFaturamentoService _acoesFaturamentoService;
    private readonly ITiposFaturamentoService _tiposFaturamentoService;
    private readonly INFeService _nFeService;
    private readonly IExibicaoFaturamentoService _exibicaoFaturamentoService;
    private readonly IExibicaoProdutoFaturamentoService _exibicaoProdutoFaturamentoService;
    private readonly IParametrizacoesService _parametrizacoesService;
    IDadosDevolucaoService _dadosDevolucaoService;

    private bool _envioBemSucedido;


    [ObservableProperty]
    private bool dlgLogEnvioFaturamentoVisible;
    partial void OnDlgLogEnvioFaturamentoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgLogEnvioFaturamentoVisible));

    [ObservableProperty]
    private string logEnvioFaturamento;
    partial void OnLogEnvioFaturamentoChanged(string value) => AoMudarPropriedade(nameof(LogEnvioFaturamento));


    [ObservableProperty]
    private bool dlgConfirmacaoEnvioVisible;
    partial void OnDlgConfirmacaoEnvioVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgConfirmacaoEnvioVisible));

    [ObservableProperty]
    private bool dlgDevolucaoVisible;
    partial void OnDlgDevolucaoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgDevolucaoVisible));

    [ObservableProperty]
    private bool dlgPessoasVisible;
    partial void OnDlgPessoasVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgPessoasVisible));


    [ObservableProperty]
    private bool dlgTransportadorasVisible;
    partial void OnDlgTransportadorasVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgTransportadorasVisible));

    [ObservableProperty]
    private bool dlgProdutosVisible;
    partial void OnDlgProdutosVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgProdutosVisible));

    [ObservableProperty]
    private bool dlgHistoricosVisible;
    partial void OnDlgHistoricosVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgHistoricosVisible));

    [ObservableProperty]
    private bool dlgTiposDeOperacaoVisible;
    partial void OnDlgTiposDeOperacaoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgTiposDeOperacaoVisible));

    [ObservableProperty]
    private bool dlgCondicoesDePagamentoVisible;
    partial void OnDlgCondicoesDePagamentoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgCondicoesDePagamentoVisible));

    [ObservableProperty]
    private bool dlgCentrosDeCustoVisible;
    partial void OnDlgCentrosDeCustoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgCentrosDeCustoVisible));

    [ObservableProperty]
    private bool dlgOcorrenciasVisible;
    partial void OnDlgOcorrenciasVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgOcorrenciasVisible));
    [ObservableProperty]
    private bool dlgFreteVisible;
    partial void OnDlgFreteVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgFreteVisible));

    [ObservableProperty]
    private FaturamentoModel faturamento;
    partial void OnFaturamentoChanged(FaturamentoModel value)
    {
        TipoFaturamentoEnabled = value.Id == 0;
        AcoesFaturamento = AcoesPorTipoFaturamento.Acoes(_acoesFaturamentoService, value.Tipo);
        AoMudarPropriedade(nameof(Faturamento));
    }

    [ObservableProperty]
    private List<ListaModel> acoesFaturamento;

    public NavigationManager Navigation { get; set; }
    public ItemFaturamentoModel ItemFaturamento { get; set; } = new ItemFaturamentoModel();
    public ProdutoModel Produto { get; set; } = new ProdutoModel();
    public SfDialog Dlg { get; set; }
    public string MascaraCNPJCPF => string.IsNullOrEmpty(Faturamento.Pessoa.CNPJ) || Faturamento.Pessoa.CNPJ.Length > 11 ? "00.000.000/0000-00" : "000.000.000-00";
    public string CNPJCPFLbl => string.IsNullOrEmpty(Faturamento.Pessoa.CNPJ) || Faturamento.Pessoa.CNPJ.Length > 11 ? "CNPJ" : "CPF";
    public PessoaModel PessoaSelecionada { get; set; } = new PessoaModel();
    public List<PessoaModel> Pessoas { get; set; } = new List<PessoaModel>();
    public List<FaturamentoModel> Faturamentos { get; set; } = new List<FaturamentoModel>();
    public List<ProdutoModel> Produtos { get; set; } = new List<ProdutoModel>();
    public bool TipoFaturamentoEnabled { get; set; } = true;
    public bool ProdutosBtnEnabled { get; set; } = true;
    public bool PessoasBtnEnabled { get; set; } = true;
    public ProdutoModel ProdutoSelecionado { get; set; }
    public bool PesquisandoProdutos { get; set; }
    public bool PesquisandoPessoas { get; set; }
    public bool Calculando { get; set; }
    public bool CentrosDeCustoBtnEnabled { get; set; } = true;
    public CentroDeCustoModel CentroDeCustoSelecionado { get; set; }
    public List<CentroDeCustoModel> CentrosDeCusto { get; set; } = new List<CentroDeCustoModel>();
    public bool PesquisandoCentrosDeCusto { get; set; }
    public bool CondicoesDePagamentoBtnEnabled { get; set; } = true;
    public CondicaoDePagamentoModel CondicaoDePagamentoSelecionada { get; set; }
    public List<CondicaoDePagamentoModel> CondicoesDePagamento { get; set; } = new List<CondicaoDePagamentoModel>();
    public bool PesquisandoCondicoesDePagamento { get; set; }
    public bool TiposDeOperacaoBtnEnabled { get; set; } = true;
    public TipoDeOperacaoModel TipoDeOperacaoSelecionado { get; set; }
    public List<TipoDeOperacaoModel> TiposDeOperacao { get; set; } = new List<TipoDeOperacaoModel>();
    public bool PesquisandoTiposDeOperacao { get; set; }
    public bool TransportadorasBtnEnabled { get; set; } = true;
    public PessoaModel TransportadoraSelecionada { get; set; }
    public List<PessoaModel> Transportadoras { get; set; } = new List<PessoaModel>();
    public bool PesquisandoTransportadoras { get; set; }
    public bool HistoricosBtnEnabled { get; set; } = true;
    public HistoricoModel HistoricoSelecionado { get; set; }
    public ParametrizacaoModel Parametrizacao { get; set; } = new();
    public List<HistoricoModel> Historicos { get; set; } = new List<HistoricoModel>();
    public bool PesquisandoHistoricos { get; set; }
    public string TextoDlgConfirmacao { get; set; } = "Confirma o envio/gravacao do documento?";
    public SfNumericTextBox<long> NumeroInput { get; set; }
    public SfNumericTextBox<long> IdHistoricoInput { get; set; }
    public SfNumericTextBox<long> IdPessoaInput { get; set; }
    public SfNumericTextBox<long> IdCondPgtoInput { get; set; }
    public SfNumericTextBox<long> IdTipoDeOperacaoInput { get; set; }
    public SfNumericTextBox<long> IdCentroDeCustoInput { get; set; }
    public SfNumericTextBox<long> IdTransportadoraInput { get; set; }
    public SfNumericTextBox<decimal> QtdItemInput { get; set; }
    public SfTextBox CodigoProdutoInput { get; set; }
    public List<ListaModel> TiposFaturamento { get; set; }
    public List<ListaModel> TiposFrete { get; }

    public TipoAcaoFaturamento? AcaoFaturamento { get; set; }
    public bool DlgConfirmacaoDuplicacaoVisible { get; set; }
    public bool DlgConfirmacaoCancelamentoVisible { get; set; }
    public bool DlgConfirmacaoInutilizacaoVisible { get; set; }

    public bool NomeProdutoImputEnabled => ItemFaturamento.Produto?.Id == 0;
    public bool NomePessoaImputEnabled => Faturamento.Pessoa?.Id == 0;
    public bool DescricaoHistoricoImputEnabled => Faturamento.Historico?.Id == 0;

    public List<OcorrenciaFaturamentoModel> Ocorrencias { get; set; } = new List<OcorrenciaFaturamentoModel>();

    public string UrlRedirecionamento
    {
        get
        {
            if (!PodeAcessar)
            {
                return "/Login";
            }
            if (!_loginStore.LogadoEmEmpresa)
            {
                return "/SelecionaEmpresaLogada";
            }
            return string.Empty;
        }
    }

    public string CidadePrincipal
    {
        get
        {
            var endereco = Faturamento.Pessoa.EnderecoPrincipal;
            return endereco != null ? endereco.Cidade.Nome + " - " + endereco.Cidade.UF : "";
        }
    }

    public bool PodeAcessar
    {
        get
        {
            var tipoClaim = (TipoFaturamento)Faturamento.Tipo switch
            {
                TipoFaturamento.PedidoDeVenda or TipoFaturamento.OrdemDeServico or TipoFaturamento.Orcamento or TipoFaturamento.NotaFiscal => "Faturamento",
                TipoFaturamento.NotaCompra or TipoFaturamento.PedidoDeCompra => "Compra",
                _ => "Tipo Desconhecido"
            };
            var claimPessoa = _loginStore.Claims.FirstOrDefault(c => c.Type == tipoClaim);
            return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimPessoa != null);
        }
    }

    public FaturamentoBaseVM(IFaturamentoBaseService faturamentosService,
        IPessoasService pessoasService,
        ILoginStore loginStore,
        IProdutosService produtosService,
        IHistoricosService historicosService,
        ICondicoesDePagamentoService condicoesDePagamentoService,
        ICentrosDeCustoService centrosDeCustoService,
        ITiposDeOperacaoService tiposDeOperacaoService,
        ITiposFreteService tiposFreteService,
        IPesquisaFaturamentoVM pesquisaFaturamentoVM,
        ITimeZoneService timeZoneService,
        IAcoesFaturamentoService acoesFaturamentoService,
        ITiposFaturamentoService tiposFaturamentoService,
        IEnvioFaturamentoService envioFaturamentoService,
        INFeService nFeService,
        IDadosDevolucaoService dadosDevolucaoService,
        IExibicaoFaturamentoService exibicaoFaturamentoService,
        IExibicaoProdutoFaturamentoService exibicaoProdutoFaturamentoService,
        IParametrizacoesService parametrizacoesService,
        NavigationManager navigation
        )
    {
        _loginStore = loginStore;
        _faturamentosService = faturamentosService;
        _pessoasService = pessoasService;
        _produtosService = produtosService;
        _historicosService = historicosService;
        _condicoesDePagamentoService = condicoesDePagamentoService;
        _centrosDeCustoService = centrosDeCustoService;
        _tiposDeOperacaoService = tiposDeOperacaoService;
        _acoesFaturamentoService = acoesFaturamentoService;
        _tiposFaturamentoService = tiposFaturamentoService;
        _envioFaturamentoService = envioFaturamentoService;
        _dadosDevolucaoService = dadosDevolucaoService;
        _exibicaoFaturamentoService = exibicaoFaturamentoService;
        _exibicaoProdutoFaturamentoService = exibicaoProdutoFaturamentoService;
        Faturamento = new FaturamentoModel { Vendedores = new ObservableCollection<VendedorFaturamentoModel>() };
        Produto = new ProdutoModel();
        MensagemResultadoEnvio = " ";
        _pesquisaFaturamentoVM = pesquisaFaturamentoVM;
        _pesquisaFaturamentoVM.VMChamador = this;
        _nFeService = nFeService;
        TiposFrete = tiposFreteService.Itens();
        AcoesFaturamento = AcoesPorTipoFaturamento.Acoes(_acoesFaturamentoService, Faturamento.Tipo);
        _parametrizacoesService = parametrizacoesService;
        Navigation = navigation;

        if (this is IFaturamentoVM vM)
        {
            TiposFaturamento = _tiposFaturamentoService.ItensSaida();
        }
        else
        {
            TiposFaturamento = _tiposFaturamentoService.ItensEntrada();
        }
    }

    public async void Enviar()
    {
        if (this is IFaturamentoVM && (Faturamento.Tipo == (int)TipoFaturamento.NotaFiscal || Faturamento.Tipo == (int)TipoFaturamento.NotaCompra) && !Faturamento.IdLancamentoOrigem.HasValue)
        {
            await ExibirMensagemToast($"Não é possível lançar diretamente um documento do tipo: {Faturamento.DescricaoTipoFaturamento}.", TipoToast.Aviso);
            return;
        }

        if (!ValidacaoDadosEnvioFaturamento.Validar(this))
        {
            return;
        }
        DlgConfirmacaoEnvioVisible = true;
    }

    public void AoFecharDlg(object e)
    {
        DlgVisible = false;
    }

    public async Task AoClicarNovoRegistroBtn(MouseEventArgs e)
    {
        await LimparTela();
        if (this is IFaturamentoVM vM)
        {
            await vM.PreencherFormularioPadrao();
            vM.Faturamento.Tipo = (int)TipoFaturamento.PedidoDeVenda;
        }
        await Task.Delay(500);
        await IdPessoaInput.FocusAsync();
    }

    public void AoFecharDlgPessoas(object e)
    {
        DlgPessoasVisible = false;
    }

    public async void AoClicarOKBtnDlgPessoas(MouseEventArgs e)
    {
        DlgPessoasVisible = false;
        if (PessoaSelecionada == null) return;
        var resultPesquisa = await ExibicaoPessoaFaturamento.Exibir(this, _pessoasService, _faturamentosService, PessoaSelecionada.Id);
        if (!resultPesquisa.Sucesso)
        {
            await ExibirMensagemToast((string)resultPesquisa.Retorno, TipoToast.Erro);
            return;
        }

        await Task.Delay(300);
        await SetaFocoAposInformarPessoaFaturamento.Setar(this);

    }

    public async void AoClicarSelecionarBtnGridPessoas(CommandClickEventArgs<PessoaModel> e)
    {
        DlgPessoasVisible = false;
        PessoaSelecionada = e.RowData;
        await ExibicaoPessoaFaturamento.Exibir(this, _pessoasService, _faturamentosService, PessoaSelecionada.Id);
        await Task.Delay(300);
        await SetaFocoAposInformarPessoaFaturamento.Setar(this);
    }

    public async void AoClicarSelecionarBtnGridTransportadoras(CommandClickEventArgs<PessoaModel> e)
    {
        TransportadoraSelecionada = e.RowData;
        await ExibirTransportadora();
    }

    public void AoSelecionarPessoa(RowSelectEventArgs<PessoaModel> e)
    {
        PessoaSelecionada = e.Data;
    }

    public void AoClicarLocalizarBtn(MouseEventArgs e)
    {
        //_pesquisaFaturamentoVM.PesquisaFaturamento.Tipo = Faturamento.Tipo;
        //_pesquisaFaturamentoVM.TiposFaturamento = TiposFaturamento;
        _pesquisaFaturamentoVM.VMChamador = this;
        if (_pesquisaFaturamentoVM.Faturamentos.Count > 0 && _pesquisaFaturamentoVM.Faturamentos.FirstOrDefault().Tipo != Faturamento.Tipo)
        {
            _pesquisaFaturamentoVM.Faturamentos = new List<FaturamentoModel>();
        }
        
        if (this is IFaturamentoVM)
        {
            Navigation.NavigateTo("/Usuario/Movimentacoes/PesquisaFaturamento/false");
            return;
        }

        if (this is ICompraVM)
        {
            Navigation.NavigateTo("/Usuario/Movimentacoes/PesquisaCompra/false");
            return;
        }


    }

    public async void IdPessoaAoSairAsync()
    {
        if (Faturamento.Pessoa.Id == 0)
        {
            Faturamento.Pessoa = new PessoaModel();
            return;
        }
        var resultPesquisa = await ExibicaoPessoaFaturamento.Exibir(this, _pessoasService, _faturamentosService, Faturamento.Pessoa.Id);
        if (Faturamento.IdPessoa == 0)
        {
            Faturamento.Pessoa.Id = 0;
            await IdPessoaInput.FocusAsync();
        }

        if (!resultPesquisa.Sucesso)
        {
            await ExibirMensagemToast((string)resultPesquisa.Retorno, TipoToast.Erro);
            return;
        }

        await Task.Delay(300);
        await SetaFocoAposInformarPessoaFaturamento.Setar(this);
    }

    public async Task AoClicarRetirarItemBtnAsync(MouseEventArgs e)
    {
        await SpinnerPrincipal.ShowAsync();
        RetiraItemFaturamento.Retirar(this);
        var resposta = await CalculoFaturamento.Calcular(Faturamento, _faturamentosService);
        AoMudarPropriedade(nameof(Faturamento));
        await SpinnerPrincipal.HideAsync();
        await CodigoProdutoInput.FocusAsync();
        if (!resposta.Sucesso)
        {
            await ExibirMensagemToast(resposta.MensagemErro, TipoToast.Erro);
        }
    }

    public async Task AoClicarInserirItemBtnAsync(MouseEventArgs e)
    {
        var resultado = await InsereItemFaturamento.Add(this, _faturamentosService);

        if (!resultado.Sucesso)
        {
            await ExibirMensagemToast(resultado.MensagemErro, TipoToast.Erro);
        };
        await CodigoProdutoInput.FocusAsync();
    }

    public void AoFecharDlgProdutos(object e)
    {
        DlgProdutosVisible = false;
    }

    public async Task AoClicarOKBtnDlgProdutos(MouseEventArgs e)
    {
        await ExibirProdutoSelecionadoDlgProdutos();
        await Task.Delay(100);
        await QtdItemInput.FocusAsync();
    }

    private async Task ExibirProdutoSelecionadoDlgProdutos()
    {
        DlgProdutosVisible = false;
        if (ProdutoSelecionado == null) return;
        await _exibicaoProdutoFaturamentoService.ExibirByCodigo(this, ProdutoSelecionado.Codigo);
    }

    public async void AoClicarProdutosBtnAsync(MouseEventArgs e)
    {
        if (ItemFaturamento.Produto.Id > 0)
        {
            ItemFaturamento.Produto = new ProdutoModel();
        }
        string resultado = await LocalizaProduto.LocalizarAsync(this, _produtosService);
        if (resultado != "")
        {
            await ExibirMensagemToast(resultado, TipoToast.Erro);
        }
    }

    public async void AoClicarPessoasBtnAsync(MouseEventArgs e)
    {
        if (Faturamento.Pessoa.Id > 0)
        {
            Faturamento.Pessoa = new PessoaModel();
        }

        var resultado = await PesquisaPessoasFaturamento.Pesquisar(this, _pessoasService);
        if (resultado != "")
        {
            await ExibirMensagemToast(resultado, TipoToast.Erro);
        }
    }

    public async void CodigoProdutoAoSairAsync()
    {
        await _exibicaoProdutoFaturamentoService.ExibirByCodigo(this, ItemFaturamento.Produto.Codigo);
        if (ItemFaturamento.Produto.Id > 0)
        {
            AoMudarPropriedade(nameof(ItemFaturamento));
            await Task.Delay(100);
            await QtdItemInput.FocusAsync();
        }
    }

    public void AoSelecionarProduto(RowSelectEventArgs<ProdutoModel> e)
    {
        ProdutoSelecionado = e.Data;
    }

    public void AoSelecionarItemFaturamento(RowSelectEventArgs<ItemFaturamentoModel> e)
    {
        ItemFaturamento = e.Data;
    }

    public void AoSelecionarCentroDeCusto(RowSelectEventArgs<CentroDeCustoModel> e)
    {
        CentroDeCustoSelecionado = e.Data;
    }

    public async void AoClicarCentrosDeCustoBtnAsync(MouseEventArgs e)
    {
        await LocalizaCentroDeCusto.LocalizarAsync(this, _centrosDeCustoService);
    }

    public async void AoClicarOKBtnDlgCentrosDeCusto(MouseEventArgs e)
    {
        await ExibirCentroDeCusto();
    }

    private async Task ExibirCentroDeCusto()
    {
        DlgCentrosDeCustoVisible = false;
        if (CentroDeCustoSelecionado == null) return;
        Faturamento.CentroDeCusto = await CentroDeCustoById.Get(this, _centrosDeCustoService, CentroDeCustoSelecionado.Id);
        Faturamento.IdCentroDeCusto = Faturamento.CentroDeCusto.Id;
        AoMudarPropriedade(nameof(Faturamento.CentroDeCusto));
        await CodigoProdutoInput.FocusAsync();
    }

    public void AoFecharDlgCentrosDeCusto(object e)
    {
        DlgCentrosDeCustoVisible = false;
    }

    public async void IdCentroDeCustoAoSairAsync()
    {
        Faturamento.CentroDeCusto = await CentroDeCustoById.Get(this, _centrosDeCustoService, Faturamento.CentroDeCusto.Id);

        AoMudarPropriedade(nameof(Faturamento.CentroDeCusto));
    }

    public void AoSelecionarCondicaoDePagamento(RowSelectEventArgs<CondicaoDePagamentoModel> e)
    {
        CondicaoDePagamentoSelecionada = e.Data;
    }

    public async void AoClicarCondicoesDePagamentoBtnAsync(MouseEventArgs e)
    {
        await LocalizaCondicaoDePagamento.LocalizarAsync(this, _condicoesDePagamentoService);
        PesquisandoCondicoesDePagamento = false;
    }

    public async void AoClicarOKBtnDlgCondicoesDePagamento(MouseEventArgs e)
    {
        await ExibirCondicaoDePagamento();
    }

    private async Task ExibirCondicaoDePagamento()
    {
        DlgCondicoesDePagamentoVisible = false;
        if (CondicaoDePagamentoSelecionada == null) return;
        Faturamento.CondicaoDePagamento = await CondicaoDePagamentoById.Get(this, _condicoesDePagamentoService, CondicaoDePagamentoSelecionada.Id);
        Faturamento.IdCondicaoDePagamento = Faturamento.CondicaoDePagamento.Id;
        AoMudarPropriedade(nameof(Faturamento.CondicaoDePagamento));
        await IdCentroDeCustoInput.FocusAsync();
    }

    public void AoFecharDlgCondicoesDePagamento(object e)
    {
        DlgCondicoesDePagamentoVisible = false;
    }

    public async void IdCondicaoDePagamentoAoSairAsync()
    {
        Faturamento.CondicaoDePagamento = await CondicaoDePagamentoById.Get(this, _condicoesDePagamentoService, Faturamento.CondicaoDePagamento.Id);
        AoMudarPropriedade(nameof(Faturamento.CondicaoDePagamento));
    }

    public void AoSelecionarTipoDeOperacao(RowSelectEventArgs<TipoDeOperacaoModel> e)
    {
        TipoDeOperacaoSelecionado = e.Data;
    }

    public async void AoClicarTiposDeOperacaoBtnAsync(MouseEventArgs e)
    {
        await LocalizaTipoDeOperacao.LocalizarAsync(this, _tiposDeOperacaoService);
    }

    public async void AoClicarOKBtnDlgTiposDeOperacao(MouseEventArgs e)
    {
        await ExibirTipoDeOperacao();
    }

    private async Task ExibirTipoDeOperacao()
    {
        DlgTiposDeOperacaoVisible = false;
        if (TipoDeOperacaoSelecionado == null) return;
        Faturamento.TipoDeOperacao = await TipoDeOperacaoById.Get(this, _tiposDeOperacaoService, TipoDeOperacaoSelecionado.Id);
        Faturamento.IdTipoDeOperacao = Faturamento.TipoDeOperacao.Id;
        AoMudarPropriedade(nameof(Faturamento.TipoDeOperacao));
        await IdCondPgtoInput.FocusAsync();
    }

    public void AoFecharDlgTiposDeOperacao(object e)
    {
        DlgTiposDeOperacaoVisible = false;
    }

    public async void IdTipoDeOperacaoAoSairAsync()
    {
        await ExibirTipoDeOperacao();
    }
    public void AoSelecionarTransportadora(RowSelectEventArgs<PessoaModel> e)
    {
        TransportadoraSelecionada = e.Data;
    }

    public async void AoClicarTransportadorasBtnAsync(MouseEventArgs e)
    {
        PesquisandoTransportadoras = true;
        var retorno = await _pessoasService.GetAsync(URIsFaturamentos.GetPessoasAsync, "Id,Nome,CNPJ,Ativo,DtCadastro", "ehtransportadora eq true and ativo eq true", "nome", "");
        Transportadoras = (List<PessoaModel>)retorno.Objeto;
        PesquisandoTransportadoras = false;
        DlgTransportadorasVisible = true;
    }

    public void AoFecharDlgTransportadoras(object e)
    {
        DlgTransportadorasVisible = false;
    }

    public async void AoClicarOKBtnDlgTransportadoras(MouseEventArgs e)
    {
        await ExibirTransportadora();
    }

    private async Task ExibirTransportadora()
    {
        DlgTransportadorasVisible = false;
        if (TransportadoraSelecionada == null) return;
        await ExibicaoTransportadoraFaturamento.Exibir(this, _pessoasService, TransportadoraSelecionada.Id);
    }

    public async void IdTransportadoraAoSairAsync()
    {
        await ExibicaoTransportadoraFaturamento.Exibir(this, _pessoasService, Faturamento.Transportadora.Id);
    }

    public async Task LimparTela()
    {
        LimpaTelaFaturamento.Limpar(this);
        _envioBemSucedido = false; // seta sempre como false
        await Task.Delay(100);
    }

    public async void VlrFreteAoSairAsync()
    {
        if (Faturamento.Itens.Count == 0) return;
        
        await SpinnerPrincipal.ShowAsync();
        var resposta = await CalculoFaturamento.Calcular(Faturamento, _faturamentosService);
        await SpinnerPrincipal.HideAsync();
        AoMudarPropriedade(nameof(Faturamento.Valor));
        if (!resposta.Sucesso)
        {
            await ExibirMensagemToast(resposta.MensagemErro, TipoToast.Erro);
        }
    }

    public async void VlrSeguroAoSairAsync()
    {
        if (Faturamento.Itens.Count == 0) return;
        await SpinnerPrincipal.ShowAsync();
        var resposta = await CalculoFaturamento.Calcular(Faturamento, _faturamentosService);
        await SpinnerPrincipal.HideAsync();
        AoMudarPropriedade(nameof(Faturamento.Valor));
        if (!resposta.Sucesso)
        {
            await ExibirMensagemToast(resposta.MensagemErro, TipoToast.Erro);
        }
    }

    public async void VlrOutDespAcessAoSairAsync()
    {
        if (Faturamento.Itens.Count > 0)
        {
            await SpinnerPrincipal.ShowAsync();
            var resposta = await CalculoFaturamento.Calcular(Faturamento, _faturamentosService);
            await SpinnerPrincipal.HideAsync();
            AoMudarPropriedade(nameof(Faturamento.Valor));
            if (!resposta.Sucesso)
            {
                await ExibirMensagemToast(resposta.MensagemErro, TipoToast.Erro);
            }
        }
    }

    public async Task AoClicarSimBtnDlgConfirmacaoEnvio(MouseEventArgs e)
    {
        _envioBemSucedido = false;

        try
        {
            DlgConfirmacaoEnvioVisible = false;
            Enviando = true;
            await SpinnerPrincipal.ShowAsync();
            DlgLogEnvioFaturamentoVisible = true;

            var resultEnviarFaturamento = await _envioFaturamentoService.CriarAlterarAsync(this);

            if (!resultEnviarFaturamento.Sucesso)
            {
                await ExibirMensagemToast((string)resultEnviarFaturamento.Retorno, TipoToast.Erro);
                return;
            }
            _envioBemSucedido = true;

        }
        finally
        {
            await SpinnerPrincipal.HideAsync();
            Enviando = false;
            await Task.Delay(100);
            AoMudarPropriedade(nameof(Faturamento));
        }
    }

    public void AoClicarNaoBtnDlgConfirmacaoEnvio(MouseEventArgs e)
    {
        DlgConfirmacaoEnvioVisible = false;
    }

    public async Task AoClicarSelecionarBtnGridProdutos(CommandClickEventArgs<ProdutoModel> e)
    {
        ProdutoSelecionado = e.RowData;
        await ExibirProdutoSelecionadoDlgProdutos();
        await Task.Delay(100);
        await QtdItemInput.FocusAsync();
    }

    public async void AoClicarHistoricosBtnAsync(MouseEventArgs e)
    {
        if (Faturamento.Historico.Id > 0)
        {
            Faturamento.Historico = new HistoricoModel();
        }
        await LocalizaHistorico.LocalizarAsync(this, _historicosService);
    }

    public async void AoClicarOKBtnDlgHistoricos(MouseEventArgs e)
    {
        DlgHistoricosVisible = false;
        if (HistoricoSelecionado == null) return;
        await ExibicaoHistoricoFaturamento.Exibir(this, _historicosService, _faturamentosService, HistoricoSelecionado.Id);
    }

    public void AoFecharDlgHistoricos(object e)
    {
        DlgHistoricosVisible = false;
    }

    public async void IdHistoricoAoSairAsync()
    {
        await ExibicaoHistoricoFaturamento.Exibir(this, _historicosService, _faturamentosService, Faturamento.Historico.Id);
    }

    public void AoSelecionarHistorico(RowSelectEventArgs<HistoricoModel> e)
    {
        HistoricoSelecionado = e.Data;
    }

    public async Task<bool> AoSelecionarAcao(MenuEventArgs e)
    {
        bool sucesso = true;
        AcaoFaturamento = (TipoAcaoFaturamento)Convert.ToUInt32(e.Item.Id);
        var resultValidacao = ValidacaoDocumentoPelaAcao.Validar(Faturamento, AcaoFaturamento.Value);
        if (!resultValidacao.Sucesso)
        {
            await ExibirMensagemToast((string)resultValidacao.Retorno, TipoToast.Aviso);
            return false;
        }
        switch (AcaoFaturamento)
        {
            case TipoAcaoFaturamento.Faturar:
                sucesso = await AlteracaoFaturamentoParaNotaFiscal.Alterar((IFaturamentoVM)this, _faturamentosService);
                Enviar();
                break;
            case TipoAcaoFaturamento.Devolver:
                await DevolverRetornarNota(TipoAcaoFaturamento.Devolver);
                break;
            case TipoAcaoFaturamento.Retornar:
                await DevolverRetornarNota(TipoAcaoFaturamento.Retornar);
                break;
            case TipoAcaoFaturamento.Cancelar:
                DlgConfirmacaoCancelamentoVisible = true;
                break;
            case TipoAcaoFaturamento.Inutilizar:
                DlgConfirmacaoInutilizacaoVisible = true;
                break;
            case TipoAcaoFaturamento.Excluir:
                break;
            case TipoAcaoFaturamento.Duplicar:
                DuplicacaoDocumento.PedirConfirmacao(this);
                break;
            case TipoAcaoFaturamento.ImprimirDANFE:
                await ImpressaoDANFE.Imprimir(this, _nFeService, faturamento.ChaveEletronica);
                break;
            case TipoAcaoFaturamento.EnviarEMail:
                EnvioEMailNFe.PedirConfirmacao(this);
                break;
            case TipoAcaoFaturamento.Estornar:
                ((IFaturamentoVM)this).DlgMotivoEstornoVisible = true;
                break;
            case TipoAcaoFaturamento.Informacao:
                await ExibicaoOcorrenciasFaturamento.Exibir(this, _faturamentosService);
                break;
            default:
                break;
        }
        return sucesso;
    }

    public async Task DevolverRetornarNota(TipoAcaoFaturamento tipoAcaoFaturamento)
    {
        string acao = tipoAcaoFaturamento == TipoAcaoFaturamento.Devolver ? "devolver" : "retornar";
        if (Faturamento.TudoDevolvidoRetornado)
        {
            await ExibirMensagemToast($"Essa nota não possui saldo para {acao}", TipoToast.Aviso);
            return;
        }
        _dadosDevolucaoService.FaturamentoADevolver = Faturamento;
        _dadosDevolucaoService.TipoAcaoFaturamento = tipoAcaoFaturamento;
        DlgDevolucaoVisible = true;
    }

    public abstract void NumeroAoSairAsync();


    public void AoMudarTipoDeFaturamento(ChangeEventArgs<int, ListaModel> args)
    {
        AcoesFaturamento = AcoesPorTipoFaturamento.Acoes(_acoesFaturamentoService, Faturamento.Tipo);
    }

    public async Task AoClicarSimBtnDlgConfirmacaoDuplicacao(MouseEventArgs e)
    {
        DlgConfirmacaoDuplicacaoVisible = false;
        await DuplicacaoDocumento.Duplicar(this, _envioFaturamentoService, Faturamento);
    }

    public void AoClicarNaoBtnDlgConfirmacaoDuplicacao(MouseEventArgs e)
    {
        DlgConfirmacaoDuplicacaoVisible = false;
    }

    public async void AoClicarSelecionarBtnGridHistoricos(CommandClickEventArgs<HistoricoModel> e)
    {
        DlgHistoricosVisible = false;
        await ExibicaoHistoricoFaturamento.Exibir(this, _historicosService, _faturamentosService, e.RowData.Id);
    }

    public async void AoClicarSelecionarBtnGridCentrosDeCusto(CommandClickEventArgs<CentroDeCustoModel> e)
    {
        Faturamento.CentroDeCusto = e.RowData;
        await ExibirCentroDeCusto();
        await CodigoProdutoInput.FocusAsync();
    }

    public async void AoClicarSelecionarBtnGridCondPgto(CommandClickEventArgs<CondicaoDePagamentoModel> e)
    {
        Faturamento.CondicaoDePagamento = e.RowData;
        await ExibirCondicaoDePagamento();
        await IdCentroDeCustoInput.FocusAsync();
    }

    public async void AoClicarSelecionarBtnGridTipoDeOperacao(CommandClickEventArgs<TipoDeOperacaoModel> e)
    {
        Faturamento.TipoDeOperacao = e.RowData;
        await ExibirTipoDeOperacao();
        await IdCondPgtoInput.FocusAsync();
    }

    public async Task AoClicarCommandBtnGridItemFaturamento(CommandClickEventArgs<ItemFaturamentoModel> e)
    {
        if (e.CommandColumn.Title == "Excluir")
        {
            Faturamento.Itens.Remove(e.RowData);
            await SpinnerPrincipal.ShowAsync();
            var resposta = await CalculoFaturamento.Calcular(Faturamento, _faturamentosService);
            AoMudarPropriedade(nameof(Faturamento));
            await SpinnerPrincipal.HideAsync();
            if (!resposta.Sucesso)
            {
                await ExibirMensagemToast(resposta.MensagemErro, TipoToast.Erro);
            }
        }
    }

    public async Task AoPressionarTeclaProdutosGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && ProdutoSelecionado?.Id > 0)
        {
            await ExibirProdutoSelecionadoDlgProdutos();
            await Task.Delay(100);
            await QtdItemInput.FocusAsync();
        }
    }

    public async Task AoPressionarTeclaPessoasGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && PessoaSelecionada?.Id > 0)
        {
            DlgPessoasVisible = false;
            await ExibicaoPessoaFaturamento.Exibir(this, _pessoasService, _faturamentosService, PessoaSelecionada.Id);
            await CodigoProdutoInput.FocusAsync();
        }
    }

    public async Task AoPressionarTeclaTiposDeOperacaoGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && TipoDeOperacaoSelecionado?.Id > 0)
        {
            await ExibirTipoDeOperacao();
        }
    }

    public async Task AoPressionarTeclaTransportadorasGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && TransportadoraSelecionada?.Id > 0)
        {
            await ExibirTransportadora();
        }
    }

    public async Task AoPressionarTeclaCondicoesDePagamentoGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && CondicaoDePagamentoSelecionada?.Id > 0)
        {
            await ExibirCondicaoDePagamento();
        }
    }

    public async Task AoPressionarTeclaCentrosDeCustoGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && CentroDeCustoSelecionado?.Id > 0)
        {
            await ExibirCentroDeCusto();
        }
    }

    public async Task AoPressionarTeclaHistoricosGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && HistoricoSelecionado?.Id > 0)
        {
            DlgHistoricosVisible = false;
            await ExibicaoHistoricoFaturamento.Exibir(this, _historicosService, _faturamentosService, HistoricoSelecionado.Id);
            await CodigoProdutoInput.FocusAsync();
        }
    }

    public async Task NomeProdutoAoSairAsync()
    {
        string resultado = "";
        if (ItemFaturamento.Produto.Id == 0 && !string.IsNullOrEmpty(ItemFaturamento.Produto.Nome))
        {
            resultado = await LocalizaProduto.LocalizarAsync(this, _produtosService);
        }
        if (resultado != "")
        {
            await ExibirMensagemToast(resultado, TipoToast.Erro);
        }
        if (ItemFaturamento.Produto.Id > 0)
        {
            ItemFaturamento.IdProduto = ItemFaturamento.Produto.Id;
            ItemFaturamento.VlrUnit = ItemFaturamento.Produto.VlrVenda;
        }
    }

    public async Task NomePessoaAoSairAsync()
    {
        string resultado = "";
        if (Faturamento.Pessoa.Id == 0 && !string.IsNullOrEmpty(Faturamento.Pessoa.Nome))
        {
            resultado = await PesquisaPessoasFaturamento.Pesquisar(this, _pessoasService);
        }
        if (resultado != "")
        {
            await ExibirMensagemToast(resultado, TipoToast.Erro);
        }
        if (Faturamento.Pessoa.Id > 0)
        {
            Faturamento.IdPessoa = Faturamento.Pessoa.Id;
            await ExibicaoPessoaFaturamento.Exibir(this, _pessoasService, _faturamentosService, Faturamento.Pessoa.Id);
        }
    }

    public async Task DescricaoHistoricoAoSairAsync()
    {
        string resultado = "";
        if (Faturamento.Historico.Id == 0 && !string.IsNullOrEmpty(Faturamento.Historico.Descricao))
        {
            resultado = await LocalizaHistorico.LocalizarAsync(this, _historicosService);
        }
        if (resultado != "")
        {
            await ExibirMensagemToast(resultado, TipoToast.Erro);
        }
        if (Faturamento.Historico.Id > 0)
        {
            Faturamento.IdHistorico = Faturamento.Historico.Id;
        }
    }

    public void AoFecharDlgOcorrencias(object e)
    {
        DlgOcorrenciasVisible = false;
    }

    public void AoClicarOKBtnDlgOcorrencias(MouseEventArgs e)
    {
        DlgOcorrenciasVisible = false;
    }

    public async void AoFecharDlgLogEnvioFaturamento(object e)
    {
        DlgLogEnvioFaturamentoVisible = false;
        LogEnvioFaturamento = "";
        if (_envioBemSucedido)
        {
            await LimparTela();
        }
    }

    void ICadastroBaseVM.LimparTela()
    {
        throw new NotImplementedException();
    }

    public void AoClicarOKBtnDlgFrete(MouseEventArgs e)
    {
        DlgFreteVisible = false;

        if (this is IFaturamentoVM vM && Parametrizacao.UsaVendedor) // se usar vendedor, chama a caixa de diálogo dos vendedores
        {
            vM.DlgInsercaoVendedoresVisible = true;
            return;
        }
        
    }

    public async Task GetParametrizacoes()
    {
        var param = await _parametrizacoesService.GetAsync(URIsParametrizacoes.GetAsync);
        if (!param.Sucesso)
        {
            await ExibirMensagemToast(param.MensagemErro, TipoToast.Erro);
            return;
        }
        Parametrizacao = (ParametrizacaoModel)param.Objeto;
    }

}
