﻿using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase
{
    public static class FaturamentoById
    {
        public static async Task<FaturamentoModel> Get(IFaturamentoBaseVM viewModel, IFaturamentoBaseService faturamentosService, long id, TipoFaturamento tipo)
        {
            var faturamento = new FaturamentoModel();
            if (id == 0) return faturamento;

            RespostaAPIModel resposta;

            try
            {
                viewModel.Localizando = true;
                if (viewModel.SpinnerPrincipal != null)
                {
                    await viewModel.SpinnerPrincipal.ShowAsync();
                }
                resposta = await faturamentosService.GetByIdAndTipoAsync(id, tipo);

                if (!resposta.Sucesso)
                {
                    viewModel.Mensagem = resposta.MensagemErro;
                    viewModel.TipoToast = TipoToast.Aviso;
                    viewModel.AoMudarPropriedade("TipoToast");
                    await viewModel.MeuToast.Show();
                }
                else
                {
                    faturamento = (FaturamentoModel)resposta.Objeto;
                }
            }
            finally
            {
                if (viewModel.SpinnerPrincipal != null)
                {
                    await viewModel.SpinnerPrincipal.HideAsync();
                }
                    
                viewModel.Localizando = false;
            }

            // Se estiver chamando na nota fiscal para devolver a compra
            if (viewModel is IFaturamentoVM && faturamento.Tipo == (int)TipoFaturamento.NotaCompra)
            {
                faturamento.Formulario = new FormularioModel();
            }

            return faturamento;
        }
    }
}
