﻿using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Usuario.Compra;
using GAFNETBlazor.ViewModels.Usuario.Faturamento;
using GAFNETBlazor.ViewModels.Usuario.PesquisaFaturamento;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Syncfusion.Blazor.Spinner;
using System;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase;

public static class ImpressaoDANFE
{
    public static async Task Imprimir(BaseVM vM, INFeService nFeService, string chaveEletronica)
    {
        SfSpinner spinner = GetSpinner(vM);

        try
        {
            await spinner.ShowAsync();
            await ExecutarCodigo(vM, nFeService, chaveEletronica);
        }
        finally
        {
            await spinner.HideAsync();
        }
    }

    private static async Task ExecutarCodigo(BaseVM vM, INFeService nFeService, string chaveEletronica)
    {
        ResultadoModel resultDANFE;
        try
        {
            resultDANFE = await nFeService.ImprimirDANFE(chaveEletronica);
            if (!resultDANFE.Sucesso)
            {
                await vM.ExibirMensagemToast($"Erro: {(string)resultDANFE.Retorno}", TipoToast.Erro);
            }
        }
        catch (Exception e)
        {
            await vM.ExibirMensagemToast($"Ocorreu um erro ao tentar gerar a DANFE. Erro: {e.GetBaseException().Message}", TipoToast.Erro);
        }
    }

    private static SfSpinner GetSpinner(BaseVM vM)
    {
        return vM switch
        {
            var tipo when tipo is FaturamentoVM => tipo.SpinnerPrincipal,
            var tipo when tipo is CompraVM => tipo.SpinnerPrincipal,
            var tipo when tipo is PesquisaFaturamentoVM => tipo.SpinnerPrincipal,
            _ => new SfSpinner()
        };
    }
}
