﻿using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using System.Collections.ObjectModel;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase;

public static class InsereItemFaturamento
{
    public static async Task<RespostaAPIModel> Add(IFaturamentoBaseVM viewModel, IFaturamentoBaseService faturamentosService)
    {
        var resposta = await ValidarItem(viewModel);

        if (!resposta.Sucesso)
        {
            await viewModel.ExibirMensagemToast(resposta.MensagemErro, TipoToast.Erro);
            return resposta;
        } 

        viewModel.Faturamento.Itens = viewModel.Faturamento.Itens ?? new ObservableCollection<ItemFaturamentoModel>();
        RemoverItemSeJaLancado(viewModel);
        viewModel.ItemFaturamento.QtdLiberada = viewModel.ItemFaturamento.QtdLancada;
        long idProduto = viewModel.ItemFaturamento.IdProduto;
        viewModel.Faturamento.Itens.Insert(0, viewModel.ItemFaturamento);
        viewModel.ItemFaturamento = new ItemFaturamentoModel();

        await viewModel.SpinnerPrincipal.ShowAsync();
        var respCalc = await CalculoFaturamento.Calcular(viewModel.Faturamento, faturamentosService);
        await viewModel.SpinnerPrincipal.HideAsync();

        if (!respCalc.Sucesso)
        {
            DesfazerInsercaoItemMalSucedida(viewModel.Faturamento, idProduto);
            //await viewModel.ExibirMensagemToast(resposta.MensagemErro, TipoToast.Erro);
            return respCalc;
        }
        //await Task.Delay(100);
        viewModel.AoMudarPropriedade(nameof(Faturamento));
        await viewModel.CodigoProdutoInput.FocusAsync();

        return new RespostaAPIModel { Sucesso = true };
    }

    private static void RemoverItemSeJaLancado(IFaturamentoBaseVM viewModel)
    {
        var itemFaturamento = viewModel.Faturamento.Itens.FirstOrDefault(e => e.Produto.Id == viewModel.ItemFaturamento.Produto.Id);
        if (itemFaturamento != null)
        {
            viewModel.Faturamento.Itens.Remove(itemFaturamento);
        }
    }

    private static async Task<RespostaAPIModel> ValidarItem(IFaturamentoBaseVM viewModel)
    {
        var itemFaturamento = viewModel.ItemFaturamento;
        var resultFormulario = await ValidarFormulario(viewModel);
        if (!resultFormulario.Sucesso) return resultFormulario;

        var resultHistorico = await ValidarHistorico(viewModel);
        if (!resultHistorico.Sucesso) return resultHistorico;

        if (viewModel.Faturamento.Pessoa.Id == 0)
        {
            await viewModel.IdPessoaInput.FocusAsync();
            return new RespostaAPIModel { Sucesso = false, MensagemErro = "É necessário informar a pessoa" };
        }

        if (itemFaturamento.Produto.Id == 0)
        {
            await viewModel.CodigoProdutoInput.FocusAsync();
            return new RespostaAPIModel { Sucesso = false, MensagemErro = "É necessário informar o produto" };
        }

        if (itemFaturamento.QtdLancada <= 0)
        {
            return new RespostaAPIModel { Sucesso = false, MensagemErro = "É necessário informar a quantidade" };
        }

        if (itemFaturamento.VlrUnit <= 0)
        {
            return new RespostaAPIModel { Sucesso = false, MensagemErro = "É necessário informar um valor unitário válido." };
        }

        if (itemFaturamento.VlrDescUnit >= viewModel.ItemFaturamento.VlrUnit)
        {
            return new RespostaAPIModel { Sucesso = false, MensagemErro = "Não é possível dar um desconto maior ou igual ao valor unitário." };
        }

        if (itemFaturamento.VlrDescUnit > 0 && viewModel.ItemFaturamento.PercDesc > 0)
        {
            return new RespostaAPIModel { Sucesso = false, MensagemErro = "Não é possível dar um desconto em valor e em percentual em um mesmo item." };
        }

        if (itemFaturamento.PercDesc >= 100)
        {
            return new RespostaAPIModel { Sucesso = false, MensagemErro = "Não é possível dar um desconto maior ou igual a 100%." };
        }

        return new RespostaAPIModel { Sucesso = true };
    }

    private static async Task<RespostaAPIModel> ValidarFormulario(IFaturamentoBaseVM viewModel)
    {
        if (!viewModel.Faturamento.UsaFormulario) return new RespostaAPIModel { Sucesso = true };

        if (viewModel.Faturamento.Formulario.Id == 0)
        {
            await ((IFaturamentoVM)viewModel).IdFormularioInput.FocusAsync();
            return new RespostaAPIModel { Sucesso = false, MensagemErro = "É necessário informar o formulário" };
        }
        return new RespostaAPIModel { Sucesso = true };
    }

    private static async Task<RespostaAPIModel> ValidarHistorico(IFaturamentoBaseVM viewModel)
    {
        if (!viewModel.Faturamento.UsaHistorico) return new RespostaAPIModel { Sucesso = true };

        if (viewModel.Faturamento.Historico.Id == 0)
        {
            await viewModel.IdHistoricoInput.FocusAsync();
            return new RespostaAPIModel { Sucesso = false, MensagemErro = "É necessário informar o histórico" };
        }

        return new RespostaAPIModel { Sucesso = true };
    }

    private static void DesfazerInsercaoItemMalSucedida(FaturamentoModel faturamento, long idProduto)
    {
        var item = faturamento.Itens.FirstOrDefault(p => p.Produto.Id == idProduto);

        if (item == null) return;

        if (faturamento.Itens.Count > 1)
        {
            faturamento.Itens.Remove(item);
            return;
        }

        faturamento.Itens = new ObservableCollection<ItemFaturamentoModel>();
    }

}
