﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Collections.Generic;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase;

public static class LocalizaHistorico
{
            
    private const string campos = "*&$Expand=HistoricoEstorno($Select=id,descricao,entradasaida)";
    public static async Task<string> LocalizarAsync(IFaturamentoBaseVM viewModel, IHistoricosService service)
    {
        if (viewModel == null) return "";
        HistoricoModel historicoFiltro = new() { Descricao = viewModel.Faturamento.Historico.Descricao };
        viewModel.HistoricosBtnEnabled = false;
        viewModel.PesquisandoHistoricos = true;

        ResultadoModel retorno;

        if (string.IsNullOrEmpty(historicoFiltro.Descricao))
        {
            retorno = await LocalizarTodos(viewModel, service);
        }
        else
        {
            retorno = await LocalizarComFiltro(viewModel, service, historicoFiltro);
        }


        viewModel.PesquisandoHistoricos = false;
        viewModel.HistoricosBtnEnabled = true;

        viewModel.AoMudarPropriedade(nameof(viewModel.Faturamento.Historico));
        
        if (!retorno.Sucesso)
        {
            return (string)retorno.Retorno;
        }

        return "";
    }

    public static async Task<ResultadoModel> LocalizarTodos(IFaturamentoBaseVM viewModel, IHistoricosService service)
    {
        string filtro = "";
        if (viewModel is IFaturamentoVM)
        {
            filtro = $"Ativo eq true and EntradaSaida eq 1 and FinNfe ne {(int)FinNFeHistorico.Devolucao} ";
        }
        if (viewModel is ICompraVM)
        {
            filtro = "ativo eq true and entradasaida eq 0";
        }

        var retorno = await service?.GetAsync(URIsFaturamentos.GetHistoricosAsync, campos, filtro, "", "");
        viewModel.PesquisandoHistoricos = false;
        viewModel.HistoricosBtnEnabled = true;

        if (!retorno.Sucesso)
        {
            return new ResultadoModel { Sucesso = false, Retorno = retorno.MensagemErro };
        }
        viewModel.Historicos = (List<HistoricoModel>)retorno.Objeto;
        viewModel.DlgHistoricosVisible = true;
        return new ResultadoModel { Sucesso = true, Retorno = (List<HistoricoModel>)retorno.Objeto };

    }

    public static async Task<ResultadoModel> LocalizarComFiltro(IFaturamentoBaseVM viewModel, IHistoricosService service, HistoricoModel historicoFiltro)
    {
        var retorno = await service?.GetAsync(URIsFaturamentos.GetHistoricosAsync, campos, $"ativo eq true and startswith(descricao, '{historicoFiltro.Descricao}')", "", "");

        if (!retorno.Sucesso)
        {
            return new ResultadoModel { Sucesso = false, Retorno = retorno.MensagemErro };
        }

        viewModel.Historicos = (List<HistoricoModel>)retorno.Objeto;
        if (viewModel.Historicos.Count == 1)
        {
            viewModel.Faturamento.Historico = viewModel.Historicos[0];
        }
        else if (viewModel.Historicos.Count > 1)
        {
            viewModel.DlgHistoricosVisible = true;
        }
        else // caso não tenha encontrado nenhum, faz a pesquisa completa
        {
            viewModel.Faturamento.Historico.Descricao = "";
            await LocalizarTodos(viewModel, service);
        }
        return new ResultadoModel { Sucesso = true, Retorno = (List<HistoricoModel>)retorno.Objeto };
    }
}
