﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Collections.Generic;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase
{
    public static class LocalizaProduto
    {
        public static async Task<string> LocalizarAsync(IFaturamentoBaseVM viewModel, IProdutosService service)
        {
            if (viewModel == null) return "";
            ProdutoModel produtoFiltro = new() { Nome = viewModel.ItemFaturamento.Produto.Nome };
            viewModel.ItemFaturamento.Produto = new(); // para apagar qualquer dado informado no item
            viewModel.ProdutosBtnEnabled = false;
            viewModel.PesquisandoProdutos = true;

            ResultadoModel retorno;

            if (string.IsNullOrEmpty(produtoFiltro.Nome))
            {
                retorno = await LocalizarTodos(viewModel, service);
            }
            else
            {
                retorno = await LocalizarComFiltro(viewModel, service, produtoFiltro);
            }

            viewModel.PesquisandoProdutos = false;

            viewModel.ProdutosBtnEnabled = true;

            viewModel.AoMudarPropriedade(nameof(viewModel.ItemFaturamento));

            if (!retorno.Sucesso)
            {
                return (string)retorno.Retorno;
            }
            return "";
        }

        public static async Task<ResultadoModel> LocalizarTodos(IFaturamentoBaseVM viewModel, IProdutosService service)
        {
            var retorno = await service?.GetAsync(URIsFaturamentos.GetProdutosAsync, StatusAtivo.Ativos);

            if (!retorno.Sucesso)
            {
                return new ResultadoModel { Sucesso = false, Retorno = retorno.MensagemErro };
            }
            viewModel.Produtos = (List<ProdutoModel>)retorno.Objeto;
            viewModel.DlgProdutosVisible = true;
            return new ResultadoModel { Sucesso = true, Retorno = (List<ProdutoModel>)retorno.Objeto };
        }

        public static async Task<ResultadoModel> LocalizarComFiltro(IFaturamentoBaseVM viewModel, IProdutosService service, ProdutoModel produtoFiltro)
        {
            var retorno = await service?.GetAsync(URIsFaturamentos.GetProdutosAsync, "id,codigo,nome,vlrvenda", $"ativo eq true and startswith(nome, '{produtoFiltro.Nome}')", "", "");

            if (!retorno.Sucesso)
            {
                return new ResultadoModel { Sucesso = false, Retorno = retorno.MensagemErro };
            }

            viewModel.Produtos = (List<ProdutoModel>)retorno.Objeto;
            if (viewModel.Produtos.Count == 1)
            {
                viewModel.ItemFaturamento.Produto = viewModel.Produtos[0];
                await viewModel.QtdItemInput.FocusAsync();
            }
            else if(viewModel.Produtos.Count > 1)
            {
                viewModel.DlgProdutosVisible = true;
            }
            else // caso não tenha encontrado nenhum, faz a pesquisa completa
            {
                viewModel.ItemFaturamento.Produto.Nome = "";
                await LocalizarTodos(viewModel, service);
            }
            return new ResultadoModel { Sucesso = true, Retorno = (List<ProdutoModel>)retorno.Objeto };
        }

    }
}
