﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase
{
    public static class PessoaById
    {
        public static async Task<ResultadoModel> Get(IFaturamentoBaseVM viewModel, IPessoasService pessoasService, long id, TipoCadastroPessoa tipoCadastro)
        {
            PessoaModel pessoa;
            if (id == 0) return new ResultadoModel { Sucesso = false, Retorno = "Não informado o id da pessoa." };

            
            switch (tipoCadastro)
            {
                case TipoCadastroPessoa.Cliente:
                    viewModel.PesquisandoPessoas = true;
                    break;
                case TipoCadastroPessoa.Transportadora:
                    viewModel.PesquisandoTransportadoras = true;
                    break;
                default:
                    break;
            }

            string filtro = "ativo eq true" + tipoCadastro switch {
                TipoCadastroPessoa.Cliente => " and ehcliente eq true",
                TipoCadastroPessoa.Fornecedor => " and ehfornecedor eq true",
                TipoCadastroPessoa.Transportadora => " and ehtransportadora eq true",
                TipoCadastroPessoa.Vendedor => " and ehvendedor eq true",
                _=> ""
            };

            string campos = "id,nome,cnpj&$expand=enderecoprincipal($select=ehprincipal;$expand=cidade($select=id,nome,uf)),historico($select=id,descricao,gerafinanceiro,ativo)";
            campos += ",centrodecusto($select=id,nome,ativo),condicaodepagamento($select=id,descricao,descricaoparcelas,ativo),tipodeoperacao($select=id,descricao,ativo), planodeconta($select=id,descricao,ativo)";
            var resposta = await pessoasService.GetByIdAsync(URIsFaturamentos.GetPessoasAsync, id, campos, filtro);

            if (!resposta.Sucesso)
            {
                return new ResultadoModel { Sucesso = false, Retorno = resposta.MensagemErro };
            }
                
            pessoa = (PessoaModel)resposta.Objeto;
            
            return new ResultadoModel { Sucesso = true, Retorno = pessoa };
        }
    }
}
