﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.FaturamentoBase
{
    public static class ProdutoByCodigo
    {
        public static async Task<ProdutoModel> Get(IFaturamentoBaseVM viewModel, IProdutosService produtosService, string codigo)
        {
            if (string.IsNullOrEmpty(codigo)) return new ProdutoModel();

            await viewModel.SpinnerPrincipal.ShowAsync();
            viewModel.ProdutosBtnEnabled = false;

            var resposta = await produtosService?.GetByCodigoAsync(URIsFaturamentos.GetProdutosAsync, codigo, StatusAtivo.Ativos);
            await viewModel.SpinnerPrincipal.HideAsync();
            viewModel.ProdutosBtnEnabled = true;

            viewModel.Mensagem = resposta.MensagemErro;

            if (resposta.Sucesso)
            {
                return (ProdutoModel)resposta.Objeto;
            }
            
            viewModel.Mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um item ativo com o código {codigo}" :
                $"Ocorreu um erro ao consultar o item com o código {codigo}. Erro {resposta.MensagemErro}";

            viewModel.TipoToast = TipoToast.Aviso;
            viewModel.AoMudarPropriedade("TipoToast");
            await viewModel.MeuToast.Show();
            await viewModel.CodigoProdutoInput.FocusAsync();
            return new ProdutoModel();
        }
    }
}
