﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.PagarReceber;

public static class CentroDeCustoById
{
    public static async Task<CentroDeCustoModel> Get(IPagarReceberVM viewModel, ICentrosDeCustoService centrosDeCustoService, long id)
    {
        if (id == 0) return new CentroDeCustoModel();
        var resposta = new RespostaAPIModel();
        try
        {
            viewModel.CentrosDeCustoBtnEnabled = false;
            await viewModel.SpinnerPrincipal.ShowAsync();
            resposta = await centrosDeCustoService.GetByIdAsync(URIsPagarReceber.GetCentrosDeCustoAsync, id, StatusAtivo.Ativos);
        }
        finally
        {
            viewModel.CentrosDeCustoBtnEnabled = true;
            await viewModel.SpinnerPrincipal.HideAsync();
        }
        
        if (!resposta.Sucesso)
        {
            viewModel.Mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um centro de custo ativo com o Id {id}" :
                $"Ocorreu um erro ao consultar o centro de custo com o código {id}. Erro {resposta.MensagemErro}";

            await viewModel.ExibirMensagemToast(viewModel.Mensagem, TipoToast.Aviso);
            await viewModel.IdCentroDeCustoInput.FocusAsync();
            return new CentroDeCustoModel();
        }

        return (CentroDeCustoModel)resposta.Objeto;
    }
}
