﻿using GAFNETBlazor.Models;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using Syncfusion.Blazor.Grids;
using System;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.PagarReceber;

public static class EditaDetalhamentoPagarReceber
{
    public static async Task Editar(IPagarReceberVM viewModel, ActionEventArgs<DetalhamentoPagarReceberModel> args)
    {
        var detalhamento = args.Data;
        if (detalhamento.Valor != args.PreviousData.Valor)
        {
            await EditarValor(viewModel, args);
        }
    }

    private static async Task CancelarEdicaoValor(IPagarReceberVM viewModel, ActionEventArgs<DetalhamentoPagarReceberModel> args, string mensagem)
    {
        if (!string.IsNullOrEmpty(mensagem))
        {
            args.Data.Valor = args.PreviousData.Valor;
            await viewModel.ItensGrid.CloseEditAsync();
            await viewModel.ExibirMensagemToast(mensagem, TipoToast.Aviso);
            return;
        }
    }

    private static async Task EditarValor(IPagarReceberVM viewModel, ActionEventArgs<DetalhamentoPagarReceberModel> args)
    {
        var detalhamento = args.Data;
        detalhamento.PlanoDeConta = viewModel.DetalhamentoPagarReceber.PlanoDeConta;
        detalhamento.CentroDeCusto = viewModel.DetalhamentoPagarReceber.CentroDeCusto;
        detalhamento.TipoDeOperacao = viewModel.DetalhamentoPagarReceber.TipoDeOperacao;
        detalhamento.ContaApropriacao = viewModel.DetalhamentoPagarReceber.ContaApropriacao;
        var detalhamentos = viewModel.ItemPagarReceberSelecionado.DetalhamentosPagarReceber;
        int nParcelasADividir = detalhamentos.Count(r => r.NDetalhamento > detalhamento.NDetalhamento);

        decimal vlrAnterior = 0;
        decimal vlrADividir;
        decimal vlrParcela;
        decimal vlrResto;
        int ultParcela = detalhamentos.Max(r => r.NDetalhamento);

        DetalhamentoPagarReceberModel ultDetalhamento = detalhamentos[detalhamentos.Count - 1];

        if (ultDetalhamento.NDetalhamento == viewModel.DetalhamentoPagarReceber.NDetalhamento)
        {
            decimal saldoItem = viewModel.ItemPagarReceberSelecionado.Valor - detalhamentos
                .Where(d => d.NDetalhamento < ultDetalhamento.NDetalhamento)
                .Sum(r => r.Valor) - args.Data.Valor;

            if (saldoItem < 0)
            {
                await CancelarEdicaoValor(viewModel, args, "Valor inserido maior que saldo do item.");
                return;
            }

            if (saldoItem > 0)
            {
                viewModel.ItemPagarReceberSelecionado.DetalhamentosPagarReceber.Add(new DetalhamentoPagarReceberModel
                {
                    Valor = saldoItem,
                    NDetalhamento = ultDetalhamento.NDetalhamento + 1,
                    CentroDeCusto = ultDetalhamento.CentroDeCusto,
                    ContaApropriacao = ultDetalhamento.ContaApropriacao,
                    IdCentroDeCusto = ultDetalhamento.IdCentroDeCusto,
                    IdContaApropriacao = ultDetalhamento.IdContaApropriacao,
                    IdItemPagarReceber = ultDetalhamento.IdItemPagarReceber,
                    IdPlanoDeConta = ultDetalhamento.IdPlanoDeConta,
                    IdTipoDeOperacao = ultDetalhamento.IdTipoDeOperacao,
                    PlanoDeConta = ultDetalhamento.PlanoDeConta,
                    TipoDeOperacao = ultDetalhamento.TipoDeOperacao,
                });
            }

            return;
        }


        for (int i = 0; i <= detalhamento.NDetalhamento - 2; i++)
        {
            vlrAnterior += detalhamentos[i].Valor;
        }

        vlrAnterior += detalhamento.Valor; // soma o valor da parcela que está sendo editada
        vlrADividir = viewModel.ItemPagarReceberSelecionado.Valor - vlrAnterior;
        vlrParcela = Math.Round(vlrADividir / nParcelasADividir, 2);
        vlrResto = vlrADividir - (vlrParcela * nParcelasADividir);

        if (vlrADividir <= 0)
        {
            await CancelarEdicaoValor(viewModel, args, "Valor inserido maior ou igual ao saldo do item.");
            return;
        }

        for (int i = detalhamento.NDetalhamento - 1; i < detalhamentos.Count; i++)
        {
            detalhamentos[i].Valor = detalhamentos[i].NDetalhamento == ultParcela ? vlrParcela + vlrResto : vlrParcela;
        }

    }
}
