﻿using GAFNETBlazor.Models;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using Syncfusion.Blazor.Grids;
using System;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.PagarReceber;

public static class EditaItemPagarReceber
{
    public static async Task Editar(IPagarReceberVM viewModel, ActionEventArgs<ItemPagarReceberModel> args)
    {
        var item = args.Data;
        var itens = viewModel.PagarReceber.Itens;
        string mensagem = string.Empty;

        if (item.Valor == args.PreviousData.Valor)
        {
            return;
        }

        int nParcelasADividir = itens.Count(r => r.NParcela > item.NParcela);

        if (nParcelasADividir <= 0)
        {
            await CancelarEdicao(viewModel, args, "Não é possível alterar diretamente a última parcela.");
            return;
        }

        if (item.Valor != item.VlrSaldo)
        {
            await CancelarEdicao(viewModel, args, "Esta parcela já possui movimentação financeira. Não é possível alterar seu valor.");
            return;
        }

        decimal vlrAnterior = 0;
        decimal vlrADividir;
        decimal vlrParcela;
        decimal vlrResto;
        int ultParcela = itens.Max(r => r.NParcela);

        for (int i = 0; i <= item.NParcela - 2; i++)
        {
            vlrAnterior += itens[i].Valor;
        }
        vlrAnterior += item.Valor; // soma o valor da parcela que está sendo editada
        vlrADividir = viewModel.PagarReceber.Valor - vlrAnterior;
        vlrParcela = Math.Round(vlrADividir / nParcelasADividir, 2);
        vlrResto = vlrADividir - (vlrParcela * nParcelasADividir);

        if (vlrADividir <= 0)
        {
            await CancelarEdicao(viewModel, args, "Valor inserido maior ou igual ao saldo do documento.");
            return;
        }

        for (int i = item.NParcela - 1; i < itens.Count; i++)
        {
            itens[i].Valor = itens[i].NParcela == ultParcela ? vlrParcela + vlrResto : vlrParcela;
        }

        var detalhamento = item.DetalhamentosPagarReceber[0] with { Valor = item.Valor };
        item.DetalhamentosPagarReceber.Clear();
        item.DetalhamentosPagarReceber.Add(detalhamento);
    }

    private static async Task CancelarEdicao(IPagarReceberVM viewModel, ActionEventArgs<ItemPagarReceberModel> args, string mensagem)
    {
        if (!string.IsNullOrEmpty(mensagem))
        {
            args.Data.Valor = args.PreviousData.Valor;
            await viewModel.ItensGrid.CloseEditAsync();
            await viewModel.ExibirMensagemToast(mensagem, TipoToast.Aviso);
            return;
        }
    }
}
