﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;
using Newtonsoft.Json;
using Syncfusion.Blazor.DropDowns;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.PagarReceber;

public partial class PagarReceberVM : CadastroBaseVM, IPagarReceberVM
{
    private readonly IPagarReceberService _pagarReceberService;
    private readonly IPessoasService _pessoasService;
    private readonly ILoginStore _loginStore;
    private readonly ITiposDeOperacaoService _tiposDeOperacaoService;
    private readonly ICentrosDeCustoService _centrosDeCustoService;
    private readonly IPlanosDeContaService _planosDeContaService;
    private readonly IPesquisaPagarReceberVM _pesquisaPagarReceberVM;


    [ObservableProperty]
    private bool dlgConfirmacaoEnvioVisible;
    partial void OnDlgConfirmacaoEnvioVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgConfirmacaoEnvioVisible));

    [ObservableProperty]
    private bool dlgPessoasVisible;
    partial void OnDlgPessoasVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgPessoasVisible));

    [ObservableProperty]
    private bool dlgPlanosDeContaVisible;
    partial void OnDlgPlanosDeContaVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgPlanosDeContaVisible));
    [ObservableProperty]
    private bool dlgTiposDeOperacaoVisible;
    partial void OnDlgTiposDeOperacaoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgTiposDeOperacaoVisible));

    [ObservableProperty]
    private bool dlgCentrosDeCustoVisible;
    partial void OnDlgCentrosDeCustoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgCentrosDeCustoVisible));

    [ObservableProperty]
    private bool dlgOcorrenciasVisible;
    partial void OnDlgOcorrenciasVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgOcorrenciasVisible));
    [ObservableProperty]
    private bool dlgFreteVisible;
    partial void OnDlgFreteVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgFreteVisible));
    [ObservableProperty]
    private PagarReceberModel pagarReceber;
    partial void OnPagarReceberChanged(PagarReceberModel value) => AoMudarPropriedade(nameof(PagarReceber));
    [ObservableProperty]
    private PlanoDeContaModel planoDeConta;
    partial void OnPlanoDeContaChanged(PlanoDeContaModel value) => AoMudarPropriedade(nameof(PlanoDeConta));

    [ObservableProperty]
    private PlanoDeContaModel contaApropriacao;
    partial void OnContaApropriacaoChanged(PlanoDeContaModel value) => AoMudarPropriedade(nameof(ContaApropriacao));


    [ObservableProperty]
    private CentroDeCustoModel centroDeCusto;
    partial void OnCentroDeCustoChanged(CentroDeCustoModel value) => AoMudarPropriedade(nameof(CentroDeCusto));

    [ObservableProperty]
    private TipoDeOperacaoModel tipoDeOperacao;
    partial void OnTipoDeOperacaoChanged(TipoDeOperacaoModel value) => AoMudarPropriedade(nameof(TipoDeOperacao));

    [ObservableProperty]
    private bool pesquisandoContasApropriacao;
    partial void OnPesquisandoContasApropriacaoChanged(bool value) => AoMudarPropriedade(nameof(PesquisandoContasApropriacao));
    [ObservableProperty]
    private bool dlgContasApropriacaoVisible;
    partial void OnDlgContasApropriacaoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgContasApropriacaoVisible));


    public PlanoDeContaModel PlanoDeContaSelecionado { get; set; } = new();
    public PlanoDeContaModel ContaApropriacaoSelecionado { get; set; } = new();
    public List<ListaModel> TiposPagarReceber { get; set; }
    public NavigationManager Navigation { get; set; }
    public ItemPagarReceberModel ItemPagarReceberSelecionado { get; set; } = new ItemPagarReceberModel();
    public DetalhamentoPagarReceberModel DetalhamentoPagarReceber { get; set; } = new DetalhamentoPagarReceberModel();
    public string MascaraCNPJCPF => string.IsNullOrEmpty(PagarReceber.Pessoa.CNPJ) || PagarReceber.Pessoa.CNPJ.Length > 11 ? "00.000.000/0000-00" : "000.000.000-00";
    public string CNPJCPFLbl => string.IsNullOrEmpty(PagarReceber.Pessoa.CNPJ) || PagarReceber.Pessoa.CNPJ.Length > 11 ? "CNPJ" : "CPF";
    public PessoaModel PessoaSelecionada { get; set; } = new PessoaModel();
    public List<PessoaModel> Pessoas { get; set; } = new List<PessoaModel>();
    public List<PagarReceberModel> ContasPagarReceber { get; set; } = new List<PagarReceberModel>();
    public bool PessoasBtnEnabled { get; set; } = true;
    public bool PesquisandoPessoas { get; set; }
    public bool CentrosDeCustoBtnEnabled { get; set; } = true;
    public CentroDeCustoModel CentroDeCustoSelecionado { get; set; } = new();
    public List<CentroDeCustoModel> CentrosDeCusto { get; set; } = new List<CentroDeCustoModel>();
    public bool PesquisandoCentrosDeCusto { get; set; }
    public bool TiposDeOperacaoBtnEnabled { get; set; } = true;
    public TipoDeOperacaoModel TipoDeOperacaoSelecionado { get; set; } = new();
    public List<TipoDeOperacaoModel> TiposDeOperacao { get; set; } = new List<TipoDeOperacaoModel>();
    public List<PlanoDeContaModel> PlanosDeConta { get; set; } = new List<PlanoDeContaModel>();
    public bool PesquisandoTiposDeOperacao { get; set; }
    public string TextoDlgConfirmacao { get; set; } = "Confirma o envio/gravacao do documento?";
    public SfNumericTextBox<long> NumeroInput { get; set; }
    public SfNumericTextBox<long> IdPessoaInput { get; set; }
    public SfNumericTextBox<long> IdTipoDeOperacaoInput { get; set; }
    public SfNumericTextBox<long> IdCentroDeCustoInput { get; set; }
    public bool NomePessoaImputEnabled => PagarReceber.Pessoa?.Id == 0;
    public string UrlRedirecionamento
    {
        get
        {
            if (!PodeAcessar)
            {
                return "/Login";
            }
            if (!_loginStore.LogadoEmEmpresa)
            {
                return "/SelecionaEmpresaLogada";
            }
            return string.Empty;
        }
    }

    public bool PodeAcessar
    {
        get
        {
            var claimPagarReceber = _loginStore.Claims.FirstOrDefault(c => c.Type == "PagarReceber");
            return (_loginStore.EhAdmin || _loginStore.EhRepresentante || claimPagarReceber != null);
        }
    }

    public SfNumericTextBox<long> IdPlanoDeContaInput { get; set; }
    public SfNumericTextBox<long> IdContaApropriacaoInput { get; set; }
    public bool PlanosDeContaBtnEnabled { get; set; } = true;
    public bool ContasApropriacaoBtnEnabled { get; set; } = true;
    public bool PesquisandoPlanosDeConta { get; set; }
    public SfTextBox NDoctoInput { get; set; }
    public SfGrid<ItemPagarReceberModel> ItensGrid { get; set; }
    public SfGrid<DetalhamentoPagarReceberModel> DetalhamentosGrid { get; set; }

    public PagarReceberVM(IPagarReceberService pagarReceberService,
        IPessoasService pessoasService,
        ILoginStore loginStore,
        ICentrosDeCustoService centrosDeCustoService,
        ITiposDeOperacaoService tiposDeOperacaoService,
        ITiposPagarReceberService tiposPagarReceberService,
        IPesquisaPagarReceberVM pesquisaPagarReceberVM,
        IPlanosDeContaService planosDeContaService,
        NavigationManager navigation
        )
    {
        _loginStore = loginStore;
        _pagarReceberService = pagarReceberService;
        _pessoasService = pessoasService;
        _centrosDeCustoService = centrosDeCustoService;
        _tiposDeOperacaoService = tiposDeOperacaoService;
        _planosDeContaService = planosDeContaService;
        TiposPagarReceber = tiposPagarReceberService.Itens();
        PagarReceber = new PagarReceberModel() { DtEmissao = DateTime.Today, Tipo = (int)TipoPagarReceber.Receber };
        PlanoDeConta = new PlanoDeContaModel();
        ContaApropriacao = new PlanoDeContaModel();
        TipoDeOperacao = new TipoDeOperacaoModel();
        CentroDeCusto = new CentroDeCustoModel();
        MensagemResultadoEnvio = " ";
        _pesquisaPagarReceberVM = pesquisaPagarReceberVM;
        Navigation = navigation;
    }

    public void Enviar()
    {
        if (!ValidacaoDadosEnvio.Validar(this))
        {
            return;
        }
        DlgConfirmacaoEnvioVisible = true;
    }

    public void AoFecharDlg(object e)
    {
        DlgVisible = false;
    }

    public async Task AoClicarNovoRegistroBtn(MouseEventArgs e)
    {
        LimparTela();
        await Task.Delay(300);
        await NDoctoInput.FocusAsync();
    }

    public void AoFecharDlgPessoas(object e)
    {
        DlgPessoasVisible = false;
    }

    public async void AoClicarOKBtnDlgPessoas(MouseEventArgs e)
    {
        DlgPessoasVisible = false;
        if (PessoaSelecionada == null) return;
        var resultPesquisa = await ExibicaoPessoa.Exibir(this, _pessoasService, PessoaSelecionada.Id);
        if (!resultPesquisa.Sucesso)
        {
            await ExibirMensagemToast((string)resultPesquisa.Retorno, TipoToast.Erro);
            return;
        }

        if (PlanoDeConta.Id > 0)
        {
            await IdPlanoDeContaInput.FocusAsync();
        }

    }

    public async void AoClicarSelecionarBtnGridPessoas(CommandClickEventArgs<PessoaModel> e)
    {
        DlgPessoasVisible = false;
        PessoaSelecionada = e.RowData;
        await ExibicaoPessoa.Exibir(this, _pessoasService, PessoaSelecionada.Id);
        if (PlanoDeConta.Id > 0)
        {
            await IdPlanoDeContaInput.FocusAsync();
        }
    }

    public void AoSelecionarPessoa(RowSelectEventArgs<PessoaModel> e)
    {
        PessoaSelecionada = e.Data;
    }

    public void AoClicarLocalizarBtn(MouseEventArgs e)
    {
        _pesquisaPagarReceberVM.Parametros.Tipo = PagarReceber.Tipo;
        _pesquisaPagarReceberVM.TiposPagarReceber = TiposPagarReceber;
        if (_pesquisaPagarReceberVM.ContasPagarReceber.Count > 0 && _pesquisaPagarReceberVM.ContasPagarReceber.FirstOrDefault().Tipo != PagarReceber.Tipo)
        {
            _pesquisaPagarReceberVM.ContasPagarReceber = new List<PagarReceberModel>();
        }
        Navigation.NavigateTo("/Usuario/Movimentacoes/PesquisaPagarReceber/false");
    }

    public async void IdPessoaAoSairAsync()
    {
        if (PagarReceber.Pessoa.Id == 0)
        {
            PagarReceber.Pessoa = new PessoaModel();
            return;
        }

        var resultPesquisa = await ExibicaoPessoa.Exibir(this, _pessoasService, PagarReceber.Pessoa.Id);

        if (!resultPesquisa.Sucesso)
        {
            await ExibirMensagemToast((string)resultPesquisa.Retorno, TipoToast.Erro);
            return;
        }

        if (PagarReceber.IdPessoa == 0)
        {
            PagarReceber.Pessoa.Id = 0;
            await IdPessoaInput.FocusAsync();
            return;
        }

        await IdPlanoDeContaInput.FocusAsync();


    }

    public async void AoClicarPessoasBtnAsync(MouseEventArgs e)
    {
        if (PagarReceber.Pessoa.Id > 0)
        {
            PagarReceber.Pessoa = new PessoaModel();
        }

        var resultado = await PesquisaPessoas.Pesquisar(this, _pessoasService);
        if (resultado != "")
        {
            await ExibirMensagemToast(resultado, TipoToast.Erro);
        }
    }

    public void AoSelecionarItemPagarReceber(RowSelectEventArgs<ItemPagarReceberModel> e)
    {
        ItemPagarReceberSelecionado = e.Data;
    }

    public void AoSelecionarDetalhamentoPagarReceber(RowSelectEventArgs<DetalhamentoPagarReceberModel> e)
    {
        DetalhamentoPagarReceber = e.Data;
    }

    public void AoSelecionarCentroDeCusto(RowSelectEventArgs<CentroDeCustoModel> e)
    {
        CentroDeCustoSelecionado = e.Data;
    }

    public async void AoClicarCentrosDeCustoBtnAsync(MouseEventArgs e)
    {
        await LocalizaCentroDeCusto.LocalizarAsync(this, _centrosDeCustoService);
    }

    public async void AoClicarOKBtnDlgCentrosDeCusto(MouseEventArgs e)
    {
        await ExibirCentroDeCusto();
    }

    private async Task ExibirCentroDeCusto()
    {
        DlgCentrosDeCustoVisible = false;
        if (CentroDeCustoSelecionado.Id == 0) return;
        CentroDeCusto = await CentroDeCustoById.Get(this, _centrosDeCustoService, CentroDeCustoSelecionado.Id);
        await IdTipoDeOperacaoInput.FocusAsync();
        AoMudarPropriedade(nameof(CentroDeCusto));
    }

    public void AoFecharDlgCentrosDeCusto(object e)
    {
        DlgCentrosDeCustoVisible = false;
    }

    public async void IdCentroDeCustoAoSairAsync()
    {
        if (CentroDeCusto.Id == 0)
        {
            CentroDeCusto = new CentroDeCustoModel();
            return;
        }
        CentroDeCusto = await CentroDeCustoById.Get(this, _centrosDeCustoService, CentroDeCusto.Id);

        if (TipoDeOperacao.Id == 0)
        {
            await IdTipoDeOperacaoInput.FocusAsync();
        }

        AoMudarPropriedade(nameof(CentroDeCusto));
    }

    public void AoSelecionarTipoDeOperacao(RowSelectEventArgs<TipoDeOperacaoModel> e)
    {
        TipoDeOperacaoSelecionado = e.Data;
    }

    public async void AoClicarTiposDeOperacaoBtnAsync(MouseEventArgs e)
    {
        await LocalizaTipoDeOperacao.LocalizarAsync(this, _tiposDeOperacaoService);
    }

    public async void AoClicarOKBtnDlgTiposDeOperacao(MouseEventArgs e)
    {
        TipoDeOperacao = TipoDeOperacaoSelecionado;
        await ExibirTipoDeOperacaoSelecionado();
    }

    private async Task ExibirTipoDeOperacaoSelecionado()
    {
        DlgTiposDeOperacaoVisible = false;
        TipoDeOperacao = await TipoDeOperacaoById.Get(this, _tiposDeOperacaoService, TipoDeOperacaoSelecionado.Id);
        AoMudarPropriedade(nameof(TipoDeOperacao));
        await IdContaApropriacaoInput.FocusAsync();
    }

    public void AoFecharDlgTiposDeOperacao(object e)
    {
        DlgTiposDeOperacaoVisible = false;
    }

    public async void IdTipoDeOperacaoAoSairAsync()
    {
        if (TipoDeOperacao.Id == 0)
        {
            TipoDeOperacao = new TipoDeOperacaoModel();
            return;
        }
        TipoDeOperacao = await TipoDeOperacaoById.Get(this, _tiposDeOperacaoService, TipoDeOperacao.Id);
        AoMudarPropriedade(nameof(TipoDeOperacao));
        if (ContaApropriacao.Id == 0)
        {
            await IdContaApropriacaoInput.FocusAsync();
        }

    }



    public async Task AoClicarSimBtnDlgConfirmacaoEnvio(MouseEventArgs e)
    {
        try
        {
            DlgConfirmacaoEnvioVisible = false;
            Enviando = true;
            await SpinnerPrincipal.ShowAsync();

            var resultEnviarPagarReceber = await _pagarReceberService.CriarAlterarAsync(this);

            if (!resultEnviarPagarReceber.Sucesso)
            {
                await ExibirMensagemToast(resultEnviarPagarReceber.MensagemErro, TipoToast.Erro);
                return;
            }

        }
        finally
        {
            await SpinnerPrincipal.HideAsync();
            Enviando = false;
            await Task.Delay(100);
            AoMudarPropriedade(nameof(PagarReceber));
        }
    }

    public void AoClicarNaoBtnDlgConfirmacaoEnvio(MouseEventArgs e)
    {
        DlgConfirmacaoEnvioVisible = false;
    }

    public async void AoClicarSelecionarBtnGridCentrosDeCusto(CommandClickEventArgs<CentroDeCustoModel> e)
    {
        CentroDeCustoSelecionado = e.RowData;
        await ExibirCentroDeCusto();
    }

    public async void AoClicarSelecionarBtnGridTipoDeOperacao(CommandClickEventArgs<TipoDeOperacaoModel> e)
    {
        TipoDeOperacaoSelecionado = e.RowData;
        await ExibirTipoDeOperacaoSelecionado();
    }

    public async Task AoPressionarTeclaPessoasGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && PessoaSelecionada?.Id > 0)
        {
            DlgPessoasVisible = false;
            await ExibicaoPessoa.Exibir(this, _pessoasService, PessoaSelecionada.Id);
        }
    }

    public async Task AoPressionarTeclaTiposDeOperacaoGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && TipoDeOperacaoSelecionado?.Id > 0)
        {
            await ExibirTipoDeOperacaoSelecionado();
        }
    }

    public async Task AoPressionarTeclaCentrosDeCustoGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && CentroDeCustoSelecionado?.Id > 0)
        {
            await ExibirCentroDeCusto();
        }
    }

    public async Task NomePessoaAoSairAsync()
    {
        string resultado = "";
        if (PagarReceber.Pessoa.Id == 0 && !string.IsNullOrEmpty(PagarReceber.Pessoa.Nome))
        {
            resultado = await PesquisaPessoas.Pesquisar(this, _pessoasService);
        }
        if (resultado != "")
        {
            await ExibirMensagemToast(resultado, TipoToast.Erro);
        }
        if (PagarReceber.Pessoa.Id > 0)
        {
            PagarReceber.IdPessoa = PagarReceber.Pessoa.Id;
            await ExibicaoPessoa.Exibir(this, _pessoasService, PagarReceber.Pessoa.Id);
        }
    }

    public void LimparTela()
    {
        LimpaTela.Limpar(this);
    }

    public async void NDoctoAoSairAsync()
    {
        await GetDocumento();
    }

    public void AoMudarTipoPagarReceber(ChangeEventArgs<int, ListaModel> args)
    {
        LimparTela();
    }

    public Task AoClicarCommandBtnGridItemPagarReceber(CommandClickEventArgs<ItemPagarReceberModel> e)
    {
        throw new NotImplementedException();
    }

    public Task AoClicarCommandBtnGridDetalhamentoPagarReceber(CommandClickEventArgs<DetalhamentoPagarReceberModel> e)
    {
        throw new NotImplementedException();
    }

    private async Task GetDocumento()
    {
        if (PagarReceber.IdPessoa == 0 || string.IsNullOrEmpty(PagarReceber.NDocto) || string.IsNullOrEmpty(PagarReceber.SerieDocto))
        {
            return;
        }

        try
        {
            await SpinnerPrincipal.ShowAsync();
            await _pagarReceberService.GetDocumento(URIsPagarReceber.GetAsync, PagarReceber.Tipo, PagarReceber.IdPessoa, PagarReceber.NDocto, PagarReceber.SerieDocto);
        }
        finally
        {
            await SpinnerPrincipal.HideAsync();
        }

    }

    public async void AoClicarPlanosDeContaBtnAsync(MouseEventArgs e)
    {
        await LocalizaPlanoDeConta.LocalizarAsync(this, _planosDeContaService);
    }

    public async void IdPlanoDeContaAoSairAsync()
    {
        if (PlanoDeConta.Id == 0)
        {
            PlanoDeConta = new PlanoDeContaModel();
            return;
        }

        try
        {
            PesquisandoPlanosDeConta = true;
            PlanoDeConta = await PlanoDeContaById.Get(this, _planosDeContaService, PlanoDeConta.Id);
            if (CentroDeCusto.Id == 0)
            {
                await IdCentroDeCustoInput.FocusAsync();
            }
        }
        finally
        {
            PesquisandoPlanosDeConta = false;
            AoMudarPropriedade(nameof(PlanoDeConta));
        }
    }

    public async void AoClicarGerarItensBtn(MouseEventArgs e)
    {
        if (await ValidacaoGeracaoItens.Validar(this) == false)
        {
            return;
        }

        var retorno = await _pagarReceberService.GerarItensAsync(PagarReceber);

        if (!retorno.Sucesso)
        {
            await ExibirMensagemToast(retorno.MensagemErro, TipoToast.Erro);
            return;
        }

        PagarReceber = JsonConvert.DeserializeObject<PagarReceberModel>(retorno.ConteudoRetorno);

        foreach (var item in PagarReceber.Itens)
        {
            foreach (var detalhamento in item.DetalhamentosPagarReceber)
            {
                detalhamento.TipoDeOperacao = TipoDeOperacao;
                detalhamento.IdTipoDeOperacao = TipoDeOperacao.Id;

                detalhamento.CentroDeCusto = CentroDeCusto;
                detalhamento.IdCentroDeCusto = CentroDeCusto.Id;

                detalhamento.PlanoDeConta = PlanoDeConta;
                detalhamento.IdPlanoDeConta = PlanoDeConta.Id;

                detalhamento.ContaApropriacao = ContaApropriacao;
                detalhamento.IdContaApropriacao = ContaApropriacao?.Id;
            }
        }

        await Task.Delay(500);
        await ItensGrid.SelectRowAsync(0);
        await ItensGrid.StartEditAsync();
    }

    public void AoFecharDlgPlanosDeConta(object e)
    {
        DlgPlanosDeContaVisible = false;
    }

    public async void AoClicarOkBtnDlgPlanosDeConta(MouseEventArgs e)
    {
        DlgPlanosDeContaVisible = false;
        if (PlanoDeContaSelecionado.Id == 0) return;
        PlanoDeConta = await PlanoDeContaById.Get(this, _planosDeContaService, PlanoDeContaSelecionado.Id);
        await IdCentroDeCustoInput.FocusAsync();
        AoMudarPropriedade(nameof(PlanoDeConta));
    }

    public void AoSelecionarPlanoDeConta(RowSelectEventArgs<PlanoDeContaModel> e)
    {
        PlanoDeContaSelecionado = e.Data;
    }

    public async void AoClicarSelecionarBtnGridPlanosDeConta(CommandClickEventArgs<PlanoDeContaModel> e)
    {
        PlanoDeContaSelecionado = e.RowData;
        await ExibirPlanoDeContaSelecionado();
    }

    private async Task ExibirPlanoDeContaSelecionado()
    {
        ErrosValidacaoFormulario.Clear();
        DlgPlanosDeContaVisible = false;
        PlanoDeConta = await PlanoDeContaById.Get(this, _planosDeContaService, PlanoDeContaSelecionado.Id);
        await IdCentroDeCustoInput.FocusAsync();
    }

    public async Task AoPressionarTeclaPlanosDeContaGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && PlanoDeContaSelecionado.Id > 0)
        {
            await IdCentroDeCustoInput.FocusAsync();
            await ExibirPlanoDeContaSelecionado();
        }
    }

    public async void AoClicarContasApropriacaoBtnAsync(MouseEventArgs e)
    {
        await LocalizaContaApropriacao.LocalizarAsync(this, _planosDeContaService);
    }

    public void AoFecharDlgContasApropriacao(object e)
    {
        DlgContasApropriacaoVisible = false;
    }

    public async void AoClicarOkBtnDlgContasApropriacao(MouseEventArgs e)
    {
        DlgContasApropriacaoVisible = false;
        if (ContaApropriacaoSelecionado.Id == 0) return;
        ContaApropriacao = await PlanoDeContaById.Get(this, _planosDeContaService, ContaApropriacaoSelecionado.Id);
        AoMudarPropriedade(nameof(ContaApropriacao));
    }

    public void AoSelecionarContaApropriacao(RowSelectEventArgs<PlanoDeContaModel> e)
    {
        ContaApropriacaoSelecionado = e.Data;
    }

    public async void AoClicarSelecionarBtnGridContasApropriacao(CommandClickEventArgs<PlanoDeContaModel> e)
    {
        ContaApropriacaoSelecionado = e.RowData;
        await ExibirContaApropriacaoSelecionado();
    }

    private async Task ExibirContaApropriacaoSelecionado()
    {
        ErrosValidacaoFormulario.Clear();
        DlgContasApropriacaoVisible = false;
        ContaApropriacao = await PlanoDeContaById.Get(this, _planosDeContaService, ContaApropriacaoSelecionado.Id);
    }

    public async Task AoPressionarTeclaContasApropriacaoGrid(KeyboardEventArgs args)
    {
        if (args.Key == "Enter" && ContaApropriacaoSelecionado.Id > 0)
        {
            await ExibirContaApropriacaoSelecionado();
        }
    }

    public async void IdContaApropriacaoAoSairAsync()
    {
        if (ContaApropriacao.Id == 0)
        {
            ContaApropriacao = new PlanoDeContaModel();
            return;
        }

        try
        {
            PesquisandoContasApropriacao = true;
            ContaApropriacao = await PlanoDeContaById.Get(this, _planosDeContaService, ContaApropriacao.Id);
            AoMudarPropriedade(nameof(ContaApropriacao));
        }
        finally
        {
            PesquisandoContasApropriacao = false;

        }
    }

    public async void AoComecarAcaoItensGrid(ActionEventArgs<ItemPagarReceberModel> args)
    {
        if (args.Action == "Edit")
        {
            await EditaItemPagarReceber.Editar(this, args);
        }
    }

    public async void AoComecarAcaoDetalhamentosGrid(ActionEventArgs<DetalhamentoPagarReceberModel> args)
    {
        if (args.Action == "Edit")
        {
            await EditaDetalhamentoPagarReceber.Editar(this, args);
        }
    }

    public async Task IdCentroDeCustoGridAoSairAsync(NumericBlurEventArgs<long> args)
    {
        if (args.Value == 0)
        {
            DetalhamentoPagarReceber.CentroDeCusto = new CentroDeCustoModel();
            return;
        }
        DetalhamentoPagarReceber.IdCentroDeCusto = args.Value;
        DetalhamentoPagarReceber.CentroDeCusto = await CentroDeCustoById.Get(this, _centrosDeCustoService, args.Value);
    }

    public async Task IdPlanoDeContaGridAoSairAsync(NumericBlurEventArgs<long> args)
    {
        if (args.Value == 0)
        {
            DetalhamentoPagarReceber.PlanoDeConta = new PlanoDeContaModel();
            return;
        }
        DetalhamentoPagarReceber.IdPlanoDeConta = args.Value;
        DetalhamentoPagarReceber.PlanoDeConta = await PlanoDeContaById.Get(this, _planosDeContaService, args.Value);
    }

    public async Task IdTipoDeOperacaoGridAoSairAsync(NumericBlurEventArgs<long> args)
    {
        if (args.Value == 0)
        {
            DetalhamentoPagarReceber.TipoDeOperacao = new TipoDeOperacaoModel();
            return;
        }
        DetalhamentoPagarReceber.IdTipoDeOperacao = args.Value;
        DetalhamentoPagarReceber.TipoDeOperacao = await TipoDeOperacaoById.Get(this, _tiposDeOperacaoService, args.Value);
    }
}
