﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.PagarReceber;

public static class PlanoDeContaById
{
    public static async Task<PlanoDeContaModel> Get(IPagarReceberVM viewModel, IPlanosDeContaService planosDeContaService, long id)
    {
        if (id == 0) return new PlanoDeContaModel();

        var resposta = new RespostaAPIModel();
        try
        {
            await viewModel.SpinnerPrincipal.ShowAsync();
            resposta = await planosDeContaService.GetByIdAsync(URIsPagarReceber.GetPlanosDeContaAsync, id, StatusAtivo.Ativos);
        }
        finally
        {
            await viewModel.SpinnerPrincipal.HideAsync();
        }

        if (!resposta.Sucesso)
        {
            string mensagemErro = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um plano de contas ativo com o Id {id}" :
                $"Ocorreu um erro ao consultar o plano de contas com o id {id}. Erro {resposta.MensagemErro}";

            await viewModel.ExibirMensagemToast(mensagemErro, TipoToast.Erro);
            return new PlanoDeContaModel();

        }

        return (PlanoDeContaModel)resposta.Objeto;

    }
}