﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.PagarReceber;

public static class TipoDeOperacaoById
{
    public static async Task<TipoDeOperacaoModel> Get(IPagarReceberVM viewModel, ITiposDeOperacaoService tiposDeOperacaoService, long id)
    {
        if (id == 0) return new TipoDeOperacaoModel();
        var resposta = new RespostaAPIModel();
        try
        {
            viewModel.PesquisandoTiposDeOperacao = true;
            viewModel.TiposDeOperacaoBtnEnabled = false;
            await viewModel.SpinnerPrincipal.ShowAsync();
            resposta = await tiposDeOperacaoService.GetByIdAsync(URIsFaturamentos.GetTiposDeOperacaoAsync, id, StatusAtivo.Ativos);
        }
        finally
        {
            await viewModel.SpinnerPrincipal.HideAsync();
            viewModel.TiposDeOperacaoBtnEnabled = true;
            viewModel.PesquisandoTiposDeOperacao = false;
        }

        if (resposta.Sucesso)
        {
            return (TipoDeOperacaoModel)resposta.Objeto;
        }
        else
        {
            viewModel.Mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um tipo de operação ativo com o Id {id}" :
                $"Ocorreu um erro ao consultar o tipo de operação com o id {id}. Erro {resposta.MensagemErro}";

            viewModel.TipoToast = TipoToast.Aviso;
            viewModel.AoMudarPropriedade("TipoToast");
            await viewModel.MeuToast.Show();
            return new TipoDeOperacaoModel();
        }
    }
}
