﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.PesquisaFaturamento
{
    public static class FormularioById
    {
        public static async Task<FormularioModel> Get(IPesquisaFaturamentoVM viewModel, IFormulariosService formulariosService, long id)
        {
            if (id == 0) return new FormularioModel();

            RespostaAPIModel resposta = new();
            try
            {
                viewModel.FormulariosBtnEnabled = false;
                await viewModel.SpinnerPrincipal.ShowAsync();
                resposta = await formulariosService.GetByIdAsync(URIsFaturamentos.GetFormulariosAsync, id, StatusAtivo.Todos);
            }
            finally
            {
                await viewModel.SpinnerPrincipal.HideAsync();
                viewModel.FormulariosBtnEnabled = true;
            }



            if (resposta.Sucesso)
            {
                return (FormularioModel)resposta.Objeto;
            }
            else
            {
                viewModel.Mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um formulário com o Id {id}" :
                    $"Ocorreu um erro ao consultar o formulário com o id {id}. Erro {resposta.MensagemErro}";

                viewModel.TipoToast = TipoToast.Aviso;
                viewModel.AoMudarPropriedade("TipoToast");
                await viewModel.MeuToast.Show();
                await viewModel.IdFormularioInput.FocusAsync();
                return new FormularioModel();
            }
        }
    }
}
