﻿using GAFNETBlazor.Constantes;
using GAFNETLibrary.Enums;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using System.Threading.Tasks;
using GAFNETLibrary.Models;

namespace GAFNETBlazor.ViewModels.Usuario.PesquisaFaturamento
{
    public static class HistoricoById
    {
        public static async Task<HistoricoModel> Get(IPesquisaFaturamentoVM viewModel, IHistoricosService historicosService, long id)
        {
            if (id == 0) return new HistoricoModel();

            RespostaAPIModel resposta = new();
            try
            {
                viewModel.HistoricosBtnEnabled = false;
                await viewModel.SpinnerPrincipal.ShowAsync();
                resposta = await historicosService.GetByIdAsync(URIsFaturamentos.GetHistoricosAsync, id, StatusAtivo.Todos);
            }
            finally
            {
                await viewModel.SpinnerPrincipal.HideAsync();
                viewModel.HistoricosBtnEnabled = true;
            }



            if (resposta.Sucesso)
            {
                return (HistoricoModel)resposta.Objeto;
            }
            else
            {
                viewModel.Mensagem = resposta.Status == System.Net.HttpStatusCode.NotFound ? $"Não foi encontrado um histórico com o Id {id}" :
                    $"Ocorreu um erro ao consultar o histórico com o id {id}. Erro {resposta.MensagemErro}";

                viewModel.TipoToast = TipoToast.Aviso;
                viewModel.AoMudarPropriedade("TipoToast");
                await viewModel.MeuToast.Show();
                await viewModel.IdHistoricoInput.FocusAsync();
                return new HistoricoModel();
            }
        }
    }
}
