﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETBlazor.ViewModels.Usuario.Faturamento;
using GAFNETBlazor.ViewModels.Usuario.FaturamentoBase;
using GAFNETLibrary.Enums;
using GAFNETLibrary.Models;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;
using Newtonsoft.Json;
using Syncfusion.Blazor.DropDowns;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using Syncfusion.Blazor.SplitButtons;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.PesquisaFaturamento;

public partial class PesquisaFaturamentoVM : BaseVM, IPesquisaFaturamentoVM
{
    private readonly IPesquisasFaturamentoService _pesquisasFaturamentoService;
    private readonly IHistoricosService _historicosService;
    private readonly IFormulariosService _formulariosService;
    private readonly ITiposDeOperacaoService _tiposDeOperacaoService;
    private readonly IPlanosDeContaService _planosDeContaService;
    private readonly ICentrosDeCustoService _centrosDeCustoService;
    private readonly IFaturamentosService _faturamentosService;
    private readonly IEnvioFaturamentoService _envioFaturamentoService;
    private readonly ITimeZoneService _timeZoneService;
    private readonly IAcoesFaturamentoService _acoesFaturamentoService;
    private readonly INFeService _nFeService;
    private readonly IDadosDevolucaoService _dadosDevolucaoService;
    private readonly IExibicaoFaturamentoService _exibicaoFaturamentoService;
    
    private readonly NavigationManager _navigationManager;

    [ObservableProperty]
    private bool dlgDevolucaoVisible;
    partial void OnDlgDevolucaoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgDevolucaoVisible));
    [ObservableProperty]
    private bool dlgConfirmacaoEnvioEMailNFeVisible;
    partial void OnDlgConfirmacaoEnvioEMailNFeVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgConfirmacaoEnvioEMailNFeVisible));
    [ObservableProperty]
    private bool dlgLogEnvioCancelamentoVisible;
    partial void OnDlgLogEnvioCancelamentoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgLogEnvioCancelamentoVisible));

    [ObservableProperty]
    private string logEnvioCancelamento;
    partial void OnLogEnvioCancelamentoChanged(string value) => AoMudarPropriedade(nameof(LogEnvioCancelamento));
    [ObservableProperty]
    private bool dlgConfirmacaoCancelamentoVisible;
    partial void OnDlgConfirmacaoCancelamentoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgConfirmacaoCancelamentoVisible));
    [ObservableProperty]
    private bool dlgMotivoCancelamentoVisible;
    partial void OnDlgMotivoCancelamentoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgMotivoCancelamentoVisible));
    [ObservableProperty]
    private bool dlgLogEnvioInutilizacaoVisible;
    partial void OnDlgLogEnvioInutilizacaoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgLogEnvioInutilizacaoVisible));

    [ObservableProperty]
    private string logEnvioInutilizacao;
    partial void OnLogEnvioInutilizacaoChanged(string value) => AoMudarPropriedade(nameof(LogEnvioInutilizacao));
    [ObservableProperty]
    private bool dlgConfirmacaoInutilizacaoVisible;
    partial void OnDlgConfirmacaoInutilizacaoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgConfirmacaoInutilizacaoVisible));
    [ObservableProperty]
    private bool dlgMotivoInutilizacaoVisible;
    partial void OnDlgMotivoInutilizacaoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgMotivoInutilizacaoVisible));
    [ObservableProperty]
    private bool dlgLogEnvioEstornoVisible;
    partial void OnDlgLogEnvioEstornoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgLogEnvioEstornoVisible));
    [ObservableProperty]
    private string logEnvioEstorno;
    partial void OnLogEnvioEstornoChanged(string value) => AoMudarPropriedade(nameof(LogEnvioEstorno));

    [ObservableProperty]
    private bool dlgLogEnvioFaturamentoVisible;
    partial void OnDlgLogEnvioFaturamentoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgLogEnvioFaturamentoVisible));

    [ObservableProperty]
    private string logEnvioFaturamento;
    partial void OnLogEnvioFaturamentoChanged(string value) => AoMudarPropriedade(nameof(LogEnvioFaturamento));

    [ObservableProperty]
    private bool dlgMotivoEstornoVisible;
    partial void OnDlgMotivoEstornoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgMotivoEstornoVisible));

    [ObservableProperty]
    private List<ListaModel> tiposFaturamento;
    partial void OnTiposFaturamentoChanged(List<ListaModel> value) => AoMudarPropriedade(nameof(TiposFaturamento));

    [ObservableProperty]
    private bool pesquisando;
    partial void OnPesquisandoChanged(bool value) => AoMudarPropriedade(nameof(Pesquisando));

    [ObservableProperty]
    private bool dlgHistoricosVisible;
    partial void OnDlgHistoricosVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgHistoricosVisible));

    [ObservableProperty]
    private bool dlgFormulariosVisible;
    partial void OnDlgFormulariosVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgFormulariosVisible));

    [ObservableProperty]
    private bool dlgTiposDeOperacaoVisible;
    partial void OnDlgTiposDeOperacaoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgTiposDeOperacaoVisible));

    [ObservableProperty]
    private bool dlgCentrosDeCustoVisible;
    partial void OnDlgCentrosDeCustoVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgCentrosDeCustoVisible));

    [ObservableProperty]
    private bool dlgPlanosDeContaVisible;
    partial void OnDlgPlanosDeContaVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgPlanosDeContaVisible));

    [ObservableProperty]
    private PesquisaFaturamentoModel pesquisaFaturamento;
    partial void OnPesquisaFaturamentoChanged(PesquisaFaturamentoModel value) => AoMudarPropriedade(nameof(PesquisaFaturamento));

    [ObservableProperty]
    private bool dlgOcorrenciasVisible;
    partial void OnDlgOcorrenciasVisibleChanged(bool value) => AoMudarPropriedade(nameof(DlgOcorrenciasVisible));


    public SfDialog Dlg { get; set; }
    public List<FaturamentoModel> Faturamentos { get; set; } = new List<FaturamentoModel>();
    public bool HistoricosBtnEnabled { get; set; } = true;
    public bool FormulariosBtnEnabled { get; set; } = true;
    public HistoricoModel HistoricoSelecionado { get; set; }
    public FormularioModel FormularioSelecionado { get; set; }
    public List<HistoricoModel> Historicos { get; set; } = new List<HistoricoModel>();
    public List<FormularioModel> Formularios { get; set; } = new List<FormularioModel>();
    public bool PesquisandoHistoricos { get; set; }
    public bool PesquisandoFormularios { get; set; }
    public bool PlanosDeContaBtnEnabled { get; set; } = true;
    public PlanoDeContaModel PlanoDeContaSelecionado { get; set; }
    public List<PlanoDeContaModel> PlanosDeContas { get; set; } = new List<PlanoDeContaModel>();
    public bool PesquisandoPlanosDeConta { get; set; }
    public bool CentrosDeCustoBtnEnabled { get; set; } = true;
    public CentroDeCustoModel CentroDeCustoSelecionado { get; set; }
    public List<CentroDeCustoModel> CentrosDeCusto { get; set; } = new List<CentroDeCustoModel>();
    public bool PesquisandoCentrosDeCusto { get; set; }
    public bool TiposDeOperacaoBtnEnabled { get; set; } = true;
    public TipoDeOperacaoModel TipoDeOperacaoSelecionado { get; set; }
    public List<TipoDeOperacaoModel> TiposDeOperacao { get; set; } = new List<TipoDeOperacaoModel>();
    public bool PesquisandoTiposDeOperacao { get; set; }
    public FaturamentoModel FaturamentoSelecionado { get; set; }
    public SfNumericTextBox<long> IdPlanoDeContaInput { get; set; }
    public SfNumericTextBox<long> IdCondPgtoInput { get; set; }
    public SfNumericTextBox<long> IdTipoDeOperacaoInput { get; set; }
    public SfNumericTextBox<long> IdCentroDeCustoInput { get; set; }
    public SfNumericTextBox<long> IdHistoricoInput { get; set; }
    public SfNumericTextBox<long> IdTransportadoraInput { get; set; }
    public SfNumericTextBox<long> IdFormularioInput { get; set; }
    public List<ListaModel> TiposPesquisaNome { get; }
    public bool PesquisarBtnEnabled { get; set; } = true;
    public IFaturamentoBaseVM VMChamador { get; set; }
    public List<ListaModel> AcoesFaturamentoGrid { get; set; } = new List<ListaModel>();
    public bool DlgConfirmacaoDuplicacaoVisible { get; set; }
    public List<OcorrenciaFaturamentoModel> Ocorrencias { get; set; } = new List<OcorrenciaFaturamentoModel>();

    public PesquisaFaturamentoVM(IPesquisasFaturamentoService pesquisasFaturamentoService,
        IHistoricosService historicosService,
        IPlanosDeContaService planosDeContaService,
        ICentrosDeCustoService centrosDeCustoService,
        ITiposDeOperacaoService tiposDeOperacaoService,
        ITiposPesquisaStringService tiposPesquisaStringService,
        IFormulariosService formulariosService,
        IAcoesFaturamentoService acoesFaturamentoService,
        IFaturamentosService faturamentosService,
        ITimeZoneService timeZoneService,
        IEnvioFaturamentoService envioFaturamentoService,
        INFeService nFeService,
        IExibicaoFaturamentoService exibicaoFaturamentoService,
        IDadosDevolucaoService dadosDevolucaoService,
        NavigationManager navigationManager)
    {
        _pesquisasFaturamentoService = pesquisasFaturamentoService;
        _historicosService = historicosService;
        _planosDeContaService = planosDeContaService;
        _centrosDeCustoService = centrosDeCustoService;
        _formulariosService = formulariosService;
        _acoesFaturamentoService = acoesFaturamentoService;
        _tiposDeOperacaoService = tiposDeOperacaoService;
        _faturamentosService = faturamentosService;
        PesquisaFaturamento = new PesquisaFaturamentoModel();
        TiposPesquisaNome = tiposPesquisaStringService.Itens();
        _timeZoneService = timeZoneService;
        _envioFaturamentoService = envioFaturamentoService;
        _nFeService = nFeService;
        _dadosDevolucaoService = dadosDevolucaoService;
        _exibicaoFaturamentoService = exibicaoFaturamentoService;
        _navigationManager = navigationManager;
        _nFeService.OnEtapaCancelamentoNFe += AoEtapaCancelamentoNFe;
        _nFeService.OnEtapaInutilizacaoNFe += AoEtapaInutilizacaoNFe;
        _nFeService.OnEtapaEnvioNFe += AoEtapaEnvioNFe;

    }

    public void AoClicarLimparFiltrosBtnAsync(MouseEventArgs e)
    {
        LimparTela();
        AoMudarPropriedade(nameof(PesquisaFaturamento));
    }

    public async Task<List<FaturamentoModel>> GetFaturamentos()
    {
        var retorno = await _pesquisasFaturamentoService.GetAsync(PesquisaFaturamento);

        if (!retorno.Sucesso || !string.IsNullOrEmpty(retorno.MensagemErro))
        {
            Mensagem = retorno.MensagemErro;
            return new List<FaturamentoModel>();
        }

        var faturamentos = (List<FaturamentoModel>)retorno.Objeto;
        foreach (var faturamento in faturamentos)
        {
            faturamento.DtHrLancamento = _timeZoneService.GetLocalDateTime(faturamento.DtHrLancamento);
        }
        return faturamentos;
    }

    public async void AoClicarLocalizarBtn()
    {
        Pesquisando = true;
        Faturamentos = await GetFaturamentos();
        Pesquisando = false;
    }

    public async void AoClicarHistoricosBtnAsync()
    {
        await LocalizaHistorico.LocalizarAsync(this, _historicosService);
    }

    public void AoClicarOkBtnDlgHistoricos(MouseEventArgs e)
    {
        DlgHistoricosVisible = false;
        PesquisaFaturamento.Historico = HistoricoSelecionado ?? new HistoricoModel();// await HistoricoById.Get(this, _historicosService, HistoricoSelecionado.Id);
        AoMudarPropriedade(nameof(PesquisaFaturamento.Historico));
    }

    public void AoClicarOkBtnDlgFormularios(MouseEventArgs e)
    {
        DlgFormulariosVisible = false;
        PesquisaFaturamento.Formulario = FormularioSelecionado ?? new FormularioModel();// await FormularioById.Get(this, _formulariosService, FormularioSelecionado.Id);
        AoMudarPropriedade(nameof(PesquisaFaturamento));
    }

    public void AoFecharDlgHistoricos(object e)
    {
        DlgHistoricosVisible = false;
    }

    public void AoFecharDlgFormularios(object e)
    {
        DlgFormulariosVisible = false;
    }

    public async void IdHistoricoAoSairAsync()
    {
        PesquisaFaturamento.Historico = await HistoricoById.Get(this, _historicosService, PesquisaFaturamento.Historico.Id);
        AoMudarPropriedade(nameof(PesquisaFaturamento.Historico));
    }

    public void AoSelecionarHistorico(RowSelectEventArgs<HistoricoModel> e)
    {
        HistoricoSelecionado = e.Data;
    }

    public async void IdFormularioAoSairAsync()
    {
        PesquisaFaturamento.Formulario = await FormularioById.Get(this, _formulariosService, PesquisaFaturamento.Formulario.Id);
        AoMudarPropriedade(nameof(PesquisaFaturamento.Formulario));
    }

    public void AoSelecionarFormulario(RowSelectEventArgs<FormularioModel> e)
    {
        FormularioSelecionado = e.Data;
    }

    public async void AoClicarFormulariosBtnAsync(MouseEventArgs e)
    {
        await LocalizaFormulario.LocalizarAsync(this, _formulariosService);
    }

    public void AoSelecionarPlanoDeConta(RowSelectEventArgs<PlanoDeContaModel> e)
    {
        PlanoDeContaSelecionado = e.Data;
    }

    public async void AoClicarPlanosDeContaBtnAsync(MouseEventArgs e)
    {
        await LocalizaPlanoDeConta.LocalizarAsync(this, _planosDeContaService);
    }

    public async void AoClicarOkBtnDlgPlanosDeConta(MouseEventArgs e)
    {
        DlgPlanosDeContaVisible = false;
        if (PlanoDeContaSelecionado == null) return;
        PesquisaFaturamento.PlanoDeConta = await PlanoDeContaById.Get(this, _planosDeContaService, PlanoDeContaSelecionado.Id);
        AoMudarPropriedade(nameof(PesquisaFaturamento.PlanoDeConta));
    }

    public void AoFecharDlgPlanosDeConta(object e)
    {
        DlgPlanosDeContaVisible = false;
    }

    public async void IdPlanoDeContaAoSairAsync()
    {
        PesquisaFaturamento.PlanoDeConta = await PlanoDeContaById.Get(this, _planosDeContaService, PesquisaFaturamento.PlanoDeConta.Id);

        AoMudarPropriedade(nameof(PesquisaFaturamento.PlanoDeConta));
    }

    public void AoSelecionarCentroDeCusto(RowSelectEventArgs<CentroDeCustoModel> e)
    {
        CentroDeCustoSelecionado = e.Data;
    }

    public async void AoClicarCentrosDeCustoBtnAsync(MouseEventArgs e)
    {
        await LocalizaCentroDeCusto.LocalizarAsync(this, _centrosDeCustoService);
    }

    public async void AoClicarOkBtnDlgCentrosDeCusto(MouseEventArgs e)
    {
        DlgCentrosDeCustoVisible = false;
        if (CentroDeCustoSelecionado == null)
        {
            return;
        }
        PesquisaFaturamento.CentroDeCusto = await CentroDeCustoById.Get(this, _centrosDeCustoService, CentroDeCustoSelecionado.Id);
        AoMudarPropriedade(nameof(PesquisaFaturamento.CentroDeCusto));
    }

    public void AoFecharDlgCentrosDeCusto(object e)
    {
        DlgCentrosDeCustoVisible = false;
    }

    public async void IdCentroDeCustoAoSairAsync()
    {
        PesquisaFaturamento.CentroDeCusto = await CentroDeCustoById.Get(this, _centrosDeCustoService, PesquisaFaturamento.CentroDeCusto.Id);

        AoMudarPropriedade(nameof(PesquisaFaturamento.CentroDeCusto));
    }

    public void AoSelecionarTipoDeOperacao(RowSelectEventArgs<TipoDeOperacaoModel> e)
    {
        TipoDeOperacaoSelecionado = e.Data;
    }

    public async void AoClicarTiposDeOperacaoBtnAsync(MouseEventArgs e)
    {
        await LocalizaTipoDeOperacao.LocalizarAsync(this, _tiposDeOperacaoService);
    }

    public async void AoClicarOkBtnDlgTiposDeOperacao(MouseEventArgs e)
    {
        DlgTiposDeOperacaoVisible = false;
        if (TipoDeOperacaoSelecionado == null) return;
        PesquisaFaturamento.TipoDeOperacao = await TipoDeOperacaoById.Get(this, _tiposDeOperacaoService, TipoDeOperacaoSelecionado.Id);
        AoMudarPropriedade(nameof(PesquisaFaturamento.TipoDeOperacao));
    }

    public void AoFecharDlgTiposDeOperacao(object e)
    {
        DlgTiposDeOperacaoVisible = false;
    }

    public async void IdTipoDeOperacaoAoSairAsync()
    {
        PesquisaFaturamento.TipoDeOperacao = await TipoDeOperacaoById.Get(this, _tiposDeOperacaoService, PesquisaFaturamento.TipoDeOperacao.Id);
        AoMudarPropriedade(nameof(PesquisaFaturamento.TipoDeOperacao));
    }


    public void LimparTela()
    {
        LimpaTela.Limpar(this);
    }

    public void AoSelecionarFaturamento(RowSelectEventArgs<FaturamentoModel> e)
    {
        FaturamentoSelecionado = e.Data;
    }

    public async void AoClicarPesquisarBtn(MouseEventArgs e)
    {
        await Pesquisar();
    }

    public async Task Pesquisar()
    {
        Pesquisando = true;
        Faturamentos = await GetFaturamentos();
        GetAcoesFaturamentoGrid();
        Pesquisando = false;
    }

    public async Task AoClicarSelecionarBtnGridFaturamentos(MouseEventArgs e)
    {
        
        if (VMChamador is IFaturamentoVM)
        {
            _navigationManager.NavigateTo("/Usuario/Movimentacoes/Faturamento");
        }
        else
        {
            _navigationManager.NavigateTo("/Usuario/Movimentacoes/Compra");
        }
        await Task.Delay(1000);
        await _exibicaoFaturamentoService.ExibirById(VMChamador, FaturamentoSelecionado.Id, FaturamentoSelecionado.Tipo);
    }

    public async Task AoPressionarTeclaFaturamentosGrid(KeyboardEventArgs args)
    {
        if (!(args.Key == "Enter" && FaturamentoSelecionado?.Id > 0)) return;
        
        if (VMChamador is IFaturamentoVM)
        {
            _navigationManager.NavigateTo("/Usuario/Movimentacoes/Faturamento");
        }
        else
        {
            _navigationManager.NavigateTo("/Usuario/Movimentacoes/Compra");
        }
        await Task.Delay(1000);
        await _exibicaoFaturamentoService.ExibirById(VMChamador, FaturamentoSelecionado.Id, FaturamentoSelecionado.Tipo);
    }

    public void AoMudarTipoFaturamento(ChangeEventArgs<int, ListaModel> args)
    {
        PesquisaFaturamento.Tipo = args.Value;
        Faturamentos = new List<FaturamentoModel>();
        AoMudarPropriedade(nameof(PesquisaFaturamento));
    }

    private void GetAcoesFaturamentoGrid()
    {
        AcoesFaturamentoGrid.Clear();
        var acoes = (TipoFaturamento)PesquisaFaturamento.Tipo switch
        {
            TipoFaturamento.PedidoDeVenda => _acoesFaturamentoService.ItensPedidoGrid(),
            TipoFaturamento.NotaFiscal => _acoesFaturamentoService.ItensNotaFiscalGrid(),
            TipoFaturamento.NotaCompra => _acoesFaturamentoService.ItensCompraGrid(),
            _ => new List<ListaModel>()
        };
        AcoesFaturamentoGrid.AddRange(acoes);
    }

    public async Task AoSelecionarAcaoGrid(MenuEventArgs e)
    {
        if (FaturamentoSelecionado == null) return;

        switch (e.Item.Text)
        {
            case "Duplicar":
                DuplicacaoDocumento.PedirConfirmacao(this);
                return;
            case "Informação":
                await ExibirOcorrencias();
                return;
            case "Imprimir":
                await ImpressaoDANFE.Imprimir(this, _nFeService, FaturamentoSelecionado.ChaveEletronica);
                return;
            case "Enviar e-mail":
                EnvioEMailNFe.PedirConfirmacao(this);
                return;
            case "Cancelar":
                DlgConfirmacaoCancelamentoVisible = true;
                return;
            case "Inutilizar":
                DlgConfirmacaoInutilizacaoVisible = true;
                return;
            case "Estornar":
                //Feita essa consulta para que os dados da nota venham completos, quando mandar para a NFe
                await SetarFaturamentoSelecionadoCompleto();
                DlgMotivoEstornoVisible = true;
                return;
            case "Devolver":
                await DevolverRetornarNota(TipoAcaoFaturamento.Devolver);
                break;
            case "Retornar":
                await DevolverRetornarNota(TipoAcaoFaturamento.Retornar);
                break;
            default:
                break;
        }
    }

    private async Task DevolverRetornarNota(TipoAcaoFaturamento tipoAcaoFaturamento)
    {
        await SetarFaturamentoSelecionadoCompleto();
        string acao = tipoAcaoFaturamento == TipoAcaoFaturamento.Devolver ? "devolver" : "retornar";
        if (FaturamentoSelecionado.TudoDevolvidoRetornado)
        {
            await ExibirMensagemToast($"Essa nota não possui saldo para {acao}", TipoToast.Aviso);
            return;
        }
        _dadosDevolucaoService.FaturamentoADevolver = FaturamentoSelecionado;
        _dadosDevolucaoService.TipoAcaoFaturamento = tipoAcaoFaturamento;
        DlgDevolucaoVisible = true;
    }

    private async Task SetarFaturamentoSelecionadoCompleto()
    {
        var resposta = await _faturamentosService.GetByIdAndTipoAsync(FaturamentoSelecionado.Id, (TipoFaturamento)FaturamentoSelecionado.Tipo);
        var faturamento = (FaturamentoModel)resposta.Objeto;
        FaturamentoSelecionado = faturamento;
    }

    public async Task AoClicarSimBtnDlgConfirmacaoDuplicacao(MouseEventArgs e)
    {
        DlgConfirmacaoDuplicacaoVisible = false;
        await DuplicacaoDocumento.Duplicar(this, _envioFaturamentoService, FaturamentoSelecionado);
    }

    public async Task AoClicarOKBtnDlgMotivoCancelamento(MouseEventArgs e)
    {
        DlgMotivoCancelamentoVisible = false;
        await CancelamentoDocumento.Cancelar(this, FaturamentoSelecionado, _nFeService, _envioFaturamentoService);
    }

    public void AoClicarSimBtnDlgConfirmacaoCancelamento(MouseEventArgs e)
    {
        DlgConfirmacaoCancelamentoVisible = false;
        DlgMotivoCancelamentoVisible = true;
    }

    public void AoClicarNaoBtnDlgConfirmacaoCancelamento(MouseEventArgs e)
    {
        DlgConfirmacaoCancelamentoVisible = false;
    }
    public async Task AoClicarOKBtnDlgMotivoInutilizacao(MouseEventArgs e)
    {
        DlgMotivoInutilizacaoVisible = false;
        await InutilizacaoDocumento.Inutilizar(this, FaturamentoSelecionado, _nFeService, _envioFaturamentoService);
    }
    public void AoClicarSimBtnDlgConfirmacaoInutilizacao(MouseEventArgs e)
    {
        DlgConfirmacaoInutilizacaoVisible = false;
        DlgMotivoInutilizacaoVisible = true;
    }
    public void AoClicarNaoBtnDlgConfirmacaoInutilizacao(MouseEventArgs e)
    {
        DlgConfirmacaoInutilizacaoVisible = false;
    }
    public void AoClicarNaoBtnDlgConfirmacaoDuplicacao(MouseEventArgs e)
    {
        DlgConfirmacaoDuplicacaoVisible = false;
    }

    private async Task ExibirOcorrencias()
    {
        Ocorrencias = new List<OcorrenciaFaturamentoModel>();
        
        await SpinnerPrincipal.ShowAsync();

        var retOcorrencias = await _faturamentosService.GetOcorrenciasAsync(FaturamentoSelecionado);

        if (SpinnerPrincipal != null)
        {
            await SpinnerPrincipal.HideAsync();
        }

        if (!retOcorrencias.Sucesso)
        {
            await ExibirMensagemToast($"Ocorreu um erro ao tentar pesquisar as ocorrências. Erro: {retOcorrencias.MensagemErro}", TipoToast.Erro);
            return;
        }

        Ocorrencias = JsonConvert.DeserializeObject<List<OcorrenciaFaturamentoModel>>(retOcorrencias.ConteudoRetorno);
        if (Ocorrencias.Count == 0)
        {
            await ExibirMensagemToast("Não há ocorrências para este documento.", TipoToast.Informacao);
            return;
        }
        DlgOcorrenciasVisible = true;
    }

    public void AoFecharDlgOcorrencias(object e)
    {
        DlgOcorrenciasVisible = false;
    }

    public void AoClicarOkBtnDlgOcorrencias(MouseEventArgs e)
    {
        DlgOcorrenciasVisible = false;
    }

    public async Task AoClicarSimBtnDlgConfirmacaoEnvioEMailNFe(MouseEventArgs e)
    {
        DlgConfirmacaoEnvioEMailNFeVisible = false;
        await EnvioEMailNFe.EnviarXMLEPDF(this, _nFeService, FaturamentoSelecionado.ChaveEletronica);
    }

    public void AoClicarNaoBtnDlgConfirmacaoEnvioEMailNFe(MouseEventArgs e)
    {
        DlgConfirmacaoEnvioEMailNFeVisible = false;
    }

    public async Task AoClicarOKBtnDlgMotivoEstorno(MouseEventArgs e)
    {
        DlgMotivoEstornoVisible = false;
        await EstornoNF.Estornar(this, _envioFaturamentoService, _nFeService, FaturamentoSelecionado);
    }

    private void AoEtapaCancelamentoNFe(string mensagem)
    {
        LogEnvioCancelamento += $"\n\r{mensagem}";
    }

    private void AoEtapaInutilizacaoNFe(string mensagem)
    {
        logEnvioInutilizacao += $"\n\r{mensagem}";
    }

    private void AoEtapaEnvioNFe(string mensagem)
    {
        LogEnvioFaturamento += $"\n\r{mensagem}";
        if (DlgLogEnvioEstornoVisible)
        {
            LogEnvioEstorno += $"\n\r{mensagem}";
        }
    }

    public void AoFecharDlgDevolucao(object e)
    {
        DlgDevolucaoVisible = false;
    }

    public async void AoClicarOkBtnDlgDevolucao(MouseEventArgs e)
    {
        try
        {
            await SpinnerPrincipal.ShowAsync();
            var resultado = await _dadosDevolucaoService.DevolucaoVM.PrepararFaturamentoParaDevolver();
            if (!resultado.Sucesso)
            {
                await ExibirMensagemToast((string)resultado.Retorno, TipoToast.Erro);
                return;
            }
            DlgDevolucaoVisible = false;
            _navigationManager.NavigateTo($"/Usuario/Movimentacoes/Faturamento");

        }
        finally
        {
            await SpinnerPrincipal.HideAsync();
        }
    }
}
