﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.DropDowns;
using Syncfusion.Blazor.Grids;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.PesquisaPagarReceber;

public partial class PesquisaPagarReceberVM : BaseVM, IPesquisaPagarReceberVM
{
    private readonly IPesquisasPagarReceberService _pesquisasPagarReceberService;
    private readonly ITimeZoneService _timeZoneService;
    private readonly NavigationManager _navigationManager;

    public event Action<long> OnNavegarParaPagarReceberVM;


    [ObservableProperty]
    private List<ListaModel> tiposPagarReceber;
    partial void OnTiposPagarReceberChanged(List<ListaModel> value) => AoMudarPropriedade(nameof(TiposPagarReceber));


    [ObservableProperty]
    private bool pesquisando;
    partial void OnPesquisandoChanged(bool value) => AoMudarPropriedade(nameof(Pesquisando));

    [ObservableProperty]
    private PesquisaPagarReceberModel parametros;
    partial void OnParametrosChanged(PesquisaPagarReceberModel value) => AoMudarPropriedade(nameof(Parametros));


    public List<PagarReceberModel> ContasPagarReceber { get; set; } = new List<PagarReceberModel>();
    public PagarReceberModel PagarReceberSelecionado { get; set; }
    public List<ListaModel> TiposPesquisaNome { get; }
    public bool PesquisarBtnEnabled { get; set; } = true;

    public PesquisaPagarReceberVM(IPesquisasPagarReceberService pesquisasPagarReceberService,
        ITiposPesquisaStringService tiposPesquisaStringService,
        ITimeZoneService timeZoneService,
        NavigationManager navigationManager)
    {
        _pesquisasPagarReceberService = pesquisasPagarReceberService;
        Parametros = new PesquisaPagarReceberModel();
        TiposPesquisaNome = tiposPesquisaStringService.Itens();
        _timeZoneService = timeZoneService;
        _navigationManager = navigationManager;
    }

    public void AoClicarLimparFiltrosBtnAsync(MouseEventArgs e)
    {
        LimparTela();
        AoMudarPropriedade(nameof(Parametros));
    }

    public async Task<List<PagarReceberModel>> GetContasPagarReceber()
    {
        var retorno = await _pesquisasPagarReceberService.GetAsync(Parametros);

        if (!retorno.Sucesso || !string.IsNullOrEmpty(retorno.MensagemErro))
        {
            Mensagem = retorno.MensagemErro;
            return new List<PagarReceberModel>();
        }

        var contasPagarReceber = (List<PagarReceberModel>)retorno.Objeto;
        foreach (var pagarReceber in contasPagarReceber)
        {
            pagarReceber.DtEmissao = _timeZoneService.GetLocalDateTime(pagarReceber.DtEmissao);
        }
        return contasPagarReceber;
    }

    public async void AoClicarLocalizarBtn()
    {
        Pesquisando = true;
        ContasPagarReceber = await GetContasPagarReceber();
        Pesquisando = false;
    }

    public void LimparTela()
    {
        LimpaTela.Limpar(this);
    }

    public void AoSelecionarPagarReceber(RowSelectEventArgs<PagarReceberModel> e)
    {
        PagarReceberSelecionado = e.Data;
    }

    public async void AoClicarPesquisarBtn(MouseEventArgs e)
    {
        await Pesquisar();
    }

    public async Task Pesquisar()
    {
        Pesquisando = true;
        ContasPagarReceber = await GetContasPagarReceber();
        Pesquisando = false;
    }

    public async Task AoClicarSelecionarBtnGridPagarReceber(MouseEventArgs e)
    {

        _navigationManager.NavigateTo("/Usuario/Movimentacoes/PagarReceber");

        await Task.Delay(200);
        OnNavegarParaPagarReceberVM?.Invoke(PagarReceberSelecionado.Id);

    }

    public async Task AoPressionarTeclaPagarReceberGrid(KeyboardEventArgs args)
    {
        if (!(args.Key == "Enter" && PagarReceberSelecionado?.Id > 0)) return;

        await Task.Delay(200);
        OnNavegarParaPagarReceberVM?.Invoke(PagarReceberSelecionado.Id);
    }

    public void AoMudarTipoPagarReceber(ChangeEventArgs<int, ListaModel> args)
    {
        Parametros.Tipo = args.Value;
        ContasPagarReceber = new List<PagarReceberModel>();
        AoMudarPropriedade(nameof(Parametros));
    }
}
