﻿using CommunityToolkit.Mvvm.ComponentModel;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Grids;
using Syncfusion.Blazor.Popups;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.PesquisaPessoa;

public partial class PesquisaPessoaVM : BaseVM, IPesquisaPessoaVM
{
    private readonly IPessoasService _pessoasService;

    public event Action<PessoaModel> OnSelecionarBtnClick;
    public event Action<PessoaModel> OnSelecionarPessoa;
    public event Action<KeyboardEventArgs> OnPressionarTecla;
    public event Action<string> OnNovoBtnClick;

    [ObservableProperty]
    private List<ListaModel> tiposPessoa;
    partial void OnTiposPessoaChanged(List<ListaModel> value) => AoMudarPropriedade(nameof(TiposPessoa));

    [ObservableProperty]
    private bool pesquisando;
    partial void OnPesquisandoChanged(bool value) => AoMudarPropriedade(nameof(Pesquisando));

    [ObservableProperty]
    private PesquisaPessoaModel pesquisaPessoa;
    partial void OnPesquisaPessoaChanged(PesquisaPessoaModel value) => AoMudarPropriedade(nameof(PesquisaPessoa));


    public SfDialog Dlg { get; set; }
    public List<PessoaModel> Pessoas { get; set; } = new List<PessoaModel>();
    public PessoaModel PessoaSelecionada { get; set; }
    public List<ListaModel> TiposPesquisaNome { get; }
    public bool PesquisarBtnEnabled { get; set; } = true;


    public PesquisaPessoaVM(IPessoasService pessoasService,
        NavigationManager navigationManager,
        ITiposPesquisaStringService tiposPesquisaStringService)
    {
        _pessoasService = pessoasService;
        PesquisaPessoa = new PesquisaPessoaModel();
    }

    public void AoClicarLimparFiltrosBtnAsync(MouseEventArgs e)
    {
        LimparTela();
        AoMudarPropriedade(nameof(PesquisaPessoa));
    }

    public async Task<List<PessoaModel>> GetPessoas()
    {
        var retorno = await _pessoasService.GetAsync(PesquisaPessoa);

        if (!retorno.Sucesso || !string.IsNullOrEmpty(retorno.MensagemErro))
        {
            Mensagem = retorno.MensagemErro;
            return new List<PessoaModel>();
        }

        var pessoas = (List<PessoaModel>)retorno.Objeto;
        return pessoas;
    }

    public void LimparTela()
    {
        LimpaTelaPesquisaPessoa.Limpar(this);
    }

    public void AoSelecionarPessoa(RowSelectEventArgs<PessoaModel> e)
    {
        PessoaSelecionada = e.Data;
        OnSelecionarPessoa?.Invoke(e.Data);
    }

    public async void AoClicarPesquisarBtn(MouseEventArgs e)
    {
        await Pesquisar();
    }

    public async Task Pesquisar()
    {
        Pesquisando = true;
        Pessoas = await GetPessoas();
        Pesquisando = false;
    }


    public void AoClicarSelecionarBtnGridPessoas(CommandClickEventArgs<PessoaModel> e)
    {
        OnSelecionarBtnClick?.Invoke(e.RowData);
    }

    public void AoPressionarTeclaPessoasGrid(KeyboardEventArgs args)
    {
        if (!(args.Key == "Enter" && PessoaSelecionada?.Id > 0)) return;
        OnPressionarTecla?.Invoke(args);
    }

    public void AoClicarNovoBtn(MouseEventArgs e)
    {
        OnNovoBtnClick?.Invoke("");
    }
}
