﻿using GAFNETBlazor.Constantes;
using GAFNETBlazor.Models;
using GAFNETBlazor.Services;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario.Relatorios;
using GAFNETLibrary.Enums;
using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Web;
using Syncfusion.Blazor.Popups;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace GAFNETBlazor.ViewModels.Usuario.Relatorios.Pessoas;

public class ParamRelatPessoasVM : BaseVM, IParamRelatPessoasVM
{
    private bool _executando;
    private List<ListaModel> _listaStatusAtivo;
    private List<ListaModel> _nomesRelatorios;
    private readonly INomesRelatoriosService _nomesRelatoriosService;
    private readonly IStatusAtivoService _statusAtivoService;
    private readonly IParametrosRelatorioService _parametrosRelatorioService;
    private readonly ITipoOrdenacaoService _tipoOrdenacaoService;
    private readonly NavigationManager _navigationManager;
    private ParamRelatPessoaModel _parametros;
    private bool _relatorioBtnEnabled;

    public bool RelatorioBtnEnabled
    {
        get { return _relatorioBtnEnabled; }
        set
        {
            _relatorioBtnEnabled = value;
            OnPropertyChanged(nameof(RelatorioBtnEnabled));
        }
    }

    public List<ListaModel> ListaStatusAtivo
    {
        get { return _listaStatusAtivo; }
        set
        {
            _listaStatusAtivo = value;
            OnPropertyChanged(nameof(ListaStatusAtivo));
        }
    }

    public List<ListaModel> ListaNomesRelatorios
    {
        get { return _nomesRelatorios; }
        set
        {
            _nomesRelatorios = value;
            OnPropertyChanged(nameof(ListaNomesRelatorios));
        }
    }

    public bool Executando
    {
        get { return _executando; }
        set
        {
            _executando = value;
            OnPropertyChanged(nameof(Executando));
        }
    }

    public ParamRelatPessoaModel Parametros
    {
        get { return _parametros; }
        set
        {
            _parametros = value;
            OnPropertyChanged(nameof(Parametros));
        }
    }

    public SfDialog Dlg { get; set; }
    public List<PessoaModel> Pessoas { get; set; } = new List<PessoaModel>();
    public string NomeRelatorio { get; set; }
    public OrdenacaoModel Ordenacao { get; set; } = new();
    public List<ListaModel> ListaTipoOrdenacao { get; }
    public List<ListaModel> ListaCamposOrdenacao => new()
    {
        new()
        {
            Descricao = "Nome",
            StrValor = "Nome"
        },
        new()
        {
            Descricao = "Cidade",
            StrValor = "EnderecoPrincipal/Cidade/Nome"
        },
        new()
        {
            Descricao = "UF",
            StrValor = "EnderecoPrincipal/Cidade/UF"
        }
    };

    public List<string> ListaUFs => UFsService.UFs();


    public ParamRelatPessoasVM(INomesRelatoriosService nomesRelatoriosService,
        IStatusAtivoService statusAtivoService,
        IParametrosRelatorioService parametrosRelatorioService,
        ITipoOrdenacaoService tipoOrdenacaoService,
        NavigationManager navigationManager)
    {
        _nomesRelatoriosService = nomesRelatoriosService;
        _statusAtivoService = statusAtivoService;
        _parametrosRelatorioService = parametrosRelatorioService;
        _navigationManager = navigationManager;
        _relatorioBtnEnabled = true;
        _tipoOrdenacaoService = tipoOrdenacaoService;
        Parametros = new();
        ListaStatusAtivo = _statusAtivoService.Itens();
        _nomesRelatorios = _nomesRelatoriosService.Itens(EntidadeRelatorio.Pessoa);
        ListaTipoOrdenacao = _tipoOrdenacaoService.Itens();
    }

    public void AoClicarResetarParametrosBtn(MouseEventArgs e)
    {
        ResetarParametros();
        OnPropertyChanged(nameof(Parametros));
    }

    public void ResetarParametros()
    {
        Parametros.ResetarParametros();
    }

    public async void AoClicarRelatorioBtn(MouseEventArgs e)
    {
        Executando = true;
        await Executar();
        Executando = false;
    }

    public async Task Executar()
    {
        if (string.IsNullOrEmpty(NomeRelatorio))
        {
            await ExibirMensagemToast("Não foi selecionado um relatório", TipoToast.Informacao);
            return;
        }

        if (NomeRelatorio == "listagem-clientes-sintetica.rdlc")
        {
            _parametrosRelatorioService.Campos = "id,nome,cnpj";
        }

        //_parametrosRelatorioService.Campos = "id,nome,cnpj,&$expand=enderecoprincipal($expand=cidade,pais)";
        _parametrosRelatorioService.Filtros = GetFiltros();
        _parametrosRelatorioService.NomeRelatorio = NomeRelatorio;
        _parametrosRelatorioService.Ordenacao = Parametros.GetStrOrdenacao();
        _parametrosRelatorioService.QtdRegistros = Parametros.QtdRegistros;
        _parametrosRelatorioService.UriBase = URIsPessoas.GetAsync;
        _parametrosRelatorioService.ServicoDados = ServicoDados.PessoasService;

        _navigationManager.NavigateTo("/ReportViewer");
    }

    public void AoClicarInserirOrdenacaoBtn(MouseEventArgs e)
    {
        if (string.IsNullOrEmpty(Ordenacao.Campo)) return;

        if (Parametros.Ordenacao.Any(o => o.StrValor == Ordenacao.Campo))
        {
            Parametros.Ordenacao.Remove(Parametros.Ordenacao.FirstOrDefault(o => o.StrValor == Ordenacao.Campo));
        }


        string descricao = ListaCamposOrdenacao.FirstOrDefault(c => c.StrValor == Ordenacao.Campo).Descricao;
        descricao += Ordenacao.TipoOrdenacao == (int)TipoOrdenacao.Descendente ? " -> Desc" : "";
        string strValor = Ordenacao.Campo;
        strValor += Ordenacao.TipoOrdenacao == (int)TipoOrdenacao.Descendente ? " desc" : "";

        Parametros.Ordenacao.Add(new ListaModel { Descricao = descricao, StrValor = strValor });
        OnPropertyChanged("Parametros.Ordenacao");
    }

    public void AoClicarLimparOrdenacaoBtn(MouseEventArgs e)
    {
        Parametros.Ordenacao.Clear();
        OnPropertyChanged(nameof(Parametros));
    }

    private string GetFiltros()
    {
        string filtro = "";
        filtro += !string.IsNullOrEmpty(Parametros.NomeInicial) ? $"nome ge '{Parametros.NomeInicial}'" : "";
        if (!string.IsNullOrEmpty(Parametros.NomeFinal))
        {
            filtro += !string.IsNullOrEmpty(filtro) ? " and " : "";
            filtro += $"nome le '{Parametros.NomeFinal}'";
        }

        string filtroTipoPessoa = "";
        if (Parametros.Clientes)
        {
            filtroTipoPessoa += "ehcliente eq true";
        }

        if (Parametros.Fornecedores)
        {
            filtroTipoPessoa += !string.IsNullOrEmpty(filtroTipoPessoa) ? " or " : "";
            filtroTipoPessoa += "ehfornecedor eq true";
        }

        if (Parametros.Transportadoras)
        {
            filtroTipoPessoa += !string.IsNullOrEmpty(filtroTipoPessoa) ? " or " : "";
            filtroTipoPessoa += "ehtransportadora eq true";
        }

        if (Parametros.Vendedores)
        {
            filtroTipoPessoa += !string.IsNullOrEmpty(filtroTipoPessoa) ? " or " : "";
            filtroTipoPessoa += "ehvendedor eq true";
        }

        if (!string.IsNullOrEmpty(filtroTipoPessoa))
        {
            filtro += !string.IsNullOrEmpty(filtro) ? " and " : "";
            filtro += $"({filtroTipoPessoa})";
        }


        if (Parametros.StatusAtivo != (int)StatusAtivo.Todos)
        {
            filtro += !string.IsNullOrEmpty(filtro) ? " and " : "";
            filtro += Parametros.StatusAtivo == (int)StatusAtivo.Ativos ? "ativo eq true" : "ativo eq false";
        }

        if (!string.IsNullOrEmpty(Parametros.NomeCidade))
        {
            filtro += !string.IsNullOrEmpty(filtro) ? " and " : "";
            filtro += $"enderecos/any(e: e/ehprincipal eq true and startswith(e/cidade/nome,'{ Parametros.NomeCidade}'))";
        }

        if (!string.IsNullOrEmpty(Parametros.UF))
        {
            filtro += !string.IsNullOrEmpty(filtro) ? " and " : "";
            filtro += $"enderecos/any(e: e/ehprincipal eq true and e/cidade/uf eq '{ Parametros.UF}')";
        }

        return filtro;
    }

}
