﻿using FluentValidation;
using GAFNETBlazor.Models;
using GAFNETBlazor.Models.Validations;
using GAFNETBlazor.Services.Interfaces;
using GAFNETBlazor.Stores.Interfaces;
using GAFNETBlazor.ViewModels.Interfaces.Usuario;
using GAFNETLibrary.Models;
using Syncfusion.Blazor.Inputs;
using Syncfusion.Blazor.Popups;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net.Http;
using System.Security.Claims;

namespace GAFNETBlazor.ViewModels.Usuario
{
    public class TrocaSenhaUsuarioVM : ITrocaSenhaUsuarioVM
    {
        private readonly IUsuariosService _usuariosService;
        private readonly ILoginStore _loginStore;
        private bool _dlgVisible = false;
        private string _mensagemResultadoEnvio = string.Empty;
        public string MensagemErro { get; set; }
        public bool Enviando { get; set; }
        public List<string> ErrosValidacaoFormulario { get; } = new List<string>();
        public UsuarioModel UsuarioModel { get; set; }
        public SfTextBox SenhaTextBox { get; set; }
        public bool DlgVisible
        {
            get { return _dlgVisible; }
            set
            {
                _dlgVisible = value;
                OnPropertyChanged(nameof(DlgVisible));
            }
        }

        public string MensagemResultadoEnvio
        {
            get { return _mensagemResultadoEnvio; }
            set
            {
                _mensagemResultadoEnvio = value;
                OnPropertyChanged(nameof(MensagemResultadoEnvio));
            }
        }

        public SfDialog Dialogo { get; set; }

        public TrocaSenhaUsuarioVM(IUsuariosService usuariosService, ILoginStore loginStore)
        {
            _usuariosService = usuariosService;
            _loginStore = loginStore;

            UsuarioModel = new UsuarioModel

            {
                Id = _loginStore.Claims.FirstOrDefault(c => c.Type == "IdUsuario").ToString(),
                Nome = _loginStore.Claims.FirstOrDefault(c => c.Type == ClaimTypes.Name).Value,
                Email = _loginStore.Claims.FirstOrDefault(c => c.Type == ClaimTypes.Email).Value
            };
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public async void Enviar()
        {
            if (!DadosValidos())
            {
                return;
            }

            Enviando = true;

            string mensagemSucesso = "Senha alterada com sucesso";

            HttpResponseMessage retorno;
            AlteraSenhaUsuarioModel model = new()
            {
                EMailUsuario = UsuarioModel.Email,
                NovaSenha = UsuarioModel.PasswordHash
            };
            retorno = await _usuariosService.AlterarSenhaUsuario(model);

            if (!retorno.IsSuccessStatusCode)
            {
                MensagemErro = await retorno.Content.ReadAsStringAsync();
                MensagemResultadoEnvio = MensagemErro;
            }
            else
            {
                MensagemErro = string.Empty;
                MensagemResultadoEnvio = mensagemSucesso;
            }
            Enviando = false;
            DlgVisible = true;
        }

        public void AoCriarSenhaTextBox(object e)
        {
            SenhaTextBox.FocusAsync();
        }
        public bool DadosValidos()
        {
            var validator = new AlteraSenhaUsuarioModelValidator();
            ErrosValidacaoFormulario.Clear();
            List<string> erros = new();

            var results = validator.Validate(UsuarioModel);
            if (!results.IsValid)
            {
                foreach (var erro in results.Errors)
                {
                    erros.Add(erro.ErrorMessage);
                }
                ErrosValidacaoFormulario.AddRange(erros);
                return false;
            }
            return true;
        }

        public void AoFecharDlg(object e)
        {
            DlgVisible = false;
            SenhaTextBox.FocusAsync();
        }

        public void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
